@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,10 +17,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -347,6 +347,10 @@ typedef struct FT_OutlineGlyphRec_* FT_OutlineGlyph;
 
 int cc_ftglue_available(void);
 
+/* FIXME: some of the typedef'ed function signatures below does not
+   match the correct function def, as given in
+   /usr/include/freetype2/freetype/freetype.h. 20050711 mortene. */
+
 FT_Error cc_ftglue_FT_Init_FreeType(FT_Library * library);
 void cc_ftglue_FT_Library_Version(void * library, int * major, int * minor, int * patch);
 void cc_ftglue_FT_Done_FreeType(void * library);
