@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,10 +17,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -64,8 +64,11 @@ typedef struct cc_glglue cc_glglue;
   doesn't crash with those.)
 */
 COIN_DLL_API const cc_glglue * cc_glglue_instance(int contextid);
-COIN_DLL_API const cc_glglue * cc_glglue_instance_from_context_ptr(void * ctx);
 
+/*
+  Don't use this function. It's not safe.
+*/
+const cc_glglue * cc_glglue_instance_from_context_ptr(void * ptr);
 
 /*** General interface. ***********************************************/
 
@@ -227,71 +230,76 @@ COIN_DLL_API SbBool cc_glglue_has_2d_proxy_textures(const cc_glglue * glue);
 
 COIN_DLL_API SbBool cc_glglue_has_texture_edge_clamp(const cc_glglue * glue);
 
+#ifdef COIN_NEXT_MINOR
+COIN_DLL_API void cc_glglue_glPushClientAttrib(const cc_glglue * glue, GLbitfield mask);
+COIN_DLL_API void cc_glglue_glPopClientAttrib(const cc_glglue * glue);
+#endif /* COIN_NEXT_MINOR */
+
 
 /* Texture compression ***/
 
 COIN_DLL_API SbBool cc_glue_has_texture_compression(const cc_glglue * glue);
 
 COIN_DLL_API void cc_glglue_glCompressedTexImage3D(const cc_glglue * glue,
-                                                   GLenum target, 
-                                                   GLint level, 
-                                                   GLenum internalformat, 
-                                                   GLsizei width, 
-                                                   GLsizei height, 
-                                                   GLsizei depth, 
-                                                   GLint border, 
-                                                   GLsizei imageSize, 
+                                                   GLenum target,
+                                                   GLint level,
+                                                   GLenum internalformat,
+                                                   GLsizei width,
+                                                   GLsizei height,
+                                                   GLsizei depth,
+                                                   GLint border,
+                                                   GLsizei imageSize,
                                                    const GLvoid * data);
 COIN_DLL_API void cc_glglue_glCompressedTexImage2D(const cc_glglue * glue,
-                                                   GLenum target, 
-                                                   GLint level, 
-                                                   GLenum internalformat, 
-                                                   GLsizei width, 
-                                                   GLsizei height, 
-                                                   GLint border, 
-                                                   GLsizei imageSize, 
+                                                   GLenum target,
+                                                   GLint level,
+                                                   GLenum internalformat,
+                                                   GLsizei width,
+                                                   GLsizei height,
+                                                   GLint border,
+                                                   GLsizei imageSize,
                                                    const GLvoid *data);
 COIN_DLL_API void cc_glglue_glCompressedTexImage1D(const cc_glglue * glue,
-                                                   GLenum target, 
-                                                   GLint level, 
-                                                   GLenum internalformat, 
-                                                   GLsizei width, 
-                                                   GLint border, 
-                                                   GLsizei imageSize, 
+                                                   GLenum target,
+                                                   GLint level,
+                                                   GLenum internalformat,
+                                                   GLsizei width,
+                                                   GLint border,
+                                                   GLsizei imageSize,
                                                    const GLvoid *data);
 COIN_DLL_API void cc_glglue_glCompressedTexSubImage3D(const cc_glglue * glue,
-                                                      GLenum target, 
-                                                      GLint level, 
-                                                      GLint xoffset, 
-                                                      GLint yoffset, 
-                                                      GLint zoffset, 
-                                                      GLsizei width, 
-                                                      GLsizei height, 
-                                                      GLsizei depth, 
-                                                      GLenum format, 
-                                                      GLsizei imageSize, 
+                                                      GLenum target,
+                                                      GLint level,
+                                                      GLint xoffset,
+                                                      GLint yoffset,
+                                                      GLint zoffset,
+                                                      GLsizei width,
+                                                      GLsizei height,
+                                                      GLsizei depth,
+                                                      GLenum format,
+                                                      GLsizei imageSize,
                                                       const GLvoid *data);
 COIN_DLL_API void cc_glglue_glCompressedTexSubImage2D(const cc_glglue * glue,
-                                                      GLenum target, 
-                                                      GLint level, 
-                                                      GLint xoffset, 
-                                                      GLint yoffset, 
-                                                      GLsizei width, 
-                                                      GLsizei height, 
-                                                      GLenum format, 
-                                                      GLsizei imageSize, 
+                                                      GLenum target,
+                                                      GLint level,
+                                                      GLint xoffset,
+                                                      GLint yoffset,
+                                                      GLsizei width,
+                                                      GLsizei height,
+                                                      GLenum format,
+                                                      GLsizei imageSize,
                                                       const GLvoid *data);
 COIN_DLL_API void cc_glglue_glCompressedTexSubImage1D(const cc_glglue * glue,
-                                                      GLenum target, 
-                                                      GLint level, 
-                                                      GLint xoffset, 
-                                                      GLsizei width, 
-                                                      GLenum format, 
-                                                      GLsizei imageSize, 
+                                                      GLenum target,
+                                                      GLint level,
+                                                      GLint xoffset,
+                                                      GLsizei width,
+                                                      GLenum format,
+                                                      GLsizei imageSize,
                                                       const GLvoid *data);
 COIN_DLL_API void cc_glglue_glGetCompressedTexImage(const cc_glglue * glue,
-                                                    GLenum target, 
-                                                    GLint level, 
+                                                    GLenum target,
+                                                    GLint level,
                                                     void *img);
 
 
@@ -304,11 +312,11 @@ COIN_DLL_API SbBool cc_glglue_has_color_subtables(const cc_glglue * glue);
 COIN_DLL_API SbBool cc_glglue_has_paletted_textures(const cc_glglue * glue);
 
 COIN_DLL_API void cc_glglue_glColorTable(const cc_glglue * glue,
-                                         GLenum target, 
-                                         GLenum internalFormat, 
-                                         GLsizei width, 
-                                         GLenum format, 
-                                         GLenum type, 
+                                         GLenum target,
+                                         GLenum internalFormat,
+                                         GLsizei width,
+                                         GLenum format,
+                                         GLenum type,
                                          const GLvoid *table);
 COIN_DLL_API void cc_glglue_glColorSubTable(const cc_glglue * glue,
                                             GLenum target,
@@ -318,17 +326,17 @@ COIN_DLL_API void cc_glglue_glColorSubTable(const cc_glglue * glue,
                                             GLenum type,
                                             const GLvoid * data);
 COIN_DLL_API void cc_glglue_glGetColorTable(const cc_glglue * glue,
-                                            GLenum target, 
-                                            GLenum format, 
-                                            GLenum type, 
+                                            GLenum target,
+                                            GLenum format,
+                                            GLenum type,
                                             GLvoid *data);
 COIN_DLL_API void cc_glglue_glGetColorTableParameteriv(const cc_glglue * glue,
-                                                       GLenum target, 
-                                                       GLenum pname, 
+                                                       GLenum target,
+                                                       GLenum pname,
                                                        GLint *params);
 COIN_DLL_API void cc_glglue_glGetColorTableParameterfv(const cc_glglue * glue,
-                                                       GLenum target, 
-                                                       GLenum pname, 
+                                                       GLenum target,
+                                                       GLenum pname,
                                                        GLfloat *params);
 
 
@@ -352,13 +360,13 @@ COIN_DLL_API void cc_glglue_glIndexPointer (const cc_glglue * glue,
                                             GLenum type, GLsizei stride, const GLvoid * pointer);
 COIN_DLL_API void cc_glglue_glEnableClientState(const cc_glglue * glue, GLenum array);
 COIN_DLL_API void cc_glglue_glDisableClientState(const cc_glglue * glue, GLenum array);
-COIN_DLL_API void cc_glglue_glInterleavedArrays(const cc_glglue * glue, 
+COIN_DLL_API void cc_glglue_glInterleavedArrays(const cc_glglue * glue,
                                                 GLenum format, GLsizei stride, const GLvoid * pointer);
-COIN_DLL_API void cc_glglue_glDrawArrays(const cc_glglue * glue, 
+COIN_DLL_API void cc_glglue_glDrawArrays(const cc_glglue * glue,
                                          GLenum mode, GLint first, GLsizei count);
-COIN_DLL_API void cc_glglue_glDrawElements(const cc_glglue * glue, 
+COIN_DLL_API void cc_glglue_glDrawElements(const cc_glglue * glue,
                                            GLenum mode, GLsizei count, GLenum type, const GLvoid * indices);
-COIN_DLL_API void cc_glglue_glDrawRangeElements(const cc_glglue * glue, 
+COIN_DLL_API void cc_glglue_glDrawRangeElements(const cc_glglue * glue,
                                                 GLenum mode, GLuint start, GLuint end, GLsizei count, GLenum type, const GLvoid * indices);
 COIN_DLL_API void cc_glglue_glArrayElement(const cc_glglue * glue, GLint i);
 
@@ -366,9 +374,9 @@ COIN_DLL_API SbBool cc_glglue_has_multitexture_vertex_array(const cc_glglue * gl
 COIN_DLL_API int cc_glglue_max_texture_units(const cc_glglue * glue);
 COIN_DLL_API int cc_glglue_has_multidraw_vertex_arrays(const cc_glglue * glue);
 
-COIN_DLL_API void cc_glglue_glMultiDrawArrays(const cc_glglue * glue, GLenum mode, const GLint * first, 
+COIN_DLL_API void cc_glglue_glMultiDrawArrays(const cc_glglue * glue, GLenum mode, const GLint * first,
                                               const GLsizei * count, GLsizei primcount);
-COIN_DLL_API void cc_glglue_glMultiDrawElements(const cc_glglue * glue, GLenum mode, const GLsizei * count, 
+COIN_DLL_API void cc_glglue_glMultiDrawElements(const cc_glglue * glue, GLenum mode, const GLsizei * count,
                                                 GLenum type, const GLvoid ** indices, GLsizei primcount);
 
 /* NV_vertex_array_range */
@@ -387,75 +395,75 @@ COIN_DLL_API void cc_glglue_glDeleteBuffers(const cc_glglue * glue, GLsizei n, c
 COIN_DLL_API void cc_glglue_glGenBuffers(const cc_glglue * glue, GLsizei n, GLuint *buffers);
 COIN_DLL_API GLboolean cc_glglue_glIsBuffer(const cc_glglue * glue, GLuint buffer);
 COIN_DLL_API void cc_glglue_glBufferData(const cc_glglue * glue,
-                                         GLenum target, 
-                                         intptr_t size, /* 64 bit on 64 bit systems */ 
-                                         const GLvoid *data, 
+                                         GLenum target,
+                                         intptr_t size, /* 64 bit on 64 bit systems */
+                                         const GLvoid *data,
                                          GLenum usage);
 COIN_DLL_API void cc_glglue_glBufferSubData(const cc_glglue * glue,
-                                            GLenum target, 
-                                            intptr_t offset, /* 64 bit */ 
-                                            intptr_t size, /* 64 bit */ 
+                                            GLenum target,
+                                            intptr_t offset, /* 64 bit */
+                                            intptr_t size, /* 64 bit */
                                             const GLvoid * data);
 COIN_DLL_API void cc_glglue_glGetBufferSubData(const cc_glglue * glue,
-                                               GLenum target, 
-                                               intptr_t offset, /* 64 bit */ 
-                                               intptr_t size, /* 64 bit */ 
+                                               GLenum target,
+                                               intptr_t offset, /* 64 bit */
+                                               intptr_t size, /* 64 bit */
                                                GLvoid *data);
 COIN_DLL_API GLvoid * cc_glglue_glMapBuffer(const cc_glglue * glue,
                                             GLenum target, GLenum access);
 COIN_DLL_API GLboolean cc_glglue_glUnmapBuffer(const cc_glglue * glue,
                                                GLenum target);
 COIN_DLL_API void cc_glglue_glGetBufferParameteriv(const cc_glglue * glue,
-                                                   GLenum target, 
-                                                   GLenum pname, 
+                                                   GLenum target,
+                                                   GLenum pname,
                                                    GLint * params);
 COIN_DLL_API void cc_glglue_glGetBufferPointerv(const cc_glglue * glue,
-                                                GLenum target, 
-                                                GLenum pname, 
+                                                GLenum target,
+                                                GLenum pname,
                                                 GLvoid ** params);
 
 /* GL_ARB_fragment_program */
 COIN_DLL_API SbBool cc_glglue_has_arb_fragment_program(const cc_glglue * glue);
-COIN_DLL_API void cc_glglue_glProgramString(const cc_glglue * glue, GLenum target, GLenum format, 
+COIN_DLL_API void cc_glglue_glProgramString(const cc_glglue * glue, GLenum target, GLenum format,
                                             GLsizei len, const GLvoid *string);
-COIN_DLL_API void cc_glglue_glBindProgram(const cc_glglue * glue, GLenum target, 
+COIN_DLL_API void cc_glglue_glBindProgram(const cc_glglue * glue, GLenum target,
                                           GLuint program);
-COIN_DLL_API void cc_glglue_glDeletePrograms(const cc_glglue * glue, GLsizei n, 
+COIN_DLL_API void cc_glglue_glDeletePrograms(const cc_glglue * glue, GLsizei n,
                                              const GLuint *programs);
 COIN_DLL_API void cc_glglue_glGenPrograms(const cc_glglue * glue, GLsizei n, GLuint *programs);
 COIN_DLL_API void cc_glglue_glProgramEnvParameter4d(const cc_glglue * glue, GLenum target,
-                                                    GLuint index, GLdouble x, GLdouble y, 
+                                                    GLuint index, GLdouble x, GLdouble y,
                                                     GLdouble z, GLdouble w);
 COIN_DLL_API void cc_glglue_glProgramEnvParameter4dv(const cc_glglue * glue, GLenum target,
                                                      GLuint index, const GLdouble *params);
-COIN_DLL_API void cc_glglue_glProgramEnvParameter4f(const cc_glglue * glue, GLenum target, 
-                                                    GLuint index, GLfloat x, 
-                                                    GLfloat y, GLfloat z, 
+COIN_DLL_API void cc_glglue_glProgramEnvParameter4f(const cc_glglue * glue, GLenum target,
+                                                    GLuint index, GLfloat x,
+                                                    GLfloat y, GLfloat z,
                                                     GLfloat w);
-COIN_DLL_API void cc_glglue_glProgramEnvParameter4fv(const cc_glglue * glue, GLenum target, 
+COIN_DLL_API void cc_glglue_glProgramEnvParameter4fv(const cc_glglue * glue, GLenum target,
                                                      GLuint index, const GLfloat *params);
-COIN_DLL_API void cc_glglue_glProgramLocalParameter4d(const cc_glglue * glue, GLenum target, 
-                                                      GLuint index, GLdouble x, 
-                                                      GLdouble y, GLdouble z, 
+COIN_DLL_API void cc_glglue_glProgramLocalParameter4d(const cc_glglue * glue, GLenum target,
+                                                      GLuint index, GLdouble x,
+                                                      GLdouble y, GLdouble z,
                                                       GLdouble w);
-COIN_DLL_API void cc_glglue_glProgramLocalParameter4dv(const cc_glglue * glue, GLenum target, 
+COIN_DLL_API void cc_glglue_glProgramLocalParameter4dv(const cc_glglue * glue, GLenum target,
                                                        GLuint index, const GLdouble *params);
-COIN_DLL_API void cc_glglue_glProgramLocalParameter4f(const cc_glglue * glue, GLenum target, 
-                                                      GLuint index, GLfloat x, GLfloat y, 
+COIN_DLL_API void cc_glglue_glProgramLocalParameter4f(const cc_glglue * glue, GLenum target,
+                                                      GLuint index, GLfloat x, GLfloat y,
                                                       GLfloat z, GLfloat w);
-COIN_DLL_API void cc_glglue_glProgramLocalParameter4fv(const cc_glglue * glue, GLenum target, 
+COIN_DLL_API void cc_glglue_glProgramLocalParameter4fv(const cc_glglue * glue, GLenum target,
                                                        GLuint index, const GLfloat *params);
-COIN_DLL_API void cc_glglue_glGetProgramEnvParameterdv(const cc_glglue * glue, GLenum target, 
+COIN_DLL_API void cc_glglue_glGetProgramEnvParameterdv(const cc_glglue * glue, GLenum target,
                                                        GLuint index, GLdouble *params);
-COIN_DLL_API void cc_glglue_glGetProgramEnvParameterfv(const cc_glglue * glue, GLenum target, 
+COIN_DLL_API void cc_glglue_glGetProgramEnvParameterfv(const cc_glglue * glue, GLenum target,
                                                        GLuint index, GLfloat *params);
-COIN_DLL_API void cc_glglue_glGetProgramLocalParameterdv(const cc_glglue * glue, GLenum target, 
+COIN_DLL_API void cc_glglue_glGetProgramLocalParameterdv(const cc_glglue * glue, GLenum target,
                                                          GLuint index, GLdouble *params);
-COIN_DLL_API void cc_glglue_glGetProgramLocalParameterfv(const cc_glglue * glue, GLenum target, 
+COIN_DLL_API void cc_glglue_glGetProgramLocalParameterfv(const cc_glglue * glue, GLenum target,
                                                          GLuint index, GLfloat *params);
-COIN_DLL_API void cc_glglue_glGetProgramiv(const cc_glglue * glue, GLenum target, 
+COIN_DLL_API void cc_glglue_glGetProgramiv(const cc_glglue * glue, GLenum target,
                                            GLenum pname, GLint *params);
-COIN_DLL_API void cc_glglue_glGetProgramString(const cc_glglue * glue, GLenum target, 
+COIN_DLL_API void cc_glglue_glGetProgramString(const cc_glglue * glue, GLenum target,
                                                GLenum pname, GLvoid *string);
 COIN_DLL_API SbBool cc_glglue_glIsProgram(const cc_glglue * glue, GLuint program);
 
@@ -467,19 +475,19 @@ COIN_DLL_API void cc_glglue_glVertexAttrib1d(const cc_glglue * glue, GLuint inde
 COIN_DLL_API void cc_glglue_glVertexAttrib2s(const cc_glglue * glue, GLuint index, GLshort x, GLshort y);
 COIN_DLL_API void cc_glglue_glVertexAttrib2f(const cc_glglue * glue, GLuint index, GLfloat x, GLfloat y);
 COIN_DLL_API void cc_glglue_glVertexAttrib2d(const cc_glglue * glue, GLuint index, GLdouble x, GLdouble y);
-COIN_DLL_API void cc_glglue_glVertexAttrib3s(const cc_glglue * glue, GLuint index, 
+COIN_DLL_API void cc_glglue_glVertexAttrib3s(const cc_glglue * glue, GLuint index,
                                              GLshort x, GLshort y, GLshort z);
-COIN_DLL_API void cc_glglue_glVertexAttrib3f(const cc_glglue * glue, GLuint index, 
+COIN_DLL_API void cc_glglue_glVertexAttrib3f(const cc_glglue * glue, GLuint index,
                                              GLfloat x, GLfloat y, GLfloat z);
-COIN_DLL_API void cc_glglue_glVertexAttrib3d(const cc_glglue * glue, GLuint index, 
+COIN_DLL_API void cc_glglue_glVertexAttrib3d(const cc_glglue * glue, GLuint index,
                                              GLdouble x, GLdouble y, GLdouble z);
-COIN_DLL_API void cc_glglue_glVertexAttrib4s(const cc_glglue * glue, GLuint index, GLshort x, 
+COIN_DLL_API void cc_glglue_glVertexAttrib4s(const cc_glglue * glue, GLuint index, GLshort x,
                                              GLshort y, GLshort z, GLshort w);
-COIN_DLL_API void cc_glglue_glVertexAttrib4f(const cc_glglue * glue, GLuint index, GLfloat x, 
+COIN_DLL_API void cc_glglue_glVertexAttrib4f(const cc_glglue * glue, GLuint index, GLfloat x,
                                              GLfloat y, GLfloat z, GLfloat w);
-COIN_DLL_API void cc_glglue_glVertexAttrib4d(const cc_glglue * glue, GLuint index, GLdouble x, 
+COIN_DLL_API void cc_glglue_glVertexAttrib4d(const cc_glglue * glue, GLuint index, GLdouble x,
                                              GLdouble y, GLdouble z, GLdouble w);
-COIN_DLL_API void cc_glglue_glVertexAttrib4Nub(const cc_glglue * glue, GLuint index, GLubyte x, 
+COIN_DLL_API void cc_glglue_glVertexAttrib4Nub(const cc_glglue * glue, GLuint index, GLubyte x,
                                                GLubyte y, GLubyte z, GLubyte w);
 COIN_DLL_API void cc_glglue_glVertexAttrib1sv(const cc_glglue * glue, GLuint index, const GLshort *v);
 COIN_DLL_API void cc_glglue_glVertexAttrib1fv(const cc_glglue * glue, GLuint index, const GLfloat *v);
@@ -504,33 +512,63 @@ COIN_DLL_API void cc_glglue_glVertexAttrib4Niv(const cc_glglue * glue, GLuint in
 COIN_DLL_API void cc_glglue_glVertexAttrib4Nubv(const cc_glglue * glue, GLuint index, const GLubyte *v);
 COIN_DLL_API void cc_glglue_glVertexAttrib4Nusv(const cc_glglue * glue, GLuint index, const GLushort *v);
 COIN_DLL_API void cc_glglue_glVertexAttrib4Nuiv(const cc_glglue * glue, GLuint index, const GLuint *v);
-COIN_DLL_API void cc_glglue_glVertexAttribPointer(const cc_glglue * glue, GLuint index, GLint size, 
-                                                  GLenum type, GLboolean normalized, GLsizei stride, 
+COIN_DLL_API void cc_glglue_glVertexAttribPointer(const cc_glglue * glue, GLuint index, GLint size,
+                                                  GLenum type, GLboolean normalized, GLsizei stride,
                                                   const GLvoid *pointer);
 COIN_DLL_API void cc_glglue_glEnableVertexAttribArray(const cc_glglue * glue, GLuint index);
 COIN_DLL_API void cc_glglue_glDisableVertexAttribArray(const cc_glglue * glue, GLuint index);
-COIN_DLL_API void cc_glglue_glGetVertexAttribdv(const cc_glglue * glue, GLuint index, GLenum pname, 
+COIN_DLL_API void cc_glglue_glGetVertexAttribdv(const cc_glglue * glue, GLuint index, GLenum pname,
                                                 GLdouble *params);
-COIN_DLL_API void cc_glglue_glGetVertexAttribfv(const cc_glglue * glue, GLuint index, GLenum pname, 
+COIN_DLL_API void cc_glglue_glGetVertexAttribfv(const cc_glglue * glue, GLuint index, GLenum pname,
                                                 GLfloat *params);
-COIN_DLL_API void cc_glglue_glGetVertexAttribiv(const cc_glglue * glue, GLuint index, GLenum pname, 
+COIN_DLL_API void cc_glglue_glGetVertexAttribiv(const cc_glglue * glue, GLuint index, GLenum pname,
                                                 GLint *params);
-COIN_DLL_API void cc_glglue_glGetVertexAttribPointerv(const cc_glglue * glue, GLuint index, GLenum pname, 
+COIN_DLL_API void cc_glglue_glGetVertexAttribPointerv(const cc_glglue * glue, GLuint index, GLenum pname,
                                                       GLvoid **pointer);
 
+/* ARB_occlusion_query */
+#ifdef COIN_NEXT_MINOR
+COIN_DLL_API SbBool cc_glglue_has_occlusion_query(const cc_glglue * glue);
+COIN_DLL_API void cc_glglue_glGenQueries(const cc_glglue * glue, 
+                                         GLsizei n, GLuint * ids);
+COIN_DLL_API void cc_glglue_glDeleteQueries(const cc_glglue * glue, 
+                                            GLsizei n, const GLuint *ids);
+COIN_DLL_API GLboolean cc_glglue_glIsQuery(const cc_glglue * glue, 
+                                         GLuint id);
+COIN_DLL_API void cc_glglue_glBeginQuery(const cc_glglue * glue, 
+                                         GLenum target, GLuint id);
+COIN_DLL_API void cc_glglue_glEndQuery(const cc_glglue * glue, 
+                                       GLenum target);
+COIN_DLL_API void cc_glglue_glGetQueryiv(const cc_glglue * glue, 
+                                         GLenum target, GLenum pname, 
+                                         GLint * params);
+COIN_DLL_API void cc_glglue_glGetQueryObjectiv(const cc_glglue * glue, 
+                                               GLuint id, GLenum pname, 
+                                               GLint * params);
+COIN_DLL_API void cc_glglue_glGetQueryObjectuiv(const cc_glglue * glue, 
+                                                GLuint id, GLenum pname, 
+                                                GLuint * params);
+#endif /* COIN_NEXT_MINOR */
 
 /* GL feature queries */
 COIN_DLL_API SbBool cc_glglue_can_do_bumpmapping(const cc_glglue * glue);
 COIN_DLL_API SbBool cc_glglue_can_do_sortedlayersblend(const cc_glglue * glue);
+#ifdef COIN_NEXT_MINOR
+COIN_DLL_API SbBool cc_glglue_can_do_anisotropic_filtering(const cc_glglue * glue);
+COIN_DLL_API float cc_glglue_get_max_anisotropy(const cc_glglue * glue);
+#endif /* COIN_NEXT_MINOR */
 
 /* GL limits */
 COIN_DLL_API int cc_glglue_get_max_lights(const cc_glglue * glue);
 COIN_DLL_API const float * cc_glglue_get_line_width_range(const cc_glglue * glue);
 COIN_DLL_API const float * cc_glglue_get_point_size_range(const cc_glglue * glue);
-/* DEPRECATED for internal use. Kept for ABI compatibility */
-COIN_DLL_API SbBool cc_glglue_is_texture_size_legal(const cc_glglue * glw, 
-                                                    int xsize, int ysize, int zsize, 
-                                                    int bytespertexel, 
+/*
+   DEPRECATED for internal use. Kept for ABI compatibility
+   FIXME: Remove for Coin-3 (kintel 20050608)
+*/
+COIN_DLL_API SbBool cc_glglue_is_texture_size_legal(const cc_glglue * glw,
+                                                    int xsize, int ysize, int zsize,
+                                                    int bytespertexel,
                                                     SbBool mipmap);
 
 /* ********************************************************************** */
@@ -568,6 +606,31 @@ COIN_DLL_API SbBool cc_glglue_context_can_render_to_texture(void * ctx);
 
 /* ********************************************************************** */
 
+/* Interface for setting external offscreen renderer functionality:
+ * Makes it possible to provide an external implementation for doing
+ * offscreen rendering (such as accelerated rendering into a hidden
+ * window). This is useful to avoid having to do slow software
+ * rendering when pBuffers cannot be used (not available on the
+ * system, buggy implementation, mismatch of onscreen context and
+ * offscreen context).
+ */
+
+typedef void * cc_glglue_offscreen_data;
+
+typedef struct cc_glglue_offscreen_cb_functions {
+    cc_glglue_offscreen_data (*create_offscreen)(unsigned int width, unsigned int height);
+    SbBool (*make_current)(cc_glglue_offscreen_data context);
+    void (*reinstate_previous)(cc_glglue_offscreen_data context);
+    void (*destruct)(cc_glglue_offscreen_data context);
+} cc_glglue_offscreen_cb_functions; 
+
+#ifdef COIN_NEXT_MINOR
+/* Set callback functions for external offscreen rendering. Pass NULL 
+   to restore default, built-in offscreen rendering. 
+ */
+COIN_DLL_API void cc_glglue_context_set_offscreen_cb_functions(cc_glglue_offscreen_cb_functions* p);
+#endif // COIN_NEXT_MINOR
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
