@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,10 +17,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -127,6 +127,9 @@ typedef void (APIENTRY * COIN_PFNGLMULTITEXCOORD3FVPROC)(GLenum target,
 typedef void (APIENTRY * COIN_PFNGLMULTITEXCOORD4FVPROC)(GLenum target,
                                                          const GLfloat * v);
 
+typedef void (APIENTRY * COIN_PFNGLPUSHCLIENTATTRIBPROC)(GLbitfield mask);
+typedef void (APIENTRY * COIN_PFNGLPOPCLIENTATTRIBPROC)(void);
+
 /* typedefs for texture compression */
 typedef void (APIENTRY * COIN_PFNGLCOMPRESSEDTEXIMAGE3DPROC)(GLenum target,
                                                              GLint level,
@@ -503,9 +506,21 @@ typedef void (APIENTRY * COIN_PFNGLUNIFORMMATRIX3FVARBPROC)(COIN_GLhandle, GLsiz
 typedef void (APIENTRY * COIN_PFNGLUNIFORMMATRIX4FVARBPROC)(COIN_GLhandle, GLsizei, GLboolean, const GLfloat *);
 
 
+/* Typedefs for occlusion queries -- GL_ARB_occlusion_query */
+
+typedef void (APIENTRY * COIN_PFNGLGENQUERIESPROC)(GLsizei n, GLuint * ids);
+typedef void (APIENTRY * COIN_PFNGLDELETEQUERIESPROC)(GLsizei n, const GLuint *ids);
+typedef GLboolean (APIENTRY * COIN_PFNGLISQUERYPROC)(GLuint id);
+typedef void (APIENTRY * COIN_PFNGLBEGINQUERYPROC)(GLenum target, GLuint id);
+typedef void (APIENTRY * COIN_PFNGLENDQUERYPROC)(GLenum target);
+typedef void (APIENTRY * COIN_PFNGLGETQUERYIVPROC)(GLenum target, GLenum pname, GLint * params);
+typedef void (APIENTRY * COIN_PFNGLGETQUERYOBJECTIVPROC)(GLuint id, GLenum pname, GLint * params);
+typedef void (APIENTRY * COIN_PFNGLGETQUERYOBJECTUIVPROC)(GLuint id, GLenum pname, GLuint * params);
+
 /* Typedefs for GLX functions. */
 typedef void *(APIENTRY * COIN_PFNGLXGETCURRENTDISPLAYPROC)(void);
 
+
 /* ********************************************************************** */
 
 /* Type specification for GLX info storage structure, embedded within
@@ -536,6 +551,7 @@ struct cc_glxglue {
    initialized for each new GL context id. */
 struct cc_glglue {
 
+  uint32_t contextid;
   struct { /* OpenGL versioning. */
     unsigned int major, minor, release;
   } version;
@@ -727,7 +743,19 @@ struct cc_glglue {
   COIN_PFNGLUNIFORMMATRIX2FVARBPROC glUniformMatrix2fvARB;
   COIN_PFNGLUNIFORMMATRIX3FVARBPROC glUniformMatrix3fvARB;
   COIN_PFNGLUNIFORMMATRIX4FVARBPROC glUniformMatrix4fvARB;
- 
+
+  COIN_PFNGLPUSHCLIENTATTRIBPROC glPushClientAttrib;
+  COIN_PFNGLPOPCLIENTATTRIBPROC glPopClientAttrib;
+
+  COIN_PFNGLGENQUERIESPROC glGenQueries;
+  COIN_PFNGLDELETEQUERIESPROC glDeleteQueries;
+  COIN_PFNGLISQUERYPROC glIsQuery;
+  COIN_PFNGLBEGINQUERYPROC glBeginQuery;
+  COIN_PFNGLENDQUERYPROC glEndQuery;
+  COIN_PFNGLGETQUERYIVPROC glGetQueryiv;
+  COIN_PFNGLGETQUERYOBJECTIVPROC glGetQueryObjectiv;
+  COIN_PFNGLGETQUERYOBJECTUIVPROC glGetQueryObjectuiv;
+
   const char * versionstr;
   const char * vendorstr;
   SbBool vendor_is_SGI;
@@ -757,6 +785,7 @@ struct cc_glglue {
   SbBool has_arb_shader_objects;
   SbBool has_texture_env_combine;
 
+  SbBool vbo_in_displaylist_ok;
   int max_lights;
   float line_width_range[2];
   float point_size_range[2];
@@ -766,6 +795,13 @@ struct cc_glglue {
 
 /* ********************************************************************** */
 
+/* Called from SoContextHandler::destructingContext() to be able
+   to deallocated the cc_glglue instance. */
+
+void coin_glglue_destruct(uint32_t contextid);
+
+/* ********************************************************************** */
+
 /* Primarily used internally from functions that are badly designed,
    lacking a cc_glglue* argument in the function signature.
 
@@ -782,10 +818,12 @@ const char * coin_glerror_string(GLenum errorcode);
 
 /* ********************************************************************** */
 
-/* Exported internally to gl_glx.c and gl_wgl.c. */
+/* Exported internally to gl_glx.c / gl_wgl.c / gl_agl.c. */
 int coin_glglue_debug(void);
 int coin_glglue_extension_available(const char * extensions, const char * ext);
 
+int coin_glglue_stencil_bits_hack(void);
+
 /* ********************************************************************** */
 
 /* needed for bumpmap rendering */
@@ -808,6 +846,13 @@ GLint coin_glglue_get_internal_texture_format(int numcomponents,
                                             SbBool compress);
 
 GLenum coin_glglue_get_texture_format(int numcomponents);
+SbBool coin_glglue_vbo_in_displaylist_supported(const cc_glglue * glw);
+
+/* context creation callback */
+typedef void coin_glglue_instance_created_cb(const uint32_t contextid, void * closure);
+void coin_glglue_add_instance_created_callback(coin_glglue_instance_created_cb * cb,
+                                               void * closure);
+
 
 #ifdef __cplusplus
 }
