@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,10 +17,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -60,6 +60,8 @@ extern "C" {
 typedef int JSBool;
 typedef long jsword;
 typedef jsword jsval;
+typedef jsword jsid;
+typedef int intN;
 typedef unsigned int uintN;
 typedef uint16_t jschar;
 
@@ -95,6 +97,11 @@ typedef enum JSGCStatus {
   JSGC_FINALIZE_END
 } JSGCStatus;
 
+struct JSIdArray {
+    int32_t length;
+    jsid vector[1];
+};
+
 typedef void JSRuntime;
 typedef void JSContext;
 typedef void JSObject;
@@ -240,10 +247,20 @@ struct JSErrorReport {
 #define JSVAL_FALSE  BOOLEAN_TO_JSVAL(JS_FALSE)
 #define JSVAL_TRUE   BOOLEAN_TO_JSVAL(JS_TRUE)
 
-#define JSCLASS_HAS_PRIVATE (1<<0)
+#define JSCLASS_HAS_PRIVATE             (1<<0)
+#define JSCLASS_NEW_ENUMERATE           (1<<1)
+#define JSCLASS_NEW_RESOLVE             (1<<2)
+#define JSCLASS_PRIVATE_IS_NSISUPPORTS  (1<<3)
+#define JSCLASS_SHARE_ALL_PROPERTIES    (1<<4)
+#define JSCLASS_NEW_RESOLVE_GETS_START  (1<<5)
 
 #define JSFUN_BOUND_METHOD 0x40
 
+#define JSOPTION_STRICT                 JS_BIT(0)
+#define JSOPTION_WERROR                 JS_BIT(1)
+#define JSOPTION_VAROBJFIX              JS_BIT(2)
+#define JSOPTION_PRIVATE_IS_NSISUPPORTS JS_BIT(3)
+#define JSOPTION_COMPILE_N_GO           JS_BIT(4)
 
 /* Function typedefs. *************************************************** */
 
@@ -309,13 +326,17 @@ typedef JSBool (* JS_DefineProperty_t)(JSContext *, JSObject *, const char *, js
 typedef JSScript * (* JS_CompileFile_t)(JSContext *, JSObject *, const char *);
 typedef JSBool (* JS_ValueToObject_t)(JSContext *, jsval, JSObject **);
 typedef JSBool (* JS_ExecuteScript_t)(JSContext *, JSObject *, JSScript *, jsval *);
+typedef JSBool (* JS_IsExceptionPending_t)(JSContext *);
 typedef JSBool (* JS_GetPendingException_t)(JSContext *, jsval *);
+typedef void (* JS_SetPendingException_t)(JSContext *, jsval);
+typedef void (* JS_ClearPendingException_t)(JSContext *);
 typedef double * (* JS_NewDouble_t)(JSContext *, double);
 typedef JSBool (* JS_CallFunction_t)(JSContext *, JSObject *, JSFunction *, uintN, jsval *, jsval *);
 typedef JSFunction * (* JS_ValueToFunction_t)(JSContext *, jsval);
 typedef void (* JS_ReportError_t)(JSContext *, const char *, ...);
 typedef JSBool (* JS_IsArrayObject_t)(JSContext *, JSObject *);
-typedef JSBool (* JS_ValueToInt32_t)(JSContext *, jsval, int32_t *);
+typedef JSBool (* JS_ObjectIsFunction_t)(JSContext *, JSObject *);
+typedef JSBool (* JS_ValueToECMAInt32_t)(JSContext *, jsval, int32_t *);
 typedef JSFunction * (* JS_DefineFunction_t)(JSContext *, JSObject *, const char *, JSNative, uintN, uintN);
 typedef JSObject * (* JS_GetGlobalObject_t)(JSContext *);
 typedef JSGCCallback (* JS_SetGCCallback_t)(JSContext *, JSGCCallback);
@@ -334,7 +355,12 @@ typedef JSBool (* JS_GetPropertyAttributes_t)(JSContext *, JSObject *, const cha
 typedef JSClass * (* JS_GetClass_t)(JSObject *);
 typedef JSObject * (* JS_GetPrototype_t)(JSContext *, JSObject *);
 typedef JSObject * (* JS_SetPrototype_t)(JSContext *, JSObject *, JSObject *);
-
+typedef intN (* JS_CompareStrings_t)(JSString *, JSString *);
+typedef uint32_t (* JS_GetOptions_t)(JSContext *);
+typedef uint32_t (* JS_SetOptions_t)(JSContext *, uint32_t);
+typedef uint32_t (* JS_ToggleOptions_t)(JSContext *, uint32_t);
+typedef struct JSIdArray * (* JS_Enumerate_t)(JSContext *, JSObject *);
+typedef JSBool (* JS_IdToValue_t)(JSContext *, jsid, jsval *);
 
 
 /* Access interface. **************************************************** */
@@ -398,13 +424,20 @@ typedef struct {
   JS_CompileFile_t JS_CompileFile;
   JS_ValueToObject_t JS_ValueToObject;
   JS_ExecuteScript_t JS_ExecuteScript;
+  JS_IsExceptionPending_t JS_IsExceptionPending;
   JS_GetPendingException_t JS_GetPendingException;
+  JS_SetPendingException_t JS_SetPendingException;
+  JS_ClearPendingException_t JS_ClearPendingException;
   JS_NewDouble_t JS_NewDouble;
   JS_CallFunction_t JS_CallFunction;
   JS_ValueToFunction_t JS_ValueToFunction;
   JS_ReportError_t JS_ReportError;
   JS_IsArrayObject_t JS_IsArrayObject;
-  JS_ValueToInt32_t JS_ValueToInt32;
+  JS_ObjectIsFunction_t JS_ObjectIsFunction;
+  // Note: We use this function instead of JS_ValueToInt32() since the 
+  // latter is buggy in versions of SpiderMonkey older than 2005-09-29, 
+  // see Mozilla bug #284032.
+  JS_ValueToECMAInt32_t JS_ValueToECMAInt32;
   JS_DefineFunction_t JS_DefineFunction;
   JS_GetGlobalObject_t JS_GetGlobalObject;
   JS_SetGCCallback_t JS_SetGCCallback;
@@ -420,6 +453,12 @@ typedef struct {
   JS_GetPropertyAttributes_t JS_GetPropertyAttributes;
   JS_GetPrototype_t JS_GetPrototype;
   JS_SetPrototype_t JS_SetPrototype;
+  JS_CompareStrings_t JS_CompareStrings;
+  JS_GetOptions_t JS_GetOptions;
+  JS_SetOptions_t JS_SetOptions;
+  JS_ToggleOptions_t JS_ToggleOptions;
+  JS_Enumerate_t JS_Enumerate;
+  JS_IdToValue_t JS_IdToValue;
 
 } SpiderMonkey_t;
 
