@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,15 +17,25 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
 #include <Inventor/C/basic.h>  /* COIN_DLL_API */
 
+/* ********************************************************************** */
+
+/* Implementation note: it is important that this header file can be
+   included even when Coin was built with no threads support.
+
+   (This simplifies client code, as we get away with far less #ifdef
+   HAVE_THREADS wrapping.) */
+
+/* ********************************************************************** */
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
