@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,10 +17,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -66,6 +66,38 @@ void coin_atexit_func(const char * name, coin_atexit_f * fp, int32_t priority);
 void coin_atexit_cleanup(void);
 SbBool coin_is_exiting(void);
 
+/* this enum contains all values that are available for use for the
+   third argument to coin_atexit_func() -- no other values than these
+   should be used! */
+enum coin_atexit_priorities {
+
+  /* clean-ups of client code should be done before any clean-up code
+     within Coin happens, so this is (2^31 - 1): */
+  CC_ATEXIT_EXTERNAL = 2147483647,
+
+  /* clean-up code with no particular dependencies should use this
+     code: */
+  CC_ATEXIT_NORMAL = 0,
+
+  /* later, in case e.g. some nodes' clean-up depends on the font
+     subsystem still being up'n'running: */
+  CC_ATEXIT_FONT_SUBSYSTEM = CC_ATEXIT_NORMAL - 100,
+
+  /* still later, so clean-up code can use e.g. SoDebugError to report
+     problems, output debugging info, etc: */
+  CC_ATEXIT_MSG_SUBSYSTEM = CC_ATEXIT_NORMAL - 200,
+
+  /* needs to happen late, since CC_ATEXIT_NORMAL clean-up routines
+     will for instance often want to dealloc mutexes: */
+  CC_ATEXIT_THREADING_SUBSYSTEM = CC_ATEXIT_NORMAL - 1000,
+  
+  /* dynamically loaded libraries should be the last to go, as other
+     code in Coin will be dependent on functionality in these in its
+     own clean-up code: */
+  CC_ATEXIT_DYNLIBS = -2147483647
+
+};
+
 /* ********************************************************************** */
 
 /*
@@ -147,6 +179,21 @@ unsigned long coin_geq_prime_number(unsigned long num);
 
 /* ********************************************************************** */
 
+enum CoinOSType { 
+  COIN_UNIX, 
+  COIN_OS_X, 
+  COIN_MSWINDOWS 
+};
+
+int coin_runtime_os(void);
+
+/* ********************************************************************** */
+
+int coin_debug_extra(void);
+int coin_debug_normalize(void);
+
+/* ********************************************************************** */
+
 #ifdef __cplusplus
 } /* extern "C" */
 #endif /* __cplusplus */
