@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -27,8 +27,10 @@
 #include <Inventor/nodes/SoSubNode.h>
 #include <Inventor/VRMLnodes/SoVRMLIndexedLine.h>
 
-#undef SO_END_LINE_INDEX
-#define SO_END_LINE_INDEX -1
+#ifndef SO_END_LINE_INDEX // also defined in SoIndexedFaceSet.h
+#define SO_END_LINE_INDEX (-1)
+#endif // !SO_END_LINE_INDEX
+
 
 class COIN_DLL_API SoVRMLIndexedLineSet : public SoVRMLIndexedLine
 {
