@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -29,6 +29,10 @@
 #include <Inventor/fields/SoSFVec3f.h>
 #include <Inventor/fields/SoMFString.h>
 
+#ifndef COIN_INTERNAL
+#include <Inventor/actions/SoCallbackAction.h>
+#endif // !COIN_INTERNAL
+
 class SbColor;
 class SoVRMLInline;
 class SoVRMLInlineP;
@@ -87,9 +91,10 @@ public:
   virtual void getPrimitiveCount(SoGetPrimitiveCountAction * action);
   virtual SoChildList * getChildren(void) const;
 
-private:
+protected:
   virtual ~SoVRMLInline();
 
+private:
   virtual void addBoundingBoxChild(SbVec3f center, SbVec3f size);
   virtual SbBool readInstance(SoInput * in, unsigned short flags);
   virtual void copyContents(const SoFieldContainer * from, SbBool copyconn);
@@ -101,5 +106,3 @@ private:
 };
 
 #endif // ! COIN_SOVRMLINLINE_H
-
-
