@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,10 +17,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -34,6 +34,9 @@
 
 typedef void SoGLRenderPassCB(void * userdata);
 typedef void SoGLPreRenderCB(void * userdata, class SoGLRenderAction * action);
+#ifdef COIN_NEXT_MINOR
+typedef float SoGLSortedObjectOrderCB(void * userdata, SoGLRenderAction * action);
+#endif // COIN_NEXT_MINOR
 
 class COIN_DLL_API SoGLRenderAction : public SoAction {
   typedef SoAction inherited;
@@ -59,6 +62,14 @@ public:
   enum AbortCode {
     CONTINUE, ABORT, PRUNE, DELAY
   };
+#ifdef COIN_NEXT_MINOR
+  enum SortedObjectOrderStrategy {
+    BBOX_CENTER,
+    BBOX_CLOSEST_CORNER,
+    BBOX_FARTHEST_CORNER,
+    CUSTOM_CALLBACK
+  };
+#endif // COIN_NEXT_MINOR  
 
   typedef AbortCode SoGLRenderAbortCB(void * userdata);
 
@@ -96,7 +107,13 @@ public:
  
   void setSortedLayersNumPasses(int num);
   int getSortedLayersNumPasses() const;
-  
+
+#ifdef COIN_NEXT_MINOR
+  void setSortedObjectOrderStrategy(const SortedObjectOrderStrategy strategy,
+                                    SoGLSortedObjectOrderCB * cb = NULL,
+                                    void * closure = NULL);
+#endif // COIN_NEXT_MINOR
+
 protected:
   virtual void beginTraversal(SoNode * node);
   virtual void endTraversal(SoNode * node);
@@ -110,3 +127,4 @@ private:
 };
 
 #endif // !COIN_SOGLRENDERACTION_H
+
