@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,10 +17,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -110,6 +110,7 @@ _classname_::atexit_cleanup(void) \
   _classname_::enabledElements = NULL; \
   delete _classname_::methods; \
   _classname_::methods = NULL; \
+  _classname_::classTypeId STATIC_SOTYPE_INIT; \
 }
 
 // *************************************************************************
@@ -123,6 +124,9 @@ _classname_::atexit_cleanup(void) \
     _classname_::methods = new SoActionMethodList(_parentclassname_::getClassActionMethods()); \
   } while (0)
 
+#define SO_ACTION_EXIT_CLASS(_class_) \
+  _class_::atexit_cleanup()
+
 // *************************************************************************
 
 #endif // !COIN_SOSUBACTION_H
