@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,10 +17,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -28,6 +28,7 @@
 
 class SoGLRenderCache;
 class SoGLLazyElementP;
+class SoVBO;
 
 class COIN_DLL_API SoGLLazyElement : public SoLazyElement {
   typedef SoLazyElement inherited;
@@ -140,7 +141,11 @@ public:
 
   static SbBool preCacheCall(SoState * state, SoGLLazyElement::GLState * prestate);
   static void postCacheCall(SoState * state, SoGLLazyElement::GLState * poststate);
-
+  
+#ifdef COIN_NEXT_MINOR
+  void updateColorVBO(SoVBO * vbo);
+#endif // COIN_NEXT_MINOR
+  
 protected:
   virtual void lazyDidSet(uint32_t mask);
   virtual void lazyDidntSet(uint32_t mask);
