@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,10 +17,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -283,6 +283,15 @@ public:
   }
   void reallocate(const int32_t size);
 
+#ifdef COIN_NEXT_MINOR
+  uint32_t getDiffuseId(void) const {
+    return this->diffuseid;
+  }
+  uint32_t getTranspId(void) const {
+    return this->transpid;
+  }
+#endif // COIN_NEXT_MINOR
+
 private:
   uint32_t transpid;
   uint32_t diffuseid;
