@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -37,6 +37,7 @@
 #endif // !COIN_INTERNAL
 
 #include <Inventor/engines/SoSubEngine.h>
+#include <Inventor/C/tidbitsp.h>
 
 // Be aware that any changes to the SO_ENGINE_INTERNAL_CONSTRUCTOR
 // macro should be matched by similar changes to the constructor in
@@ -56,6 +57,7 @@
   do { \
     const char * classname = SO__QUOTE(_class_); \
     PRIVATE_COMMON_ENGINE_INIT_CODE(_class_, &classname[2], &_class_::createInstance, inherited); \
+    coin_atexit((coin_atexit_f*)_class_::atexit_cleanup, 0); \
   } while (0)
 
 
@@ -122,6 +124,19 @@ const SoEngineOutputData * _class_::getOutputData(void) const \
 void * _class_::createInstance(void) \
 { \
   return new _class_; \
+} \
+ \
+void \
+_class_::atexit_cleanup(void) \
+{ \
+  delete _class_::inputdata; \
+  delete _class_::outputdata; \
+  _class_::inputdata = NULL; \
+  _class_::outputdata = NULL; \
+  _class_::parentinputdata = NULL; \
+  _class_::parentoutputdata = NULL; \
+  _class_::classTypeId STATIC_SOTYPE_INIT; \
+  _class_::classinstances = 0; \
 }
 
 #endif // !COIN_SOSUBENGINEP_H
