@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -34,6 +34,7 @@ protected: \
 public: \
   virtual const SoEngineOutputData * getOutputData(void) const; \
 private: \
+  static void atexit_cleanupnodeengine(void); \
   static SoEngineOutputData * outputdata; \
   static const SoEngineOutputData ** parentoutputdata
 
@@ -57,6 +58,14 @@ const SoEngineOutputData * \
 _class_::getOutputData(void) const \
 { \
   return _class_::outputdata; \
+} \
+ \
+void \
+_class_::atexit_cleanupnodeengine(void) { \
+  delete _class_::outputdata; \
+  _class_::outputdata = NULL; \
+  _class_::parentoutputdata = NULL; \
+  _class_::classTypeId STATIC_SOTYPE_INIT; \
 }
 
 #define SO_NODEENGINE_SOURCE(_class_) \
@@ -116,7 +125,8 @@ _class_::createInstance(void) \
     PRIVATE_COMMON_INIT_CODE(_class_, classname, &_class_::createInstance, _parentclass_); \
   } while (0)
 
-
+#define SO_NODEENGINE_EXIT_CLASS(_class_) \
+  _class_::atexit_cleanupnodeengine();
 
 #define SO_NODEENGINE_INIT_ABSTRACT_CLASS(_class_, _parentclass_, _parentname_) \
   do { \
