@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,10 +17,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -35,6 +35,15 @@
 #include <Inventor/SoOutput.h>
 #endif // !COIN_INTERNAL
 
+/**************************************************************************
+ *
+ * Common source macros
+ *
+ **************************************************************************/
+
+#define SO_FIELD_EXIT_CLASS(_class_) \
+  _class_::atexit_cleanup()
+
 /**************************************************************************
  *
  * Header macros for single-value fields.
@@ -50,6 +59,7 @@ public: \
 #define SO_SFIELD_REQUIRED_HEADER(_class_) \
 private: \
   static SoType classTypeId; \
+  static void atexit_cleanup(void) { _class_::classTypeId STATIC_SOTYPE_INIT; } \
 public: \
   static void * createInstance(void); \
   static SoType getClassTypeId(void); \
@@ -126,8 +136,6 @@ public: \
     PRIVATE_FIELD_INIT_CLASS(_class_, classname, _parent_, &_class_::createInstance); \
   } while (0)
 
-
-
 #define SO_SFIELD_CONSTRUCTOR_SOURCE(_class_) \
 _class_::_class_(void) { assert(_class_::classTypeId != SoType::badType()); } \
 _class_::~_class_() { }
@@ -405,6 +413,7 @@ _class_::allocValues(int newnum) \
   /* or not they should be matched with modifications in that method */ \
   /* aswell. */ \
  \
+  int oldmaxnum; /* must be here as a gcc 4.0.0 bug workaround */ \
   assert(newnum >= 0); \
  \
   if (newnum == 0) { \
@@ -423,7 +432,7 @@ _class_::allocValues(int newnum) \
       /* */ \
       /* I think this will handle both cases quite gracefully: */ \
       /* 1) newnum > this->maxNum, 2) newnum < num */ \
-      int oldmaxnum = this->maxNum; \
+      oldmaxnum = this->maxNum; \
       while (newnum > this->maxNum) this->maxNum *= 2; \
       while ((this->maxNum / 2) >= newnum) this->maxNum /= 2; \
  \
