@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,10 +17,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -63,13 +63,13 @@ public:
     SUBIMAGE,
     MIPMAP
   };
-
   typedef SbBool SoGLImageResizeCB(SoState * state,
                                    const SbVec3s &newsize, 
                                    unsigned char * destbuffer, 
                                    ResizeReason reason,
                                    void * closure,
                                    class SoGLImage * image);
+
 #endif // COIN_NEXT_MINOR
 
   void setPBuffer(SoState * state,
@@ -130,11 +130,11 @@ public:
     INVINCIBLE                = 0x0200, // never die of old age
 
     // use GL_NV_texture_rectangle or GL_EXT_texture_rectangle
+    RECTANGLE                 = 0x0400,
+
     // Compress texture if available from OpenGL
     COMPRESSED                = 0x0800,
 
-    RECTANGLE                 = 0x0400,
-
     // use quality value to decide mipmap, filtering and scaling. This
     // is the default.
     USE_QUALITY_VALUE         = 0X8000
@@ -151,7 +151,10 @@ public:
   Wrap getWrapS(void) const;
   Wrap getWrapT(void) const;
   Wrap getWrapR(void) const;
-  
+
+#ifdef COIN_NEXT_MINOR
+  float getQuality(void) const;
+#endif // COIN_NEXT_MINOR
   uint32_t getGLImageId(void) const;
   
 protected:
@@ -165,6 +168,7 @@ private:
 
   class SoGLImageP * pimpl;
   friend class SoGLImageP;
+  static void cleanupClass(void);
   
 public:
   // internal methods for texture resource management
