@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,10 +17,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -29,7 +29,15 @@
 #include <Inventor/fields/SoSFEnum.h>
 #include <stddef.h> // NULL
 
+#ifndef COIN_INTERNAL
+#include <Inventor/actions/SoCallbackAction.h>
+#endif // !COIN_INTERNAL
+
 class SbColor;
+class SbVec3f;
+class SbVec2f;
+class SbVec2s;
+class SbViewportRegion;
 class SoPrimitiveVertex;
 
 // This shouldn't strictly be necessary, but the OSF1/cxx compiler
@@ -99,6 +107,16 @@ public:
   virtual void handleEvent(SoHandleEventAction * action);
   virtual void GLRenderBelowPath(SoGLRenderAction * action);
 
+#ifdef COIN_NEXT_MINOR
+  void select(SoNode * root, int numcoords, SbVec2f * lasso, 
+              const SbViewportRegion & vp, SbBool shiftpolicy);
+  void select(SoNode * root, int numcoords, SbVec3f * lasso,
+              const SbViewportRegion & vp, SbBool shiftkeypolicy);
+  const SbVec2s * getLassoCoordsDC(int & numCoords);
+  const SbVec3f * getLassoCoordsWC(int & numCoords);
+  const SoPathList & getSelectionPathList() const;
+#endif
+
   void setLassoFilterCallback(SoLassoSelectionFilterCB * f, void * userdata = NULL,
                               const SbBool callonlyifselectable = TRUE);
 
