@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,10 +17,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -35,6 +35,7 @@ class SoPickedPoint;
 class SoFaceDetail;
 class SoState;
 class SoTextureCoordinateElement;
+class SoCoordinateElement;
 class SbVec2f;
 class SoMaterialBundle;
 class SoBoundingBoxCache;
@@ -68,7 +69,7 @@ public:
                             SbVec2s & rectsize);
   static float getDecimatedComplexity(SoState * state, float complexity);
   const SoBoundingBoxCache * getBoundingBoxCache(void) const;
-  
+
 protected:
   SoShape(void);
   virtual ~SoShape();
@@ -128,6 +129,19 @@ protected:
                       const float r,
                       const SbVec3f & normal);
 
+#ifdef COIN_NEXT_MINOR
+  SbBool startVertexArray(SoGLRenderAction * action,
+                          const SoCoordinateElement * coords,
+                          const SbVec3f * pervertexnormals,
+                          const SbBool texpervertex,
+                          const SbBool colorpervertex);
+
+  void finishVertexArray(SoGLRenderAction * action,
+                         const SbBool vbo,
+                         const SbBool normpervertex,
+                         const SbBool texpervertex,
+                         const SbBool colorpervertex);
+#endif // COIN_NEXT_MINOR
 private:
   class SoShapeP * pimpl;
 
