@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,10 +17,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -114,12 +114,15 @@ _class_::atexit_cleanup(void) \
 { \
   delete _class_::fieldData; \
   _class_::fieldData = NULL; \
+  _class_::parentFieldData = NULL; \
+  _class_::classTypeId STATIC_SOTYPE_INIT; \
+  _class_::classinstances = 0; \
 }
 
 // FIXME: document. 20000103 mortene.
 #define SO_NODE_SOURCE(_class_) \
 SO_NODE_ABSTRACT_SOURCE(_class_); \
-\
+ \
 /*! \
   Creates a new instance of the class type corresponding to the SoType object. \
 */ \
@@ -202,6 +205,9 @@ _class_::createInstance(void) \
     PRIVATE_COMMON_INIT_CODE(_class_, classname, &_class_::createInstance, _parentclass_); \
   } while (0)
 
+// FIXME: document. 20060208 kintel.
+#define SO_NODE_EXIT_CLASS(_class_) \
+  _class_::atexit_cleanup()
 
 // FIXME: document. 20000103 mortene.
 #define SO_NODE_INIT_ABSTRACT_CLASS(_class_, _parentclass_, _parentname_) \
