@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,10 +17,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -56,6 +56,7 @@
  \
     /* Store parent's fielddata pointer for later use in the constructor. */ \
     _class_::parentFieldData = _parentclass_::getFieldDataPtr(); \
+    coin_atexit((coin_atexit_f*)_class_::atexit_cleanup, 0); \
   } while (0)
 
 #define SO_NODE_INTERNAL_CONSTRUCTOR(_class_) \
@@ -65,9 +66,6 @@
     /* Restore value of isBuiltIn flag (which is set to FALSE */ \
     /* in the SO_NODE_CONSTRUCTOR() macro. */ \
     this->isBuiltIn = TRUE; \
-    if (SO_NODE_IS_FIRST_INSTANCE()) { \
-      coin_atexit((coin_atexit_f*)_class_::atexit_cleanup, 0); \
-    } \
     SoBase::staticDataUnlock(); \
   } while (0)
 
