@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,10 +17,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -385,13 +385,56 @@
 /* some glVertexBufferObject defines */
 #ifndef GL_ARRAY_BUFFER
 #define GL_ARRAY_BUFFER 0x8892
-#endif /* GL_ARRAY_BUFFER_ARB */
+#endif /* GL_ARRAY_BUFFER */
 #ifndef GL_ELEMENT_ARRAY_BUFFER
 #define GL_ELEMENT_ARRAY_BUFFER 0x8893
-#endif /* GL_ELEMENT_ARRAY_BUFFER_ARB */
+#endif /* GL_ELEMENT_ARRAY_BUFFER */
+#ifndef GL_READ_ONLY
+#define GL_READ_ONLY 0x88B8
+#endif /* GL_READ_ONLY */
+#ifndef GL_WRITE_ONLY
+#define GL_WRITE_ONLY 0x88B9
+#endif /* GL_WRITE_ONLY */
+#ifndef GL_READ_WRITE
+#define GL_READ_WRITE 0x88BA
+#endif /* GL_READ_WRITE */
+#ifndef GL_BUFFER_ACCESS
+#define GL_BUFFER_ACCESS 0x88BB
+#endif /* GL_BUFFER_ACCESS */
+#ifndef GL_BUFFER_MAPPED
+#define GL_BUFFER_MAPPED 0x88BC
+#endif /* GL_BUFFER_MAPPED */
+#ifndef GL_BUFFER_MAP_POINTER
+#define GL_BUFFER_MAP_POINTER 0x88BD
+#endif /* GL_BUFFER_MAP_POINTER */
+#ifndef GL_STREAM_DRAW
+#define GL_STREAM_DRAW 0x88E0
+#endif /* GL_STREAM_DRAW */
+#ifndef GL_STREAM_READ
+#define GL_STREAM_READ 0x88E1
+#endif /* GL_STREAM_READ */
+#ifndef GL_STREAM_COPY
+#define GL_STREAM_COPY 0x88E2
+#endif /* GL_STREAM_COPY */
 #ifndef GL_STATIC_DRAW
 #define GL_STATIC_DRAW 0x88E4
-#endif /* GL_STATIC_DRAW_ARB */
+#endif /* GL_STATIC_DRAW */
+#ifndef GL_STATIC_READ
+#define GL_STATIC_READ 0x88E5
+#endif /* GL_STATIC_READ */
+#ifndef GL_STATIC_COPY
+#define GL_STATIC_COPY 0x88E6
+#endif /* GL_STATIC_COPY */
+#ifndef GL_DYNAMIC_DRAW
+#define GL_DYNAMIC_DRAW 0x88E8
+#endif /* GL_DYNAMIC_DRAW */
+#ifndef GL_DYNAMIC_READ
+#define GL_DYNAMIC_READ 0x88E9
+#endif /* GL_DYNAMIC_READ */
+#ifndef GL_DYNAMIC_COPY
+#define GL_DYNAMIC_COPY 0x88EA
+#endif /* GL_DYNAMIC_COPY */
+
 
 /* NViDIA GL_NV_register_combiners extension */
 #ifndef GL_REGISTER_COMBINERS_NV 
@@ -1407,6 +1450,45 @@
 #define GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT 0x84FF
 #endif /* GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT */
 
+/* GL_ARB_occlusion_query */
+#ifndef GL_SAMPLES_PASSED
+#define GL_SAMPLES_PASSED 0x8914
+#endif /* GL_SAMPLES_PASSED */
+
+#ifndef GL_QUERY_COUNTER_BITS
+#define GL_QUERY_COUNTER_BITS 0x8864
+#endif /* GL_QUERY_COUNTER_BITS */
+
+#ifndef GL_CURRENT_QUERY
+#define GL_CURRENT_QUERY 0x8865
+#endif /* GL_CURRENT_QUERY */
+
+#ifndef GL_QUERY_RESULT
+#define GL_QUERY_RESULT 0x8866
+#endif /* GL_QUERY_RESULT */
+
+#ifndef GL_QUERY_RESULT_AVAILABLE
+#define GL_QUERY_RESULT_AVAILABLE 0x8867
+#endif /* GL_QUERY_RESULT_AVAILABLE */
+
+
+/* glClientPush/PopAttrib() defines */
+#ifndef GL_CLIENT_PIXEL_STORE_BIT
+#define GL_CLIENT_PIXEL_STORE_BIT 0x00000001
+#endif /* GL_CLIENT_PIXEL_STORE_BIT */
+
+#ifndef GL_CLIENT_VERTEX_ARRAY_BIT
+#define GL_CLIENT_VERTEX_ARRAY_BIT  0x00000002
+#endif /* GL_CLIENT_VERTEX_ARRAY_BIT */
+
+#ifndef GL_CLIENT_ALL_ATTRIB_BITS
+#define GL_CLIENT_ALL_ATTRIB_BITS 0xFFFFFFFF
+#endif /* GL_CLIENT_ALL_ATTRIB_BITS */
+
+#ifndef GL_ALL_CLIENT_ATTRIB_BITS
+#define GL_ALL_CLIENT_ATTRIB_BITS 0xFFFFFFFF
+#endif /* GL_ALL_CLIENT_ATTRIB_BITS */
+
 /*** GL enums, end ****************************************************/
 /**********************************************************************/
 
@@ -1615,7 +1697,21 @@
 #ifndef GLU_TESS_COMBINE
 #define GLU_TESS_COMBINE 100105
 #endif
-
+#ifndef GLU_TESS_MISSING_BEGIN_POLYGON
+#define GLU_TESS_MISSING_BEGIN_POLYGON 100151
+#endif /* ! GLU_TESS_MISSING_BEGIN_POLYGON */
+#ifndef GLU_TESS_MISSING_BEGIN_CONTOUR
+#define GLU_TESS_MISSING_BEGIN_CONTOUR 100152
+#endif /* ! GLU_TESS_MISSING_BEGIN_CONTOUR */
+#ifndef GLU_TESS_MISSING_END_POLYGON
+#define GLU_TESS_MISSING_END_POLYGON 100153
+#endif /* ! GLU_TESS_MISSING_END_POLYGON */
+#ifndef GLU_TESS_MISSING_END_CONTOUR
+#define GLU_TESS_MISSING_END_CONTOUR 100154
+#endif /* ! GLU_TESS_MISSING_END_CONTOUR */
+#ifndef GLU_TESS_NEED_COMBINE_CALLBACK
+#define GLU_TESS_NEED_COMBINE_CALLBACK 100156
+#endif /* ! GLU_TESS_NEED_COMBINE_CALLBACK */
 
 /*** GLU enums, end ***************************************************/
 /**********************************************************************/
@@ -1631,6 +1727,9 @@
 #ifndef WGL_DEPTH_BITS_ARB
 #define WGL_DEPTH_BITS_ARB 0x2022
 #endif /* WGL_DEPTH_BITS_ARB */
+#ifndef WGL_STENCIL_BITS_ARB
+#define WGL_STENCIL_BITS_ARB 0x2023
+#endif /* WGL_STENCIL_BITS_ARB */
 #ifndef WGL_DRAW_TO_PBUFFER_ARB
 #define WGL_DRAW_TO_PBUFFER_ARB 0x202D
 #endif /* WGL_DRAW_TO_PBUFFER_ARB */
