@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -390,7 +390,7 @@ SoAction::initClass(void)
                                     SoOverrideElement::getClassStackIndex());
 
   SoAction::initClasses();
-  coin_atexit((coin_atexit_f*) SoAction::atexit_cleanup, 0);
+  coin_atexit((coin_atexit_f*) SoAction::atexit_cleanup, CC_ATEXIT_NORMAL);
 }
 
 // private cleanup method
@@ -401,6 +401,7 @@ SoAction::atexit_cleanup(void)
   SoAction::enabledElements = NULL;
   delete SoAction::methods;
   SoAction::methods = NULL;
+  SoAction::classTypeId STATIC_SOTYPE_INIT;
 }
 
 /*!
@@ -424,6 +425,13 @@ SoAction::initClasses(void)
   SoWriteAction::initClass();
   SoAudioRenderAction::initClass();  
   SoIntersectionDetectionAction::initClass();
+  
+  SoToVRMLAction::initClass();
+  SoToVRML2Action::initClass();
+#ifdef COIN_NEXT_MINOR
+  SoSimplifyAction::initClass();
+  SoReorganizeAction::initClass();
+#endif // COIN_NEXT_MINOR
 }
 
 /*!
@@ -537,6 +545,12 @@ SoAction::apply(SoNode * root)
 /*!
   Applies the action to the parts of the graph defined by \a path.
 
+  Note that an SoPath will also contain all nodes that may influence
+  e.g. geometry nodes in the path. So for instance applying an
+  SoGLRenderAction on an SoPath will render that path as expected in
+  the view, where geometry will get its materials, textures, and other
+  appearance settings correctly.
+
   If the \a path ends in an SoGroup node, the action will also
   traverse the tail node's children.
 */
@@ -606,6 +620,8 @@ SoAction::apply(SoPath * path)
   All paths must start at the same head node. All paths must be sorted
   in traversal order. The paths must be unique. No path can continue
   through the end point of another path.
+
+  \sa SoAction::apply(SoPath * path)
 */
 void
 SoAction::apply(const SoPathList & pathlist, SbBool obeysrules)
