@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -99,6 +99,8 @@
 #include <Inventor/nodes/SoNode.h>
 #include <Inventor/nodes/SoShape.h>
 #include <Inventor/nodes/SoShapeHints.h>
+#include <Inventor/C/tidbits.h>
+#include <stdlib.h>
 
 // *************************************************************************
 
@@ -422,6 +424,63 @@
   \sa setAbortCallback()
 */
 
+#ifdef COIN_NEXT_MINOR
+
+/*!
+  \typedef float SoGLSortedObjectOrderCB(void * userdata, SoGLRenderAction * action)
+
+  A callback used for controlling the transparency sorting order.
+
+  \sa setSortedObjectOrderStrategy().
+  \since Coin 2.5
+*/
+
+/*! 
+   \enum SortedObjectOrderStrategy
+
+   Used for enumerating the different transparency sorting strategies.
+
+  \sa setSortedObjectOrderStrategy().
+  \since Coin 2.5
+*/
+
+/*!
+  \var SoGLRenderAction::SortedObjectOrderStrategy SoGLRenderAction::BBOX_CENTER
+
+  Do the sorting based on the center of the object bounding box.
+
+  \sa setSortedObjectOrderStrategy().
+  \since Coin 2.5
+*/
+
+/*!
+  \var SoGLRenderAction::SortedObjectOrderStrategy BBOX_CLOSEST_CORNER
+
+  Do the sorting based on the bounding box corner closest to the camera.
+
+  \sa setSortedObjectOrderStrategy().
+  \since Coin 2.5
+*/
+
+/*!
+  \var SoGLRenderAction::SortedObjectOrderStrategy SoGLRenderAction::BBOX_FARTHEST_CORNER
+
+  Do the sorting based on the bounding box corner farthest from the camera.
+
+  \sa setSortedObjectOrderStrategy().
+  \since Coin 2.5
+*/
+
+/*!
+  \var SoGLRenderAction::SortedObjectOrderStrategy SoGLRenderAction::CUSTOM_CALLBACK
+
+  Use a custom callback to determine the sorting order. 
+  
+  \sa setSortedObjectOrderStrategy().
+  \since Coin 2.5
+*/
+#endif // COIN_NEXT_MINOR
+
 // *************************************************************************
 
 class SoGLRenderActionP {
@@ -470,6 +529,12 @@ public:
   int sortedlayersblendcounter;
   SbBool usenvidiaregistercombiners;
 
+#ifdef COIN_NEXT_MINOR
+  SoGLRenderAction::SortedObjectOrderStrategy sortedobjectstrategy;
+  SoGLSortedObjectOrderCB * sortedobjectcb;
+  void * sortedobjectclosure;
+#endif // COIN_NEXT_MINOR
+
   void setupSortedLayersBlendTextures(const SoState * state);
   void doSortedLayersBlendRendering(const SoState * state, SoNode * node);
   void initSortedLayersBlendRendering(const SoState * state);
@@ -499,6 +564,7 @@ public:
 
 SO_ACTION_SOURCE(SoGLRenderAction);
 
+static int COIN_GLBBOX = 0;
 // *************************************************************************
 
 // Override from parent class.
@@ -518,6 +584,14 @@ SoGLRenderAction::initClass(void)
   SO_ENABLE(SoGLRenderAction, SoWindowElement);
   SO_ENABLE(SoGLRenderAction, SoGLViewportRegionElement);
   SO_ENABLE(SoGLRenderAction, SoGLCacheContextElement);
+
+  const char * env = coin_getenv("COIN_GLBBOX");
+  if (env) {
+    COIN_GLBBOX = atoi(env);
+  }
+  else {
+    COIN_GLBBOX = 0;
+  }
 }
 
 // *************************************************************************
@@ -571,7 +645,7 @@ SoGLRenderAction::SoGLRenderAction(const SbViewportRegion & viewportregion)
   THIS->transparencyrender = FALSE;
   THIS->isrendering = FALSE;
   THIS->passupdate = FALSE;
-  THIS->bboxaction = NULL;
+  THIS->bboxaction = new SoGetBoundingBoxAction(viewportregion);
   THIS->updateorigin.setValue(0.0f, 0.0f);
   THIS->updatesize.setValue(1.0f, 1.0f);
   THIS->rendering = SoGLRenderActionP::RENDERING_UNSET;
@@ -585,7 +659,11 @@ SoGLRenderAction::SoGLRenderAction(const SbViewportRegion & viewportregion)
   THIS->sortedlayersblendinitialized = FALSE;
   THIS->sortedlayersblendcounter = 0;
   THIS->usenvidiaregistercombiners = FALSE;
-
+#ifdef COIN_NEXT_MINOR
+  THIS->sortedobjectstrategy = BBOX_CENTER;
+  THIS->sortedobjectcb = NULL;
+  THIS->sortedobjectclosure = NULL;
+#endif // COIN_NEXT_MINOR
 }
 
 /*!
@@ -605,7 +683,7 @@ void
 SoGLRenderAction::setViewportRegion(const SbViewportRegion & newregion)
 {
   THIS->viewport = newregion;
-
+  THIS->bboxaction->setViewportRegion(newregion);
   // The SoViewportRegionElement is not set here, as it is always
   // initialized before redraw in beginTraversal().
 }
@@ -866,6 +944,14 @@ SoGLRenderAction::beginTraversal(SoNode * node)
     return;
   }
 
+  // If the environment variable COIN_GLBBOX is set to 1, apply a bbox
+  // action before rendering.  This will make sure bounding box caches
+  // are updated (needed for view frustum culling). The default
+  // SoQt/SoWin/SoXt viewers will also apply a SoGetBoundingBoxAction
+  // so we don't do this by default yet.
+  if (COIN_GLBBOX) {
+    THIS->bboxaction->apply(node);
+  }
   int err_before_init = glGetError();
 
   if (THIS->needglinit) {
@@ -1173,7 +1259,7 @@ SoGLRenderAction::doPathSort(void)
       dtmp = darray[i];
       ptmp = plist->get(i);
       j = i;
-      while (j >= distance && darray[j-distance] > dtmp) {
+      while (j >= distance && darray[j-distance] < dtmp) {
         darray[j] = darray[j-distance];
         plist->set(j, plist->get(j-distance));
         j -= distance;
@@ -1225,6 +1311,37 @@ SoGLRenderAction::removePreRenderCallback(SoGLPreRenderCB * func, void * userdat
   THIS->precblist.removeCallback((SoCallbackListCB*) func, userdata);
 }
 
+#ifdef COIN_NEXT_MINOR
+/*!
+  
+  Sets the strategy used for sorting transparent objects.
+
+  The \e CUSTOM_CALLBACK strategy enables the user to supply a
+  callback which is called for each transparent shape. This strategy
+  can be used if the built in sorting strategies aren't sufficient.
+
+  The callback should return a floating point value to be used when
+  sorting the objects in Coin. This floating point value is
+  interpreted as a distance to the camera, and objects with higher
+  values will be sorted behind objects with lower values.
+  
+  The callback will supply the SoGLRenderAction instance, and the path
+  to the current object can be found using SoAction::getCurPath().
+
+  \since Coin 2.5 
+
+*/
+void 
+SoGLRenderAction::setSortedObjectOrderStrategy(const SortedObjectOrderStrategy strategy,
+                                               SoGLSortedObjectOrderCB * cb,
+                                               void * closure)
+{
+  THIS->sortedobjectstrategy = strategy;
+  THIS->sortedobjectcb = cb;
+  THIS->sortedobjectclosure = closure;
+}
+#endif // COIN_NEXT_MINOR
+
 #undef THIS
 
 // *************************************************************************
@@ -1237,10 +1354,19 @@ void
 SoGLRenderActionP::addSortTransPath(SoPath * path) 
 {
   this->sorttranspobjpaths.append(path);
+#ifdef COIN_NEXT_MINOR
+  // check and handle callback first
+  if ((this->sortedobjectstrategy == SoGLRenderAction::CUSTOM_CALLBACK) &&
+      (this->sortedobjectcb != NULL)) {
+    this->sorttranspobjdistances.append(this->sortedobjectcb(this->sortedobjectclosure,
+                                                             this->action));
+    return;
+  }
+#endif // COIN_NEXT_MINOR  
 
   SoNode * tail = ((SoFullPath*)path)->getTail();
   float dist;
-
+  SbBox3f bbox;
   // test if we can find the bbox using SoShape::getBoundingBoxCache()
   // or SoShape::computeBBox. This is the common case, and quite a lot
   // faster than using an SoGetBoundingBoxAction.
@@ -1252,24 +1378,54 @@ SoGLRenderActionP::addSortTransPath(SoPath * path)
     if (bboxcache && bboxcache->isValid(action->state)) {
       if (bboxcache->isCenterSet()) center = bboxcache->getCenter();
       center = bboxcache->getProjectedBox().getCenter();
+      bbox = bboxcache->getProjectedBox();
     }
     else {
-      SbBox3f dummy;
-      tailshape->computeBBox(action, dummy, center);
+      tailshape->computeBBox(action, bbox, center);
     }
     SoModelMatrixElement::get(action->state).multVecMatrix(center, center);
-    dist = SoViewVolumeElement::get(action->state).getPlane(0.0f).getDistance(center);
+    dist = -SoViewVolumeElement::get(action->state).getPlane(0.0f).getDistance(center);
   }
   else {
-    if (this->bboxaction == NULL) {
-      this->bboxaction =
-        new SoGetBoundingBoxAction(SoViewportRegionElement::get(action->state));
-    }
     this->bboxaction->setViewportRegion(SoViewportRegionElement::get(action->state));
     this->bboxaction->apply(path);
     SbVec3f center = this->bboxaction->getBoundingBox().getCenter();
-    dist = SoViewVolumeElement::get(action->state).getPlane(0.0f).getDistance(center);
+    bbox = this->bboxaction->getBoundingBox();
+    bbox.transform(SoModelMatrixElement::get(action->state).inverse());
+    dist = -SoViewVolumeElement::get(action->state).getPlane(0.0f).getDistance(center);
   }
+#ifdef COIN_NEXT_MINOR
+  if ((this->sortedobjectstrategy == SoGLRenderAction::BBOX_CLOSEST_CORNER) ||
+      (this->sortedobjectstrategy == SoGLRenderAction::BBOX_FARTHEST_CORNER)) {
+    const SbMatrix & m = SoModelMatrixElement::get(action->state);
+    const SbPlane & plane = SoViewVolumeElement::get(action->state).getPlane(0.0f);
+    SbVec3f bmin, bmax;
+    bmin = bbox.getMin();
+    bmax = bbox.getMax();
+    
+    for (int i = 0; i < 8; i++) {
+      SbVec3f tmp(i&1 ? bmin[0] : bmax[0],
+                  i&2 ? bmin[1] : bmax[1],
+                  i&4 ? bmin[2] : bmax[2]);
+      m.multVecMatrix(tmp, tmp);
+      float tmpdist = -plane.getDistance(tmp);
+      if (i == 0) dist = tmpdist;
+      else {
+        switch (this->sortedobjectstrategy) {
+        case SoGLRenderAction::BBOX_CLOSEST_CORNER: 
+          if (tmpdist < dist) dist = tmpdist;
+          break;
+        case SoGLRenderAction::BBOX_FARTHEST_CORNER:
+          if (tmpdist > dist) dist = tmpdist;
+          break;
+        default:
+          assert(0 && "unknown sorting strategy");
+          break;
+        }
+      }
+    }
+  }
+#endif // COIN_NEXT_MINOR
   this->sorttranspobjdistances.append(dist);
 }
 
@@ -1516,9 +1672,9 @@ SoGLRenderActionP::doSortedLayersBlendRendering(const SoState * state, SoNode *
 
   glDisable(GL_BLEND);
 
-  // The 'sortedlayersblendcounter' is global as it must be reached by
-  // 'setupNVRegisterCombiners()' at all times during the scenegraph
-  // traversals.
+  // The 'sortedlayersblendcounter' must be global so that it can be
+  // reached by 'setupNVRegisterCombiners()' at all times during the
+  // scenegraph traversals.
   for(this->sortedlayersblendcounter=0; 
       this->sortedlayersblendcounter < this->sortedlayersblendpasses; 
       this->sortedlayersblendcounter++) {
