@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -327,6 +327,13 @@ void
 SoRayPickAction::setRay(const SbVec3f & start, const SbVec3f & direction,
                         float neardistance, float fardistance)
 {
+#if COIN_DEBUG
+  if (direction == SbVec3f(0.0f, 0.0f, 0.0f)) {
+    SoDebugError::postWarning("SoRayPickAction::setRay",
+                              "Ray has no direction");
+    
+  }
+#endif // COIN_DEBUG
   if (neardistance >= 0.0f) PRIVATE(this)->setFlag(SoRayPickActionP::CLIP_NEAR);
   else {
     PRIVATE(this)->clearFlag(SoRayPickActionP::CLIP_NEAR);
@@ -351,7 +358,7 @@ SoRayPickAction::setRay(const SbVec3f & start, const SbVec3f & direction,
 
   PRIVATE(this)->raystart.setValue(start);
   PRIVATE(this)->raydirection.setValue(direction);
-  PRIVATE(this)->raydirection.normalize();
+  (void) PRIVATE(this)->raydirection.normalize();
   PRIVATE(this)->raynear = neardistance;
   PRIVATE(this)->rayfar = fardistance;
   PRIVATE(this)->wsline = SbDPLine(PRIVATE(this)->raystart, 
@@ -500,7 +507,8 @@ SoRayPickAction::computeWorldSpaceRay(void)
     PRIVATE(this)->rayradiusdelta = 0.0;
     if (vv.getProjectionType() == SbViewVolume::PERSPECTIVE) {
       SbVec3d dir(0.0f, vv.getHeight()*0.5f, vv.getNearDist());
-      dir.normalize();
+      // no need to test here, we know vv isn't empty
+      (void) dir.normalize();
       SbVec3d upperfar = dir * double(vv.getNearDist()+vv.getDepth()) /
         dir.dot(SbVec3d(0.0, 0.0, 1.0));
 
@@ -598,30 +606,27 @@ SoRayPickAction::intersect(const SbVec3f & v0_in,
   else front = FALSE;
 
   // create some more intuitive barycentric coordinate names
-  float & u = barycentric[1];
-  float & v = barycentric[2];
-  float & w = barycentric[0];
-
+  double u, v, w;
   double inv_det = 1.0 / det;
 
   // calculate distance from v0 to ray origin
   SbVec3d tvec = orig - v0;
 
   // calculate U parameter and test bounds
-  u = float(tvec.dot(pvec) * inv_det);
+  u = tvec.dot(pvec) * inv_det;
   if (u < 0.0 || u > 1.0)
     return FALSE;
-
+  
   // prepare to test V parameter
   SbVec3d qvec = tvec.cross(edge1);
 
   // calculate V parameter and test bounds
-  v = float(dir.dot(qvec) * inv_det);
+  v = dir.dot(qvec) * inv_det;
   if (v < 0.0 || u + v > 1.0)
     return FALSE;
 
   // third barycentric coordinate
-  w = 1.0f - u - v;
+  w = 1.0 - u - v;
 
   // calculate t and intersection point
   double t = edge2.dot(qvec) * inv_det;
@@ -629,6 +634,11 @@ SoRayPickAction::intersect(const SbVec3f & v0_in,
   SbVec3d itmp = orig + t * dir;
   intersection.setValue(itmp);
 
+  // set the barycentric coordinates before returning
+  barycentric[0] = (float) w;
+  barycentric[1] = (float) u;
+  barycentric[2] = (float) v;
+
   return TRUE;
 }
 
