@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -123,6 +123,7 @@
 #include <Inventor/SoPrimitiveVertex.h>
 #include <Inventor/actions/SoCallbackAction.h>
 #include <Inventor/actions/SoGetPrimitiveCountAction.h>
+#include <Inventor/actions/SoGetBoundingBoxAction.h>
 #include <Inventor/actions/SoSearchAction.h>
 #include <Inventor/actions/SoWriteAction.h>
 #include <Inventor/elements/SoCoordinateElement.h>
@@ -137,17 +138,41 @@
 #include <Inventor/nodekits/SoBaseKit.h>
 #include <Inventor/nodes/SoNodes.h>
 #include <Inventor/nodes/SoTransform.h>
+#include <float.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif // HAVE_CONFIG_H
 
+// default values for cases where a viewport is needed
+#define DEFAULT_VIEWPORT_WIDTH 1024
+#define DEFAULT_VIEWPORT_HEIGHT 768
+
+
 // *************************************************************************
 
 SO_ACTION_SOURCE(SoToVRML2Action);
 
 // *************************************************************************
 
+// helper function needed to copy the name of a node
+static SoNode * tovrml2_new_node(SoNode * newnode, const SoNode * oldnode)
+{
+  const SbName name = oldnode->getName();
+  if (name != SbName("")) newnode->setName(name);
+  return newnode;
+}
+
+// We use SoType::createInstance() instead of simply new'ing to make
+// an instance, as this makes SoType::overrideType() influence the
+// conversion process.
+
+#define NEW_NODE(_type_, _oldnode_) \
+        (_type_*) tovrml2_new_node((SoNode *)_type_::getClassTypeId().createInstance(), \
+                                   _oldnode_)
+
+// *************************************************************************
+
 // Overridden from parent class.
 void
 SoToVRML2Action::initClass(void)
@@ -268,6 +293,7 @@ public:
     this->reuseGeometryNodes = FALSE;
     this->vrml2path = NULL;
     this->vrml2root = NULL;
+    this->bboxaction = NULL;
   }
 
   void init(void)
@@ -282,6 +308,7 @@ public:
 
     recentTex2 = NULL;
     do_post_primitives = FALSE;
+    didpush = FALSE;
 
     this->vrmlcoords = new SbList <SoVRMLCoordinate *>;
     this->vrmlnormals = new SbList <SoVRMLNormal *>;
@@ -302,6 +329,29 @@ public:
     this->vrml2path->setHead(this->vrml2root);
   }
 
+  SoGetBoundingBoxAction * getBBoxAction(void) {
+    if (this->bboxaction == NULL) {
+      SbViewportRegion vp(DEFAULT_VIEWPORT_WIDTH, DEFAULT_VIEWPORT_HEIGHT);
+      this->bboxaction = new SoGetBoundingBoxAction(vp);
+    }
+    return this->bboxaction;
+  }
+  float getBBoxDistance(const SbViewVolume & vv, const float screenarea, const float h) {
+    const float h2 = h * 0.5f; // use half the height for simplicity
+    float screenheight = float(sqrt(screenarea)) * 0.5f; // wanted height in pixels
+    float vvheight = vv.getHeight() * 0.5f; // total height of view volume    
+    float neardist = vv.getNearDist();
+
+    float projheight = (screenheight / 768.0f) * vvheight; // wanted projected height
+
+    // now, find the distance the bbox must be at the achieve this projheight
+    if (projheight > 0.0f) { 
+      return (neardist / projheight) * h2;
+    }
+    return FLT_MAX; // never switch
+  }
+
+
   SoToVRML2Action * master;
   SbBool nodefuse;
   SbBool reuseAppearanceNodes;
@@ -318,9 +368,11 @@ public:
   SbList <int32_t> * normalidx;
   SbList <int32_t> * texidx;
   SbList <int32_t> * coloridx;
+  SoGetBoundingBoxAction * bboxaction;
 
   SoTexture2 * recentTex2;
   SbBool do_post_primitives;
+  SbBool didpush;
 
   static SoCallbackAction::Response unsupported_cb(void *, SoCallbackAction *, const SoNode *);
 
@@ -367,6 +419,7 @@ public:
   static SoCallbackAction::Response push_switch_cb(void *, SoCallbackAction *, const SoNode *);
   static SoCallbackAction::Response pop_switch_cb(void *, SoCallbackAction *, const SoNode *);
   static SoCallbackAction::Response push_lod_cb(void *, SoCallbackAction *, const SoNode *);
+  static SoCallbackAction::Response push_levelofdetail_cb(void *, SoCallbackAction *, const SoNode *);
 
   // Other nodes
   static SoCallbackAction::Response sopercam_cb(void *, SoCallbackAction *, const SoNode *);
@@ -395,6 +448,25 @@ public:
 
 // *************************************************************************
 
+// add type and all shapes inheriting type to the list of shapes
+// already handled by the action
+static void add_shape_handled(const SoType & type, SoTypeList & addlist)
+{
+  SoTypeList shapes;
+  (void) SoType::getAllDerivedFrom(type, shapes);
+  int i;
+  
+  for (i = 0; i < shapes.getLength(); i++) {
+    SoType s = shapes[i];
+    if (s.canCreateInstance() && (addlist.find(s) < 0)) {
+      addlist.append(s);
+    }
+  }
+}
+
+// *************************************************************************
+
+
 SoToVRML2Action::SoToVRML2Action(void)
 {
   SO_ACTION_CONSTRUCTOR(SoToVRML2Action);
@@ -411,10 +483,11 @@ SoToVRML2Action::SoToVRML2Action(void)
   PRIVATE(this)->cbaction.addTriangleCallback(_node_::getClassTypeId(), SoToVRML2ActionP::triangle_cb, PRIVATE(this))
 #define ADD_SHAPE_CB(_node_, _cb_) \
   ADD_PRE_CB(_node_, _cb_); ADD_TRIANGLE_CB(_node_); ADD_POST_CB(_node_, post_primitives_cb); \
-  shapehandledlist.append(_node_::getClassTypeId());
+  add_shape_handled(_node_::getClassTypeId(), shapehandledlist);
+
 #define ADD_SO_TO_IFS(_node_) \
   ADD_PRE_CB(_node_, sotoifs_cb); ADD_TRIANGLE_CB(_node_); ADD_POST_CB(_node_, post_primitives_cb); \
-  shapehandledlist.append(_node_::getClassTypeId());
+  add_shape_handled(_node_::getClassTypeId(), shapehandledlist);
 
   SoTypeList shapehandledlist;
 
@@ -442,6 +515,7 @@ SoToVRML2Action::SoToVRML2Action(void)
   ADD_PRE_CB(SoSwitch, push_switch_cb);
   ADD_POST_CB(SoSwitch, pop_switch_cb);
   ADD_PRE_CB(SoLOD, push_lod_cb);
+  ADD_PRE_CB(SoLevelOfDetail, push_levelofdetail_cb);
   ADD_UNSUPPORTED(SoWWWAnchor); // Convert to SoVRMLAnchor
 
   // Other nodes
@@ -464,8 +538,6 @@ SoToVRML2Action::SoToVRML2Action(void)
   ADD_SO_TO_IFS(SoIndexedNurbsCurve);
   ADD_SO_TO_IFS(SoIndexedNurbsSurface);
 
-  ADD_SO_TO_IFS(SoProfile); // FIXME: Should this be here? 20020805 kristian.
-
   // find all shapes not handled earlier, and add generic triangle
   // handling for them
   //
@@ -493,6 +565,9 @@ SoToVRML2Action::SoToVRML2Action(void)
 
 SoToVRML2Action::~SoToVRML2Action(void)
 {
+  if (PRIVATE(this)->bboxaction) {
+    delete PRIVATE(this)->bboxaction;
+  }
   if (PRIVATE(this)->vrml2path) {
     PRIVATE(this)->vrml2path->unref();
   }
@@ -769,7 +844,7 @@ SoToVRML2ActionP::insert_shape(SoCallbackAction * action, SoVRMLGeometry * geom)
       SbVec2s size;
       int numComponents;
       const unsigned char * image = this->recentTex2->image.getValue(size, numComponents);
-      if (size[0] > 0 && size[1] > 0) {
+      if (!this->recentTex2->filename.isDefault() || (size[0] > 0 && size[1] > 0)) {
         SoVRMLTexture * tex;
         if (!this->recentTex2->filename.isDefault()) {
           tex = new SoVRMLImageTexture;
@@ -829,7 +904,7 @@ SoToVRML2ActionP::push_sep_cb(void * closure, SoCallbackAction * action, const S
   }
 
   // Push a new SoVRMLGroup on the tail of the path
-  SoVRMLGroup * newgroup = new SoVRMLGroup;
+  SoVRMLGroup * newgroup = NEW_NODE(SoVRMLGroup, node);
   prevgroup->addChild(newgroup);
   thisp->vrml2path->append(newgroup);
   return SoCallbackAction::CONTINUE;
@@ -868,7 +943,7 @@ SoToVRML2ActionP::push_switch_cb(void * closure, SoCallbackAction * action, cons
   }
 
   const SoSwitch * oldswitch = (const SoSwitch *) node;
-  SoVRMLSwitch * newswitch = new SoVRMLSwitch;
+  SoVRMLSwitch * newswitch = NEW_NODE(SoVRMLSwitch, node);
 
   // SO_SWITCH_INHERIT is not supported in VRML97, so just translate
   // it here. We could perhaps consider creating a ROUTE from the
@@ -945,6 +1020,66 @@ SoToVRML2ActionP::pop_switch_cb(void * closure, SoCallbackAction * action, const
   return SoCallbackAction::CONTINUE;
 }
 
+SoCallbackAction::Response
+SoToVRML2ActionP::push_levelofdetail_cb(void * closure, SoCallbackAction * action, const SoNode * node)
+{
+  SoToVRML2ActionP * thisp = (SoToVRML2ActionP*) closure;
+  SoGroup * prevgroup = THISP(closure)->get_current_tail();
+
+  SoGroup * vp;
+  if (THISP(closure)->dict.get(node, vp)) {
+    // Re-use previous subgraph
+    prevgroup->addChild(vp);
+    return SoCallbackAction::PRUNE;
+  }
+
+  const SoLevelOfDetail * oldlod = (const SoLevelOfDetail *) node;
+  SoVRMLLOD * newlod = NEW_NODE(SoVRMLLOD, node);
+
+  // calculate bbox of children to find a reasonable conversion to range 
+  SoGetBoundingBoxAction * bboxAction = thisp->getBBoxAction();
+  SbViewportRegion viewport(DEFAULT_VIEWPORT_WIDTH, DEFAULT_VIEWPORT_HEIGHT);
+  bboxAction->setViewportRegion(viewport);
+  // need to apply on the current path, not on the node, since we
+  // might need coordinates from the state. Also, we need to set the
+  // reset path so that we get the local bounding box for the nodes
+  // below this node.
+  bboxAction->setResetPath(action->getCurPath());
+  bboxAction->apply((SoPath*) action->getCurPath()); // find bbox of all children
+  SbBox3f bbox = bboxAction->getBoundingBox();
+
+  float dx, dy, dz;
+  bbox.getSize(dx,dy,dz);
+  const float h = SbMax(SbMax(dx,dy), dz);
+
+  // create a typical view volume
+  SbViewVolume vv;
+  vv.perspective(float(M_PI)/4.0f, DEFAULT_VIEWPORT_WIDTH/DEFAULT_VIEWPORT_HEIGHT,
+                 1.0f, 10.0f);
+
+
+  newlod->range.setNum(oldlod->screenArea.getNum());
+  float * rangeptr = newlod->range.startEditing();
+  int i;
+  for (i = 0; i < oldlod->screenArea.getNum(); i++) {
+    rangeptr[i] = thisp->getBBoxDistance(vv, oldlod->screenArea[i], h);
+  }
+  newlod->range.finishEditing();
+
+  prevgroup->addChild(newlod);
+  thisp->vrml2path->append(newlod);
+
+  // Traverse all children separately, that is, save and restore state between each
+  int n = oldlod->getNumChildren();
+  for (i=0; i < n; i++) {
+    action->switchToNodeTraversal(oldlod->getChild(i));
+  }
+
+  thisp->vrml2path->pop();
+  THISP(closure)->dict.put(node, newlod);
+  return SoCallbackAction::PRUNE;
+}
+
 SoCallbackAction::Response
 SoToVRML2ActionP::push_lod_cb(void * closure, SoCallbackAction * action, const SoNode * node)
 {
@@ -959,7 +1094,7 @@ SoToVRML2ActionP::push_lod_cb(void * closure, SoCallbackAction * action, const S
   }
 
   const SoLOD * oldlod = (const SoLOD *) node;
-  SoVRMLLOD * newlod = new SoVRMLLOD;
+  SoVRMLLOD * newlod = NEW_NODE(SoVRMLLOD, node);
 
   newlod->range.setValues(0, oldlod->range.getNum(), oldlod->range.getValues(0));
   newlod->center = oldlod->center.getValue();
@@ -981,7 +1116,7 @@ SoToVRML2ActionP::push_lod_cb(void * closure, SoCallbackAction * action, const S
 SoCallbackAction::Response
 SoToVRML2ActionP::unsupported_cb(void * closure, SoCallbackAction * action, const SoNode * node)
 {
-  SoVRMLWorldInfo * info = new SoVRMLWorldInfo;
+  SoVRMLWorldInfo * info = NEW_NODE(SoVRMLWorldInfo, node);
   SbString str;
   str.sprintf("Unsupported node: %s",
               node->getTypeId().getName().getString());
@@ -1002,7 +1137,7 @@ SoToVRML2ActionP::soasciitext_cb(void * closure, SoCallbackAction * action, cons
   if (action->getDrawStyle() != SoDrawStyle::FILLED) {
       return SoToVRML2ActionP::sotoifs_cb(closure, action, node);
   }
-  SoVRMLText * text = new SoVRMLText;
+  SoVRMLText * text = NEW_NODE(SoVRMLText, node);
   const SoAsciiText * oldtext = (const SoAsciiText*) node;
 
   text->string = oldtext->string;
@@ -1023,7 +1158,7 @@ SoToVRML2ActionP::socube_cb(void * closure, SoCallbackAction * action, const SoN
   if (action->getDrawStyle() != SoDrawStyle::FILLED) {
       return SoToVRML2ActionP::sotoifs_cb(closure, action, node);
   }
-  SoVRMLBox * box = new SoVRMLBox;
+  SoVRMLBox * box = NEW_NODE(SoVRMLBox, node);
   const SoCube * cube = (const SoCube*) node;
   if (box->size.getValue()[0] != cube->width.getValue() ||
       box->size.getValue()[1] != cube->height.getValue() ||
@@ -1040,7 +1175,7 @@ SoToVRML2ActionP::socone_cb(void * closure, SoCallbackAction * action, const SoN
   if (action->getDrawStyle() != SoDrawStyle::FILLED) {
       return SoToVRML2ActionP::sotoifs_cb(closure, action, node);
   }
-  SoVRMLCone * cone = new SoVRMLCone;
+  SoVRMLCone * cone = NEW_NODE(SoVRMLCone, node);
   const SoCone * oldcone = (const SoCone*) node;
 
   if (oldcone->bottomRadius != cone->bottomRadius)
@@ -1062,7 +1197,7 @@ SoToVRML2ActionP::socylinder_cb(void * closure, SoCallbackAction * action, const
   if (action->getDrawStyle() != SoDrawStyle::FILLED) {
       return SoToVRML2ActionP::sotoifs_cb(closure, action, node);
   }
-  SoVRMLCylinder * cyl = new SoVRMLCylinder;
+  SoVRMLCylinder * cyl = NEW_NODE(SoVRMLCylinder, node);
   const SoCylinder * oldcyl = (const SoCylinder*) node;
 
   if (oldcyl->radius != cyl->radius)
@@ -1095,7 +1230,7 @@ SoToVRML2ActionP::soifs_cb(void * closure, SoCallbackAction * action, const SoNo
 
   SoToVRML2ActionP * thisp = (SoToVRML2ActionP*) closure;
   SoGroup * tail = thisp->get_current_tail();
-  SoVRMLIndexedFaceSet * ifs = new SoVRMLIndexedFaceSet;
+  SoVRMLIndexedFaceSet * ifs = NEW_NODE(SoVRMLIndexedFaceSet, node);
 
   // Set the values from the current ShapeHints
   ifs->creaseAngle = action->getCreaseAngle();
@@ -1104,9 +1239,13 @@ SoToVRML2ActionP::soifs_cb(void * closure, SoCallbackAction * action, const SoNo
   ifs->convex = action->getFaceType() == SoShapeHints::CONVEX;
 
   // If there is a VertexProperty node set we need to put it on the state stack
-  if (oldifs->vertexProperty.getValue() != NULL) {
+  SoNode *vpnode = oldifs->vertexProperty.getValue();
+  SoVertexProperty *vp = 
+    (vpnode && vpnode->isOfType(SoVertexProperty::getClassTypeId())) ?
+    (SoVertexProperty *)vpnode : NULL;
+  if (vp) {
     action->getState()->push();
-    ((SoVertexProperty *)oldifs->vertexProperty.getValue())->callback(action);
+    vp->callback(action);
   }
 
   const SoCoordinateElement * coordElem = SoCoordinateElement::getInstance(action->getState());
@@ -1157,10 +1296,6 @@ SoToVRML2ActionP::soifs_cb(void * closure, SoCallbackAction * action, const SoNo
                                                        texcoordElem->getNum());
   }
 
-  if (oldifs->vertexProperty.getValue() != NULL) {
-    action->getState()->pop();
-  }
-
   ifs->coordIndex.setValues(0, oldifs->coordIndex.getNum(),
                             oldifs->coordIndex.getValues(0));
   if (!oldifs->textureCoordIndex.isDefault() &&
@@ -1178,15 +1313,21 @@ SoToVRML2ActionP::soifs_cb(void * closure, SoCallbackAction * action, const SoNo
   }
 
   THISP(closure)->insert_shape(action, ifs);
+
+  // it's important to pop state _after_ inserting the shape to get
+  // the correct material from SoVertexProperty nodes.
+  if (vp) {
+    action->getState()->pop();
+  }
+
   return SoCallbackAction::PRUNE;
 }
 
 SoCallbackAction::Response
 SoToVRML2ActionP::soils_cb(void * closure, SoCallbackAction * action, const SoNode * node)
 {
-  if (action->getDrawStyle() != SoDrawStyle::FILLED) {
-      return SoToVRML2ActionP::sotoifs_cb(closure, action, node);
-  }
+  // FIXME: test for drawstyle == POINTS and convert to a point set
+  // instead.  pederb, 20060327
   SoToVRML2ActionP * thisp = (SoToVRML2ActionP*) closure;
 
   const SoIndexedLineSet * oldils = (const SoIndexedLineSet*) node;
@@ -1195,13 +1336,17 @@ SoToVRML2ActionP::soils_cb(void * closure, SoCallbackAction * action, const SoNo
       oldils->coordIndex[0] < 0)
     return SoCallbackAction::CONTINUE;
 
-  SoVRMLIndexedLineSet * ils = new SoVRMLIndexedLineSet;
+  SoVRMLIndexedLineSet * ils = NEW_NODE(SoVRMLIndexedLineSet, node);
   SoGroup * tail = thisp->get_current_tail();
 
   // If there is a VertexProperty node set we need to put it on the state stack
-  if (oldils->vertexProperty.getValue() != NULL) {
+  SoNode *vpnode = oldils->vertexProperty.getValue();
+  SoVertexProperty *vp = 
+    (vpnode && vpnode->isOfType(SoVertexProperty::getClassTypeId())) ?
+    (SoVertexProperty *)vpnode : NULL;
+  if (vp) {
     action->getState()->push();
-    ((SoVertexProperty *)oldils->vertexProperty.getValue())->callback(action);
+    vp->callback(action);
   }
 
   SoVRMLCoordinate * newcoord = NULL;
@@ -1331,20 +1476,22 @@ SoToVRML2ActionP::soils_cb(void * closure, SoCallbackAction * action, const SoNo
     ils->colorPerVertex = TRUE;
   }
 
-  if (oldils->vertexProperty.getValue() != NULL) {
+  THISP(closure)->insert_shape(action, ils);
+
+  // it's important to pop state _after_ inserting the shape to get
+  // the correct material from SoVertexProperty nodes.
+  if (vp) {
     action->getState()->pop();
   }
 
-  THISP(closure)->insert_shape(action, ils);
   return SoCallbackAction::PRUNE;
 }
 
 SoCallbackAction::Response
 SoToVRML2ActionP::solineset_cb(void * closure, SoCallbackAction * action, const SoNode * node)
 {
-  if (action->getDrawStyle() != SoDrawStyle::FILLED) {
-      return SoToVRML2ActionP::sotoifs_cb(closure, action, node);
-  }
+  // FIXME: test for drawstyle == POINTS and convert to a point set
+  // instead.  pederb, 20060327
   SoToVRML2ActionP * thisp = (SoToVRML2ActionP*) closure;
 
   const SoLineSet * oldls = (const SoLineSet*) node;
@@ -1352,13 +1499,17 @@ SoToVRML2ActionP::solineset_cb(void * closure, SoCallbackAction * action, const
   if (oldls->numVertices.getNum() == 0)
     return SoCallbackAction::CONTINUE;
 
-  SoVRMLIndexedLineSet * ils = new SoVRMLIndexedLineSet;
+  SoVRMLIndexedLineSet * ils = NEW_NODE(SoVRMLIndexedLineSet, node);
   SoGroup * tail = thisp->get_current_tail();
 
   // If there is a VertexProperty node set we need to put it on the state stack
-  if (oldls->vertexProperty.getValue() != NULL) {
+  SoNode *vpnode = oldls->vertexProperty.getValue();
+  SoVertexProperty *vp = 
+    (vpnode && vpnode->isOfType(SoVertexProperty::getClassTypeId())) ?
+    (SoVertexProperty *)vpnode : NULL;
+  if (vp) {
     action->getState()->push();
-    ((SoVertexProperty *)oldls->vertexProperty.getValue())->callback(action);
+    vp->callback(action);
   }
 
   const SoCoordinateElement * coordElem = SoCoordinateElement::getInstance(action->getState());
@@ -1454,65 +1605,73 @@ SoToVRML2ActionP::solineset_cb(void * closure, SoCallbackAction * action, const
     ils->colorPerVertex = TRUE;
   }
 
-  if (oldls->vertexProperty.getValue() != NULL) {
+  THISP(closure)->insert_shape(action, ils);
+
+  // it's important to pop state _after_ inserting the shape to get
+  // the correct material from SoVertexProperty nodes.
+  if (vp) {
     action->getState()->pop();
   }
-
-  THISP(closure)->insert_shape(action, ils);
   return SoCallbackAction::PRUNE;
 }
 
 SoCallbackAction::Response
 SoToVRML2ActionP::sopointset_cb(void * closure, SoCallbackAction * action, const SoNode * node)
 {
-  if (action->getDrawStyle() != SoDrawStyle::FILLED) {
-      return SoToVRML2ActionP::sotoifs_cb(closure, action, node);
-  }
   SoToVRML2ActionP * thisp = (SoToVRML2ActionP*) closure;
 
   const SoPointSet * oldps = (const SoPointSet*) node;
 
-  SoVRMLPointSet * ps = new SoVRMLPointSet;
+  SoVRMLPointSet * ps = NEW_NODE(SoVRMLPointSet, node);
   SoGroup * tail = thisp->get_current_tail();
 
   // If there is a VertexProperty node set we need to put it on the state stack
-  if (oldps->vertexProperty.getValue() != NULL) {
+  SoNode *vpnode = oldps->vertexProperty.getValue();
+  SoVertexProperty *vp = 
+    (vpnode && vpnode->isOfType(SoVertexProperty::getClassTypeId())) ?
+    (SoVertexProperty *)vpnode : NULL;
+  if (vp) {
     action->getState()->push();
-    ((SoVertexProperty *)oldps->vertexProperty.getValue())->callback(action);
+    vp->callback(action);
   }
 
+
   const SoCoordinateElement * coordElem = SoCoordinateElement::getInstance(action->getState());
-  if (coordElem->getNum() > 0) {
+
+  int numpts = oldps->numPoints.getValue();
+  // if numPts == -1, use all coordinates on the stack
+  if (numpts < 0 || numpts > coordElem->getNum()) numpts = coordElem->getNum();
+
+  if (numpts) {
     if (coordElem->getArrayPtr3() != NULL) {
-      ps->coord = thisp->get_or_create_coordinate(coordElem->getArrayPtr3(),
-                                                  SbMin(coordElem->getNum(),
-                                                        oldps->numPoints.getValue()));
-    } else {
-      ps->coord = thisp->get_or_create_coordinate(coordElem->getArrayPtr4(),
-                                                  SbMin(coordElem->getNum(),
-                                                        oldps->numPoints.getValue()));
+      ps->coord = thisp->get_or_create_coordinate(coordElem->getArrayPtr3(), numpts);
+    } 
+    else {
+      ps->coord = thisp->get_or_create_coordinate(coordElem->getArrayPtr4(), numpts);
     }
   }
 
   if (action->getMaterialBinding() != SoMaterialBinding::OVERALL) {
     const SoLazyElement * colorElem = SoLazyElement::getInstance(action->getState());
-    int n = ((SoVRMLCoordinate*)ps->coord.getValue())->point.getNum();
-    if (colorElem->getNumDiffuse() >= n) {
+    if (colorElem->getNumDiffuse() >= numpts) {
       if (colorElem->isPacked()) {
         ps->color = thisp->get_or_create_color(colorElem->getPackedPointer(),
-                                               n);
-      } else {
+                                               numpts);
+      } 
+      else {
         ps->color = thisp->get_or_create_color(colorElem->getDiffusePointer(),
-                                               n);
+                                               numpts);
       }
     }
   }
 
-  if (oldps->vertexProperty.getValue() != NULL) {
+  THISP(closure)->insert_shape(action, ps);
+  // it's important to pop state _after_ inserting the shape to get
+  // the correct material from SoVertexProperty nodes.
+  if (vp) {
     action->getState()->pop();
   }
 
-  THISP(closure)->insert_shape(action, ps);
   return SoCallbackAction::PRUNE;
 }
 
@@ -1522,7 +1681,7 @@ SoToVRML2ActionP::sosphere_cb(void * closure, SoCallbackAction * action, const S
   if (action->getDrawStyle() != SoDrawStyle::FILLED) {
       return SoToVRML2ActionP::sotoifs_cb(closure, action, node);
   }
-  SoVRMLSphere * sphere = new SoVRMLSphere;
+  SoVRMLSphere * sphere = NEW_NODE(SoVRMLSphere, node);
   const SoSphere * oldsphere = (const SoSphere*) node;
   if (oldsphere->radius != sphere->radius)
     sphere->radius = oldsphere->radius.getValue();
@@ -1535,7 +1694,7 @@ SoCallbackAction::Response
 SoToVRML2ActionP::soinfo_cb(void * closure, SoCallbackAction * action, const SoNode * node)
 {
   const SoInfo * oldinfo = (const SoInfo*) node;
-  SoVRMLWorldInfo * info = new SoVRMLWorldInfo;
+  SoVRMLWorldInfo * info = NEW_NODE(SoVRMLWorldInfo, node);
   info->title = oldinfo->string;
   THISP(closure)->get_current_tail()->addChild(info);
   return SoCallbackAction::CONTINUE;
@@ -1545,7 +1704,7 @@ SoCallbackAction::Response
 SoToVRML2ActionP::somattrans_cb(void * closure, SoCallbackAction * action, const SoNode * node)
 {
   const SoMatrixTransform * oldt = (const SoMatrixTransform*) node;
-  SoVRMLTransform * newt = new SoVRMLTransform;
+  SoVRMLTransform * newt = NEW_NODE(SoVRMLTransform, node);
 
   SbVec3f translation, scaleFactor;
   SbRotation rotation, scaleOrientation;
@@ -1564,7 +1723,7 @@ SoCallbackAction::Response
 SoToVRML2ActionP::sorotation_cb(void * closure, SoCallbackAction * action, const SoNode * node)
 {
   const SoRotation * oldt = (const SoRotation*) node;
-  SoVRMLTransform * newt = new SoVRMLTransform;
+  SoVRMLTransform * newt = NEW_NODE(SoVRMLTransform, node);
   newt->rotation = oldt->rotation.getValue();
   THISP(closure)->get_current_tail()->addChild(newt);
   THISP(closure)->vrml2path->append(newt);
@@ -1575,7 +1734,7 @@ SoCallbackAction::Response
 SoToVRML2ActionP::sorotationxyz_cb(void * closure, SoCallbackAction * action, const SoNode * node)
 {
   const SoRotationXYZ * oldt = (const SoRotationXYZ*) node;
-  SoVRMLTransform * newt = new SoVRMLTransform;
+  SoVRMLTransform * newt = NEW_NODE(SoVRMLTransform, node);
   newt->rotation = oldt->getRotation();
   THISP(closure)->get_current_tail()->addChild(newt);
   THISP(closure)->vrml2path->append(newt);
@@ -1586,7 +1745,7 @@ SoCallbackAction::Response
 SoToVRML2ActionP::soscale_cb(void * closure, SoCallbackAction * action, const SoNode * node)
 {
   const SoScale * oldt = (const SoScale*) node;
-  SoVRMLTransform * newt = new SoVRMLTransform;
+  SoVRMLTransform * newt = NEW_NODE(SoVRMLTransform, node);
   newt->scale = oldt->scaleFactor.getValue();
   THISP(closure)->get_current_tail()->addChild(newt);
   THISP(closure)->vrml2path->append(newt);
@@ -1597,7 +1756,7 @@ SoCallbackAction::Response
 SoToVRML2ActionP::sotransform_cb(void * closure, SoCallbackAction * action, const SoNode * node)
 {
   const SoTransform * oldt = (const SoTransform*) node;
-  SoVRMLTransform * newt = new SoVRMLTransform;
+  SoVRMLTransform * newt = NEW_NODE(SoVRMLTransform, node);
 
   newt->translation = oldt->translation.getValue();
   newt->rotation = oldt->rotation.getValue();
@@ -1613,7 +1772,7 @@ SoCallbackAction::Response
 SoToVRML2ActionP::sotranslation_cb(void * closure, SoCallbackAction * action, const SoNode * node)
 {
   const SoTranslation * oldt = (const SoTranslation*) node;
-  SoVRMLTransform * newt = new SoVRMLTransform;
+  SoVRMLTransform * newt = NEW_NODE(SoVRMLTransform, node);
   newt->translation = oldt->translation.getValue();
   THISP(closure)->get_current_tail()->addChild(newt);
   THISP(closure)->vrml2path->append(newt);
@@ -1630,7 +1789,7 @@ SoToVRML2ActionP::sopercam_cb(void * closure, SoCallbackAction * action, const S
 SoCallbackAction::Response
 SoToVRML2ActionP::sodirlight_cb(void * closure, SoCallbackAction * action, const SoNode * node)
 {
-  SoVRMLDirectionalLight * dl = new SoVRMLDirectionalLight;
+  SoVRMLDirectionalLight * dl = NEW_NODE(SoVRMLDirectionalLight, node);
   const SoDirectionalLight * olddl = (const SoDirectionalLight *) node;
 
   dl->direction = olddl->direction.getValue();
@@ -1647,7 +1806,7 @@ SoToVRML2ActionP::sodirlight_cb(void * closure, SoCallbackAction * action, const
 SoCallbackAction::Response
 SoToVRML2ActionP::sowwwinl_cb(void * closure, SoCallbackAction * action, const SoNode * node)
 {
-  SoVRMLInline * inl = new SoVRMLInline;
+  SoVRMLInline * inl = NEW_NODE(SoVRMLInline, node);
   const SoWWWInline * oldinl = (const SoWWWInline *) node;
 
   inl->url = oldinl->name.getValue();
@@ -1665,6 +1824,19 @@ SoToVRML2ActionP::sotoifs_cb(void * closure, SoCallbackAction * action, const So
   SoToVRML2ActionP * thisp = (SoToVRML2ActionP*) closure;
   SoGroup * tail = thisp->get_current_tail();
 
+  thisp->didpush = FALSE;
+  // push state to handle SoVertexProperty node
+  if (node->isOfType(SoVertexShape::getClassTypeId())) {
+    SoNode *vpnode = ((SoVertexShape *) node)->vertexProperty.getValue();
+    SoVertexProperty *vp =
+      (vpnode && vpnode->isOfType(SoVertexProperty::getClassTypeId())) ?
+      (SoVertexProperty *)vpnode : NULL;
+    if (vp) {
+      action->getState()->push();
+      vp->callback(action);
+      thisp->didpush = TRUE;
+    }
+  }
   thisp->bsptree = new SbBSPTree;
   thisp->bsptreenormal = new SbBSPTree;
 
@@ -1727,7 +1899,7 @@ SoToVRML2ActionP::post_primitives_cb(void * closure, SoCallbackAction * action,
 
   SoVRMLGeometry * is;
   if (action->getDrawStyle() == SoDrawStyle::POINTS) {
-    SoVRMLPointSet * ps = new SoVRMLPointSet;
+    SoVRMLPointSet * ps = NEW_NODE(SoVRMLPointSet, node);
     is = ps;
 
     ps->coord = thisp->get_or_create_coordinate(thisp->bsptree->getPointsArrayPtr(),
@@ -1750,7 +1922,7 @@ SoToVRML2ActionP::post_primitives_cb(void * closure, SoCallbackAction * action,
 
   } else
   if (action->getDrawStyle() == SoDrawStyle::LINES) {
-    SoVRMLIndexedLineSet * ils = new SoVRMLIndexedLineSet;
+    SoVRMLIndexedLineSet * ils = NEW_NODE(SoVRMLIndexedLineSet, node);
     is = ils;
 
     ils->coord = thisp->get_or_create_coordinate(thisp->bsptree->getPointsArrayPtr(),
@@ -1787,7 +1959,7 @@ SoToVRML2ActionP::post_primitives_cb(void * closure, SoCallbackAction * action,
                               l.getArrayPtr());
 
   } else {
-    SoVRMLIndexedFaceSet * ifs = new SoVRMLIndexedFaceSet;
+    SoVRMLIndexedFaceSet * ifs = NEW_NODE(SoVRMLIndexedFaceSet, node);
     is = ifs;
 
     // Set the values from the current ShapeHints
@@ -1858,7 +2030,16 @@ SoToVRML2ActionP::post_primitives_cb(void * closure, SoCallbackAction * action,
   delete thisp->coloridx; thisp->coloridx = NULL;
 
   thisp->insert_shape(action, is);
+
+  if (thisp->didpush) {
+    action->getState()->pop();
+  }
   return SoCallbackAction::CONTINUE;
 }
 
+#undef NEW_NODE
+#undef DEFAULT_VIEWPORT_WIDTH
+#undef DEFAULT_VIEWPORT_HEIGHT
+
 #endif // HAVE_VRML97
+
