@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -102,6 +102,8 @@
 
 #include <Inventor/SoOutput.h>
 #include <Inventor/nodes/SoNode.h>
+#include <Inventor/sensors/SoNodeSensor.h>
+#include <Inventor/errors/SoDebugError.h>
 #include "../io/SoWriterefCounter.h"
 
 SO_ACTION_SOURCE(SoWriteAction);
@@ -192,6 +194,14 @@ SoWriteAction::continueToApply(SoPath * path)
   this->continuing = wascontinuing;
 }
 
+#if COIN_DEBUG
+static void sensorCB(void * data, SoSensor * sensor)
+{
+  SoDebugError::postWarning("SoWriteAction::SoWriteAction", 
+                            "Scenegraph changed during SoWriteAction().");
+}
+#endif
+
 // Documented for Doxygen in superclass.
 //
 // Overridden from parent class, as the write action is actually done
@@ -203,11 +213,23 @@ SoWriteAction::continueToApply(SoPath * path)
 void
 SoWriteAction::beginTraversal(SoNode * node)
 {
+#if COIN_DEBUG
+  SoNodeSensor *sensor = NULL;
+#endif
   if (this->continuing == FALSE) { // Run through both stages.
     // call SoWriterefCounter::instance() before traversing to set the
     // "current" pointer in SoWriterefCounter. This is needed to be
     // backwards compatible with old code that uses the writeref
     // system in SoBase.
+
+#if COIN_DEBUG
+    if (SoWriterefCounter::debugWriterefs()) {
+      sensor = new SoNodeSensor(sensorCB, NULL);
+      sensor->setPriority(0);
+      sensor->attach(node);
+    }
+#endif
+
     (void) SoWriterefCounter::instance(this->getOutput());
     this->outobj->setStage(SoOutput::COUNT_REFS);
     this->traverse(node);
@@ -220,6 +242,9 @@ SoWriteAction::beginTraversal(SoNode * node)
   }
   if (!this->continuing) {
     SoWriterefCounter::instance(this->getOutput())->debugCleanup();
+#if COIN_DEBUG
+    delete sensor;
+#endif
   }
 }
 
