@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -385,7 +385,15 @@ SbBox3f::getSpan(const SbVec3f & dir, float & dmin, float & dmax) const
   SbVec3f points[2]={this->min, this->max};
   SbVec3f corner;
   SbVec3f normdir(dir);
-  normdir.normalize(); // To avoid a division
+  if (normdir.normalize() == 0.0f) {
+#if COIN_DEBUG
+    SoDebugError::postWarning("SbBox3f::getSpan",
+                              "The direction is a null vector.");
+#endif // COIN_DEBUG
+
+    dmin = dmax = 0.0f;
+    return;
+  }
 
   for (int i=0;i<8;i++) {
     //Find all corners the "binary" way :-)
