@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -122,7 +122,8 @@ SbClip::clip(const SbPlane & plane)
 
     if (d0 >= 0.0f && d1 < 0.0f) { // exit plane
       SbVec3f dir = v1-v0;
-      dir.normalize();
+      // we know that v0 != v1 since we got here
+      (void) dir.normalize();
       float dot = dir.dot(planeN);
       SbVec3f newvertex = v0 - dir * (d0/dot);
       void * newdata = NULL;
@@ -133,7 +134,8 @@ SbClip::clip(const SbPlane & plane)
     }
     else if (d0 < 0.0f && d1 >= 0.0f) { // enter plane
       SbVec3f dir = v1-v0;
-      dir.normalize();
+      // we know that v0 != v1 since we got here
+      (void) dir.normalize();
       float dot = dir.dot(planeN);
       SbVec3f newvertex = v0 - dir * (d0/dot);
       void * newdata = NULL;
