@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -160,7 +160,13 @@ SbDPRotation::setValue(const double q0, const double q1,
   this->quat[1] = q1;
   this->quat[2] = q2;
   this->quat[3] = q3;
-  this->quat.normalize();
+  if (this->quat.normalize() == 0.0f) {
+#if COIN_DEBUG
+    SoDebugError::postWarning("SbRotation::setValue",
+                              "Quarternion has zero length => "
+                              "undefined rotation.");
+#endif // COIN_DEBUG    
+  }
   return *this;
 }
 
@@ -293,7 +299,13 @@ SbDPRotation::setValue(const double q[4])
   this->quat[1] = q[1];
   this->quat[2] = q[2];
   this->quat[3] = q[3];
-  this->quat.normalize();
+  if (this->quat.normalize() == 0.0f) {
+#if COIN_DEBUG
+    SoDebugError::postWarning("SbRotation::setValue",
+                              "Quarternion has zero length => "
+                              "undefined rotation.");
+#endif // COIN_DEBUG    
+  }
   return *this;
 }
 
@@ -362,7 +374,8 @@ SbDPRotation::setValue(const SbVec3d & axis, const double radians)
 
   const double sineval = (double)sin(radians/2);
   SbVec3d a = axis;
-  a.normalize();
+  // we test for a null vector above
+  (void) a.normalize();
   this->quat[0] = a[0] * sineval;
   this->quat[1] = a[1] * sineval;
   this->quat[2] = a[2] * sineval;
@@ -393,18 +406,20 @@ SbDPRotation::setValue(const SbVec3d & rotateFrom, const SbVec3d & rotateTo)
 #endif // COIN_DEBUG
 
   SbVec3d from(rotateFrom);
-  from.normalize();
+  // we test for a null vector above
+  (void) from.normalize();
   SbVec3d to(rotateTo);
-  to.normalize();
+  // we test for a null vector above
+  (void) to.normalize();
 
   const double dot = from.dot(to);
   SbVec3d crossvec = from.cross(to);
-  const double crosslen = crossvec.length();
+  const double crosslen = crossvec.normalize();
 
   if (crosslen == 0.0f) { // Parallel vectors
     // Check if they are pointing in the same direction.
-    if (dot > 0.0f) {
-      this->setValue(0.0f, 0.0f, 0.0f, 1.0f);
+    if (dot > 0.0) {
+      this->setValue(0.0, 0.0, 0.0, 1.0);
     }
     // Ok, so they are parallel and pointing in the opposite direction
     // of each other.
@@ -412,14 +427,14 @@ SbDPRotation::setValue(const SbVec3d & rotateFrom, const SbVec3d & rotateTo)
       // Try crossing with x axis.
       SbVec3d t = from.cross(SbVec3d(1.0f, 0.0f, 0.0f));
       // If not ok, cross with y axis.
-      if(t.length() == 0.0f) t = from.cross(SbVec3d(0.0f, 1.0f, 0.0f));
-
-      t.normalize();
+      if (t.normalize() == 0.0) {
+        t = from.cross(SbVec3d(0.0f, 1.0f, 0.0f));
+        (void) t.normalize();
+      }
       this->setValue(t[0], t[1], t[2], 0.0f);
     }
   }
   else { // Vectors are not parallel
-    crossvec.normalize();
     // The fabs() wrapping is to avoid problems when `dot' "overflows"
     // a tiny wee bit, which can lead to sqrt() returning NaN.
     crossvec *= (double)sqrt(0.5f * fabs(1.0f - dot));
