@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -479,16 +479,18 @@ SbImage::operator==(const SbImage & image) const
   this->readLock();
   int ret = 0;
   if (!PRIVATE(this)->schedulecb && !PRIVATE(&image)->schedulecb) {
-    if (PRIVATE(this)->size != PRIVATE(&image)->size) return FALSE;
-    if (PRIVATE(this)->bpp != PRIVATE(&image)->bpp) return FALSE;
-    if (PRIVATE(this)->bytes == NULL || PRIVATE(&image)->bytes == NULL) {
-      return (PRIVATE(this)->bytes == PRIVATE(&image)->bytes);
+    if (PRIVATE(this)->size != PRIVATE(&image)->size) ret = 0;
+    else if (PRIVATE(this)->bpp != PRIVATE(&image)->bpp) ret = 0;
+    else if (PRIVATE(this)->bytes == NULL || PRIVATE(&image)->bytes == NULL) {
+      ret = (PRIVATE(this)->bytes == PRIVATE(&image)->bytes);
+    }
+    else {
+      ret = memcmp(PRIVATE(this)->bytes, PRIVATE(&image)->bytes,
+                   int(PRIVATE(this)->size[0]) *
+                   int(PRIVATE(this)->size[1]) *
+                   int(PRIVATE(this)->size[2]==0?1:PRIVATE(this)->size[2]) * 
+                   PRIVATE(this)->bpp) == 0;
     }
-    SbBool ret = memcmp(PRIVATE(this)->bytes, PRIVATE(&image)->bytes,
-                        int(PRIVATE(this)->size[0]) *
-                        int(PRIVATE(this)->size[1]) *
-                        int(PRIVATE(this)->size[2]==0?1:PRIVATE(this)->size[2]) * 
-                        PRIVATE(this)->bpp) == 0;
   }
   this->readUnlock();
   return ret;
