@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -99,7 +99,8 @@ SbLine::setValue(const SbVec3f& origin, const SbVec3f& point)
   }
 #endif // COIN_DEBUG
 
-  this->dir.normalize();
+  // we test for a null vector above, just normalize
+  (void) this->dir.normalize();
 }
 
 /*!
@@ -207,7 +208,9 @@ SbLine::getClosestPoints(const SbLine& line2,
   SbVec3f D0 = this->dir;
   SbVec3f D1 = line2.dir;
   SbVec3f D0N = D0;
-  D0N.normalize();
+
+  // we warn about lines with no direction above, just normalize
+  (void) D0N.normalize();
 
   float c[3], d[3];
 
