@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -68,7 +68,8 @@ SbPlane::SbPlane(const SbVec3f& normalref, const float D)
 #endif // COIN_DEBUG
 
   this->normal = normalref;
-  this->normal.normalize();
+  // we test for a null vector above, just normalize
+  (void) this->normal.normalize();
   this->distance = D;
 }
 
@@ -95,7 +96,8 @@ SbPlane::SbPlane(const SbVec3f& p0, const SbVec3f& p1, const SbVec3f& p2)
   }
 #endif // COIN_DEBUG
 
-  this->normal.normalize();
+  // we test and warn about a null vector above
+  (void) this->normal.normalize();
   //     Npoint
   // d = -------, |N| == 1
   //       |N|
@@ -117,7 +119,9 @@ SbPlane::SbPlane(const SbVec3f& normalref, const SbVec3f& point)
 #endif // COIN_DEBUG
 
   this->normal = normalref;
-  this->normal.normalize();
+
+  // we test and warn about a null vector above
+  (void) this->normal.normalize();
 
   //     Npoint
   // d = -------, |N| == 1
@@ -207,8 +211,13 @@ SbPlane::transform(const SbMatrix& matrix)
 
   // the point should be transformed using the original matrix
   matrix.multVecMatrix(ptInPlane, ptInPlane);
-
-  this->normal.normalize();
+  
+  if (this->normal.normalize() == 0.0f) {
+#if COIN_DEBUG
+    SoDebugError::postWarning("SbPlane::transform",
+                              "The transformation invalidated the plane.");
+#endif // COIN_DEBUG
+  }
   this->distance = this->normal.dot(ptInPlane);
 }
 
