@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -76,10 +76,17 @@
 
   This class is not part of the original Open Inventor API.
 
-  (Another option for tessellating polygons is the tessellator of the
+
+  Another option for tessellating polygons is the tessellator of the
   GLU library. It has some features not part of SbTesselator (like
-  handling hulls), but the GLU library is known to have bugs in various
-  implementations and doesn't do Delaunay triangulation.)
+  handling hulls), but the GLU library is known to have bugs in
+  various implementations and doesn't do Delaunay triangulation. If
+  you however still prefer to use the GLU tessellator instead of this
+  one, that can be forced by setting an environment variable:
+
+  \code
+  (void) coin_setenv("COIN_PREFER_GLU_TESSELLATOR", "1", 1);
+  \endcode
 */
 
 
@@ -89,6 +96,7 @@
 #include <Inventor/SbBSPTree.h>
 #include <Inventor/SbSphere.h>
 #include <Inventor/lists/SbList.h>
+#include <Inventor/errors/SoDebugError.h>
 #include <stdio.h>
 #include <limits.h>
 #include <assert.h>
@@ -131,7 +139,7 @@ SbTesselator::heap_evaluate(void *v)
   SbTVertex *vertex = (SbTVertex*)v;
   if (vertex->dirtyweight) {
     vertex->dirtyweight = 0;
-    if (vertex->thisp->area(vertex) > PRIVATE(vertex->thisp)->epsilon &&
+    if ((vertex->thisp->area(vertex) > PRIVATE(vertex->thisp)->epsilon) &&
         vertex->thisp->isTriangle(vertex) &&
         vertex->thisp->clippable(vertex)) {
 #if 0 // testing code to avoid empty triangles
@@ -460,8 +468,28 @@ SbTesselator::pointInTriangle(SbTVertex *p, SbTVertex *t)
       (x < (v2[X] - v1[X]) * (y - v1[Y]) /  (v2[Y] - v1[Y]) + v1[X]))
     tst = (tst==FALSE ? TRUE : FALSE);
 
-#if 1
-  if (this->keepVertices && tst == FALSE) {
+  // the pointInTriangle test might fail for vertices that are on one
+  // of the triangle edges. Do a point_on_edge test for all three
+  // edges to handle this case. Example model that fails without this
+  // test:
+  // 
+  //  ShapeHints { faceType UNKNOWN_FACE_TYPE vertexOrdering CLOCKWISE }
+  //  IndexedFaceSet {
+  //    vertexProperty
+  //    VertexProperty {
+  //      vertex [ -0.3 0.05 0.0,
+  //                0.3   0.05  0.0,
+  //                0.3   0.1   0.0,
+  //                0.5   0.0   0.0,
+  //                0.3  -0.1   0.0,
+  //                0.3  -0.05  0.0,
+  //               -0.3  -0.05  0.0,
+  //               -0.3  -0.1   0.0,
+  //               -0.5   0.0   0.0,
+  //               -0.3   0.1   0.0 ]
+  //    }
+  //    coordIndex [ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, -1]
+  if (tst == FALSE) {
     if (point_on_edge(x,y,t->v.getValue(),
                       t->next->v.getValue(),X,Y, PRIVATE(this)->epsilon))
       return TRUE;
@@ -472,8 +500,6 @@ SbTesselator::pointInTriangle(SbTVertex *p, SbTVertex *t)
                       t->v.getValue(),X,Y, PRIVATE(this)->epsilon))
       return TRUE;
   }
-#endif
-
   return tst;
 }
 
@@ -534,7 +560,6 @@ SbTesselator::emitTriangle(SbTVertex *t)
 
   this->callback(t->data, t->next->data, t->next->next->data,
                  this->callbackData);
-
   this->cutTriangle(t);
 }
 
@@ -655,8 +680,13 @@ SbTesselator::calcPolygonNormal()
   polyNormal[0] += (vert1[1] - vert2[1]) * (vert1[2] + vert2[2]);
   polyNormal[1] += (vert1[2] - vert2[2]) * (vert1[0] + vert2[0]);
   polyNormal[2] += (vert1[0] - vert2[0]) * (vert1[1] + vert2[1]);
-
-  polyNormal.normalize();
+  
+  if (polyNormal.normalize() == 0.0f) {
+#if COIN_DEBUG
+    SoDebugError::postWarning("SbTesselator::calcPolygonNormal",
+                              "Polygon has no normal.");
+#endif // COIN_DEBUG
+  }
 }
 
 //
