@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -58,6 +58,7 @@
 #include <assert.h>
 #include <Inventor/SbVec3d.h>
 #include <Inventor/SbVec3f.h>
+#include <Inventor/C/tidbitsp.h> // coin_debug_normalize()
 
 #if COIN_DEBUG
 #include <Inventor/errors/SoDebugError.h>
@@ -233,14 +234,16 @@ double
 SbVec3d::normalize(void)
 {
   double len = this->length();
+  if (len > 0.0) {
+    operator/=(len);
+  }
 #if COIN_DEBUG
-  if (!(len > 0.0))
+  else if (coin_debug_normalize()) {
     SoDebugError::postWarning("SbVec3d::normalize",
                               "The length of the vector should be > 0.0 "
                               "to be able to normalize.");
+  }
 #endif // COIN_DEBUG
-
-  if (len > 0.0) operator/=(len);
   return len;
 }
 
