@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -57,6 +57,7 @@
 #include <float.h> // FLT_MAX
 #include <Inventor/SbPlane.h>
 #include <Inventor/SbVec3d.h>
+#include <Inventor/C/tidbitsp.h> // coin_debug_normalize()
 #if COIN_DEBUG
 #include <Inventor/errors/SoDebugError.h>
 #endif // COIN_DEBUG
@@ -327,19 +328,25 @@ SbVec3f::negate(void)
   length of the vector before normalization.
 
   If the vector is the null vector, no attempt at normalization will
-  be done, and if the Coin library was built in a debug version, this
-  error message will then be shown:
+  be done. If the Coin library was built in a debug version, and
+  the COIN_DEBUG_NORMALIZE environment variable is set, this error
+  message will then be shown:
 
   \verbatim
     Coin warning in SbVec3f::normalize(): The length of the vector
     should be > 0.0f to be able to normalize.
   \endverbatim
 
-  We've made Coin spit out a warning when an attempt at normalizing a
-  null-vector is made, as that seems to always be a symptom caused by
-  some graver error somewhere else -- either an internal error in Coin
-  code, a programming error in application code, or an error in an
-  input file (like for instance invalid polygon specifications).
+  We've made it possible for Coin to spit out a warning when an
+  attempt at normalizing a null-vector is made, as that seems to
+  sometimes be a symptom caused by some graver error somewhere else --
+  either an internal error in Coin code, a programming error in
+  application code, or an error in an input file (like for instance
+  invalid polygon specifications).
+
+  If you run into bugs/problems with your application or with Coin, it could be a
+  good idea to set COIN_DEBUG_NORMALIZE=1, and then restart the
+  application to see if you get any warnings from normalize().
 
   If this happens, you should run the application in a debugger and see
   how the call-stack backtrace looks when it hits. An easy way of
@@ -374,7 +381,7 @@ SbVec3f::normalize(void)
     operator/=(len);
   }
 #if COIN_DEBUG
-  else {
+  else if (coin_debug_normalize()) {
     SoDebugError::postWarning("SbVec3f::normalize",
                               "The length of the vector should be > 0.0f "
                               "to be able to normalize.");
