@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -116,7 +116,13 @@ cc_internal_gettimeofday(cc_time * t)
 static SbBool
 cc_internal_ftime(cc_time * t)
 {
-#ifdef HAVE__FTIME
+#ifdef HAVE__FTIME64_S
+  struct _timeb timebuffer;
+  _ftime64_s(&timebuffer);
+  /* FIXME: should use timezone field of struct _timeb aswell. 20011023 mortene. */
+  *t = (double)timebuffer.time + (double)timebuffer.millitm / 1000.0;
+  return TRUE;
+#elif defined(HAVE__FTIME)
   struct _timeb timebuffer;
   _ftime(&timebuffer);
   /* FIXME: should use timezone field of struct _timeb aswell. 20011023 mortene. */
