@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -45,6 +45,7 @@
 #include <Inventor/misc/SoNormalGenerator.h>
 #include <Inventor/lists/SbList.h>
 #include <Inventor/errors/SoDebugError.h>
+#include <Inventor/C/tidbitsp.h>
 
 #include "normalcache_numcoords_hack.h"
 
@@ -362,19 +363,21 @@ SoNormalCache::generatePerVertex(const SbVec3f * const coords,
                       threshold, tmpvec);
 
       // Be robust when it comes to erroneously specified triangles.
-      float len = tmpvec.length();
-
-      if (len <= 0.0f) {
+      if ((tmpvec.normalize() == 0.0f) && coin_debug_extra()) {
+#if COIN_DEBUG
         static uint32_t normgenerrors_vertex = 0;
         if (normgenerrors_vertex < 1) {
           SoDebugError::postWarning("SoNormalCache::generatePerVertex","Unable to "
                                     "generate valid normal for face %d", facenum);
         }
         normgenerrors_vertex++;
+#endif // COIN_DEBUG
       }
-
-      if (len > 0.0f) tmpvec /= len;
-      else tmpvec.setValue(1.0f, 0.0f, 0.0f); // dummy value
+      // it's really ok to have a null vector for a face/vertex, and we
+      // should not set it to some dummy vector. A null vector just
+      // means that the face is empty, and that the face shouldn't be
+      // considered when generating vertex normals.  
+      // pederb, 2005-12-21
 
       if (PRIVATE(this)->normalArray.getLength() <= nindex)
         PRIVATE(this)->normalArray.append(tmpvec);
@@ -461,16 +464,17 @@ SoNormalCache::generatePerFace(const SbVec3f * const coords,
 
     if (v0 < 0 || v1 < 0 || v2 < 0 ||
         v0 > maxcoordidx || v1 > maxcoordidx || v2 > maxcoordidx) {
-
+#if COIN_DEBUG
       SoDebugError::postWarning("SoNormalCache::generatePerFace",
                                 "Polygon with less than three valid "
                                 "vertices detected. (offset: %d, [%d %d %d]). "
                                 "Should be within [0, %d].",
                                 cind - cstart, v0, v1, v2, maxcoordidx);
+#endif // COIN_DEBUG
 
        // Insert dummy normal for robustness
       SbVec3f dummynormal;
-      dummynormal.setValue(1.0, 0.0, 0.0);
+      dummynormal.setValue(0.0f, 0.0f, 0.0f);
       PRIVATE(this)->normalArray.append(dummynormal);
 
       // Skip ahead to next possible index
@@ -498,14 +502,9 @@ SoNormalCache::generatePerFace(const SbVec3f * const coords,
         tmpvec = (coords[v2] - coords[v1]).cross(coords[v0] - coords[v1]);
 
       // Be robust when it comes to erroneously specified triangles.
-      float len = tmpvec.length();
-
-      if (len <= 0.0f) {
+      if ((tmpvec.normalize() == 0.0f) && coin_debug_extra()) {
         static uint32_t normgenerrors_face = 0;
         if (normgenerrors_face < 1) {
-          // FIXME: this is too pedantic. Wrap within a check for a
-          // run-time "pedantic" getenv() variable? Note: there are
-          // many more like this in this file. 20050407 mortene.
           SoDebugError::postWarning("SoNormalCache::generatePerFace",
                                     "Erroneous triangle specification in model "
                                     "(indices= [%d, %d, %d], "
@@ -519,9 +518,7 @@ SoNormalCache::generatePerFace(const SbVec3f * const coords,
         }
         normgenerrors_face++;
       }
-
-      if (len > 0.0f) tmpvec /= len;
-      else tmpvec.setValue(1.0f, 0.0f, 0.0f); // dummy value
+      
       PRIVATE(this)->normalArray.append(tmpvec);
       cind += 4; // goto next triangle/polygon
     }
@@ -550,9 +547,7 @@ SoNormalCache::generatePerFace(const SbVec3f * const coords,
       tmpvec[2] += ((*vert1)[0] - (*vert2)[0]) * ((*vert1)[1] + (*vert2)[1]);
 
       // Be robust when it comes to erroneously specified polygons.
-      float len = tmpvec.length();
-
-      if (len <= 0.0f) {
+      if ((tmpvec.normalize() == 0.0f) && coin_debug_extra()) {
         static uint32_t normgenerrors_face = 0;
         if (normgenerrors_face < 1) {
           SoDebugError::postWarning("SoNormalCache::generatePerFace",
@@ -564,20 +559,19 @@ SoNormalCache::generatePerFace(const SbVec3f * const coords,
         normgenerrors_face++;
       }
 
-      if (len > 0.0f) tmpvec /= len;
-      else tmpvec.setValue(1.0f, 0.0f, 0.0f); // dummy value
       PRIVATE(this)->normalArray.append(ccw ? tmpvec : -tmpvec);
       cind++; // skip the -1
     }
   }
 
   if (endptr - cind > 0) {
+#if COIN_DEBUG
     SoDebugError::postWarning("SoNormalCache::generatePerFace", "Face "
                               "specification did not end with a valid "
                               "polygon. Too few points");
-
+#endif // COIN_DEBUG
     SbVec3f dummynormal;
-    dummynormal.setValue(1.0, 0.0, 0.0);
+    dummynormal.setValue(0.0f, 0.0f, 0.0f);
     PRIVATE(this)->normalArray.append(dummynormal);
   }
 
@@ -628,15 +622,16 @@ SoNormalCache::generatePerFaceStrip(const SbVec3f * const coords,
   while (cind + 2 < endptr) {
     if (cind[0] < 0 || cind[1] < 0 || cind[2] < 0 ||
         cind[0] > maxcoordidx || cind[1] > maxcoordidx || cind[2] > maxcoordidx) {
-
+#if COIN_DEBUG
       SoDebugError::postWarning("SoNormalCache::generatePerFaceStrip", "Erroneous "
                                 "coordinate index detected (offset: %d, [%d %d %d]). Should be "
                                 "within [0, %d].",
                                 cind - cstart, *(cind), *(cind+1), *(cind+2), maxcoordidx);
+#endif // COIN_DEBUG
 
       // Insert dummy normal for robustness
       SbVec3f dummynormal;
-      dummynormal.setValue(1.0, 0.0, 0.0);
+      dummynormal.setValue(0.0, 0.0, 0.0);
       PRIVATE(this)->normalArray.append(dummynormal);
 
       // Skip to next possibly valid index
@@ -667,11 +662,8 @@ SoNormalCache::generatePerFaceStrip(const SbVec3f * const coords,
     else
       n = (*c2 - *c1).cross(*c0 - *c1);
 
-    // Be robust when it comes to erroneously specified polygons.
-    float len = n.length();
-    
     static uint32_t normgenerrors_facestrip = 0;
-    if (len <= 0.0f) {
+    if ((n.normalize() == 0.0f) && coin_debug_extra()) {
       if (normgenerrors_facestrip < 1) {
         SoDebugError::postWarning("SoNormalCache::generatePerFaceStrip",
                                   "Erroneous triangle specification in model "
@@ -688,8 +680,6 @@ SoNormalCache::generatePerFaceStrip(const SbVec3f * const coords,
       normgenerrors_facestrip++;
     }
     
-    if (len > 0.0f) n /= len;
-    else n.setValue(1.0f, 0.0f, 0.0f); // dummy value
     PRIVATE(this)->normalArray.append(n);
 
     int idx = cind < endptr ? *cind++ : -1;
@@ -703,10 +693,7 @@ SoNormalCache::generatePerFaceStrip(const SbVec3f * const coords,
       else
         n = (*c2 - *c1).cross(*c0 - *c1);
 
-      // Be robust when it comes to erroneously specified polygons.
-      float len = n.length();
-
-      if (len <= 0.0f) {
+      if ((n.normalize() == 0.0f) && coin_debug_extra()) {
         if (normgenerrors_facestrip < 1) {
           SoDebugError::postWarning("SoNormalCache::generatePerFaceStrip",
                                     "Erroneous triangle specification in model "
@@ -720,12 +707,10 @@ SoNormalCache::generatePerFaceStrip(const SbVec3f * const coords,
         normgenerrors_facestrip++;
       }
 
-      if (len > 0.0f) n /= len;
-      else n.setValue(1.0f, 0.0f, 0.0f); // dummy value
       PRIVATE(this)->normalArray.append(n);
       idx = cind < endptr ? *cind++ : -1;
     }
-
+#if COIN_DEBUG
     if (idx > maxcoordidx) {
       static uint32_t normgenerrors_facestrip = 0;
       if (normgenerrors_facestrip < 1) {
@@ -738,15 +723,17 @@ SoNormalCache::generatePerFaceStrip(const SbVec3f * const coords,
       }
       normgenerrors_facestrip++;
     }
+#endif // COIN_DEBUG
   }
 
   if (endptr - cind > 0) {
+#if COIN_DEBUG
     SoDebugError::postWarning("SoNormalCache::generatePerFaceStrip", "Strip "
                               "did not end with a valid polygon. Too few "
                               "points");
-
+#endif // COIN_DEBUG
     SbVec3f dummynormal;
-    dummynormal.setValue(1.0, 0.0, 0.0);
+    dummynormal.setValue(0.0, 0.0, 0.0);
     PRIVATE(this)->normalArray.append(dummynormal);
   }
 
@@ -798,15 +785,15 @@ SoNormalCache::generatePerStrip(const SbVec3f * const coords,
   while (cind + 2 < endptr) {
     if (cind[0] < 0 || cind[1] < 0 || cind[2] < 0 ||
         cind[0] > maxcoordidx || cind[1] > maxcoordidx || cind[2] > maxcoordidx) {
-
+#if COIN_DEBUG
       SoDebugError::postWarning("SoNormalCache::generatePerStrip", "Erroneous "
                                 "coordinate index detected (offset: %d, [%d %d %d]). Should be "
                                 "within [0, %d].",
                                 cind - cstart, *(cind), *(cind+1), *(cind+2), maxcoordidx);
-
+#endif // COIN_DEBUG
       // Insert dummy normal for robustness
       SbVec3f dummynormal;
-      dummynormal.setValue(1.0, 0.0, 0.0);
+      dummynormal.setValue(0.0f, 0.0f, 0.0f);
       PRIVATE(this)->normalArray.append(dummynormal);
 
       // Skip to next possibly valid index
@@ -850,6 +837,7 @@ SoNormalCache::generatePerStrip(const SbVec3f * const coords,
       idx = cind < endptr ? *cind++ : -1;
     }
 
+#if COIN_DEBUG
     if (idx > maxcoordidx) {
       static uint32_t normgenerrors_strip = 0;
       if (normgenerrors_strip < 1) {
@@ -862,11 +850,9 @@ SoNormalCache::generatePerStrip(const SbVec3f * const coords,
       }
       normgenerrors_strip++;
     }
+#endif // COIN_DEBUG
 
-    // Be robust when it comes to erroneously specified polygons.
-    float len = n.length();
-    
-    if (len <= 0.0f) {
+    if ((n.normalize() == 0.0f) && coin_debug_extra()) {
       static uint32_t normgenerrors_strip = 0;
       if (normgenerrors_strip < 1) {
         SoDebugError::postWarning("SoNormalCache::generatePerStrip",
@@ -879,17 +865,16 @@ SoNormalCache::generatePerStrip(const SbVec3f * const coords,
       normgenerrors_strip++;
     }
     
-    if (len > 0.0f) n /= len;
-    else n.setValue(1.0f, 0.0f, 0.0f); // dummy value
     PRIVATE(this)->normalArray.append(n);
   }
 
   if (endptr - cind > 0) {
+#if COIN_DEBUG
     SoDebugError::postWarning("SoNormalCache::generatePerStrip", "Strip did "
                               "not end with a valid polygon. Too few points");
-
+#endif // COIN_DEBUG
     SbVec3f dummynormal;
-    dummynormal.setValue(1.0, 0.0, 0.0);
+    dummynormal.setValue(0.0, 0.0, 0.0);
     PRIVATE(this)->normalArray.append(dummynormal);
   }
 
@@ -950,10 +935,7 @@ SoNormalCache::generatePerVertexQuad(const SbVec3f * const coords,
       if (j > 0 && i > 0 && idx3 < numfacenormals) n += facenormals[idx3];
       if (j > 0 && i < vPerColumn-1 && idx4 < numfacenormals) n += facenormals[idx4];
 
-      // Be robust when it comes to erroneously specified polygons.
-      float len = n.length();
-
-      if (len <= 0.0f) {
+      if ((n.normalize() == 0.0f) && coin_debug_extra()) {
         static uint32_t normgenerrors_vertexquad = 0;
         if (normgenerrors_vertexquad < 1) {
           SoDebugError::postWarning("SoNormalCache::generatePerVertexQuad",
@@ -963,10 +945,7 @@ SoNormalCache::generatePerVertexQuad(const SbVec3f * const coords,
                                     "but there might be more errors).");
         }
         normgenerrors_vertexquad++;
-      }
-        
-      if (len > 0.0f) n /= len;
-      else n.setValue(1.0f, 0.0f, 0.0f); // dummy value
+      }        
       PRIVATE(this)->normalArray.append(ccw ? -n : n);
     }
   }
@@ -1004,6 +983,7 @@ SoNormalCache::generatePerFaceQuad(const SbVec3f * const coords,
   this->clearGenerator();
   PRIVATE(this)->normalArray.truncate(0, TRUE);
 
+#if COIN_DEBUG
   if (vPerRow <= 1 || vPerColumn <= 1 || 
       (unsigned int)(vPerRow * vPerColumn) > numcoords) {
 
@@ -1014,6 +994,7 @@ SoNormalCache::generatePerFaceQuad(const SbVec3f * const coords,
                               "<= number of coordinates available.", 
                               vPerRow, vPerColumn, numcoords);
   }
+#endif // COIN_DEBUG
 
 #define IDX(r, c) ((r)*(vPerRow)+(c))
 
@@ -1027,9 +1008,7 @@ SoNormalCache::generatePerFaceQuad(const SbVec3f * const coords,
         SbVec3f n = (coords[idx2] - coords[idx1]).cross(coords[idx3] - coords[idx1]);
 
         // Be robust when it comes to erroneously specified polygons.
-        float len = n.length();
-
-        if (len <= 0.0f) {
+        if ((n.normalize() == 0.0f) && coin_debug_extra())  {
           static uint32_t normgenerrors_facequad = 0;
           if (normgenerrors_facequad < 1) {
             SoDebugError::postWarning("SoNormalCache::generatePerFaceQuad",
@@ -1046,13 +1025,11 @@ SoNormalCache::generatePerFaceQuad(const SbVec3f * const coords,
           normgenerrors_facequad++;
         }
         
-        if (len > 0.0f) n /= len;
-        else n.setValue(1.0f, 0.0f, 0.0f); // dummy value
         PRIVATE(this)->normalArray.append(ccw ? -n : n);
       }
       else {
         // Generate normals even for invalid input
-        SbVec3f dummynormal(1.0, 0.0, 0.0);
+        SbVec3f dummynormal(0.0, 0.0, 0.0);
         PRIVATE(this)->normalArray.append(ccw ? -dummynormal : dummynormal);
       }
     }
@@ -1095,9 +1072,9 @@ SoNormalCache::generatePerRowQuad(const SbVec3f * const coords,
   PRIVATE(this)->normalArray.truncate(0, TRUE);
   SbVec3f n;
 
+#if COIN_DEBUG
   if (vPerRow <= 1 || vPerColumn <= 1 || 
       (unsigned int)(vPerRow * vPerColumn) > numcoords) {
-
     SoDebugError::postWarning("SoNormalCache::generatePerRowQuad", "Illegal "
                               "facequad dimension: [%d %d] with %d coordinates "
                               "available. verticesPerRow and verticesPerColumn "
@@ -1105,6 +1082,7 @@ SoNormalCache::generatePerRowQuad(const SbVec3f * const coords,
                               "<= number of coordinates available.", 
                               vPerRow, vPerColumn, numcoords);
   }
+#endif // COIN_DEBUG
 
 #define IDX(r, c) ((r)*(vPerRow)+(c))
 
@@ -1121,25 +1099,20 @@ SoNormalCache::generatePerRowQuad(const SbVec3f * const coords,
     }
 
     // Be robust when it comes to erroneously specified polygons.
-    float len = n.length();
-
-    if (len <= 0.0f) {
+    if ((n.normalize() == 0.0f) && coin_debug_extra()) {
       static uint32_t normgenerrors_rowquad = 0;
       if (normgenerrors_rowquad < 1) {
         SoDebugError::postWarning("SoNormalCache::generatePerRowQuad",
                                   "Erroneous polygon specification in model. "
-                                  "Unable to generate valid normal, adding dummy. "
+                                  "Unable to generate valid normal, adding null vector. "
                                   "(this warning will be printed only once, "
                                   "but there might be more errors).");
       }
       normgenerrors_rowquad++;
-    }
-    
-    if (len > 0.0f) n /= len;
-    else n.setValue(1.0f, 0.0f, 0.0f); // dummy value
+    }    
     PRIVATE(this)->normalArray.append(ccw ? -n : n);
   }
-
+  
 #undef IDX
 
   if (PRIVATE(this)->normalArray.getLength()) {
