@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -42,33 +42,24 @@
 #include <Inventor/details/SoLineDetail.h>
 #include <Inventor/details/SoPointDetail.h>
 #include <Inventor/elements/SoBumpMapCoordinateElement.h>
+#include <Inventor/elements/SoViewVolumeElement.h>
+#include <Inventor/elements/SoModelMatrixElement.h>
 #include <Inventor/elements/SoGLCacheContextElement.h>
+#include <Inventor/elements/SoGLLazyElement.h>
 #include <Inventor/elements/SoLazyElement.h>
 #include <Inventor/elements/SoMultiTextureCoordinateElement.h>
 #include <Inventor/elements/SoMultiTextureEnabledElement.h>
+#include <Inventor/elements/SoGLVBOElement.h>
 #include <Inventor/lists/SbList.h>
 #include <Inventor/misc/SbHash.h>
 #include <Inventor/misc/SoGL.h>
 #include <Inventor/system/gl.h>
-#include <Inventor/misc/SoContextHandler.h>
+#include <Inventor/SbPlane.h>
+#include "../misc/SoVBO.h"
+#include "../misc/SoVertexArrayIndexer.h"
 
 // *************************************************************************
 
-#define MAX_UNITS 16
-
-typedef struct  {
-  GLuint triangleindex;
-  GLuint vertex;
-  GLuint normal;
-  GLuint texcoord0;
-  GLuint rgba;
-  GLuint multitex[MAX_UNITS];
-} SoPrimitiveVertexCache_vboidx;
-
-static unsigned long total_vbo_memory = 0;
-static unsigned long COIN_MAX_VBO_MEMORY = 0;
-static int COIN_ENABLE_VBO = -1;
-
 class SoPrimitiveVertexCacheP {
 public:
   SoPrimitiveVertexCacheP(void)
@@ -77,9 +68,16 @@ public:
       texcoordlist(256),
       bumpcoordlist(256),
       rgbalist(256),
-      indices(1024),
       vhash(1024),
-      vbodict(4) { }
+      deptharray(NULL),
+      triangleindexer(NULL),
+      lineindexer(NULL),
+      pointindexer(NULL),
+      vertexvbo(NULL),
+      normalvbo(NULL),
+      texcoord0vbo(NULL),
+      rgbavbo(NULL)
+  { }
 
   SbList <SoPrimitiveVertexCache::Vertex> vertices;
 
@@ -88,10 +86,6 @@ public:
   SbList <SbVec4f> texcoordlist;
   SbList <SbVec2f> bumpcoordlist;
   SbList <uint8_t> rgbalist;
-
-  SbList <int32_t> indices;
-  SbList <int32_t> lineindices;
-  SbList <int32_t> pointindices;
   SbHash <int32_t, SoPrimitiveVertexCache::Vertex> vhash;
   
   const SbVec2f * bumpcoords;
@@ -113,9 +107,27 @@ public:
   const SoMultiTextureCoordinateElement * multielem;
   SbList <SbVec4f> * multitexcoords;
   SoState * state;
-  SbHash<SoPrimitiveVertexCache_vboidx *, uint32_t> vbodict;
+  SbPlane prevsortplane;
+  float * deptharray;
+
+  SoVertexArrayIndexer * triangleindexer;
+  SoVertexArrayIndexer * lineindexer;
+  SoVertexArrayIndexer * pointindexer;
+  SoVBO * vertexvbo;
+  SoVBO * normalvbo;
+  SoVBO * texcoord0vbo;
+  SoVBO * rgbavbo;
+  SbList <SoVBO*> multitexvbo;
 
   void addVertex(const SoPrimitiveVertexCache::Vertex & v);
+
+  void renderImmediate(const cc_glglue * glue,
+                       const int32_t * indices,
+                       const int numindices,
+                       const SbBool color, const SbBool normal,
+                       const SbBool texture, const SbBool * enabled,
+                       const int lastenabled);
+
   void enableArrays(const cc_glglue * glue,
                     const SbBool color, const SbBool normal,
                     const SbBool texture, const SbBool * enabled,
@@ -143,9 +155,6 @@ public:
                              const SbBool texture, const SbBool * enabled,
                              const int lastenabled);
 
-  static void vbo_schedule(const uint32_t & key, SoPrimitiveVertexCache_vboidx * const & value, void * closure);
-  static void vbo_delete(void * closure, uint32_t contextid);
-  static void contextCleanup(uint32_t context, void * closure);
 };
 
 #define PRIVATE(obj) ((obj)->pimpl)
@@ -160,8 +169,6 @@ SoPrimitiveVertexCache::SoPrimitiveVertexCache(SoState * state)
 {
   PRIVATE(this) = new SoPrimitiveVertexCacheP;
   PRIVATE(this)->state = state;
-  SoContextHandler::addContextDestructionCallback(SoPrimitiveVertexCacheP::contextCleanup, 
-                                                  PRIVATE(this));
   const SoBumpMapCoordinateElement * belem =
     SoBumpMapCoordinateElement::getInstance(state);
 
@@ -210,21 +217,6 @@ SoPrimitiveVertexCache::SoPrimitiveVertexCache(SoState * state)
     // triangle callback. SoTextureCoordinateBundle might push a new
     // element.
   }
-
-  if (COIN_MAX_VBO_MEMORY == 0) {
-    COIN_MAX_VBO_MEMORY = 0xffffffff;
-    const char * env = coin_getenv("COIN_MAX_VBO_MEMORY");
-    if (env) {
-      COIN_MAX_VBO_MEMORY = (unsigned long) atol(env);
-    }
-  }
-  if (COIN_ENABLE_VBO < 0) {
-    COIN_ENABLE_VBO = 0;
-    const char * env = coin_getenv("COIN_ENABLE_VBO");
-    if (env) {
-      COIN_ENABLE_VBO = atoi(env);
-    }
-  }
 }
 
 /*!
@@ -232,13 +224,21 @@ SoPrimitiveVertexCache::SoPrimitiveVertexCache(SoState * state)
 */
 SoPrimitiveVertexCache::~SoPrimitiveVertexCache()
 {
-  SoContextHandler::removeContextDestructionCallback(SoPrimitiveVertexCacheP::contextCleanup, 
-                                                     PRIVATE(this));
+  delete PRIVATE(this)->triangleindexer;
+  delete PRIVATE(this)->lineindexer;
+  delete PRIVATE(this)->pointindexer;
+  delete PRIVATE(this)->vertexvbo;
+  delete PRIVATE(this)->normalvbo;
+  delete PRIVATE(this)->texcoord0vbo;
+  delete PRIVATE(this)->rgbavbo;
 
-  PRIVATE(this)->vbodict.apply(SoPrimitiveVertexCacheP::vbo_schedule, NULL);
+  for (int i = 0; i < PRIVATE(this)->multitexvbo.getLength(); i++) {
+    delete PRIVATE(this)->multitexvbo[i];
+  }
   if (PRIVATE(this)->lastenabled >= 1) {
     delete[] PRIVATE(this)->multitexcoords;
   }
+  delete[] PRIVATE(this)->deptharray;
   delete PRIVATE(this);
 }
 
@@ -259,30 +259,44 @@ SoPrimitiveVertexCache::renderTriangles(SoState * state, const int arrays) const
 
   const uint32_t contextid = SoGLCacheContextElement::get(state);
   const cc_glglue * glue = cc_glglue_instance((int) contextid);
-  SbBool renderasvbo = COIN_ENABLE_VBO && cc_glglue_has_vertex_buffer_object(glue);
+  
+  int numvertex = PRIVATE(this)->vertexlist.getLength();
 
-  if (renderasvbo) {
-    unsigned long size = PRIVATE(this)->countVBOSize(glue, contextid, color,
-                                                     normal, texture, enabled, lastenabled);
-    if (total_vbo_memory + size > COIN_MAX_VBO_MEMORY) {
-      renderasvbo = FALSE;
-    }
-    else {
-      total_vbo_memory += size;
-    }
-  }
+  SbBool renderasvbo = 
+    PRIVATE(this)->vertexvbo || 
+#ifdef COIN_NEXT_MINOR
+    SoGLVBOElement::shouldCreateVBO(state, PRIVATE(this)->vertexlist.getLength());
+#else
+  (numvertex >= SoVBO::getVertexCountMinLimit() && 
+   numvertex <= SoVBO::getVertexCountMaxLimit() &&
+   cc_glglue_has_vertex_buffer_object(glue));
+#endif
 
   if (renderasvbo) {
     PRIVATE(this)->enableVBOs(glue, contextid, color, normal, texture, enabled, lastenabled);
-    cc_glglue_glDrawElements(glue, GL_TRIANGLES, n, GL_UNSIGNED_INT, NULL);
+    PRIVATE(this)->triangleindexer->render(glue, TRUE, contextid);
     PRIVATE(this)->disableVBOs(glue, color, normal, texture, enabled, lastenabled);
   }
-  else {
+  else if (cc_glglue_has_vertex_array(glue)) {
     PRIVATE(this)->enableArrays(glue, color, normal, texture, enabled, lastenabled);
-    cc_glglue_glDrawElements(glue, GL_TRIANGLES, n, GL_UNSIGNED_INT,
-                             (const GLvoid*) this->getIndices());
+    PRIVATE(this)->triangleindexer->render(glue, FALSE, contextid);
     PRIVATE(this)->disableArrays(glue, color, normal, texture, enabled, lastenabled);
   }
+  else {
+    // fall back to immediate mode rendering
+    glBegin(GL_TRIANGLES);
+    PRIVATE(this)->renderImmediate(glue,
+                                   this->getIndices(),
+                                   this->getNumIndices(),
+                                   color, normal, texture, enabled, lastenabled);
+    glEnd();
+  }
+
+  // inform SoGLLazyElement that we might have changed the current color
+  if (color) {
+    SoGLLazyElement::getInstance(state)->reset(state, 
+                                               SoLazyElement::DIFFUSE_MASK);
+  }
 }
 
 void
@@ -300,11 +314,27 @@ SoPrimitiveVertexCache::renderLines(SoState * state, const int arrays) const
     enabled = SoMultiTextureEnabledElement::getEnabledUnits(state, lastenabled);
   }
   const cc_glglue * glue = sogl_glue_instance(state);
+  const uint32_t contextid = SoGLCacheContextElement::get(state);
 
-  PRIVATE(this)->enableArrays(glue, color, normal, texture, enabled, lastenabled);
-  cc_glglue_glDrawElements(glue, GL_LINES, n, GL_UNSIGNED_INT,
-                           (const GLvoid*) this->getLineIndices());
-  PRIVATE(this)->disableArrays(glue, color, normal, texture, enabled, lastenabled);
+  if (cc_glglue_has_vertex_array(glue)) {
+    PRIVATE(this)->enableArrays(glue, color, normal, texture, enabled, lastenabled);
+    PRIVATE(this)->lineindexer->render(glue, FALSE, contextid);
+    PRIVATE(this)->disableArrays(glue, color, normal, texture, enabled, lastenabled);
+  }
+  else {
+    // fall back to immediate mode rendering
+    glBegin(GL_TRIANGLES);
+    PRIVATE(this)->renderImmediate(glue,
+                                   this->getLineIndices(),
+                                   this->getNumLineIndices(),
+                                   color, normal, texture, enabled, lastenabled);
+    glEnd();
+  }
+  // inform SoGLLazyElement that we might have changed the current color
+  if (color) {
+    SoGLLazyElement::getInstance(state)->reset(state, 
+                                               SoLazyElement::DIFFUSE_MASK);
+  }
 }
 
 void
@@ -322,11 +352,27 @@ SoPrimitiveVertexCache::renderPoints(SoState * state, const int arrays) const
     enabled = SoMultiTextureEnabledElement::getEnabledUnits(state, lastenabled);
   }
   const cc_glglue * glue = sogl_glue_instance(state);
+  const uint32_t contextid = SoGLCacheContextElement::get(state);
 
-  PRIVATE(this)->enableArrays(glue, color, normal, texture, enabled, lastenabled);
-  cc_glglue_glDrawElements(glue, GL_POINTS, n, GL_UNSIGNED_INT,
-                           (const GLvoid*) this->getPointIndices());
-  PRIVATE(this)->disableArrays(glue, color, normal, texture, enabled, lastenabled);
+  if (cc_glglue_has_vertex_array(glue)) {
+    PRIVATE(this)->enableArrays(glue, color, normal, texture, enabled, lastenabled);
+    PRIVATE(this)->pointindexer->render(glue, FALSE, contextid);
+    PRIVATE(this)->disableArrays(glue, color, normal, texture, enabled, lastenabled);
+  }
+  else {
+    // fall back to immediate mode rendering
+    glBegin(GL_POINTS);
+    PRIVATE(this)->renderImmediate(glue,
+                                   this->getPointIndices(),
+                                   this->getNumPointIndices(),
+                                   color, normal, texture, enabled, lastenabled);
+    glEnd();
+  }
+  // inform SoGLLazyElement that we might have changed the current color
+  if (color) {
+    SoGLLazyElement::getInstance(state)->reset(state, 
+                                               SoLazyElement::DIFFUSE_MASK);
+  }
 }
 
 
@@ -342,6 +388,8 @@ SoPrimitiveVertexCache::addTriangle(const SoPrimitiveVertex * v0,
   }
   const SoPrimitiveVertex *vp[3] = { v0, v1, v2 };
 
+  int32_t triangleindices[3];
+
   for (int i = 0; i < 3; i++) {
     Vertex v;
     v.vertex = vp[i]->getPoint();
@@ -387,7 +435,7 @@ SoPrimitiveVertexCache::addTriangle(const SoPrimitiveVertex * v0,
       idx = PRIVATE(this)->vertexlist.getLength();
       PRIVATE(this)->vhash.put(v, idx);
       PRIVATE(this)->addVertex(v);
-      PRIVATE(this)->indices.append(idx);
+      triangleindices[i] = idx;
 
       // update texture coordinates for unit 1-n
       for (int j = 1; j <= PRIVATE(this)->lastenabled; j++) {
@@ -404,9 +452,15 @@ SoPrimitiveVertexCache::addTriangle(const SoPrimitiveVertex * v0,
       }
     }
     else {
-      PRIVATE(this)->indices.append(idx);
+      triangleindices[i] = idx;
     }
   }
+  if (PRIVATE(this)->triangleindexer == NULL) {
+    PRIVATE(this)->triangleindexer = new SoVertexArrayIndexer;
+  }
+  PRIVATE(this)->triangleindexer->addTriangle(triangleindices[0],
+                                              triangleindices[1],
+                                              triangleindices[2]);
 }
 
 void
@@ -419,6 +473,8 @@ SoPrimitiveVertexCache::addLine(const SoPrimitiveVertex * v0,
   }
   const SoPrimitiveVertex *vp[2] = { v0,v1 };
 
+  int32_t lineindices[2];
+
   for (int i = 0; i < 2; i++) {
     Vertex v;
     v.vertex = vp[i]->getPoint();
@@ -463,7 +519,7 @@ SoPrimitiveVertexCache::addLine(const SoPrimitiveVertex * v0,
       idx = PRIVATE(this)->vertexlist.getLength();
       PRIVATE(this)->vhash.put(v, idx);
       PRIVATE(this)->addVertex(v);
-      PRIVATE(this)->lineindices.append(idx);
+      lineindices[i] = idx;
 
       // update texture coordinates for unit 1-n
       for (int j = 1; j <= PRIVATE(this)->lastenabled; j++) {
@@ -480,9 +536,13 @@ SoPrimitiveVertexCache::addLine(const SoPrimitiveVertex * v0,
       }
     }
     else {
-      PRIVATE(this)->lineindices.append(idx);
+      lineindices[i] = idx;
     }
   }
+  if (PRIVATE(this)->lineindexer == NULL) {
+    PRIVATE(this)->lineindexer = new SoVertexArrayIndexer;
+  }
+  PRIVATE(this)->lineindexer->addLine(lineindices[0], lineindices[1]);
 }
 
 void
@@ -527,12 +587,16 @@ SoPrimitiveVertexCache::addPoint(const SoPrimitiveVertex * v0)
     }
   }
 
+  if (PRIVATE(this)->pointindexer == NULL) {
+    PRIVATE(this)->pointindexer = new SoVertexArrayIndexer;
+  }
+
   int32_t idx;
   if (!PRIVATE(this)->vhash.get(v, idx)) {
     idx = PRIVATE(this)->vertexlist.getLength();
     PRIVATE(this)->vhash.put(v, idx);
     PRIVATE(this)->addVertex(v);
-    PRIVATE(this)->pointindices.append(idx);
+    PRIVATE(this)->pointindexer->addPoint(idx);
 
     // update texture coordinates for unit 1-n
     for (int j = 1; j <= PRIVATE(this)->lastenabled; j++) {
@@ -549,7 +613,7 @@ SoPrimitiveVertexCache::addPoint(const SoPrimitiveVertex * v0)
     }
   }
   else {
-    PRIVATE(this)->pointindices.append(idx);
+    PRIVATE(this)->pointindexer->addPoint(idx);
   }
 }
 
@@ -592,19 +656,21 @@ SoPrimitiveVertexCache::getColorArray(void) const
 int
 SoPrimitiveVertexCache::getNumIndices(void) const
 {
-  return PRIVATE(this)->indices.getLength();
+  return PRIVATE(this)->triangleindexer ? PRIVATE(this)->triangleindexer->getNumIndices() : 0;
 }
 
 const int32_t *
 SoPrimitiveVertexCache::getIndices(void) const
 {
-  return PRIVATE(this)->indices.getArrayPtr();
+  assert(PRIVATE(this)->triangleindexer);
+  return PRIVATE(this)->triangleindexer->getIndices();
 }
 
 int32_t
 SoPrimitiveVertexCache::getIndex(const int idx) const
 {
-  return PRIVATE(this)->indices[idx];
+  assert(PRIVATE(this)->triangleindexer);
+  return PRIVATE(this)->triangleindexer->getIndices()[idx];
 }
 
 SbBool
@@ -623,99 +689,28 @@ SoPrimitiveVertexCache::getMultiTextureCoordinateArray(const int unit) const
 int
 SoPrimitiveVertexCache::getNumLineIndices(void) const
 {
-  return PRIVATE(this)->lineindices.getLength();
+  return PRIVATE(this)->lineindexer ? PRIVATE(this)->lineindexer->getNumIndices() : 0;
 }
 
 int
 SoPrimitiveVertexCache::getNumPointIndices(void) const
 {
-  return PRIVATE(this)->pointindices.getLength();
+  return PRIVATE(this)->pointindexer ? PRIVATE(this)->pointindexer->getNumIndices() : 0;
 }
 
 
 const int32_t *
 SoPrimitiveVertexCache::getLineIndices(void) const
 {
-  return PRIVATE(this)->lineindices.getArrayPtr();
+  assert(PRIVATE(this)->lineindexer);
+  return PRIVATE(this)->lineindexer->getIndices();
 }
 
 const int32_t *
 SoPrimitiveVertexCache::getPointIndices(void) const
 {
-  return PRIVATE(this)->pointindices.getArrayPtr();
-}
-
-// sort an array of three integers
-static void sort3(int32_t * arr)
-{
-  // simple bubble-sort
-  int32_t tmp;
-  if (arr[1] < arr[0]) {
-    tmp = arr[0];
-    arr[0] = arr[1];
-    arr[1] = tmp;
-  }
-  if (arr[2] < arr[1]) {
-    tmp = arr[1];
-    arr[1] = arr[2];
-    arr[2] = tmp;
-  }
-  if (arr[1] < arr[0]) {
-    tmp = arr[0];
-    arr[0] = arr[1];
-    arr[1] = tmp;
-  }
-}
-
-// qsort callback used for sorting triangles based on vertex indices
-extern "C" {
-static int
-compare_triangle(const void * v0, const void * v1)
-{
-  int i;
-  int32_t * t0 = (int*) v0;
-  int32_t * t1 = (int*) v1;
-
-  int32_t ti0[3];
-  int32_t ti1[3];
-  for (i = 0; i < 3; i++) {
-    ti0[i] = t0[i];
-    ti1[i] = t1[i];
-  }
-  sort3(ti0);
-  sort3(ti1);
-
-  for (i = 0; i < 3; i++) {
-    int32_t diff = ti0[i] - ti1[i];
-    if (diff != 0) return diff;
-  }
-  return 0;
-}
-}
-
-// qsort callback used for sorting line segments based on vertex
-// indices
-extern "C" {
-static int
-compare_line_segment(const void * v0, const void * v1)
-{
-  int i;
-  int32_t * l0 = (int*) v0;
-  int32_t * l1 = (int*) v1;
-  int32_t li0[2];
-  int32_t li1[2];
-  
-  li0[0] = SbMin(l0[0], l0[1]);
-  li0[1] = SbMax(l0[0], l0[1]);
-  li1[0] = SbMin(l1[0], l1[1]);
-  li1[1] = SbMax(l1[0], l1[1]);
-
-  for (i = 0; i < 2; i++) {
-    int diff = li0[i] - li1[i];
-    if (diff) return diff;
-  }
-  return 0;
-}
+  assert(PRIVATE(this)->pointindexer);
+  return PRIVATE(this)->pointindexer->getIndices();
 }
 
 void
@@ -726,32 +721,69 @@ SoPrimitiveVertexCache::fit(void)
   PRIVATE(this)->texcoordlist.fit();
   PRIVATE(this)->bumpcoordlist.fit();
   PRIVATE(this)->rgbalist.fit();
-  PRIVATE(this)->indices.fit();
-  PRIVATE(this)->lineindices.fit();
-  PRIVATE(this)->pointindices.fit();
   PRIVATE(this)->vhash.clear();
 
-
-  // sort triangles based on vertex indices to get more hits in the
-  // GPU vertex cache. Not the optimal solution, but should work
-  // pretty well. Example: bunny.iv (~70000 triangles) went from 238
-  // fps with no sorting to 380 fps with sorting.
-  if (PRIVATE(this)->indices.getLength()) {
-    qsort((void*) PRIVATE(this)->indices.getArrayPtr(),
-          PRIVATE(this)->indices.getLength() / 3,
-          sizeof(int32_t) * 3,
-          compare_triangle);
-  }
-  
-  // sort lines as well
-  if (PRIVATE(this)->lineindices.getLength()) {
-    qsort((void*) PRIVATE(this)->lineindices.getArrayPtr(),
-          PRIVATE(this)->lineindices.getLength() / 2,
-          sizeof(int32_t) * 2,
-          compare_line_segment);
-  }
+  if (PRIVATE(this)->triangleindexer) PRIVATE(this)->triangleindexer->close();
+  if (PRIVATE(this)->lineindexer) PRIVATE(this)->lineindexer->close();
+  if (PRIVATE(this)->pointindexer) PRIVATE(this)->pointindexer->close();
 }
 
+void 
+SoPrimitiveVertexCache::depthSortTriangles(SoState * state)
+{
+  int numv = PRIVATE(this)->vertexlist.getLength();
+  int numtri = this->getNumIndices() / 3;
+  if (numv == 0 || numtri == 0) return;
+  
+  SbPlane sortplane = SoViewVolumeElement::get(state).getPlane(0.0);
+  // move plane into object space
+  sortplane.transform(SoModelMatrixElement::get(state).inverse());
+  
+  if (PRIVATE(this)->deptharray == NULL ||
+      (sortplane != PRIVATE(this)->prevsortplane)) {
+    if (!PRIVATE(this)->deptharray) {
+      PRIVATE(this)->deptharray = new float[numtri];
+    }
+    PRIVATE(this)->prevsortplane = sortplane;
+    float * darray = PRIVATE(this)->deptharray;
+    const SbVec3f * vptr = PRIVATE(this)->vertexlist.getArrayPtr();
+    int32_t * iptr = PRIVATE(this)->triangleindexer->getWriteableIndices();
+    int i,j;
+    for (i = 0; i < numtri; i++) {
+      float acc = 0.0;
+      for (j = 0; j < 3; j++) {
+        acc += sortplane.getDistance(vptr[iptr[i*3+j]]);
+      }
+      darray[i] = acc / 3.0f;
+    }
+    int distance;
+    float dtmp;
+    int itmp[3];
+
+    // shell sort algorithm (O(nlog(n))
+    for (distance = 1; distance <= numtri/9; distance = 3*distance + 1);
+    for (; distance > 0; distance /= 3) {
+      for (i = distance; i < numtri; i++) {
+        dtmp = darray[i];
+        itmp[0] = iptr[i*3];
+        itmp[1] = iptr[i*3+1];
+        itmp[2] = iptr[i*3+2];
+        j = i;
+        while (j >= distance && darray[j-distance] > dtmp) {
+          darray[j] = darray[j-distance];
+          iptr[j*3] = iptr[(j-distance)*3];
+          iptr[j*3+1] = iptr[(j-distance)*3+1];
+          iptr[j*3+2] = iptr[(j-distance)*3+2];
+          j -= distance;
+        }
+        darray[j] = dtmp;
+        iptr[j*3] = itmp[0];
+        iptr[j*3+1] = itmp[1];
+        iptr[j*3+2] = itmp[2];
+      }
+    }
+  } 
+}
 
 SoPrimitiveVertexCache::Vertex::operator unsigned long(void) const
 {
@@ -795,10 +827,6 @@ SoPrimitiveVertexCacheP::enableArrays(const cc_glglue * glue,
                                       const int lastenabled)
 {
   int i;
-  cc_glglue_glVertexPointer(glue, 3, GL_FLOAT, 0,
-                            (GLvoid*) this->vertexlist.getArrayPtr());
-  cc_glglue_glEnableClientState(glue, GL_VERTEX_ARRAY);
-
   if (color) {
     cc_glglue_glColorPointer(glue, 4, GL_UNSIGNED_BYTE, 0,
                              (GLvoid*) this->rgbalist.getArrayPtr());
@@ -824,6 +852,10 @@ SoPrimitiveVertexCacheP::enableArrays(const cc_glglue * glue,
                               (GLvoid*) this->normallist.getArrayPtr());
     cc_glglue_glEnableClientState(glue, GL_NORMAL_ARRAY);
   }
+
+  cc_glglue_glVertexPointer(glue, 3, GL_FLOAT, 0,
+                            (GLvoid*) this->vertexlist.getArrayPtr());
+  cc_glglue_glEnableClientState(glue, GL_VERTEX_ARRAY);
 }
 
 
@@ -863,73 +895,39 @@ SoPrimitiveVertexCacheP::enableVBOs(const cc_glglue * glue,
                                     const SbBool texture, const SbBool * enabled,
                                     const int lastenabled)
 {
-  SoPrimitiveVertexCache_vboidx * vbo;
-  if (!this->vbodict.get(contextid, vbo)) {
-    vbo = new SoPrimitiveVertexCache_vboidx;
-    memset(vbo, 0, sizeof(SoPrimitiveVertexCache_vboidx));
-    (void) this->vbodict.put(contextid, vbo);
-  }
-
   int i;
-  if (vbo->vertex == 0) {
-    cc_glglue_glGenBuffers(glue, 1, &vbo->vertex);
-    cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->vertex);
-    cc_glglue_glBufferData(glue, GL_ARRAY_BUFFER,
-                           this->vertexlist.getLength()*3*sizeof(float),
-                           this->vertexlist.getArrayPtr(),
-                           GL_STATIC_DRAW);
-  }
-  else {
-    cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->vertex);
-  }
-  cc_glglue_glVertexPointer(glue, 3, GL_FLOAT, 0, NULL);
-  cc_glglue_glEnableClientState(glue, GL_VERTEX_ARRAY);
-
   if (color) {
-    if (vbo->rgba == 0) {
-      cc_glglue_glGenBuffers(glue, 1, &vbo->rgba);
-      cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->rgba);
-      cc_glglue_glBufferData(glue, GL_ARRAY_BUFFER,
-                             this->rgbalist.getLength() * sizeof(uint8_t),
-                             this->rgbalist.getArrayPtr(),
-                             GL_STATIC_DRAW);
-    }
-    else {
-      cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->rgba);
+    if (this->rgbavbo == NULL) {
+      this->rgbavbo = new SoVBO;
+      this->rgbavbo->setBufferData(this->rgbalist.getArrayPtr(), 
+                                   this->rgbalist.getLength() * sizeof(uint8_t));
     }
+    this->rgbavbo->bindBuffer(contextid);
     cc_glglue_glColorPointer(glue, 4, GL_UNSIGNED_BYTE, 0, NULL);
     cc_glglue_glEnableClientState(glue, GL_COLOR_ARRAY);
   }
   if (texture) {
-    if (vbo->texcoord0 == 0) {
-      cc_glglue_glGenBuffers(glue, 1, &vbo->texcoord0);
-      cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->texcoord0);
-      cc_glglue_glBufferData(glue, GL_ARRAY_BUFFER,
-                             this->texcoordlist.getLength()*4*sizeof(float),
-                             this->texcoordlist.getArrayPtr(),
-                             GL_STATIC_DRAW);
+    if (this->texcoord0vbo == NULL) {
+      this->texcoord0vbo = new SoVBO;
+      this->texcoord0vbo->setBufferData(this->texcoordlist.getArrayPtr(),
+                                        this->texcoordlist.getLength()*4*sizeof(float));
     }
-    else {
-      cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->texcoord0);
-    }
-
+    this->texcoord0vbo->bindBuffer(contextid);
     cc_glglue_glTexCoordPointer(glue, 4, GL_FLOAT, 0, NULL);
     cc_glglue_glEnableClientState(glue, GL_TEXTURE_COORD_ARRAY);
 
     for (i = 1; i <= lastenabled; i++) {
+      while (this->multitexvbo.getLength() <= i) {
+        this->multitexvbo.append(NULL);
+      }
       if (enabled[i]) {
-        if (vbo->multitex[i] == 0) {
-          cc_glglue_glGenBuffers(glue, 1, &vbo->multitex[i]);
-          cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->multitex[i]);
-          cc_glglue_glBufferData(glue, GL_ARRAY_BUFFER,
-                                 this->multitexcoords[i].getLength()*4*sizeof(float),
-                                 this->multitexcoords[i].getArrayPtr(),
-                                 GL_STATIC_DRAW);
+        if (this->multitexvbo[i] == NULL) {
+          SoVBO * vbo = new SoVBO;
+          vbo->setBufferData(this->multitexcoords[i].getArrayPtr(),
+                             this->multitexcoords[i].getLength()*4*sizeof(float));
+          this->multitexvbo[i] = vbo;
         }
-        else {
-          cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->multitex[i]);
-        }
-
+        this->multitexvbo[i]->bindBuffer(contextid);
         cc_glglue_glClientActiveTexture(glue, GL_TEXTURE0 + i);
         cc_glglue_glTexCoordPointer(glue, 4, GL_FLOAT, 0, NULL);
         cc_glglue_glEnableClientState(glue, GL_TEXTURE_COORD_ARRAY);
@@ -937,36 +935,26 @@ SoPrimitiveVertexCacheP::enableVBOs(const cc_glglue * glue,
     }
   }
   if (normal) {
-    if (vbo->normal == 0) {
-      cc_glglue_glGenBuffers(glue, 1, &vbo->normal);
-      cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->normal);
-      cc_glglue_glBufferData(glue, GL_ARRAY_BUFFER,
-                             this->normallist.getLength()*3*sizeof(float),
-                             this->normallist.getArrayPtr(),
-                             GL_STATIC_DRAW);
+    if (this->normalvbo == NULL) {
+      this->normalvbo = new SoVBO;
+      this->normalvbo->setBufferData(this->normallist.getArrayPtr(),
+                                     this->normallist.getLength()*3*sizeof(float));
     }
-    else {
-      cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->normal);
-    }
-
+    this->normalvbo->bindBuffer(contextid);
     cc_glglue_glNormalPointer(glue, GL_FLOAT, 0, NULL);
     cc_glglue_glEnableClientState(glue, GL_NORMAL_ARRAY);
   }
 
-  if (vbo->triangleindex == 0) {
-    cc_glglue_glGenBuffers(glue, 1, &vbo->triangleindex);
-    cc_glglue_glBindBuffer(glue, GL_ELEMENT_ARRAY_BUFFER, vbo->triangleindex);
-    cc_glglue_glBufferData(glue, GL_ELEMENT_ARRAY_BUFFER,
-                           this->indices.getLength()*sizeof(int32_t),
-                           this->indices.getArrayPtr(),
-                           GL_STATIC_DRAW);
-  }
-  else {
-    cc_glglue_glBindBuffer(glue, GL_ELEMENT_ARRAY_BUFFER, vbo->triangleindex);
+  if (this->vertexvbo == NULL) {
+    this->vertexvbo = new SoVBO;
+    this->vertexvbo->setBufferData(this->vertexlist.getArrayPtr(),
+                                   this->vertexlist.getLength()*3*sizeof(float));
   }
+  this->vertexvbo->bindBuffer(contextid);
+  cc_glglue_glVertexPointer(glue, 3, GL_FLOAT, 0, NULL);
+  cc_glglue_glEnableClientState(glue, GL_VERTEX_ARRAY);
 }
 
-
 void
 SoPrimitiveVertexCacheP::disableVBOs(const cc_glglue * glue,
                                      const SbBool color, const SbBool normal,
@@ -975,110 +963,54 @@ SoPrimitiveVertexCacheP::disableVBOs(const cc_glglue * glue,
 {
   this->disableArrays(glue, color, normal, texture, enabled, lastenabled);
   cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, 0); // Reset VBO binding
-  cc_glglue_glBindBuffer(glue, GL_ELEMENT_ARRAY_BUFFER, 0); // Reset VBO binding
 }
 
-unsigned long
-SoPrimitiveVertexCacheP::countVBOSize(const cc_glglue * glue,
-                                      const uint32_t contextid,
-                                      const SbBool color, const SbBool normal,
-                                      const SbBool texture, const SbBool * enabled,
-                                      const int lastenabled)
+void 
+SoPrimitiveVertexCacheP::renderImmediate(const cc_glglue * glue,
+                                         const int32_t * indices,
+                                         const int numindices,
+                                         const SbBool color, const SbBool normal,
+                                         const SbBool texture, const SbBool * enabled,
+                                         const int lastenabled)
 {
-  unsigned long size = 0;
-  SoPrimitiveVertexCache_vboidx * vbo;
-  if (!this->vbodict.get(contextid, vbo)) {
-    vbo = new SoPrimitiveVertexCache_vboidx;
-    memset(vbo, 0, sizeof(SoPrimitiveVertexCache_vboidx));
-    (void) this->vbodict.put(contextid, vbo);
-  }
-
-  int i;
-  if (vbo->vertex == 0) {
-    size +=  this->vertexlist.getLength()*3*sizeof(float);
-  }
+  const unsigned char * colorptr = NULL;
+  const SbVec3f * normalptr = NULL;
+  const SbVec3f * vertexptr = NULL;
+  const SbVec4f * texcoordptr = NULL;
+  
   if (color) {
-    if (vbo->rgba == 0) {
-      size += this->rgbalist.getLength() * sizeof(uint8_t);
-    }
-  }
-  if (texture) {
-    if (vbo->texcoord0 == 0) {
-      size += this->texcoordlist.getLength()*4*sizeof(float);
-    }
-    for (i = 1; i <= lastenabled; i++) {
-      if (enabled[i]) {
-        if (vbo->multitex[i] == 0) {
-          size += this->multitexcoords[i].getLength()*4*sizeof(float);
-        }
-      }
-    }
+    colorptr = this->rgbalist.getArrayPtr();
   }
   if (normal) {
-    if (vbo->normal == 0) {
-      size += this->normallist.getLength()*3*sizeof(float);
+    normalptr = this->normallist.getArrayPtr();
+  }
+  if (texture) {
+    texcoordptr = this->texcoordlist.getArrayPtr();
+  }
+  vertexptr = this->vertexlist.getArrayPtr();
+  
+  for (int i = 0; i < numindices; i++) {
+    const int idx = indices[i];
+    if (normal) {
+      glNormal3fv((const GLfloat*) &normalptr[idx]);
     }
-  }
-  if (vbo->triangleindex == 0) {
-    size += this->indices.getLength()*sizeof(int32_t);
-  }
-  return size;
-}
-
-void
-SoPrimitiveVertexCacheP::vbo_schedule(const uint32_t & key,
-                                      SoPrimitiveVertexCache_vboidx * const & value,
-                                      void * closure)
-{
-  SoGLCacheContextElement::scheduleDeleteCallback(key, vbo_delete, value);
-}
-
-void
-SoPrimitiveVertexCacheP::vbo_delete(void * closure, uint32_t contextid)
-{
-  SoPrimitiveVertexCache_vboidx * vbo =
-    (SoPrimitiveVertexCache_vboidx *) closure;
-
-  const cc_glglue * glue = cc_glglue_instance((int)contextid);
-  if (vbo->triangleindex) {
-    cc_glglue_glDeleteBuffers(glue, 1, &vbo->triangleindex);
-  }
-  if (vbo->vertex) {
-    cc_glglue_glDeleteBuffers(glue, 1, &vbo->vertex);
-  }
-  if (vbo->normal) {
-    cc_glglue_glDeleteBuffers(glue, 1, &vbo->normal);
-  }
-  if (vbo->texcoord0) {
-    cc_glglue_glDeleteBuffers(glue, 1, &vbo->texcoord0);
-  }
-  if (vbo->rgba) {
-    cc_glglue_glDeleteBuffers(glue, 1, &vbo->rgba);
-  }
-
-  for (int i = 1; i < MAX_UNITS; i++) {
-    if (vbo->multitex[i]) {
-      cc_glglue_glDeleteBuffers(glue, 1, &vbo->multitex[i]);
+    if (color) {
+      glColor3ubv((const GLubyte*) &colorptr[idx*4]);
     }
-  }
-  delete vbo;
-}
-
-//
-// Callback from SoContextHandler
-//
-void 
-SoPrimitiveVertexCacheP::contextCleanup(uint32_t context, void * closure)
-{
-  SoPrimitiveVertexCacheP * thisp = (SoPrimitiveVertexCacheP*) closure;
-
-  SoPrimitiveVertexCache_vboidx * tmp;
-  if (thisp->vbodict.get(context, tmp)) {
-    SoPrimitiveVertexCacheP::vbo_delete(tmp, context);
-    thisp->vbodict.remove(context);
+    if (texture) {
+      glTexCoord4fv((const GLfloat*) &texcoordptr[idx]);
+      
+      for (int j = 1; j <= lastenabled; j++) {
+        if (enabled[j]) {
+          const SbVec4f * mt = this->multitexcoords[j].getArrayPtr();
+          cc_glglue_glMultiTexCoord4fv(glue,
+                                       GL_TEXTURE0 + j,
+                                       (const GLfloat *) &mt[idx]);
+        }
+      }
+      }
+    glVertex3fv((const GLfloat*) &vertexptr[idx]);
   }
 }
 
-
-#undef MAX_UNITS
 #undef PRIVATE
