@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -188,7 +188,7 @@
 
 // FIXME: document DraggerCB typedef? 20010909 mortene.
 
-
+#include <Inventor/C/tidbitsp.h>
 #include <Inventor/draggers/SoDragger.h>
 #include <Inventor/nodekits/SoSubKitP.h>
 #include <Inventor/draggers/SoCenterballDragger.h>
@@ -388,6 +388,7 @@ void
 SoDragger::initClass(void)
 {
   SO_KIT_INTERNAL_INIT_CLASS(SoDragger, SO_FROM_INVENTOR_1);
+  SoDragger::minscale = 0.001f;
 
   SoDragger::initClasses();
 
@@ -1502,6 +1503,8 @@ typedef struct {
   SbViewVolume vv;
 } sodragger_vv_data;
 
+static sodragger_vv_data * vvdata = NULL;
+
 static SoCallbackAction::Response
 sodragger_vv_cb(void * userdata, SoCallbackAction * action, const SoNode * node)
 {
@@ -1510,6 +1513,12 @@ sodragger_vv_cb(void * userdata, SoCallbackAction * action, const SoNode * node)
   return SoCallbackAction::CONTINUE;
 }
 
+static void vv_data_cleanup(void) 
+{
+  delete vvdata;
+  vvdata = NULL;
+}
+
 /*!
   Store data about the current camera in the given action.
 */
@@ -1525,9 +1534,9 @@ SoDragger::setCameraInfo(SoAction * action)
   // view volume can be found directly. pederb, 2002-10-30
 
   if (THIS->draggercache && THIS->draggercache->path) {
-    static sodragger_vv_data * vvdata;
     if (vvdata == NULL) {
       vvdata = new sodragger_vv_data;
+      coin_atexit((coin_atexit_f*) vv_data_cleanup, CC_ATEXIT_NORMAL);
     }
     if (THIS->cbaction == NULL) {
       THIS->cbaction = new SoCallbackAction;
