@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -313,8 +313,9 @@ SoRotateDiscDragger::drag(void)
   plane.intersect(line, center);
   startPt -= center;
   projPt -= center;
-  startPt.normalize();
-  projPt.normalize();
+  // just normalize, null vector are ok (the angle will 0)
+  (void) startPt.normalize();
+  (void) projPt.normalize();
   SbVec3f dir = startPt.cross(projPt);
   float angle = (float) acos(SbClamp(startPt.dot(projPt), -1.0f, 1.0f));
   if (dir[2] < 0.0f) angle = -angle;
