@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -79,6 +79,10 @@
 
 #include <data/draggerDefaults/trackballDragger.h>
 
+#if COIN_DEBUG
+#include <Inventor/errors/SoDebugError.h>
+#endif // COIN_DEBUG
+
 /*!
   \var SoSFRotation SoTrackballDragger::rotation
 
@@ -544,7 +548,13 @@ SoTrackballDragger::dragStart(void)
                                          hitPt.length()));
 
     if (THIS->whatkind == WHATKIND_USERAXIS) {
-      hitPt.normalize();
+      if (hitPt.normalize() == 0.0f) {
+#if COIN_DEBUG
+        SoDebugError::postWarning("SoTrackballDragger::dragStart",
+                                  "Unable to find hitpoint direction.");
+        
+#endif // COIN_DEBUG
+      }
       SO_GET_ANY_PART(this, "userAxisRotation", SoRotation)->rotation =
         SbRotation(SbVec3f(0.0f, 1.0f, 0.0f), hitPt);
     }
@@ -608,7 +618,12 @@ SoTrackballDragger::drag(void)
     THIS->sphereProj->setViewVolume(this->getViewVolume());
     THIS->sphereProj->setWorkingSpace(wk2ws);
     SbVec3f vec = THIS->sphereProj->project(this->getNormalizedLocaterPosition());
-    vec.normalize();
+    if (vec.normalize() == 0.0f) {
+#if COIN_DEBUG
+      SoDebugError::postWarning("SoTrackballDragger::drag",
+                                "Unable to find drag direction.");
+#endif // COIN_DEBUG
+    }
     SO_GET_ANY_PART(this, "userAxisRotation", SoRotation)->rotation =
       SbRotation(SbVec3f(0.0f, 1.0f, 0.0f), vec);
   }
@@ -808,8 +823,12 @@ SoTrackballDragger::updateUserAxisSwitches(const SbBool setactive)
     SbRotation rot =
       SO_GET_ANY_PART(this, "userAxisRotation", SoRotation)->rotation.getValue();
     rot.multVec(vec, vec);
-    vec.normalize();
-
+    if (vec.normalize() == 0.0f) {
+#if COIN_DEBUG
+      SoDebugError::postWarning("SoTrackballDragger::updateUseAxisSwitches",
+                                "Invalid use axis rotation.");
+#endif // COIN_DEBUG
+    }
     if (vec[0] >= USER_AXIS_DISAPPEAR_LIMIT) val = SO_SWITCH_NONE;
     else if (vec[1] >= USER_AXIS_DISAPPEAR_LIMIT) val = SO_SWITCH_NONE;
     else if (vec[2] >= USER_AXIS_DISAPPEAR_LIMIT) val = SO_SWITCH_NONE;
