@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -218,6 +218,7 @@ void
 SoTransformerDragger::initClass(void)
 {
   SO_KIT_INTERNAL_INIT_CLASS(SoTransformerDragger, SO_FROM_INVENTOR_1);
+  SoTransformerDraggerP::colinearThreshold = 3;
 }
 
 void
@@ -1436,7 +1437,7 @@ SoTransformerDragger::dragScale()
     SbVec3f tmp = vv.getPlanePoint(vv.getNearDist(), SbVec2f(0.5f, 0.5f));
     SbVec3f dir = vv.getPlanePoint(vv.getNearDist(), SbVec2f(0.5f, 0.5f) + normmove);
     dir -= tmp;
-    dir.normalize();
+    (void) dir.normalize(); // ok if null (no movement)
     this->getWorldToWorkingMatrix().multDirMatrix(dir, dir);
     int biggest = 0;
     double bigval = fabs(dir[0]);
@@ -1625,18 +1626,18 @@ SoTransformerDragger::dragRotate(void)
     worldprojpt -= worldcenter;
     prevworldprojpt -= worldcenter;
 
-    // Normalize the vectors before finding the dotproduct angle
-    // between them.
-    worldprojpt.normalize();
-    prevworldprojpt.normalize();
-
     // FIXME: Without this test, the rotation behaves very strange,
     // especially at the edge/outside of the sphere. I will
     // investigate this some more to understand what happens. 
     // 20040804 jornskaa
-    if (worldprojpt.dot(prevworldprojpt) > 0.8f) {
-      THIS->prevWorldHitPt = wppt;
 
+    // Normalize the vectors before finding the dotproduct angle
+    // between them.
+    if ((worldprojpt.normalize() > 0.0f) &&
+        (prevworldprojpt.normalize() > 0.0f) && 
+        (worldprojpt.dot(prevworldprojpt) > 0.8f)) {
+      THIS->prevWorldHitPt = wppt;
+      
       // Calculate the rotation from previous prevworldprojpt to
       // worldprojpt
       SbRotation rot(prevworldprojpt, worldprojpt);
@@ -1722,11 +1723,6 @@ SoTransformerDragger::dragRotate(void)
     // and there is little room for error that might happen when the
     // vectors come very close to the rotation center.
     if (projpt.sqrLength() > 0.1f) {
-      // Normalize the vectors before finding the dotproduct angle
-      // between them.
-      prevworldprojpt.normalize();
-      worldprojpt.normalize();
-
       // Since we are using incremental changes, the changes will
       // never be very big, and the dot product between the
       // rotation-vectors should always be above zero, and never
@@ -1734,7 +1730,12 @@ SoTransformerDragger::dragRotate(void)
       // is above PI/2. This might happen if dragging is done over the
       // rotation center, but we do not allow that and just wait until
       // the locater comes into the valid range before rotating.
-      if (prevworldprojpt.dot(worldprojpt) > 0.3f) { // 0.3 == 72.5degrees
+
+      // Normalize the vectors before finding the dotproduct angle
+      // between them.
+      if ((prevworldprojpt.normalize() > 0.0f) &&
+          (worldprojpt.normalize() > 0.0f) &&
+          (prevworldprojpt.dot(worldprojpt) > 0.3f)) { // 0.3 == 72.5degrees
         THIS->prevWorldHitPt = wppt;
         
         // Rotate between the two points in the plane
