@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -101,6 +101,8 @@ static void soglcachecontext_cleanup(void)
     SoContextHandler::removeContextDestructionCallback(soglcache_contextdestructioncb, NULL);
     soglcache_contextdestructioncb = NULL;
   }
+
+  biggest_cache_context_id = 0;
 }
 
 //
@@ -156,7 +158,7 @@ SoGLCacheContextElement::initClass(void)
   scheduledeletelist = new SbList <SoGLDisplayList*>;
   scheduledeletecblist = new SbList <so_scheduledeletecb_info*>;
   CC_MUTEX_CONSTRUCT(glcache_mutex);
-  coin_atexit((coin_atexit_f *)soglcachecontext_cleanup, 0);
+  coin_atexit((coin_atexit_f *)soglcachecontext_cleanup, CC_ATEXIT_NORMAL);
 
   // add a callback which is called every time a GL-context is
   // destructed.  it's important that this callback is added in
@@ -290,11 +292,14 @@ SoGLCacheContextElement::extSupported(SoState * state, int extid)
   assert(extid >= 0 && extid < extsupportlist->getLength());
 
   so_glext_info * info = (*extsupportlist)[extid];
-
+  
   int currcontext = SoGLCacheContextElement::get(state);
   int n = info->context.getLength();
   for (int i = 0; i < n; i++) {
-    if (info->context[i] == currcontext) return info->supported[i];
+    if (info->context[i] == currcontext) {
+      CC_MUTEX_UNLOCK(glcache_mutex);
+      return info->supported[i];
+    }
   }
   const cc_glglue * w = sogl_glue_instance(state);
   SbBool supported = cc_glglue_glext_supported(w, info->extname.getString());
