@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -36,12 +36,22 @@
 #endif // HAVE_CONFIG_H
 
 #include <Inventor/system/gl.h>
+#include <Inventor/errors/SoDebugError.h>
+
+// *************************************************************************
 
 // static variables
+
+// Not used, because it was part of a fubar interface. Kept around in
+// Coin v2 for backward binary compatibility reasons.
 int SoGLClipPlaneElement::maxGLPlanes = -1;
 
+// *************************************************************************
+
 SO_ELEMENT_SOURCE(SoGLClipPlaneElement);
 
+// *************************************************************************
+
 /*!
   This static method initializes static data for the SoGLClipPlaneElement
   class.
@@ -94,20 +104,21 @@ SoGLClipPlaneElement::getMaxGLPlanes(void)
   // FIXME: should also make a likewise method available as part of
   // the So*GLWidget classes. 20020802 mortene.
 
-  if (SoGLClipPlaneElement::maxGLPlanes == -1) {
-    // NB: don't try to be clever and move this code to the
-    // initClass() method, as it won't work -- the GL variables may
-    // not have been initialized yet when it's called. --mortene
-    GLint val;
+  SoDebugError::postWarning("SoGLClipPlaneElement::getMaxGLPlanes",
+                            "This function is obsoleted. It should not "
+                            "be used because its interface is fubar: "
+                            "the number of clip planes available from "
+                            "the OpenGL driver depends on the context, and "
+                            "this function does not know which context this "
+                            "information is requested for.");
 
-    // FIXME: this needs a valid OpenGL context to work -- which we
-    // either should make sure we have, or post an error or warning if
-    // we don't. 20011011 mortene.
-    glGetIntegerv(GL_MAX_CLIP_PLANES, &val);
-    SoGLClipPlaneElement::maxGLPlanes = val;
-  }
+  GLint val;
+  glGetIntegerv(GL_MAX_CLIP_PLANES, &val);
 
-  return SoGLClipPlaneElement::maxGLPlanes;
+  assert(glGetError() == GL_NO_ERROR &&
+         "GL error when calling glGetInteger() -- no current GL context?");
+
+  return (int)val;
 }
 
 //! FIXME: write doc.
