@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -51,6 +51,13 @@
 
 // *************************************************************************
 
+// HACK WARNING: Please note that the "type" member is also used to store
+// the texture target for a texture object, so it's only safe to check
+// if "type" != DISPLAY_LIST. This is fixed properly in Coin-dev (using
+// a pimpl implementation). pederb, 2005-10-18
+
+// *************************************************************************
+
 /*!
   Constructor.
 */
@@ -122,7 +129,7 @@ SoGLDisplayList::SoGLDisplayList(SoState * state, Type type, int allocnum,
 
   // Reserve displaylist IDs.
 
-  if (this->type == TEXTURE_OBJECT) {
+  if (this->type != DISPLAY_LIST) {
     assert(allocnum == 1 && "it is only possible to create one texture object at a time");
     const cc_glglue * glw = cc_glglue_instance(this->context);
     if (cc_glglue_has_texture_objects(glw)) {
@@ -136,7 +143,6 @@ SoGLDisplayList::SoGLDisplayList(SoState * state, Type type, int allocnum,
       this->type = DISPLAY_LIST;
     }
   }
-
   if (this->type == DISPLAY_LIST) {
     this->firstindex = (unsigned int) glGenLists(allocnum);
     if (this->firstindex == 0) {
@@ -165,11 +171,9 @@ SoGLDisplayList::~SoGLDisplayList()
     glDeleteLists((GLuint) this->firstindex, this->numalloc);
   }
   else {
-    assert(this->type == TEXTURE_OBJECT);
-
     const cc_glglue * glw = cc_glglue_instance(this->context);
     assert(cc_glglue_has_texture_objects(glw));
-
+    
     // Use temporary variable in case GLUint != unsigned int.
     GLuint tmpindex = (GLuint) this->firstindex;
     // It is only possible to create one texture object at a time, so
@@ -211,7 +215,6 @@ SoGLDisplayList::open(SoState * state, int index)
     glNewList((GLuint) (this->firstindex+index), GL_COMPILE_AND_EXECUTE);
   }
   else {
-    assert(type == TEXTURE_OBJECT);
     assert(index == 0);
     this->bindTexture(state);
   }
@@ -244,7 +247,6 @@ SoGLDisplayList::call(SoState * state, int index)
     glCallList((GLuint) (this->firstindex + index));
   }
   else {
-    assert(type == TEXTURE_OBJECT);
     assert(index == 0);
     this->bindTexture(state);
   }
@@ -324,12 +326,69 @@ SoGLDisplayList::bindTexture(SoState *state)
   const cc_glglue * glw = cc_glglue_instance(this->context);
   assert(cc_glglue_has_texture_objects(glw));
 
-  GLenum target = GL_TEXTURE_2D;
+  GLenum target;
+  switch (this->type) {
+  case TEXTURE_OBJECT:
+  case DISPLAY_LIST:
+    target = GL_TEXTURE_2D;
+    break;
+  default:
+    target = (GLenum) this->type;
+    break;
+  }
 
+#ifndef COIN_NEXT_MINOR
+  // this is a quite ugly, since it will not work with multi
+  // texturing.  In Coin 2.5 this test will be removed, and the
+  // SoGLImage instance will set the correct texture target. 
+  // pederb, 2005-10-18
   if (SoGLTexture3EnabledElement::get(state)) {
     assert(cc_glglue_has_3d_textures(glw));
     target = GL_TEXTURE_3D;
   }
+#endif // COIN_NEXT_MINOR
 
   cc_glglue_glBindTexture(glw, target, (GLuint)this->firstindex);
 }
+
+#ifdef COIN_NEXT_MINOR
+
+/*!
+  Sets the texture target for a texture object. The texture
+  target should be the OpenGL texture target: GL_TEXTURE_2D,
+  GL_TEXTURE_3D, GL_TEXTURE_CUBE_MAP or GL_TEXTURE_RECTANGLE_EXT
+  (more targets might apply in the future).
+
+  \since Coin 2.5
+*/
+void 
+SoGLDisplayList::setTextureTarget(int target)
+{
+  // just reuse the type member. This assumes that the GL enums are !=
+  // DISPLAY_LIST (== 0), which should be safe. This is handled
+  // properly in Coin-dev using a pimpl-implementation.
+  this->type = (Type) target;
+}
+
+/*!
+  Returns the OpenGL texture target.
+
+  \sa setTextureTarget().
+  \since Coin 2.5
+*/
+int 
+SoGLDisplayList::getTextureTarget(void) const
+{
+  GLenum target;
+  switch (this->type) {
+  case TEXTURE_OBJECT:
+  case DISPLAY_LIST:
+    target = GL_TEXTURE_2D;
+    break;
+  default:
+    target = (GLenum) this->type;
+    break;
+  }
+  return (int) target;
+}
+#endif // COIN_NEXT_MINOR
