@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -58,7 +58,9 @@
 #include <Inventor/misc/SoGLImage.h>
 #include <Inventor/misc/SoState.h>
 #include <Inventor/nodes/SoNode.h>
+#include <Inventor/C/tidbits.h>
 #include <coindefs.h> // COIN_OBSOLETED
+#include "../../misc/SoVBO.h"
 
 // *************************************************************************
 
@@ -1301,6 +1303,43 @@ SoGLLazyElement::lazyDidntSet(uint32_t mask)
   this->didntsetbitmask |= mask&(~this->didsetbitmask);
 }
 
+#ifdef COIN_NEXT_MINOR
+void 
+SoGLLazyElement::updateColorVBO(SoVBO * vbo)
+{
+  if (this->colorpacker) {
+    uint32_t maxid = this->colorpacker->getDiffuseId();
+    uint32_t tid = this->colorpacker->getTranspId();
+    if (tid > maxid) {
+      maxid = tid;
+    }
+    uint32_t vboid = vbo->getBufferDataId();
+    if (vboid != maxid) {
+      const int n = this->coinstate.numdiffuse;
+      // need to update the VBO
+      const uint32_t * src = this->colorpacker->getPackedColors();
+      if (coin_host_get_endianness() == COIN_HOST_IS_BIGENDIAN) {
+        vbo->setBufferData(src, n * sizeof(uint32_t),
+                           maxid);
+      }
+      else {
+        uint32_t * dst = (uint32_t*)
+          vbo->allocBufferData(n * sizeof(uint32_t),
+                               maxid);
+        for (int i = 0; i < n; i++) {
+          uint32_t tmp = src[i];
+          dst[i] = 
+            (tmp << 24) |
+            ((tmp & 0xff00) << 8) |
+            ((tmp & 0xff0000) >> 8) |
+            (tmp >> 24);
+        }
+      }
+    }
+  }
+}
+#endif // COIN_NEXT_MINOR
+
 #undef UPDATE_GLSTATE
 #undef GLSTATE
 #undef OPENCACHEFLAGS
