@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -29,28 +29,34 @@
   FIXME: write doc.
 */
 
+// *************************************************************************
+
+#include <Inventor/elements/SoGLLightIdElement.h>
+
 #include <assert.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif // HAVE_CONFIG_H
 
-#include <Inventor/elements/SoGLLightIdElement.h>
-#include <Inventor/errors/SoDebugError.h>
-#include <Inventor/system/gl.h>
 #include <Inventor/C/glue/gl.h>
+#include <Inventor/errors/SoDebugError.h>
+#include <Inventor/errors/SoDebugError.h>
 #include <Inventor/misc/SoGL.h>
+#include <Inventor/system/gl.h>
 
+// *************************************************************************
+
+// Not used, because it was part of a fubar interface. Kept around in
+// Coin v2 for backward binary compatibility reasons.
 int32_t SoGLLightIdElement::maxGLSources = -1;
 
-/*!
-  \fn SoGLLightIdElement::maxGLSources
-
-  FIXME: write doc.
-*/
+// *************************************************************************
 
 SO_ELEMENT_SOURCE(SoGLLightIdElement);
 
+// *************************************************************************
+
 /*!
   This static method initializes static data for the
   SoGLLightIdElement class.
@@ -120,12 +126,6 @@ SoGLLightIdElement::increment(SoState * const state,
     element->data++;
     int maxl = cc_glglue_get_max_lights(glue);
 
-    // update static variable in case somebody uses the maxGLLights()
-    // function
-    if (SoGLLightIdElement::maxGLSources != maxl) {
-      SoGLLightIdElement::maxGLSources = maxl;
-    }
-
     if (element->data >= maxl) {
       element->data--;
 #if COIN_DEBUG
@@ -146,7 +146,7 @@ SoGLLightIdElement::increment(SoState * const state,
                                   "shape nodes -- audit your GLRender() "
                                   "method(s)).",
 
-                                  SoGLLightIdElement::getMaxGLSources());
+                                  maxl);
       }
 #endif
       return -1;
@@ -166,20 +166,21 @@ SoGLLightIdElement::getMaxGLSources(void)
   // FIXME: should also make a likewise method available as part of
   // the So*GLWidget classes. 20020802 mortene.
 
-  // FIXME: consider context. pederb, 20001012
-  if (SoGLLightIdElement::maxGLSources == -1) {
-    // NB: don't try to be clever and move this code to the
-    // initClass() method, as it won't work -- there will be no
-    // current GL context.
-    GLint val;
-    glGetIntegerv(GL_MAX_LIGHTS, &val);
-    assert(glGetError() == GL_NO_ERROR &&
-           "GL error when calling glGetInteger() -- no current GL context?");
+  SoDebugError::postWarning("SoGLLightIdElement::getMaxGLSources",
+                            "This function is obsoleted. It should not "
+                            "be used because its interface is fubar: "
+                            "the number of light sources available from "
+                            "the OpenGL driver depends on the context, and "
+                            "this function does not know which context this "
+                            "information is requested for.");
 
-    SoGLLightIdElement::maxGLSources = (int32_t)val;
-  }
+  GLint val;
+  glGetIntegerv(GL_MAX_LIGHTS, &val);
 
-  return SoGLLightIdElement::maxGLSources;
+  assert(glGetError() == GL_NO_ERROR &&
+         "GL error when calling glGetInteger() -- no current GL context?");
+
+  return (int32_t)val;
 }
 
 //! FIXME: write doc.
