@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -29,35 +29,42 @@
   This is a lazy element. It is evaluated in SoShape::shouldGLRender().
 */
 
+// *************************************************************************
+
 #include <Inventor/elements/SoGLTextureImageElement.h>
-#include <Inventor/elements/SoTextureQualityElement.h>
-#include <Inventor/elements/SoGLCacheContextElement.h>
-#include <Inventor/elements/SoShapeStyleElement.h>
-#include <Inventor/elements/SoLazyElement.h>
-#include <Inventor/actions/SoGLRenderAction.h>
-#include <Inventor/misc/SoGL.h> // GL wrapper.
-#include <Inventor/C/glue/glp.h>
-#include <Inventor/misc/SoGLImage.h>
-#include <Inventor/misc/SoGLBigImage.h>
-#include <Inventor/SbImage.h>
-#include <Inventor/C/tidbits.h>
+
 #include <stdlib.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif // HAVE_CONFIG_H
 
-#if COIN_DEBUG
+#include <Inventor/C/glue/glp.h>
+#include <Inventor/C/tidbits.h>
+#include <Inventor/SbImage.h>
+#include <Inventor/actions/SoGLRenderAction.h>
+#include <Inventor/elements/SoGLCacheContextElement.h>
+#include <Inventor/elements/SoLazyElement.h>
+#include <Inventor/elements/SoShapeStyleElement.h>
+#include <Inventor/elements/SoTextureQualityElement.h>
 #include <Inventor/errors/SoDebugError.h>
-#endif // COIN_DEBUG
+#include <Inventor/misc/SoGL.h> // GL wrapper.
+#include <Inventor/misc/SoGLBigImage.h>
+#include <Inventor/misc/SoGLImage.h>
+
+// *************************************************************************
 
 // Can be used as a workaround for buggy PROXY texture handling (bug
 // in our code or in the OpenGL driver).
 static int COIN_MAXIMUM_TEXTURE2_SIZE = 0;
 static int COIN_MAXIMUM_TEXTURE3_SIZE = 0;
 
+// *************************************************************************
+
 SO_ELEMENT_SOURCE(SoGLTextureImageElement);
 
+// *************************************************************************
+
 /*!
   This static method initializes static data in the
   SoGLTextureImageElement class.
@@ -211,14 +218,17 @@ SoGLTextureImageElement::hasTransparency(void) const
 int32_t
 SoGLTextureImageElement::getMaxGLTextureSize(void)
 {
-  static int32_t maxGLTextureSize = -1;
+  SoDebugError::postWarning("SoGLTextureImageElement::getMaxGLTextureSize",
+                            "This function is obsoleted. It should not "
+                            "be used because its interface is fubar: "
+                            "the maximum texture size handled by "
+                            "the OpenGL driver depends on the context, and "
+                            "this function does not know which context this "
+                            "information is requested for.");
 
-  if (maxGLTextureSize == -1) {
-    GLint val;
-    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &val);
-    maxGLTextureSize = (int32_t)val;
-  }
-  return maxGLTextureSize;
+  GLint val;
+  glGetIntegerv(GL_MAX_TEXTURE_SIZE, &val);
+  return (int32_t)val;
 }
 
 /*!
@@ -257,10 +267,20 @@ SoGLTextureImageElement::updateLazyElement(void) const
 {
   if (state->isElementEnabled(SoLazyElement::getClassStackIndex())) {
     uint32_t glimageid = this->glimage ? this->glimage->getGLImageId() : 0;
-    SbBool alphatest = this->glimage && this->glimage->getImage() && 
-      this->glimage->getImage()->hasData() ? 
-      this->glimage->useAlphaTest() : FALSE;
-    
+    SbBool alphatest = FALSE;
+    uint32_t flags = this->glimage ? this->glimage->getFlags() : 0;
+    if (flags & SoGLImage::FORCE_ALPHA_TEST_TRUE) {
+      alphatest = TRUE;      
+    }
+    else if (flags & SoGLImage::FORCE_ALPHA_TEST_FALSE) {
+      alphatest = FALSE;
+    }
+    else {
+      alphatest = this->glimage && 
+        this->glimage->getImage() && 
+        this->glimage->getImage()->hasData() ? 
+        this->glimage->useAlphaTest() : FALSE;
+    }
     SoLazyElement::setGLImageId(state, glimageid, alphatest);
   }
 }
