@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -470,6 +470,9 @@ SoElement::initElements(void)
   SoBumpMapMatrixElement::initClass();
 
   SoTextureCombineElement::initClass();
+#ifdef COIN_NEXT_MINOR
+  SoGLVBOElement::initClass();
+#endif // COIN_NEXT_MINOR
 }
 
 // Note: the following documentation for initClass() will also be
@@ -491,8 +494,7 @@ SoElement::initClass(void)
   SoElement::classStackIndex = -1;
   SoElement::initElements();
 
-  // cleanup at exit
-  coin_atexit((coin_atexit_f *)SoElement::cleanup, 0);
+  coin_atexit((coin_atexit_f *)SoElement::cleanup, CC_ATEXIT_NORMAL);
 }
 
 // atexit callback
