@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -31,17 +31,35 @@
   the speed of the time source plus logic to reset, stop and restart
   it.
 
+  Simple usage example, combined with SoComposeRotation:
+
+  \code
+  #Inventor V2.1 ascii
+  
+  Separator {
+     Transform {
+        rotation =
+        ComposeRotation { axis 0 1 0  angle =
+           ElapsedTime { }.timeOut
+        }.rotation
+     }
+     Cube { }
+  }
+  \endcode
+
   \sa SoTimeCounter
 */
 
+// *************************************************************************
+
 #include <Inventor/engines/SoElapsedTime.h>
-#include <Inventor/lists/SoEngineOutputList.h>
+
 #include <Inventor/SoDB.h>
 #include <Inventor/engines/SoSubEngineP.h>
-
-#if COIN_DEBUG
 #include <Inventor/errors/SoDebugError.h>
-#endif // COIN_DEBUG
+#include <Inventor/lists/SoEngineOutputList.h>
+
+// *************************************************************************
 
 /*!
   \var SoSFTime SoElapsedTime::timeIn
@@ -77,8 +95,12 @@
   (SoSFTime) Output time value. Starts at 0.
 */
 
+// *************************************************************************
+
 SO_ENGINE_SOURCE(SoElapsedTime);
 
+// *************************************************************************
+
 // Documented in superclass.
 void
 SoElapsedTime::initClass(void)
@@ -116,6 +138,8 @@ SoElapsedTime::~SoElapsedTime()
 {
 }
 
+// *************************************************************************
+
 // Documented in superclass.
 void
 SoElapsedTime::evaluate(void)
@@ -196,7 +220,11 @@ SoElapsedTime::writeInstance(SoOutput * out)
 
   // Re-connect to realTime field.
   if (connectfromrealTime) {
-    this->timeIn.connectFrom(connectfield);
+    // Don't send notification when reconnecting to preserve the state
+    // of the scenegraph between write passes.
+    this->timeIn.connectFrom(connectfield, TRUE);
     this->timeIn.setDefault(defaultflag);
   }
 }
+
+// *************************************************************************
