@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -119,10 +119,22 @@ SoReadError::post(const SoInput * const in, const char * const format, ...)
   error.setDebugString("Coin read error: ");
   error.appendToDebugString(formatstr.getString());
   error.appendToDebugString("\n");
+  
   SbString s;
   in->getLocationString(s);
   error.appendToDebugString(s.getString());
 
+  // for some reason the getIVVersion() function is not const. We
+  // therefore need to make the SoInput cast here to be able to test
+  // for Inventor V1.0 files
+  if (((SoInput*)in)->getIVVersion() == 1.0f) {
+    error.appendToDebugString("\nThis operation might have failed due to limited "
+                              "support for old Inventor files. If this is the case, "
+                              "please get in touch with Systems in Motion "
+                              "at <coin-support@coin3d.org>, and we will "
+                              "rectify the situation.");
+  }
+
   if (SoReadError::callback != SoError::defaultHandlerCB) {
     SoReadError::callback(&error, SoReadError::callbackData);
   }
