@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -25,12 +25,21 @@
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 
+#include <Inventor/C/tidbitsp.h>
 #include <Inventor/C/errors/debugerror.h>
 #include <stdlib.h>
 
 static cc_debugerror_cb * dbgerr_callback = (cc_debugerror_cb *)cc_error_default_handler_cb;
 static void * dbgerr_callback_data = NULL;
+static SbBool dbgerr_cleanup_function_set = FALSE;
 
+static void
+debugerror_cleanup(void)
+{
+  dbgerr_callback = (cc_debugerror_cb *)cc_error_default_handler_cb;
+  dbgerr_callback_data = NULL;
+  dbgerr_cleanup_function_set = FALSE;
+}
 
 void
 cc_debugerror_init(cc_debugerror * me)
@@ -56,6 +65,11 @@ cc_debugerror_set_handler_callback(cc_debugerror_cb * function, void * data)
 {
   dbgerr_callback = function;
   dbgerr_callback_data = data;
+
+  if (!dbgerr_cleanup_function_set) {
+    coin_atexit((coin_atexit_f*) debugerror_cleanup, CC_ATEXIT_MSG_SUBSYSTEM);
+    dbgerr_cleanup_function_set = TRUE;
+  }
 }
 
 cc_debugerror_cb *
