@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -82,7 +82,9 @@ static void
 sokeyboardevent_cleanup(void)
 {
   delete converttoprintable;
+  converttoprintable = NULL;
   delete converttoprintable_shift;
+  converttoprintable_shift = NULL;
 }
 
 static void
@@ -91,7 +93,7 @@ build_convert_dicts(void)
   int i;
   converttoprintable = new SbHash<char, int>();
   converttoprintable_shift = new SbHash<char, int>();
-  coin_atexit((coin_atexit_f *)sokeyboardevent_cleanup, 0);
+  coin_atexit((coin_atexit_f *)sokeyboardevent_cleanup, CC_ATEXIT_NORMAL);
 
 #undef ADD_KEY // just in case
 #define ADD_KEY(x,y) d->put(SoKeyboardEvent::x, y)
