@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -157,8 +157,10 @@ static const char CONNECTIONCHAR = '=';
 class SoConnectStorage {
 public:
   SoConnectStorage(SoFieldContainer * c, SoType t)
-    : container(c), fieldtype(t),
-      maptoconverter(13) // save about ~1kB vs default nr of buckets
+    : container(c), 
+    lastnotify(NULL),
+    fieldtype(t),
+    maptoconverter(13) // save about ~1kB vs default nr of buckets
     {
     }
 
@@ -189,6 +191,9 @@ public:
   // Direct auditors of us.
   SoAuditorList auditors;
 
+  // used to track the last notification (for fanIn handling)
+  void * lastnotify;
+
   // Convenience functions for adding, removing and finding mappings.
 
   void addConverter(const void * item, SoFieldConverter * converter)
@@ -213,7 +218,30 @@ public:
     if (!this->maptoconverter.get(item, val)) { return NULL; }
     return val;
   }
-  
+
+  SbBool hasFanIn(void) {
+    return (this->masterfields.getLength() + this->masterengineouts.getLength()) > 1;
+  }
+  int findFanInEngine(void) const {
+    for (int i = 0; i < this->masterengineouts.getLength(); i++) {
+      SoEngineOutput * o = this->masterengineouts[i];
+      if (o->isNodeEngineOutput()) {
+        if (((void*)(o->getNodeContainer())) == this->lastnotify) return i;
+      }
+      else {
+        if (((void*)(o->getContainer())) == this->lastnotify) return i;
+      }
+    }
+    return -1;
+  }
+  int findFanInField(void) const {
+    for (int i = 0; i < this->masterfields.getLength(); i++) {
+      if (((void*)this->masterfields[i]->getContainer() == this->lastnotify)) return i; 
+    }
+    return -1;
+  }
+
+
   // Provides us with a hack to get at a master field's type in code
   // called from its constructor (SoField::getTypeId() is virtual and
   // can't be used).
@@ -221,12 +249,12 @@ public:
   // (Used in the master::~SoField() -> slave::disconnect(master)
   // chain.)
   SoType fieldtype;
-
   void add_vrml2_routes(SoOutput * out, const SoField * f);
 
 private:
   // Dictionary of void* -> SoFieldConverter* mappings.
   SbHash<SoFieldConverter *, const void *> maptoconverter;
+
 };
 
 // helper function. Used to check if field is in a vrml2 node
@@ -237,8 +265,10 @@ is_vrml2_field(const SoField * f)
   SoFieldContainer * fc = f->getContainer();
   // test fc to support fields with no container
   if (fc && fc->isOfType(SoNode::getClassTypeId())) {
+    if (fc->isOfType(SoProtoInstance::getClassTypeId())) return TRUE;
     if (((SoNode*)fc)->getNodeType() & SoNode::VRML2) return TRUE;
   }
+
   return FALSE;
 }
 
@@ -332,10 +362,10 @@ SbHash<char **, char *> * SoFieldP::ptrhash = NULL;
 SbHash<char **, char *> *
 SoFieldP::getReallocHash(void)
 {
-  // XXX protect with mutex?
+  // FIXME: protect with mutex?
   if (SoFieldP::ptrhash == NULL) {
     SoFieldP::ptrhash = new SbHash<char **, char *>;
-    coin_atexit((coin_atexit_f *)SoFieldP::hashExitCleanup, 0);
+    coin_atexit((coin_atexit_f *)SoFieldP::hashExitCleanup, CC_ATEXIT_NORMAL);
   }
   return SoFieldP::ptrhash;
 }
@@ -351,7 +381,7 @@ SoFieldP::hashExitCleanup(void)
 void *
 SoFieldP::hashRealloc(void * bufptr, size_t size)
 {
-  char ** bufptrptr;
+  char ** bufptrptr = NULL;
   int ok = SoFieldP::ptrhash->get((char *)bufptr, bufptrptr);
   assert(ok);
 
@@ -555,7 +585,6 @@ static void
 field_mutex_cleanup(void)
 {
   CC_MUTEX_DESTRUCT(sofield_mutex);
-  sofield_mutex = NULL;
 }
 
 /*!
@@ -569,7 +598,7 @@ SoField::initClass(void)
   assert(SoField::classTypeId == SoType::badType());
 
   CC_MUTEX_CONSTRUCT(sofield_mutex);
-  coin_atexit((coin_atexit_f*) field_mutex_cleanup, 0);
+  coin_atexit((coin_atexit_f*) field_mutex_cleanup, CC_ATEXIT_NORMAL);
 
   SoField::classTypeId = SoType::createType(SoType::badType(), "Field");
   SoField::initClasses();
@@ -1345,17 +1374,21 @@ SoField::notify(SoNotList * nlist)
   }
 #endif // COIN_DEBUG
 
-  // don't process the notification if we're notified from a
-  // connection, and connection is disabled (through
-  // enableConnection())
-  if (!this->isConnectionEnabled()) {
-    // check NotRec type to find if the notification was from a
-    // connection. If someone changes the field directly we should
-    // just continue.
-    SoNotRec * rec = nlist->getLastRec();
-    if (rec) {
-      SoNotRec::Type t = nlist->getLastRec()->getType();
-      if (t == SoNotRec::ENGINE || t == SoNotRec::FIELD) return;
+  // check NotRec type to find if the notification was from a
+  // connection. If someone changes the field directly we should
+  // just continue.
+
+  SoNotRec * rec = nlist->getLastRec();
+  if (rec) {
+    SoNotRec::Type t = nlist->getLastRec()->getType();  
+    if (t == SoNotRec::ENGINE || t == SoNotRec::FIELD) {
+      if (this->hasExtendedStorage()) {
+        this->storage->lastnotify = (void*) rec->getBase();
+      }
+      // don't process the notification if we're notified from a
+      // connection, and connection is disabled (through
+      // enableConnection())
+      if (!this->isConnectionEnabled()) return;
     }
   }
 
@@ -1528,10 +1561,16 @@ SoField::shouldWrite(void) const
                          finfo.getString(), this->isDefault(),
                          this->isIgnored(), this->isConnected());
 #endif // debug
-
+  
   if (!this->isDefault()) return TRUE;
   if (this->isIgnored()) return TRUE;
+
   if (this->isConnected()) {
+#if 0 // disabled (was only needed for bidirectional connections in PROTOs)
+    // I suspect this code was here only to make the bidirectional
+    // connection hack in SoProto work. Connected PROTO instance
+    // fields should be written even if they have the default value
+    // (just like any other field). pederb, 2005-12-20
     SoFieldContainer * thecontainer = this->getContainer();
     if ( thecontainer != NULL &&
          thecontainer->isOfType(SoProtoInstance::getClassTypeId()) ) {
@@ -1541,6 +1580,7 @@ SoField::shouldWrite(void) const
       // when they are read in again later anyways. -- 20040115 larsa
       return FALSE;
     }
+#endif // disabled PROTO hack
     return TRUE;
   }
 
@@ -1904,8 +1944,9 @@ SoField::write(SoOutput * out, const SbName & name) const
       // connected
       if (this->isDefault()) return;
     }
-    // never write eventIn fields
-    if (this->getFieldType() == SoField::EVENTIN_FIELD) return;
+    // never write eventIn or eventOut fields
+    if ((this->getFieldType() == SoField::EVENTIN_FIELD) ||
+        (this->getFieldType() == SoField::EVENTOUT_FIELD)) return;
   }
   
   // ASCII write.
@@ -2323,42 +2364,46 @@ SoField::resolveWriteConnection(SbName & mastername) const
 void
 SoField::evaluateConnection(void) const
 {
+  SbBool fanin = this->storage->hasFanIn();
+  SbBool didevaluate = FALSE;
+
   // FIXME: should we evaluate from all masters in turn? 19990623 mortene.
   if (this->isConnectedFromField()) {
-    int idx = this->storage->masterfields.getLength() - 1;
-    SoField * master = this->storage->masterfields[idx];
-    // don't copy if master is destructing, or if master is currently
-    // evaluating. The master might be evaluating if we have circular
-    // field connections. If this is the case, the field will already
-    // contain the correct value, and we should not copy again.
-    if (!master->isDestructing() && !master->getStatus(FLAG_ISEVALUATING)) {
-      SoFieldConverter * converter = this->storage->findConverter(master);
-      if (converter) converter->evaluateWrapper();
-      else {
-        SoField * that = (SoField *)this; // cast away const
-        // Copy data. Disable notification first since notification
-        // has already been sent from the master.
-        SbBool oldnotify = that->enableNotify(FALSE);
-        that->copyFrom(*master);
-        (void) that->enableNotify(oldnotify);
+    int idx = fanin ? this->storage->findFanInField() : this->storage->masterfields.getLength() - 1;
+    if (idx >= 0) {
+      didevaluate = TRUE;
+      SoField * master = this->storage->masterfields[idx];
+      // don't copy if master is destructing, or if master is currently
+      // evaluating. The master might be evaluating if we have circular
+      // field connections. If this is the case, the field will already
+      // contain the correct value, and we should not copy again.
+      if (!master->isDestructing() && !master->getStatus(FLAG_ISEVALUATING)) {
+        SoFieldConverter * converter = this->storage->findConverter(master);
+        if (converter) converter->evaluateWrapper();
+        else {
+          SoField * that = (SoField *)this; // cast away const
+          // Copy data. Disable notification first since notification
+          // has already been sent from the master.
+          SbBool oldnotify = that->enableNotify(FALSE);
+          that->copyFrom(*master);
+          (void) that->enableNotify(oldnotify);
+        }
       }
     }
   }
-  else if (this->isConnectedFromEngine()) {
-    int idx = this->storage->masterengineouts.getLength() - 1;
-    SoEngineOutput * master = this->storage->masterengineouts[idx];
-    SoFieldConverter * converter = this->storage->findConverter(master);
-    if (converter) converter->evaluateWrapper();
-    else if (master->isNodeEngineOutput()) {
-      master->getNodeContainer()->evaluateWrapper();
+  if (this->isConnectedFromEngine() && !didevaluate) {
+    int idx = fanin ? this->storage->findFanInEngine() : this->storage->masterengineouts.getLength() - 1;
+    if (idx >= 0) {
+      SoEngineOutput * master = this->storage->masterengineouts[idx];
+      SoFieldConverter * converter = this->storage->findConverter(master);
+      if (converter) converter->evaluateWrapper();
+      else if (master->isNodeEngineOutput()) {
+        master->getNodeContainer()->evaluateWrapper();
+      }
+      else {
+        master->getContainer()->evaluateWrapper();
+      }
     }
-    else {
-      master->getContainer()->evaluateWrapper();
-    }
-  }
-  else {
-    // Should never happen.
-    assert(0);
   }
 }
 
@@ -2442,6 +2487,9 @@ SoField::initClasses(void)
 {
   SoSField::initClass();
   SoSFBox3s::initClass();
+#ifdef COIN_NEXT_MINOR
+  SoSFBox3f::initClass();
+#endif
   SoSFBool::initClass();
   SoSFColor::initClass();
   SoSFEngine::initClass();
