@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -176,14 +176,14 @@ SoFieldContainer::initClass(void)
     SoType::createType(inherited::getClassTypeId(), "FieldContainer", NULL);
 
   sofieldcontainer_userdata_dict = new UserDataMap;
-  coin_atexit((coin_atexit_f*) sofieldcontainer_userdata_cleanup, 0);
+  coin_atexit((coin_atexit_f*) sofieldcontainer_userdata_cleanup, CC_ATEXIT_NORMAL);
 
   sofieldcontainer_copydictstorage = 
     new SbStorage(sizeof(sofieldcontainer_copydict),
                   sofieldcontainer_construct_copydict,
                   sofieldcontainer_destruct_copydict);                 
 
-  coin_atexit((coin_atexit_f*) sofieldcontainer_copydict_cleanup, 0);
+  coin_atexit((coin_atexit_f*) sofieldcontainer_copydict_cleanup, CC_ATEXIT_NORMAL);
 }
 
 // Overridden from parent class.
@@ -618,10 +618,24 @@ SoFieldContainer::addWriteReference(SoOutput * out, SbBool isfromfield)
 }
 
 
+// Note that this documentation will also be used for all subclasses
+// which reimplements the method, so keep the doc "generic enough".
 /*!
   Writes all the fields contained in this instance to the output stream
   within \a out.
- */
+
+  This method is solely called from the write methods of fields.
+
+  Either from SoField if the write is done because of a field-to-field
+  connection, or from one of the fields which may actually write
+  SoFieldContainer instances, i.e. SoSFNode, SoMFNode, SoSFEngine,
+  SoMFEngine, SoSFPath and SoMFPath.
+
+  This method, in addition to the ordinary write() method of SoNode,
+  needs to be present since the fields don't have a write action
+  instance in their writeValue() method, and writeInstance() will
+  create a new SoWriteAction and call continueToApply() on it.
+*/
 void
 SoFieldContainer::writeInstance(SoOutput * out)
 {
@@ -806,7 +820,12 @@ SoFieldContainer::checkCopy(const SoFieldContainer * orig)
   the copy outside of this method, this will go undetected and the
   guts of \a orig will be copied multiple times into its copy.
 
-  \sa checkCopy()
+  If copyContents() is called directly (instead of using copy()), it's
+  assumed that the user only wants to copy the field values, and we
+  just return NULL here. This is done to match how it's done in SGI
+  Inventor.
+
+  \sa checkCopy() 
 */
 SoFieldContainer *
 SoFieldContainer::findCopy(const SoFieldContainer * orig,
@@ -815,6 +834,12 @@ SoFieldContainer::findCopy(const SoFieldContainer * orig,
   sofieldcontainer_copydict * copydict =
     sofieldcontainer_get_copydict();
 
+  // Sometimes copyContents() is called directly (initCopyDict() is
+  // not called first) to just copy the fields of a single node. This
+  // is often done in SGI example code. We should just return NULL
+  // here in those cases to be compatible with SGI Inventor.
+  if (copydict->copiedinstancestack->getLength() == 0) return NULL;
+
   SoFieldContainerCopyMap * copiedinstances = (*(copydict->copiedinstancestack))[0];
   ContentsCopiedMap * contentscopied  = (*(copydict->contentscopiedstack))[0];
   
@@ -858,7 +883,7 @@ SoFieldContainer::findCopy(const SoFieldContainer * orig,
   // Don't call copyContents for the proto instance root node, since
   // this is handled by the Proto node.
   if (!protoinst) {
-    SbBool copied;
+    SbBool copied = FALSE;
     SbBool chk = contentscopied->get(orig, copied);
     assert(chk);
     
