@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -41,6 +41,7 @@
 
 #include <Inventor/fields/SoGlobalField.h>
 
+#include <Inventor/SoDB.h>
 #include <Inventor/C/tidbitsp.h>
 #include <Inventor/SbName.h>
 #include <Inventor/SoInput.h>
@@ -123,7 +124,7 @@ SoGlobalField::initClass(void)
                        SoGlobalField::createInstance);
 
   SoGlobalField::allcontainers = new SoBaseList;
-  coin_atexit((coin_atexit_f *)SoGlobalField::clean, 0);
+  coin_atexit((coin_atexit_f *)SoGlobalField::clean, CC_ATEXIT_NORMAL);
 }
 
 // Free up resources.
@@ -315,7 +316,11 @@ void
 SoGlobalField::addWriteReference(SoOutput * out, SbBool isfromfield)
 {
   assert(this->classfielddata);
-  inherited::addWriteReference(out, FALSE);
+  // The global realTime field will never be written to file and
+  // should thus never be writeref'ed.
+  if (getGlobalField() != SoDB::getGlobalField("realTime")) {
+    inherited::addWriteReference(out, FALSE);
+  }
 }
 
 
