@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -296,7 +296,7 @@ SoMField::initClass(void)
   PRIVATE_FIELD_INIT_CLASS(SoMField, "MField", inherited, NULL);
 
   CC_MUTEX_CONSTRUCT(somfield_mutex);
-  coin_atexit((coin_atexit_f*) somfield_mutex_cleanup, 0);
+  coin_atexit((coin_atexit_f*) somfield_mutex_cleanup, CC_ATEXIT_NORMAL);
 }
 
 /*!
@@ -397,7 +397,7 @@ SoMField::get1(const int index, SbString & valuestring)
   if (mfield_buffer_size < STARTSIZE) {
     mfield_buffer = malloc(STARTSIZE);
     mfield_buffer_size = STARTSIZE;
-    coin_atexit((coin_atexit_f *)mfield_buffer_cleanup, 0);
+    coin_atexit((coin_atexit_f *)mfield_buffer_cleanup, CC_ATEXIT_NORMAL);
   }
 
   out.setBuffer(mfield_buffer, mfield_buffer_size,
