@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -38,7 +38,7 @@
 // *************************************************************************
 
 #include <Inventor/fields/SoSFBox3s.h>
-#include <Inventor/fields/SoSubField.h>
+#include <Inventor/fields/SoSubFieldP.h>
 #include <Inventor/SoInput.h>
 #include <Inventor/SoOutput.h>
 #include <Inventor/errors/SoDebugError.h>
@@ -54,7 +54,7 @@ SO_SFIELD_SOURCE(SoSFBox3s, SbBox3s, SbBox3s);
 void
 SoSFBox3s::initClass(void)
 {
-  SO_SFIELD_INIT_CLASS(SoSFBox3s, SoSField);
+  SO_SFIELD_INTERNAL_INIT_CLASS(SoSFBox3s);
 }
 
 // No need to document readValue() and writeValue() here, as the
@@ -74,7 +74,7 @@ SoSFBox3s::readValue(SoInput * in)
       !in->read(max[0]) || 
       !in->read(max[1]) || 
       !in->read(max[2])) {
-    SoReadError::post(in, "Couldn't read vector");
+    SoReadError::post(in, "Couldn't read SoSFBox3s");
     return FALSE;
   }
   this->setValue(min[0], min[1], min[2], max[0], max[1], max[2]);
@@ -88,11 +88,17 @@ SoSFBox3s::writeValue(SoOutput * out) const
   short max[3];
   SbBox3s b = this->getValue();
   b.getBounds(min[0], min[1], min[2], max[0], max[1], max[2]);
+
   out->write(min[0]);
+  if (!out->isBinary()) out->write(' ');
   out->write(min[1]);
+  if (!out->isBinary()) out->write(' ');
   out->write(min[2]);
+  if (!out->isBinary()) out->write(' ');
   out->write(max[0]);
+  if (!out->isBinary()) out->write(' ');
   out->write(max[1]);
+  if (!out->isBinary()) out->write(' ');
   out->write(max[2]);
 }
 
