@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -57,7 +57,6 @@
 
 static void * flw_global_lock = NULL;
 static int flw_global_font_index = 0;
-static unsigned int flw_global_glyph_index = 0;
 
 /* Debug: enable this in case code hangs waiting for a lock.  A hang
    will typically happen for one out of two reasons:
@@ -179,7 +178,8 @@ using_freetype(void)
     if (available) {
       coin_atexit((coin_atexit_f *)freetype_cleanup,
                   /* priority must be lower than for abstraction
-                     interface, so don't change this willy-nilly: */ 0);
+                     interface, so don't change this willy-nilly: */
+                  CC_ATEXIT_FONT_SUBSYSTEM - 1);
     }
   }
 
@@ -230,7 +230,8 @@ using_win32api(void)
     if (available) {
       coin_atexit((coin_atexit_f *)win32api_cleanup,
                   /* priority must be lower than for abstraction
-                     interface, so don't change this willy-nilly: */ 0);
+                     interface, so don't change this willy-nilly: */
+                  CC_ATEXIT_FONT_SUBSYSTEM - 1);
     }
   }
 
@@ -281,8 +282,8 @@ fontstruct_rmglyph(struct cc_flw_font * fs, unsigned int glyph)
   struct cc_flw_glyph * gs = flw_glyphidx2glyphptr(fs, glyph);
   assert(gs);
 
-  if (gs->bitmap && !gs->fromdefaultfont) {
-    if (gs->bitmap->buffer) { free(gs->bitmap->buffer); }
+  if (gs->bitmap) {
+    if (!gs->fromdefaultfont && gs->bitmap->buffer) { free(gs->bitmap->buffer); }
     free(gs->bitmap);
   }
 
@@ -356,8 +357,20 @@ flw_exit(void)
     struct cc_flw_font * fs = (struct cc_flw_font *)cc_dynarray_get(fontarray, n);
 
 #if COIN_DEBUG
-    cc_debugerror_postinfo("flw_exit", "font instance resource leak:");
-    dump_cc_flw_font("flw_exit", fs);
+    /* don't use cc_debugerror_post_* here since this will be called when Coin is
+       exiting => we cannot allocate new resources */
+    fprintf(stderr, "flw_exit: font instance resource leak:\n");
+    fprintf(stderr, "nativefonthandle==%p, fontname=='%s', requestname=='%s', "
+            "glyphdict==%p, sizey==%u, angle==%f, "
+            "complexity==%f, defaultfont==%s, fontindex==%d, "
+            "refcount==%d",
+            fs->nativefonthandle,
+            cc_string_get_text(fs->fontname),
+            cc_string_get_text(fs->requestname),
+            fs->glyphdict, fs->sizey, fs->angle,
+            fs->complexity,
+            fs->defaultfont ? "TRUE" : "FALSE",
+            fs->fontindex, fs->refcount);
 #endif /* COIN_DEBUG */
 
     fontstruct_rmfont(fs->fontindex);
@@ -366,6 +379,7 @@ flw_exit(void)
   cc_dynarray_destruct(fontarray);
 
   CC_MUTEX_DESTRUCT(flw_global_lock);
+  flw_global_font_index = 0;
 }
 
 static void
@@ -387,7 +401,9 @@ flw_initialize(void)
   coin_atexit((coin_atexit_f *)flw_exit,
               /* priority must be higher than for atexit cleanup of
                  underlying, native interfaces (so this is done
-                 first), so don't change this: */ 1);
+                 first), so keep this correctly in sync with other
+                 coin_atexit() invocations in the font sub-system: */
+              CC_ATEXIT_FONT_SUBSYSTEM);
 
   FLW_MUTEX_UNLOCK(flw_global_lock);
 }
