@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -44,6 +44,10 @@
 #include <Inventor/C/glue/freetype.h>
 #include <Inventor/C/glue/GLUWrapper.h>
 
+#ifdef HAVE__STRDUP
+#define strdup(a) _strdup(a)
+#endif /* HAVE__STRDUP */
+
 /* ************************************************************************* */
 
 #ifdef __cplusplus
@@ -75,7 +79,7 @@ void cc_flwft_done_font(void * font) { assert(FALSE); }
 int cc_flwft_get_num_charmaps(void * font) { assert(FALSE); return 0; }
 const char * cc_flwft_get_charmap_name(void * font, int charmap) { assert(FALSE); return NULL; }
 void cc_flwft_set_charmap(void * font, int charmap) { assert(FALSE); }
-void cc_flwft_set_char_size(void * font, int width, int height) { assert(FALSE); }
+void cc_flwft_set_char_size(void * font, int height) { assert(FALSE); }
 
 
 void cc_flwft_set_font_rotation(void * font, float angle) { assert(FALSE); }
@@ -443,7 +447,7 @@ cc_flwft_initialize(void)
     cc_dynarray_append(cc_flwft_globals.fontfiledirs, str);
   }
 
-  /* Setup temporary glyph-struct used during for tessellation */
+  /* Setup temporary glyph-struct used during tessellation */
   flwft_tessellator.vertexlist = NULL;
   flwft_tessellator.faceindexlist = NULL;
   flwft_tessellator.edgeindexlist = NULL;
@@ -469,11 +473,14 @@ cc_flwft_exit(void)
     free(cc_dynarray_get(cc_flwft_globals.fontfiledirs, i));
   }
   cc_dynarray_destruct(cc_flwft_globals.fontfiledirs);
+  cc_flwft_globals.fontfiledirs = NULL;
 
   cc_dict_apply(cc_flwft_globals.fontname2filename, clean_fontmap_hash, NULL);
   cc_dict_destruct(cc_flwft_globals.fontname2filename);
+  cc_flwft_globals.fontname2filename = NULL;
 
   cc_ftglue_FT_Done_FreeType(library);
+  library = NULL;
 }
 
 static const char *
@@ -544,7 +551,7 @@ find_font_file(const char * fontname, unsigned int pixelsize)
       cc_fcglue_FcPatternPrint(matched_pattern);
     }
 
-    foundfile = strdup((const char*)filename);
+    foundfile = cc_namemap_get_address((const char *)filename);
 
     cc_fcglue_FcPatternDestroy(font_pattern);
     cc_fcglue_FcPatternDestroy(matched_pattern);
@@ -647,7 +654,6 @@ cc_flwft_get_font(const char * fontname, const unsigned int pixelsize)
   }
 
   cc_flwft_set_charmap(face, FT_ENCODING_ADOBE_LATIN_1);
-
   return face;
 }
 
@@ -1027,11 +1033,11 @@ cc_flwft_get_vector_glyph(void * font, unsigned int glyphindex, float complexity
   g = (FT_OutlineGlyph)tmp;
   outline = g->outline;
 
-  if ( flwft_tessellator.vertexlist == NULL)
+  if (flwft_tessellator.vertexlist == NULL)
      flwft_tessellator.vertexlist = cc_list_construct();
-  if ( flwft_tessellator.faceindexlist == NULL)
+  if (flwft_tessellator.faceindexlist == NULL)
      flwft_tessellator.faceindexlist = cc_list_construct();
-  if ( flwft_tessellator.edgeindexlist == NULL)
+  if (flwft_tessellator.edgeindexlist == NULL)
      flwft_tessellator.edgeindexlist = cc_list_construct();
   if (flwft_tessellator.malloclist == NULL) {
     flwft_tessellator.malloclist = cc_list_construct();
@@ -1055,7 +1061,7 @@ cc_flwft_get_vector_glyph(void * font, unsigned int glyphindex, float complexity
   flwft_tessellator.triangle_strip_flipflop = FALSE;
   flwft_tessellator.vertex_counter = 0;
 
-  /* gluTessllator callbacks */
+  /* gluTessellator callbacks */
   GLUWrapper()->gluTessCallback(flwft_tessellator.tessellator_object, GLU_TESS_VERTEX, (gluTessCallback_cb_t) flwft_vertexCallback);
   GLUWrapper()->gluTessCallback(flwft_tessellator.tessellator_object, GLU_TESS_BEGIN, (gluTessCallback_cb_t) flwft_beginCallback);
   GLUWrapper()->gluTessCallback(flwft_tessellator.tessellator_object, GLU_TESS_END, (gluTessCallback_cb_t) flwft_endCallback);
@@ -1063,8 +1069,8 @@ cc_flwft_get_vector_glyph(void * font, unsigned int glyphindex, float complexity
   GLUWrapper()->gluTessCallback(flwft_tessellator.tessellator_object, GLU_TESS_ERROR, (gluTessCallback_cb_t) flwft_errorCallback);
 
   GLUWrapper()->gluTessBeginPolygon(flwft_tessellator.tessellator_object, NULL);
-  /* According to the SGI doc for GLU, specifying the triangle normal will speed up
-     the tessellation process. */
+  /* According to the SGI doc for GLU, specifying the triangle normal
+     will speed up the tessellation process. */
   GLUWrapper()->gluTessNormal(flwft_tessellator.tessellator_object, 0.0f, 0.0f, -1.0f);
   error = cc_ftglue_FT_Outline_Decompose(&outline, &outline_funcs, NULL);
   if (flwft_tessellator.contour_open) {
@@ -1076,6 +1082,7 @@ cc_flwft_get_vector_glyph(void * font, unsigned int glyphindex, float complexity
   }
 
   GLUWrapper()->gluTessEndPolygon(flwft_tessellator.tessellator_object);
+  /* FIXME: why not reuse the GLU tessellation object? 20060215 mortene. */
   GLUWrapper()->gluDeleteTess(flwft_tessellator.tessellator_object);
 
   cc_list_append(flwft_tessellator.faceindexlist, (void *) -1);
@@ -1122,7 +1129,10 @@ flwft_addTessVertex(double * vertex)
 
 }
 
-
+/* "move to" in this context means that a previous contour should be
+   terminated, as the virtual "pencil" is moved to start a new
+   contour. (so "move to" means "lift pencil, jump to and start new
+   contour".) */
 static int
 flwft_moveToCallback(FT_Vector * to, void * user)
 {
@@ -1144,6 +1154,8 @@ flwft_moveToCallback(FT_Vector * to, void * user)
   return 0;
 }
 
+/* "line to" in this context means that the current contour is
+   continued with a line to the given point. */
 static int
 flwft_lineToCallback(FT_Vector * to, void * user)
 {
@@ -1385,6 +1397,9 @@ flwft_endCallback(void)
 static void
 flwft_combineCallback(GLdouble coords[3], GLvoid * vertex_data, GLfloat weight[4], int **dataOut)
 {
+  /* FIXME: I believe this callback may not actually be necessary.
+     20060227 mortene. */
+
   int * ret;
   float * point;
 
@@ -1402,8 +1417,12 @@ flwft_combineCallback(GLdouble coords[3], GLvoid * vertex_data, GLfloat weight[4
 }
 
 static void
-flwft_errorCallback(GLenum error_code)
+flwft_errorCallback(GLenum errcode)
 {
+  const GLubyte * str = GLUWrapper()->gluGetString(errcode);
+  cc_debugerror_post("flwft_errorCallback",
+                     "Got following error from GLU when tessellating FreeType glyph: '%s'",
+                     str);
 }
 
 static void
