@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -36,28 +36,27 @@
 
 #include "fontlib_wrapper.h"
 #include "glyph2d.h"
+#include "glyph.h"
 
 static SbBool glyph2d_specmatch(const cc_font_specification * spec1, const cc_font_specification * spec2);
 
 struct cc_glyph2d {
-  int fontidx;    
-  int glyphidx;
-  uint32_t character;
-  float angle;
-  unsigned short width;
+  struct cc_glyph c; /* "c" for "common" glyph data (2d & 3d). */
+
+  float angle; /* FIXME: is this one really in use? 20060109 mortene. */
+  unsigned short width; /* FIXME: is this one really in use? 20060109 mortene. */
+
   unsigned short height;
   unsigned short bitmapwidth;
   unsigned short bitmapheight;
   short bitmapoffsetx;
   short bitmapoffsety;
-  cc_font_specification * fontspec;
   unsigned char * bitmap;
-  int refcount;
-  int fontid;
   SbBool mono;
 };
 
 static cc_dict * glyph2d_fonthash = NULL;
+static SbBool glyph2d_initialized = FALSE;
 
 /*
   Mutex lock for the static ang global font hash
@@ -85,27 +84,28 @@ static void
 cc_glyph2d_cleanup(void)
 {
   CC_MUTEX_DESTRUCT(glyph2d_fonthash_lock);
-  glyph2d_fonthash_lock = NULL;
   cc_dict_destruct(glyph2d_fonthash);
   glyph2d_fonthash = NULL;
+  glyph2d_initialized = FALSE;
 }
 
 static void
 cc_glyph2d_initialize()
 {
-
-  static SbBool initialized = FALSE;
   CC_MUTEX_CONSTRUCT(glyph2d_fonthash_lock);
   GLYPH2D_MUTEX_LOCK(glyph2d_fonthash_lock);
   
-  if (initialized) {
+  if (glyph2d_initialized) {
     GLYPH2D_MUTEX_UNLOCK(glyph2d_fonthash_lock);
     return;
   }
-  initialized = TRUE;
+  glyph2d_initialized = TRUE;
   
   glyph2d_fonthash = cc_dict_construct(15, 0.75);
-  coin_atexit((coin_atexit_f*) cc_glyph2d_cleanup, 0);
+
+  /* +1, so it happens before the underlying font abstraction layer
+     cleans itself up: */
+  coin_atexit((coin_atexit_f*) cc_glyph2d_cleanup, CC_ATEXIT_FONT_SUBSYSTEM + 1);
   
   GLYPH2D_MUTEX_UNLOCK(glyph2d_fonthash_lock);
 }
@@ -139,9 +139,9 @@ cc_glyph2d_ref(uint32_t character, const cc_font_specification * spec, float ang
     glyphlist = (cc_list *) val;
     for (i = 0; i < cc_list_get_length(glyphlist); ++i) {
       glyph = (cc_glyph2d *) cc_list_get(glyphlist, i);
-      if (glyph2d_specmatch(spec, glyph->fontspec)) {
+      if (glyph2d_specmatch(spec, glyph->c.fontspec)) {
         GLYPH2D_MUTEX_UNLOCK(glyph2d_fonthash_lock);
-        glyph->refcount++;
+        glyph->c.refcount++;
         return glyph;
       }
     }    
@@ -157,13 +157,13 @@ cc_glyph2d_ref(uint32_t character, const cc_font_specification * spec, float ang
 
   /* build a new glyph struct with bitmap */    
   glyph = (cc_glyph2d *) malloc(sizeof(cc_glyph2d));
-  glyph->character = character;
+  glyph->c.character = character;
   
   newspec = (cc_font_specification *) malloc(sizeof(cc_font_specification)); 
   assert(newspec);
   cc_fontspec_copy(spec, newspec);
 
-  glyph->fontspec = newspec;
+  glyph->c.fontspec = newspec;
 
   /* FIXME: fonttoload variable should be allocated on the
      stack. 20030921 mortene. */
@@ -196,7 +196,6 @@ cc_glyph2d_ref(uint32_t character, const cc_font_specification * spec, float ang
   cc_string_destruct(fonttoload);
   assert(fontidx >= 0);
 
-  glyph->fontid = fontidx;
   cc_flw_ref_font(fontidx);
 
   /* Should _always_ be able to get hold of a glyph -- if no glyph is
@@ -205,8 +204,8 @@ cc_glyph2d_ref(uint32_t character, const cc_font_specification * spec, float ang
   glyphidx = cc_flw_get_glyph(fontidx, character);
   assert(glyphidx >= 0);
 
-  glyph->glyphidx = glyphidx;
-  glyph->fontidx = fontidx;
+  glyph->c.glyphidx = glyphidx;
+  glyph->c.fontidx = fontidx;
   glyph->angle = angle;
 
   bm = cc_flw_get_bitmap(fontidx, glyphidx);
@@ -219,7 +218,7 @@ cc_glyph2d_ref(uint32_t character, const cc_font_specification * spec, float ang
   glyph->bitmapoffsety = bm->bearingY;
   glyph->bitmap = bm->buffer;
   glyph->mono = bm->mono;
-  glyph->refcount = 1;
+  glyph->c.refcount = 1;
   
   /* Store newly created glyph in the list for this character */
   cc_list_append(glyphlist, glyph);
@@ -228,39 +227,10 @@ cc_glyph2d_ref(uint32_t character, const cc_font_specification * spec, float ang
   return glyph;
 }
 
-void 
+void
 cc_glyph2d_unref(cc_glyph2d * glyph)
 {
-  glyph->refcount--;
-  if (glyph->refcount == 0) {
-  
-    cc_list * glyphlist;
-    int ret;
-    void * tmp;
-    int i;
-
-    ret = cc_dict_get(glyph2d_fonthash, (uintptr_t)glyph->character, &tmp);
-    assert(ret);
-    glyphlist = (cc_list*) tmp;
-    
-    for (i = 0; i < cc_list_get_length(glyphlist); i++) {
-      if (glyph == (cc_glyph2d*) cc_list_get(glyphlist, i)) break;
-    }    
-    assert(i < cc_list_get_length(glyphlist));
-
-    cc_list_remove_fast(glyphlist, i);
-    if (cc_list_get_length(glyphlist) == 0) {
-      (void) cc_dict_remove(glyph2d_fonthash, (uintptr_t)glyph->character);
-      cc_list_destruct(glyphlist);
-    }
-
-    cc_fontspec_clean(glyph->fontspec);
-    free(glyph->fontspec);
-    cc_flw_done_glyph(glyph->fontidx, glyph->glyphidx);
-
-    cc_flw_unref_font(glyph->fontid);
-    free(glyph);
-  }
+  cc_glyph_unref(glyph2d_fonthash, &(glyph->c), NULL);
 }
 
 static SbBool 
@@ -283,13 +253,13 @@ glyph2d_specmatch(const cc_font_specification * spec1,
 void 
 cc_glyph2d_getadvance(const cc_glyph2d * g, int * x, int * y)
 {
-  cc_flw_get_bitmap_advance(g->fontidx, g->glyphidx, x, y);
+  cc_flw_get_bitmap_advance(g->c.fontidx, g->c.glyphidx, x, y);
 }
 
 void 
 cc_glyph2d_getkerning(const cc_glyph2d * left, const cc_glyph2d * right, int * x, int * y)
 {
-  cc_flw_get_bitmap_kerning(right->fontidx, left->glyphidx, right->glyphidx, x, y);
+  cc_flw_get_bitmap_kerning(right->c.fontidx, left->c.glyphidx, right->c.glyphidx, x, y);
 }
 
 unsigned int 
