@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -196,6 +196,13 @@ SbBool
 cc_flww32_initialize(void)
 {
   OSVERSIONINFO osvi;
+  UINT previous;
+
+  cc_flww32_globals.devctx = CreateDC("DISPLAY", NULL, NULL, NULL);
+  if (cc_flww32_globals.devctx == NULL) {
+    cc_win32_print_error("cc_flww32_initialize", "CreateDC()", GetLastError());
+    return FALSE;
+  }
 
   if (cc_font_debug()) { /* list all fonts on system */
     LOGFONT logfont; /* logical font information */
@@ -206,9 +213,6 @@ cc_flww32_initialize(void)
     logfont.lfFaceName[0] = '\0';
     logfont.lfPitchAndFamily = 0;
 
-    /* FIXME: for some peculiar reason, this has stopped working --
-       font_enum_proc() is never called. It used to work..?
-       Investigate what is going on. 20030610 mortene. */
     (void)EnumFontFamiliesEx(cc_flww32_globals.devctx,
                              (LPLOGFONT)&logfont,
                              (FONTENUMPROC)font_enum_proc,
@@ -216,11 +220,11 @@ cc_flww32_initialize(void)
                              0); /* reserved, must be zero */
   }
 
-  cc_flww32_globals.devctx = CreateDC("DISPLAY", NULL, NULL, NULL);
-  if (cc_flww32_globals.devctx == NULL) {
-    cc_win32_print_error("cc_flww32_initialize", "CreateDC()", GetLastError());
-    return FALSE;
-  }
+  /* Draw fonts on the baseline. (If we don't do this, we will for
+     instance get placement mismatches when rendering text in
+     different fonts on the same line.) */
+  previous = SetTextAlign(cc_flww32_globals.devctx, TA_BASELINE);
+  assert(previous != GDI_ERROR);
 
   cc_flww32_globals.font2kerninghash = cc_dict_construct(17, 0.75);
   cc_flww32_globals.fontsizehash = cc_dict_construct(17, 0.75f);
@@ -850,6 +854,7 @@ cc_flww32_get_vector_glyph(void * font, unsigned int glyph, float complexity)
   void * tmp;
   unsigned int size;
   uintptr_t cast_aid;
+  UINT previous;
 
   if (!GLUWrapper()->available) {
     cc_debugerror_post("cc_flww32_get_vector_glyph",
@@ -893,6 +898,11 @@ cc_flww32_get_vector_glyph(void * font, unsigned int glyph, float complexity)
     return NULL;
   }
 
+  /* About this: see comment on SetTextAlign() call in
+     cc_flww32_initialize(). */
+  previous = SetTextAlign(memdc, TA_BASELINE);
+  assert(previous != GDI_ERROR);
+
   screendc = GetDC(NULL);
   if (screendc == NULL) {
     cc_win32_print_error("cc_flww32_get_vector_glyph","Error calling GetDC(). "
@@ -1170,7 +1180,8 @@ flww32_buildVertexList(struct cc_font_vector_glyph * newglyph, int size)
     /* Must flip and translate glyph due to the W32 coord system
        which has a y-axis pointing downwards */
     newglyph->vertices[i*2 + 0] = coord[0] / size;
-    newglyph->vertices[i*2 + 1] = (-coord[1] / size) + 1;
+    newglyph->vertices[i*2 + 1] = (-coord[1]) / size;
+
     free(coord);
   }
 
