@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -67,6 +67,7 @@ extern "C" {
 static GLUWrapper_t * GLU_instance = NULL;
 static cc_libhandle GLU_libhandle = NULL;
 static int GLU_failed_to_load = 0;
+static int GLU_is_initializing = 0;
 
 /* ******************************************************************** */
 
@@ -88,11 +89,17 @@ static void
 GLUWrapper_cleanup(void)
 {
 #ifdef GLU_RUNTIME_LINKING
-  if (GLU_libhandle) cc_dl_close(GLU_libhandle);
+  if (GLU_libhandle) {
+    cc_dl_close(GLU_libhandle);
+    GLU_libhandle = NULL;
+  }
 #endif /* GLU_RUNTIME_LINKING */
 
   assert(GLU_instance);
   free(GLU_instance);
+  GLU_instance = NULL;
+  GLU_failed_to_load = 0;
+  GLU_is_initializing = 0;
 }
 
 /* ******************************************************************** */
@@ -329,17 +336,14 @@ GLUWrapper(void)
   if (GLU_instance || GLU_failed_to_load) { goto wrapperexit; }
 
   /* Detect recursive calls. */
-  {
-    static int is_initializing = 0;
-    assert(is_initializing == 0);
-    is_initializing = 1;
-  }
+  assert(GLU_is_initializing == 0);
+  GLU_is_initializing = 1;
 
 
   /* First invocation, do initializations. */
   GLU_instance = gi = (GLUWrapper_t *)malloc(sizeof(GLUWrapper_t));
   /* FIXME: handle out-of-memory on malloc(). 20000928 mortene. */
-  (void)coin_atexit((coin_atexit_f*) GLUWrapper_cleanup, 0);
+  (void)coin_atexit((coin_atexit_f*) GLUWrapper_cleanup, CC_ATEXIT_DYNLIBS);
 
   gi->versionMatchesAtLeast = GLUWrapper_versionMatchesAtLeast;
 
@@ -349,14 +353,11 @@ GLUWrapper(void)
 
 #ifdef GLU_RUNTIME_LINKING
 
-#ifndef GLU_IS_PART_OF_GL
-
   {
     int idx;
+    char * libname;
 
-    /* On Mac OS X, the glu* functions are part of the OpenGL framework,
-       which at this point is already loaded, so no need to try and open
-       the library here... */
+#ifndef GLU_IS_PART_OF_GL
 
     /* FIXME: should we get the system shared library name from an
        Autoconf check? 20000930 mortene. */
@@ -378,6 +379,25 @@ GLUWrapper(void)
       GLU_libhandle = cc_dl_open(possiblelibnames[idx]);
       idx++;
     }
+    libname = (char *)possiblelibnames[idx-1];
+
+#elif (defined HAVE_OPENGL_GLU_H)
+
+    /* On Mac OS X, GLU is part of the OpenGL framework, which at this
+       point is alrady loaded -> We can resolve symbols from the current
+       process image. */
+    GLU_libhandle = cc_dl_open(NULL);
+    libname = "OpenGL.framework/Libraries/libGLU.dylib";
+
+#endif /* !GLU_IS_PART_OF_GL */
+
+  /* FIXME: Resolving GLU functions will fail on other platforms where
+     GLU is considered part of OpenGL, since we never set GLU_libhandle. 
+     We should probably try to dlopen the GL image (or the current
+     process image as on Mac OS X?) on these platforms.  
+
+     I don't know any platforms other than OS X that have GLU as part
+     of GL though... 20051216 kyrah. */
 
     if (!GLU_libhandle) {
       gi->available = 0;
@@ -392,12 +412,11 @@ GLUWrapper(void)
       else {
         cc_debugerror_postinfo("GLUWrapper",
                                "Dynamically loaded GLU library as '%s'.",
-                               possiblelibnames[idx-1]);
+                               libname);
       }
     }
   }
 
-#endif /* !GLU_IS_PART_OF_GL */
 
   /* Define GLUWRAPPER_REGISTER_FUNC macro. Casting the type is
      necessary for this file to be compatible with C++ compilers. */
@@ -420,6 +439,7 @@ GLUWrapper(void)
 
   GLUWRAPPER_REGISTER_FUNC(gluScaleImage, gluScaleImage_t);
   GLUWRAPPER_REGISTER_FUNC(gluGetString, gluGetString_t);
+  GLUWRAPPER_REGISTER_FUNC(gluErrorString, gluErrorString_t);
   GLUWRAPPER_REGISTER_FUNC(gluNewNurbsRenderer, gluNewNurbsRenderer_t);
   GLUWRAPPER_REGISTER_FUNC(gluDeleteNurbsRenderer, gluDeleteNurbsRenderer_t);
   GLUWRAPPER_REGISTER_FUNC(gluNurbsProperty, gluNurbsProperty_t);
@@ -435,6 +455,13 @@ GLUWrapper(void)
   GLUWRAPPER_REGISTER_FUNC(gluPwlCurve, gluPwlCurve_t);
   GLUWRAPPER_REGISTER_FUNC(gluNurbsCallback, gluNurbsCallback_t);
 #if defined(GLU_VERSION_1_3) || defined(GLU_RUNTIME_LINKING)
+  // Note: nurbs tessellation is also available if the GLU_EXTENSION
+  // string contains GLU_EXT_nurbs_tessellator, but most platforms
+  // should have GLU 1.3, and on Windows -- the sole platform we know
+  // of where a GLU older than 1.3 is common -- the extension is not
+  // available. So we don't bother. Yet.
+  //
+  // mortene & kyrah.
   GLUWRAPPER_REGISTER_FUNC(gluNurbsCallbackData, gluNurbsCallbackData_t);
 #else /* !gluNurbsCallbackData */
   gi->gluNurbsCallbackData = NULL;
