@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -137,10 +137,15 @@ static void
 fcglue_cleanup(void)
 {
 #ifdef FONTCONFIG_RUNTIME_LINKING
-  if (fontconfig_libhandle) { cc_dl_close(fontconfig_libhandle); }
+  if (fontconfig_libhandle) {
+    cc_dl_close(fontconfig_libhandle);
+    fontconfig_libhandle = NULL;
+  }
 #endif /* FREETYPE_RUNTIME_LINKING */
   assert(fontconfig_instance);
   free(fontconfig_instance);
+  fontconfig_instance = NULL;
+  fontconfig_failed_to_load = 0;
 }
 
 static const cc_fcglue_t *
@@ -151,7 +156,7 @@ fcglue_init(void)
   if (!fontconfig_instance && !fontconfig_failed_to_load) {
     /* First invocation, do initializations. */
     cc_fcglue_t * fi = (cc_fcglue_t *)malloc(sizeof(cc_fcglue_t));
-    (void)coin_atexit((coin_atexit_f *)fcglue_cleanup, 0);
+    (void)coin_atexit((coin_atexit_f *)fcglue_cleanup, CC_ATEXIT_DYNLIBS);
 
     /* The common case is that fontconfig is either available from the
        linking process or we're successfully going to link it in. */
@@ -311,10 +316,15 @@ static void
 ftglue_cleanup(void)
 {
 #ifdef FREETYPE_RUNTIME_LINKING
-  if (freetype_libhandle) { cc_dl_close(freetype_libhandle); }
+  if (freetype_libhandle) {
+    cc_dl_close(freetype_libhandle);
+    freetype_libhandle = NULL;
+  }
 #endif /* FREETYPE_RUNTIME_LINKING */
   assert(freetype_instance);
   free(freetype_instance);
+  freetype_instance = NULL;
+  freetype_failed_to_load = 0;
 }
 
 static const cc_ftglue_t *
@@ -325,7 +335,7 @@ ftglue_init(void)
   if (!freetype_instance && !freetype_failed_to_load) {
     /* First invocation, do initializations. */
     cc_ftglue_t * fi = (cc_ftglue_t *)malloc(sizeof(cc_ftglue_t));
-    (void)coin_atexit((coin_atexit_f *)ftglue_cleanup, 0);
+    (void)coin_atexit((coin_atexit_f *)ftglue_cleanup, CC_ATEXIT_DYNLIBS);
 
     /* The common case is that FreeType is either available from the
        linking process or we're successfully going to link it in. */
