@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -44,6 +44,10 @@
 #include <Inventor/C/glue/dlp.h>
 #include <Inventor/C/glue/gl_agl.h>
 
+#ifdef HAVE_OPENGL_CGLCURRENT_H
+#include <OpenGL/CGLCurrent.h>
+#endif
+
 /* ********************************************************************** */
 
 #ifndef HAVE_AGL
@@ -133,14 +137,13 @@ static SbBool tried_fetching_handle = FALSE;
 
 /* 
  * Since AGL does not have an aglGetProcAddress() as such, this is
- * simply a wrapper around cc_dl_*. Prsent here for consistency with
+ * simply a wrapper around cc_dl_*. Present here for consistency with
  * WGL and GLX implementations.
  */
 void *
 aglglue_getprocaddress(const char * fname)
 {
   if (!tried_fetching_handle) {
-    /* FIXME: never released -- one-off resource leak. 20041208 mortene.*/
     glhnd = cc_dl_handle_with_gl_symbols();
     tried_fetching_handle = TRUE;
   }
@@ -182,10 +185,9 @@ aglglue_get_pbuffer_enable(void)
 
 
 struct aglglue_contextdata {
+  CGLContextObj storedcontext;
   AGLDrawable drawable;
   AGLContext aglcontext;
-  AGLContext storedcontext;
-  AGLDrawable storeddrawable;
   AGLPixelFormat pixformat;
   Rect bounds;
   CGrafPtr savedport;
@@ -205,7 +207,6 @@ aglglue_contextdata_init(unsigned int width, unsigned int height)
   ctx->drawable = NULL;
   ctx->aglcontext = NULL;
   ctx->storedcontext = NULL;
-  ctx->storeddrawable = NULL;
   ctx->pixformat = NULL;
   ctx->savedport = NULL;
   ctx->savedgdh = NULL;
@@ -242,6 +243,36 @@ aglglue_context_create_software(struct aglglue_contextdata * ctx)
     AGL_NONE
   };
 
+  /* FIXME: the following is a hack to get around a problem which
+     really demands more effort to be solved properly.
+
+     The problem is that there is no way in the API of the
+     SoOffscreenRenderer class to specify what particular attributes
+     to request. This most often manifests itself as a problem for app
+     programmers in that they have made some kind of extension node
+     which uses the OpenGL stencil buffer. If no stencil buffer
+     happens to be part of the GL context format for the offscreen
+     renderer, these will not work properly. At the same time, we
+     don't want to default to requesting a stencil buffer, as that
+     takes a non-trivial amount of extra memory resources on the gfx
+     card.
+
+     So until we have implemented the proper solution for making it
+     possible to pass in a detailed specification of which attributes
+     to request from offscreen GL contexts, we provide this temporary
+     work-around: the app programmer can set an envvar with a value
+     specifying the number of stencil buffer bits he/she wants.
+
+     20060223 mortene.
+  */
+  const int v = coin_glglue_stencil_bits_hack();
+  if (v != -1) {
+    size_t i;
+    for (i = 0; i < (sizeof(attrib) / sizeof(attrib[0]) / 2); i++) {
+      if (attrib[i] == AGL_STENCIL_SIZE) { attrib[i+1] = v; }
+    }
+  }
+
   if (coin_glglue_debug()) {
     cc_debugerror_postinfo("aglglue_context_create_software",
                            "Creating software buffer.");
@@ -318,12 +349,24 @@ aglglue_context_create_pbuffer(struct aglglue_contextdata * ctx)
     AGL_RED_SIZE, 8, 
     AGL_ALPHA_SIZE, 8, 
     AGL_DEPTH_SIZE, 24, 
+    AGL_STENCIL_SIZE, 1,
     AGL_CLOSEST_POLICY, 
     AGL_ACCELERATED, 
     AGL_NO_RECOVERY, 
     AGL_NONE 
   };
 
+  /* FIXME: this is a hack. See comment elsewhere in the file where
+     this function is used, for an elaborate explanation. 20060307
+     mortene. */
+  const int v = coin_glglue_stencil_bits_hack();
+  if (v != -1) {
+    size_t i;
+    for (i = 0; i < (sizeof(attribs) / sizeof(attribs[0]) / 2); i++) {
+      if (attribs[i] == AGL_STENCIL_SIZE) { attribs[i+1] = v; }
+    }
+  }
+
   /* Note that unlike in WGL, where the ability to bind a pBuffer
      as a texture has to be specified when creating the pixel
      format, AGL works in a different way: We can assume that 
@@ -454,16 +497,13 @@ aglglue_context_make_current(void * ctx)
     PixMapHandle pixmap;
 
     if (context->aglcontext) {
-      context->storedcontext = aglGetCurrentContext();
-      context->storeddrawable = aglGetDrawable(context->storedcontext);
+      context->storedcontext = CGLGetCurrentContext();
     }
 
     if (coin_glglue_debug()) {
       cc_debugerror_postinfo("aglglue_make_context_current",
-                             "store current status first => context==%p, "
-                             "drawable==%p",
-                             context->storedcontext,
-                             context->storeddrawable);
+                             "store current status first => context==%p",
+                             context->storedcontext);
     }
 
     pixmap = GetGWorldPixMap((GWorldPtr)context->drawable);
@@ -483,7 +523,7 @@ aglglue_context_make_current(void * ctx)
 
   } else { /* pBuffer support available */
 
-    context->storedcontext = aglGetCurrentContext();
+    context->storedcontext = CGLGetCurrentContext();
     if (!aglSetCurrentContext (context->aglcontext)) {
       GLenum error = aglGetError();
       if (error != AGL_NO_ERROR) {
@@ -527,24 +567,21 @@ aglglue_context_reinstate_previous(void * ctx)
                              "releasing context");
     }
 
-    if (context->storedcontext && context->storeddrawable) {
+    if (context->storedcontext) {
 
       if (coin_glglue_debug()) {
         cc_debugerror_postinfo("aglglue_context_reinstate_previous",
-                               "restoring context %p to be current "
-                               "(drawable==%p)",
-                               context->storedcontext,
-                               context->storeddrawable);
+                               "restoring context %p to be current",
+                               context->storedcontext);
       }
 
-      aglSetDrawable(context->storedcontext, context->storeddrawable);
-      aglSetCurrentContext(context->storedcontext);
+      CGLSetCurrentContext(context->storedcontext);
     }
 
   } else { /* pBuffer support available */
 
-    if (context->storedcontext) aglSetCurrentContext(context->storedcontext);
-    else aglSetCurrentContext(NULL);
+    if (context->storedcontext) CGLSetCurrentContext(context->storedcontext);
+    else CGLSetCurrentContext(NULL);
 
   } 
 }
@@ -578,7 +615,7 @@ aglglue_context_bind_pbuffer(void * ctx)
   /* FIXME: I don't think this needs to be here. pederb, 2003-11-27 */ 
   /* aglglue_context_reinstate_previous(context); */
 
-  aglglue_aglTexImagePBuffer (context->storedcontext, context->aglpbuffer, GL_FRONT);
+  aglglue_aglTexImagePBuffer((AGLContext)context->storedcontext, context->aglpbuffer, GL_FRONT);
   context->pbufferisbound = TRUE;
   error = aglGetError();  
   if (error != AGL_NO_ERROR) {    
@@ -608,7 +645,22 @@ aglglue_context_can_render_to_texture(void * ctx)
   return context->aglpbuffer != NULL;
 }
 
+void 
+aglglue_cleanup(void)
+{
+  aglglue_aglCreatePBuffer = NULL;
+  aglglue_aglDestroyPBuffer = NULL;
+  aglglue_aglSetPBuffer = NULL;
+  aglglue_aglTexImagePBuffer = NULL;
 
+  aglglue_context_create = NULL;
+
+  if (glhnd) { 
+    cc_dl_close(glhnd); 
+    glhnd = NULL; 
+  }
+  tried_fetching_handle = FALSE;
+}
 #endif /* HAVE_AGL */
 
 
