@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -117,6 +117,9 @@ SbBool glxglue_context_pbuffer_max(void * ctx, unsigned int * lims) { assert(FAL
 
 /* ********************************************************************** */
 
+static Display * glxglue_display = NULL;
+static SbBool glxglue_opendisplay_failed = FALSE;
+
 static int glxglue_screen = -1;
 typedef void *(APIENTRY * COIN_PFNGLXGETPROCADDRESS)(const GLubyte *);
 static COIN_PFNGLXGETPROCADDRESS glxglue_glXGetProcAddress = NULL;
@@ -198,24 +201,13 @@ static COIN_PFNGLXDESTROYPBUFFER glxglue_glXDestroyPbuffer;
 static Display *
 glxglue_get_display(void)
 {
-  /* The Display resource is never deallocated explicitly (but of
-   * course implicitly by the system on application close down). This
-   * to work around some strange problems with the NVidia-driver 29.60
-   * on XFree86 v4 when using XCloseDisplay() -- like doublebuffered
-   * visuals coming up just blank.
-   *
-   *   XCloseDisplay(glxglue_display);
-   */
-
-  static SbBool failed = FALSE;
-  static Display * glxglue_display = NULL;
-  if ((glxglue_display == NULL) && !failed) {
+  if ((glxglue_display == NULL) && !glxglue_opendisplay_failed) {
     /* FIXME: should use the real display-setting. :-(  20020926 mortene. */
     glxglue_display = XOpenDisplay(NULL);
     if (glxglue_display == NULL) {
       cc_debugerror_post("glxglue_init",
                          "Couldn't open NULL display.");
-      failed = TRUE;
+      glxglue_opendisplay_failed = TRUE;
     }
     else {
       /* FIXME: should use the real screen number. :-(  20020926 mortene. */
@@ -517,8 +509,32 @@ glxglue_build_GL_attrs(int * attrs, int trynum)
      one. 20021127 handegar. */
 
   if (! (trynum & 0x04)) {
+    /* FIXME: the following is a hack to get around a problem which
+       really demands more effort to be solved properly.
+
+       The problem is that there is no way in the API of the
+       SoOffscreenRenderer class to specify what particular attributes
+       to request. This most often manifests itself as a problem for
+       app programmers in that they have made some kind of extension
+       node which uses the OpenGL stencil buffer. If no stencil buffer
+       happens to be part of the GL context format for the offscreen
+       renderer, these will not work properly. At the same time, we
+       don't want to default to requesting a stencil buffer, as that
+       takes a non-trivial amount of extra memory resources on the gfx
+       card.
+
+       So until we have implemented the proper solution for making it
+       possible to pass in a detailed specification of which
+       attributes to request from offscreen GL contexts, we provide
+       this temporary work-around: the app programmer can set an
+       envvar with a value specifying the number of stencil buffer
+       bits he/she wants.
+
+       20060223 mortene.
+    */
+    const int v = coin_glglue_stencil_bits_hack();
     attrs[pos++] = GLX_STENCIL_SIZE;
-    attrs[pos++] = 1;
+    attrs[pos++] = (v == -1) ? 1 : v;
   }
   if (! (trynum & 0x02)) {
     attrs[pos++] = GLX_ALPHA_SIZE;
@@ -727,7 +743,13 @@ glxglue_context_create_pbuffer(struct glxglue_contextdata * context)
   int fbc_cnt;
 
   /* set frame buffer attributes */
-  const int attrs[] = {
+  /* FIXME: should refactor the attribute selection / setting process
+     to one common with the software offscreen buffer creation. This
+     is lame. 20060307 mortene. */
+  int attrs[] = {
+    GLX_STENCIL_SIZE, 1, /* FIXME: must be first, due to the hack
+                            below where we may change the value of the
+                            stencil bit setting. 20060307 mortene. */
     GLX_RENDER_TYPE, GLX_RGBA_BIT,
     GLX_RED_SIZE,   8,
     GLX_GREEN_SIZE, 8,
@@ -738,6 +760,12 @@ glxglue_context_create_pbuffer(struct glxglue_contextdata * context)
     None
   };
 
+  /* FIXME: hack. See comments in source code elsewhere in this file
+     where the same function is used. 20060307 mortene. */
+  const int v = coin_glglue_stencil_bits_hack();
+  assert(attrs[0] == GLX_STENCIL_SIZE);
+  if (v != -1) { attrs[1] = v; };
+
   dpy = glxglue_get_display();
   if (!dpy) { return FALSE; }
 
@@ -1013,4 +1041,39 @@ glxglue_context_pbuffer_max(void * ctx, unsigned int * lims)
 
 /* ********************************************************************** */
 
+void glxglue_cleanup(void)
+{
+  glxglue_screen = -1;
+  glxglue_glXGetProcAddress = NULL;
+  tried_bind_glXGetProcAddress = FALSE;
+
+  glxglue_context_create = NULL;
+
+  glxglue_glXChooseFBConfig = NULL;
+  glxglue_glXCreateNewContext = NULL;
+  glxglue_glXGetFBConfigAttrib = NULL; 
+  glxglue_glXCreatePbuffer_GLX_1_3 = NULL;
+  glxglue_glXCreateGLXPbufferSGIX = NULL;
+  glxglue_glXDestroyPbuffer = NULL;
+
+  /* FIXME: We used to not close the display due to potential problems
+     on some NVidia drivers (see original comment, reproduced
+     below). I think it's wrong to not cleanup properly on all systems
+     just because of an issue with *one* driver -- so I've enabled it
+     now. When the problem re-appears, we should disable it *for that
+     specific driver*. 20060210 kyrah
+
+     Original problem description:
+
+       The Display resource is never deallocated explicitly (but of
+       course implicitly by the system on application close
+       down). This to work around some strange problems with the
+       NVidia-driver 29.60 on XFree86 v4 when using XCloseDisplay() --
+       like doublebuffered visuals coming up just blank.
+   */
+  if (glxglue_display) XCloseDisplay(glxglue_display);
+  glxglue_display = NULL;
+  glxglue_opendisplay_failed = FALSE;
+}
+
 #endif /* HAVE_GLX */
