@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -58,18 +58,24 @@
 static openal_wrapper_t * openal_instance = NULL;
 static cc_libhandle openal_libhandle = NULL;
 static int openal_failed_to_load = 0;
-
+static int openal_is_initializing = 0;
 
 /* Cleans up at exit. */
 static void
 openal_wrapper_cleanup(void)
 {
 #ifdef OPENAL_RUNTIME_LINKING
-  if (openal_libhandle) { cc_dl_close(openal_libhandle); }
+  if (openal_libhandle) {
+    cc_dl_close(openal_libhandle);
+    openal_libhandle = NULL;
+  }
 #endif /* OPENAL_RUNTIME_LINKING */
 
   assert(openal_instance);
   free(openal_instance);
+  openal_instance = NULL;
+  openal_failed_to_load = 0;
+  openal_is_initializing = 0;
 }
 
 /* Implemented by using the singleton pattern. */
@@ -84,13 +90,12 @@ openal_wrapper(void)
     openal_wrapper_t * oal = (openal_wrapper_t *)malloc(sizeof(openal_wrapper_t));
     /* FIXME: Verify that atexit works when Coin is used in an ActiveX
        dll also. 2003-01-27 thammer. */
-    coin_atexit((coin_atexit_f *)openal_wrapper_cleanup, 0);
+    coin_atexit((coin_atexit_f *)openal_wrapper_cleanup, CC_ATEXIT_DYNLIBS);
 
     /* Detect recursive calls. */
     {
-      static int is_initializing = 0;
-      assert(is_initializing == 0);
-      is_initializing = 1;
+      assert(openal_is_initializing == 0);
+      openal_is_initializing = 1;
     }
 
     /* The common case is that simage is either available from the
@@ -184,17 +189,19 @@ openal_wrapper(void)
     if (oal->available) { /* debugging */
       const unsigned char * str;
       const char * debug = coin_getenv("COIN_DEBUG_AUDIO");
-      cc_debugerror_postinfo("openal_wrapper", "OpenAL using %s linking.", 
-                             oal->runtime ? "run-time" : "build-time");
-
-      str = oal->alGetString(AL_VENDOR);
-      cc_debugerror_postinfo("openal_wrapper", "AL_VENDOR=='%s'", str);
-      str = oal->alGetString(AL_VERSION);
-      cc_debugerror_postinfo("openal_wrapper", "AL_VERSION=='%s'", str);
-      str = oal->alGetString(AL_RENDERER);
-      cc_debugerror_postinfo("openal_wrapper", "AL_RENDERER=='%s'", str);
-      str = oal->alGetString(AL_EXTENSIONS);
-      cc_debugerror_postinfo("openal_wrapper", "AL_EXTENSIONS=='%s'", str);
+      if (debug && atoi(debug)) {
+        cc_debugerror_postinfo("openal_wrapper", "OpenAL using %s linking.", 
+                               oal->runtime ? "run-time" : "build-time");
+        
+        str = oal->alGetString(AL_VENDOR);
+        cc_debugerror_postinfo("openal_wrapper", "AL_VENDOR=='%s'", str);
+        str = oal->alGetString(AL_VERSION);
+        cc_debugerror_postinfo("openal_wrapper", "AL_VERSION=='%s'", str);
+        str = oal->alGetString(AL_RENDERER);
+        cc_debugerror_postinfo("openal_wrapper", "AL_RENDERER=='%s'", str);
+        str = oal->alGetString(AL_EXTENSIONS);
+        cc_debugerror_postinfo("openal_wrapper", "AL_EXTENSIONS=='%s'", str);
+      }
     }
 
     /* Do this late, so we can detect recursive calls to this function. */
