@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -61,6 +61,8 @@ extern "C" {
 static simage_wrapper_t * simage_instance = NULL;
 static cc_libhandle simage_libhandle = NULL;
 static int simage_failed_to_load = 0;
+static int simage_is_initializing = 0;
+
 
 /* Return value of COIN_DEBUG_SIMAGE environment variable. */
 static int
@@ -79,11 +81,17 @@ static void
 simage_wrapper_cleanup(void)
 {
 #ifdef SIMAGE_RUNTIME_LINKING
-  if (simage_libhandle) { cc_dl_close(simage_libhandle); }
+  if (simage_libhandle) {
+    cc_dl_close(simage_libhandle);
+    simage_libhandle = NULL;
+  }
 #endif /* SIMAGE_RUNTIME_LINKING */
 
   assert(simage_instance);
   free(simage_instance);
+  simage_instance = NULL;
+  simage_failed_to_load = 0;
+  simage_is_initializing = 0;
 }
 
 /* backup-functions. More robust when simage is an old version, or not
@@ -243,17 +251,13 @@ simage_wrapper(void)
      this. 20020628 mortene. */
 
   if (!simage_instance && !simage_failed_to_load) {
-
     /* First invocation, do initializations. */
     simage_wrapper_t * si = (simage_wrapper_t *)malloc(sizeof(simage_wrapper_t));
-    (void)coin_atexit((coin_atexit_f *)simage_wrapper_cleanup, 0);
+    (void)coin_atexit((coin_atexit_f *)simage_wrapper_cleanup, CC_ATEXIT_DYNLIBS);
 
     /* Detect recursive calls. */
-    {
-      static int is_initializing = 0;
-      assert(is_initializing == 0);
-      is_initializing = 1;
-    }
+    assert(simage_is_initializing == 0);
+    simage_is_initializing = 1;
 
     si->versionMatchesAtLeast = simage_wrapper_versionMatchesAtLeast;
 
@@ -427,6 +431,7 @@ simage_wrapper(void)
         si->s_stream_params = NULL;
       }
     }
+    simage_is_initializing = 0;
   }
   CC_SYNC_END(simage_wrapper);
   return simage_instance;
