@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -202,6 +202,44 @@ coin_LocalFree(HLOCAL hMem) /* handle to local memory object */
 
 /* ********************************************************************** */
 
+static HGDIOBJ WINAPI
+coin_SelectObject(HDC hdc, HGDIOBJ hgdiobj)
+{
+  HGDIOBJ previous;
+  DWORD d = GetObjectType(hgdiobj);
+  if (d == 0) {
+    cc_win32_print_error("coin_SelectObject", "GetObjectType()", GetLastError());
+    assert(FALSE && "unhandled error");
+  }
+
+  previous = SelectObject(hdc, hgdiobj);
+  if (((d == OBJ_REGION) && (previous == HGDI_ERROR)) ||
+      ((d == OBJ_REGION) && (previous == NULL))) {
+    cc_win32_print_error("coin_SelectObject", "SelectObject()", GetLastError());
+    
+    /* not sure about this one, suddenly start assert'ing on
+       SelectObject() failures may be too much of a shock for
+       exisiting code... but eventually, it should go in:     (mortene) */
+    /* assert(FALSE && "unhandled error"); */
+  }
+  return previous;
+}
+
+/* ********************************************************************** */
+
+static int WINAPI
+coin_GetObject(HGDIOBJ hgdiobj, int cbBuffer, LPVOID lpvObject)
+{
+  int ret = GetObject(hgdiobj, cbBuffer, lpvObject);
+  if (ret == 0) {
+    cc_win32_print_error("coin_GetObject", "GetObject()", GetLastError());
+    assert(FALSE && "unhandled error");
+  }
+  return ret;
+}
+
+/* ********************************************************************** */
+
 /* singleton access to the structure, so we can initialize it at first
    use */
 const struct cc_win32_api *
@@ -217,6 +255,8 @@ cc_win32(void)
     instance.GetTextFace = coin_GetTextFace;
     instance.LocalFree = coin_LocalFree;
     instance.GetVersionEx = coin_GetVersionEx;
+    instance.SelectObject = coin_SelectObject;
+    instance.GetObject = coin_GetObject;
   }
 
   return &instance;
