@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -135,10 +135,15 @@ static void
 zlibglue_cleanup(void)
 {
 #ifdef ZLIB_RUNTIME_LINKING
-  if (zlib_libhandle) { cc_dl_close(zlib_libhandle); }
+  if (zlib_libhandle) {
+    cc_dl_close(zlib_libhandle);
+    zlib_libhandle = NULL;
+  }
 #endif /* ZLIB_RUNTIME_LINKING */
   assert(zlib_instance);
   free(zlib_instance);
+  zlib_instance = NULL;
+  zlib_failed_to_load = 0;
 }
 
 static const cc_zlibglue_t *
@@ -149,7 +154,7 @@ zlibglue_init(void)
   if (!zlib_instance && !zlib_failed_to_load) {
     /* First invocation, do initializations. */
     cc_zlibglue_t * zi = (cc_zlibglue_t *)malloc(sizeof(cc_zlibglue_t));
-    (void)coin_atexit((coin_atexit_f *)zlibglue_cleanup, 0);
+    (void)coin_atexit((coin_atexit_f *)zlibglue_cleanup, CC_ATEXIT_DYNLIBS);
 
     /* The common case is that zlib is either available from the
        linking process or we're successfully going to link it in. */
