@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -118,6 +118,13 @@
 
 // *************************************************************************
 
+static SbBool hardcopy_isinitialized = FALSE;
+
+static void hardcopy_cleanup(void)
+{
+  hardcopy_isinitialized = FALSE;
+}
+
 /*!
   Initialization of the hardcopy support happens automatically from
   SoDB::init(), so the application programmer should normally not have
@@ -126,12 +133,13 @@
 void
 SoHardCopy::init(void)
 {
-  static SbBool first = TRUE;
-  if (!first) { return; }
-  first = FALSE;
+  if (hardcopy_isinitialized) { return; }
 
   SoVectorizeAction::initClass();
   SoVectorizePSAction::initClass();
+
+  hardcopy_isinitialized = TRUE;
+  coin_atexit((coin_atexit_f*)hardcopy_cleanup, CC_ATEXIT_NORMAL);
 }
 
 /*!
