@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -1281,17 +1281,30 @@ SoInput::read(SbString & s)
 
         if (*buf == '\\') {
           if (!fi->get(c)) return FALSE;
-          // for VRML97, \\ and \" should be converted to \ and ".
-          // for VRML 1.0 and Inventor, only \" should be converted.
-          if (c == '\"' || (c == '\\' && this->isFileVRML2())) {
-            *buf = c;
+          if (c == '\"') {
+            // VRML 2.0 allows for strings to contain literal
+            // newlines, Inventor/VRML V1.0 doesn't.  This also checks
+            // for a special case in Inventor/VRML 1.0 files where the
+            // last two characters of a line containing a quoted
+            // string is \". In this case, the backslash should be
+            // considered literal and the quote should terminate the
+            // string.
+            if (!this->isFileVRML2() && fi->get(c)) {
+              fi->putBack(c);
+              if (c == '\n') fi->putBack('\"');
+              else *buf = '\"';
+            }
+            else {
+              *buf = c;
+            }
           }
-          else {
-            *buf = '\\';
+          // In VRML V2.0, backslashes must be quoted.
+          else if (c != '\\' || !this->isFileVRML2()) {
             fi->putBack(c);
           }
         }
       }
+      // Unquoted strings end on first encountered whitespace
       else if (fi->isSpace(*buf)) {
         fi->putBack(*buf);
         break;
@@ -1790,6 +1803,7 @@ void
 SoInput::addDirectoryIdx(const int idx, const char * dirName)
 {
   assert(idx > -2);
+  if (strlen(dirName) == 0) return; // Don't add empty dirs
   SbStringList * dirs = SoInput::dirsearchlist;
 
   if (soinput_tls) {
@@ -1974,9 +1988,7 @@ SoInput::getDirectories(void)
 void
 SoInput::init(void)
 {
-  // Debugging for memory leaks will be easier if we also clean up the
-  // static / one-off resource usage.
-  coin_atexit((coin_atexit_f *)SoInput::clean, 0);
+  coin_atexit((coin_atexit_f *)SoInput::clean, CC_ATEXIT_NORMAL);
 
   // This will catch multiple initClass() calls (unless there's a
   // removeDirectories() in between them, which is unlikely to happen
