@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -388,16 +388,6 @@ SoInput_FileInfo::readHeader(SoInput * soinput)
       this->vrml2file = TRUE;
     }
     if (this->prefunc) this->prefunc(this->userdata, soinput);
-
-    if (this->ivversion == 1.0f) {
-      SoDebugError::postWarning("SoInput_FileInfo::readHeader",
-                                "Attempting to import old Inventor V1.0 file. "
-                                "This operation may fail, due to limited "
-                                "support for old Inventor files. If it does, "
-                                "please get in touch with Systems in Motion "
-                                "at <coin-support@coin3d.org>, and we will "
-                                "rectify the situation.");
-    }
   }
   return TRUE;
 }
