@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -278,7 +278,7 @@ SoOutput_compression_list_init(void)
   if (cc_bzglue_available()) {
     SoOutput_compmethods->append(SbName("BZIP2"));
   }
-  coin_atexit((coin_atexit_f*) SoOutput_compression_list_cleanup, 0);
+  coin_atexit((coin_atexit_f*) SoOutput_compression_list_cleanup, CC_ATEXIT_NORMAL);
 }
 
 #define PRIVATE(obj) (obj->pimpl)
@@ -1483,9 +1483,34 @@ SoOutput::resolveRoutes(void)
       this->write(toname.getString());
       this->write("\n");
 
+#if COIN_DEBUG
+      if (SoWriterefCounter::debugWriterefs()) {
+        int writerefcount = PRIVATE(this)->counter->getWriteref(fromc);
+        SoDebugError::postInfo("SoOutput::resolveRoutes",
+                               "%p/%s/'%s': %d -> %d",
+                               fromc,
+                               fromc->getTypeId().getName().getString(),
+                               fromc->getName().getString(),
+                               writerefcount, writerefcount - 1);
+      }
+#endif // COIN_DEBUG
+
+#if COIN_DEBUG
+      if (SoWriterefCounter::debugWriterefs()) {
+        int writerefcount = PRIVATE(this)->counter->getWriteref(toc);
+        SoDebugError::postInfo("SoOutput::resolveRoutes",
+                               "%p/%s/'%s': %d -> %d",
+                               toc,
+                               toc->getTypeId().getName().getString(),
+                               toc->getName().getString(),
+                               writerefcount, writerefcount - 1);
+      }
+#endif // COIN_DEBUG
+
       // remove write references again
       PRIVATE(this)->counter->decrementWriteref(fromc);
       PRIVATE(this)->counter->decrementWriteref(toc);
+
     }
     list->truncate(0);
   }
