@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -94,7 +94,7 @@ private:
     if (name == "") name = "<noname>";
     
     SoDebugError::postWarning("SoWriterefCounter::<cleanup>",
-                              "Not removed from writedict: %p, %s:%s",
+                              "Not removed from writerefdict: %p, %s:%s",
                               base, base->getTypeId().getName().getString(), name.getString());
 #endif // COIN_DEBUG
   }
@@ -130,6 +130,21 @@ public:
     this->outputdata->unref();
     delete this->sobase2id;
   }
+  void appendPostfix(const SoBase *base, SbString &name, int refid)
+  {
+    // Fix to avoid writing DEFs starting with an illegal
+    // character (e.g. '+') in VRML2.
+    if (name.getLength() == 0 &&
+        base->isOfType(SoNode::getClassTypeId()) &&
+        ((SoNode *)base)->getNodeType() == SoNode::VRML2 &&
+        !SbName::isBaseNameStartChar((*refwriteprefix)[0])) {
+      name += "_";
+    }
+    
+    name += SoWriterefCounterP::refwriteprefix->getString();
+    name.addIntString(refid);
+  }
+
   SoWriterefCounter * master;
   SoOutput * out;
   SoWriterefCounterOutputData * outputdata;
@@ -142,19 +157,19 @@ public:
   static SbString * refwriteprefix;
   
   static void atexit_cleanup(void) {
+    current = NULL;
     delete refwriteprefix;
     refwriteprefix = NULL;
     delete outputdict;
     outputdict = NULL;
     CC_MUTEX_DESTRUCT(mutex);
-    mutex = NULL;
   }
 
 };
 
 void * SoWriterefCounterP::mutex;
 SoOutput2SoWriterefCounterMap *  SoWriterefCounterP::outputdict;
-SoWriterefCounter *  SoWriterefCounterP::current; // used to be backwards compatible
+SoWriterefCounter *  SoWriterefCounterP::current = NULL; // used to be backwards compatible
 SbString *  SoWriterefCounterP::refwriteprefix;
 
 #define PRIVATE(obj) obj->pimpl
@@ -204,7 +219,7 @@ SoWriterefCounter::initClass(void)
   CC_MUTEX_CONSTRUCT(SoWriterefCounterP::mutex);
   SoWriterefCounterP::outputdict = new SoOutput2SoWriterefCounterMap;
   SoWriterefCounterP::refwriteprefix = new SbString("+");
-  coin_atexit((coin_atexit_f*) SoWriterefCounterP::atexit_cleanup, 0);
+  coin_atexit((coin_atexit_f*) SoWriterefCounterP::atexit_cleanup, CC_ATEXIT_NORMAL);
 }
 
 void 
@@ -505,8 +520,7 @@ SoWriterefCounter::getWriteName(const SoBase * base) const
       writename = name.getString();
       // We have used a suffix when DEF'ing the node
       if (refid != (int) NOSUFFIX) {
-        writename += SoWriterefCounterP::refwriteprefix->getString();
-        writename.addIntString(refid);
+        PRIVATE(this)->appendPostfix(base, writename, refid);
       }
       // Detects last USE of a node, enables reuse of DEF's
       if (!multiref) out->removeDEFNode(SbName(writename));
@@ -521,25 +535,16 @@ SoWriterefCounter::getWriteName(const SoBase * base) const
       }
       else {
         // Node name is already DEF'ed or an unnamed multiref => use a suffix.
-        writename += SoWriterefCounterP::refwriteprefix->getString();
-        writename.addIntString(out->addReference(base));
+        PRIVATE(this)->appendPostfix(base, writename, out->addReference(base));
         out->addDEFNode(SbName(writename));
       }
     }
   }
   else { // Default OIV behavior
     if (multiref && firstwrite) refid = out->addReference(base);
-    if (!firstwrite) {
-      writename = name.getString();
-      writename += SoWriterefCounterP::refwriteprefix->getString();
-      writename.addIntString(refid);
-    }
-    else {
-      writename = name.getString();
-      if (multiref) {
-        writename += SoWriterefCounterP::refwriteprefix->getString();
-        writename.addIntString(refid);
-      }
+    writename = name.getString();
+    if (!firstwrite || multiref) {
+      PRIVATE(this)->appendPostfix(base, writename, refid);
     }
   }
   return writename;
@@ -586,5 +591,22 @@ SoWriterefCounter::removeSoBase2IdRef(const SoBase * base)
   PRIVATE(this)->sobase2id->remove(base);
 }
 
+/*!
+  Returns TRUE if the user wants extra debugging information regarding
+  writerefs (COIN_DEBUG_WRITEREFS=1)
+*/
+SbBool
+SoWriterefCounter::debugWriterefs(void)
+{
+  static int dbg = -1;
+  if (dbg == -1) {
+    const char * env = coin_getenv("COIN_DEBUG_WRITEREFS");
+    dbg = (env && (atoi(env) > 0)) ? 1 : 0;
+  }
+  return dbg;
+}
+
+/**********************************************************************/
+
 
 #undef PRIVATE
