@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -35,6 +35,27 @@
   used directly -- use one of it's subclasses.
 */
 
+// *************************************************************************
+
+#include <Inventor/manips/SoTransformManip.h>
+
+#include <Inventor/SoNodeKitPath.h>
+#include <Inventor/actions/SoCallbackAction.h>
+#include <Inventor/actions/SoGLRenderAction.h>
+#include <Inventor/actions/SoGetBoundingBoxAction.h>
+#include <Inventor/actions/SoGetMatrixAction.h>
+#include <Inventor/actions/SoHandleEventAction.h>
+#include <Inventor/actions/SoPickAction.h>
+#include <Inventor/actions/SoSearchAction.h>
+#include <Inventor/draggers/SoDragger.h>
+#include <Inventor/errors/SoDebugError.h>
+#include <Inventor/misc/SoChildList.h>
+#include <Inventor/nodes/SoGroup.h>
+#include <Inventor/nodes/SoSubNodeP.h>
+#include <Inventor/sensors/SoFieldSensor.h>
+
+// *************************************************************************
+
 /*!
   \var SoFieldSensor * SoTransformManip::rotateFieldSensor
   \COININTERNAL
@@ -65,27 +86,11 @@
   \COININTERNAL
 */
 
-#include <Inventor/manips/SoTransformManip.h>
-#include <Inventor/nodes/SoSubNodeP.h>
-#include <Inventor/draggers/SoDragger.h>
-#include <Inventor/actions/SoGLRenderAction.h>
-#include <Inventor/actions/SoPickAction.h>
-#include <Inventor/actions/SoGetMatrixAction.h>
-#include <Inventor/actions/SoGetBoundingBoxAction.h>
-#include <Inventor/actions/SoHandleEventAction.h>
-#include <Inventor/actions/SoCallbackAction.h>
-#include <Inventor/actions/SoSearchAction.h>
-#include <Inventor/nodes/SoGroup.h>
-#include <Inventor/misc/SoChildList.h>
-#include <Inventor/sensors/SoFieldSensor.h>
-#include <Inventor/SoNodeKitPath.h>
-
-#if COIN_DEBUG
-#include <Inventor/errors/SoDebugError.h>
-#endif // COIN_DEBUG
+// *************************************************************************
 
 SO_NODE_SOURCE(SoTransformManip);
 
+// *************************************************************************
 
 // Documented in superclass
 void
@@ -135,6 +140,8 @@ SoTransformManip::~SoTransformManip()
   delete this->children;
 }
 
+// *************************************************************************
+
 // Note: this documentation is also shared by some of the subclasses,
 // so keep it general enough to also be valid for those.
 /*!
@@ -180,6 +187,8 @@ SoTransformManip::getDragger(void)
   return NULL;
 }
 
+// *************************************************************************
+
 /*!
   Replaces a node at the tail end of \a path with this manipulator.
   The node must be an instance of a node class derived from
@@ -271,10 +280,21 @@ SoTransformManip::replaceNode(SoPath * path)
   return TRUE;
 }
 
+// *************************************************************************
+
 // Documented in superclass
 void
 SoTransformManip::doAction(SoAction * action)
 {
+  // Robustness. Catches an easily made user error:
+  if (this->getTypeId() == SoTransformManip::getClassTypeId()) {
+    SoDebugError::postWarning("SoTransformManip::replaceNode",
+                              "SoTransformManip is an abstract manipulator "
+                              "class, which should not be used in the scene "
+                              "graph. Use one of its subclasses instead.");
+  }
+  // </robustness>
+
   int numindices;
   const int *indices;
   if (action->getPathCode(numindices, indices) == SoAction::IN_PATH) {
@@ -387,6 +407,8 @@ SoTransformManip::search(SoSearchAction * action)
   SoTransformManip::doAction(action);
 }
 
+// *************************************************************************
+
 /*!
   Returns the children of this node. This node only has the dragger
   as a child.
@@ -528,3 +550,5 @@ SoTransformManip::attachSensors(const SbBool onoff)
     this->scaleOrientFieldSensor->detach();
   }
 }
+
+// *************************************************************************
