@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -33,6 +33,7 @@
 
 #include <Inventor/SbString.h>
 #include <Inventor/C/tidbits.h>
+#include <Inventor/C/tidbitsp.h>
 #include <Inventor/C/glue/openal_wrapper.h>
 
 // *************************************************************************
@@ -97,6 +98,11 @@ coin_debug_audio(void)
 
 static SbBool should_traverse = FALSE;
 
+static void audio_cleanup(void) 
+{
+  should_traverse = FALSE;
+}
+
 // Called from the constructor of SoVRMLAudioClip and SoVRMLSound.
 // After this has been called once, SoSceneManager instances will
 // start applying an SoAudioRenderAction to its scene graph before
@@ -112,6 +118,7 @@ void
 coin_sound_enable_traverse(void)
 {
   should_traverse = TRUE;
+  coin_atexit((coin_atexit_f*) audio_cleanup, CC_ATEXIT_NORMAL);
 }
 
 // Called from SoSceneManager::render() to decide whether or not to
