@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -109,10 +109,7 @@ SoAudioDevice::instance()
     // For Coin, that means one /has/ to invoke SoDB::finish() for
     // application which uses sound, or the application will hang on
     // exit for some users.
-    //
-    // FIXME: about the priority setting below; make sure this is done
-    // before SoDebugError's at-exit. 20050627 mortene.
-    coin_atexit((coin_atexit_f *)SoAudioDeviceP::clean, 1);
+    coin_atexit((coin_atexit_f *)SoAudioDeviceP::clean, CC_ATEXIT_NORMAL);
   }
   return SoAudioDeviceP::singleton;
 }
