@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -249,20 +249,6 @@ uint32_t SoBase::writecounter = 0;
 
 /**********************************************************************/
 
-
-static SbBool
-debug_writerefs(void)
-{
-  static int dbg = -1;
-  if (dbg == -1) {
-    const char * env = coin_getenv("COIN_DEBUG_WRITEREFS");
-    dbg = (env && (atoi(env) > 0)) ? 1 : 0;
-  }
-  return dbg;
-}
-
-/**********************************************************************/
-
 // This can be any "magic" bitpattern of 4 bits which seems unlikely
 // to be randomly assigned to a memory byte upon destruction. I chose
 // "1101".
@@ -420,7 +406,7 @@ SoBase::destroy(void)
   SoDebugError::postInfo("SoBase::destroy", "delete this %p", this);
 #endif // debug
 
-  // Sjlmord.
+  // Harakiri
   delete this;
 
 #if COIN_DEBUG && 0 // debug
@@ -437,7 +423,7 @@ SoBase::destroy(void)
 void
 SoBase::initClass(void)
 {
-  coin_atexit((coin_atexit_f *)SoBase::cleanClass, 0);
+  coin_atexit((coin_atexit_f *)SoBase::cleanClass, CC_ATEXIT_NORMAL);
 
   // Avoid multiple attempts at initialization.
   assert(SoBase::classTypeId == SoType::badType());
@@ -512,6 +498,9 @@ SoBase::cleanClass(void)
   CC_MUTEX_DESTRUCT(SoBaseP::name2obj_mutex);
   CC_MUTEX_DESTRUCT(SoBaseP::auditor_mutex);
   CC_MUTEX_DESTRUCT(SoBaseP::global_mutex);
+
+  SoBase::tracerefs = FALSE;
+  SoBase::writecounter = 0;
 }
 
 /*!
@@ -558,6 +547,9 @@ SoBase::assertAlive(void) const
   avoid premature deletion), but is usually called from other instances
   within the Coin library when objects are made dependent on each other.
 
+  See the class documentation of SoNode for more extensive information
+  about reference counting.
+
   \sa unref(), unrefNoDelete()
  */
 void
@@ -719,10 +711,11 @@ SbName
 SoBase::getName(void) const
 {
   // If this assert fails, SoBase::initClass() has probably not been
-  // called.
+  // called, or you have objects on the stack that is destroyed after
+  // you have invoked SoDB::cleanup().
   assert(SoBaseP::obj2name);
 
-  const char * value;
+  const char * value = NULL;
   CC_MUTEX_LOCK(SoBaseP::obj2name_mutex);
   SbBool found = SoBaseP::obj2name->get(this, value);
   CC_MUTEX_UNLOCK(SoBaseP::obj2name_mutex);
@@ -819,7 +812,7 @@ void
 SoBase::removeName(SoBase * const b, const char * const name)
 {
   CC_MUTEX_LOCK(SoBaseP::name2obj_mutex);
-  SbPList * l;
+  SbPList * l = NULL;
   SbBool found = SoBaseP::name2obj->get(name, l);
   assert(found);
 
@@ -961,7 +954,7 @@ SoBase::getAuditors(void) const
 
   if (SoBaseP::auditordict == NULL) {
     SoBaseP::auditordict = new SbHash<SoAuditorList *, const SoBase *>();
-    coin_atexit((coin_atexit_f*)SoBaseP::cleanup_auditordict, 0);
+    coin_atexit((coin_atexit_f*)SoBaseP::cleanup_auditordict, CC_ATEXIT_NORMAL);
   }
 
   SoAuditorList * l = NULL;
@@ -994,7 +987,7 @@ SoBase::addWriteReference(SoOutput * out, SbBool isfromfield)
   int refcount = SoWriterefCounter::instance(out)->getWriteref(this);
 
 #if COIN_DEBUG
-  if (debug_writerefs()) {
+  if (SoWriterefCounter::debugWriterefs()) {
     SoDebugError::postInfo("SoBase::addWriteReference",
                            "%p/%s/'%s': %d -> %d",
                            this, this->getTypeId().getName().getString(),
@@ -1123,7 +1116,8 @@ SoBase::getNamedBases(const SbName & name, SoBaseList & baselist, SoType type)
   stream is a '}' to detect the latter case.
 
   3. A child was given as the \c NULL keyword. This can happen when
-  reading the contents of SoSFNode or SoMFNode fields.
+  reading the contents of SoSFNode fields (note that NULL is not
+  allowed for SoMFNode)
 
   If \c TRUE is returned and \a base is not \c NULL upon return, the
   instance was allocated and initialized according to what was read
@@ -1132,6 +1126,28 @@ SoBase::getNamedBases(const SbName & name, SoBaseList & baselist, SoType type)
 SbBool
 SoBase::read(SoInput * in, SoBase *& base, SoType expectedtype)
 {
+  // FIXME: the interface design for this function is goddamn _awful_!
+  // We need to keep it like this, though, to be compatible with SGI
+  // Inventor. What we however /could/ do about it is:
+  //
+  // First, split out the SoBaseP class definition to a separate
+  // interface, which can be accessed internally from Coin lirbary
+  // code.
+  //
+  // Then, in this, write up /sensibly designed/ read()-function(s)
+  // which implements the actually needed functionality of this
+  // method.
+  //
+  // Third, make this function use those new functions in SoBaseP (to
+  // avoid code duplication) -- and mangle the results so that this
+  // function still conforms to the SGI Inventor behavior.
+  //
+  // Finally, start using the SoBaseP::read()-function(s) from
+  // internal Coin code instead, to clean up the messy interaction
+  // with this function from everywhere else.
+  //
+  // 20060202 mortene.
+
   assert(expectedtype != SoType::badType());
   base = NULL;
 
@@ -1295,6 +1311,13 @@ SoBase::writeHeader(SoOutput * out, SbBool isgroup, SbBool isengine) const
     out->write(USE_KEYWORD);
     if (!out->isBinary()) out->write(' ');
     out->write(writename.getString());
+
+    if (SoWriterefCounter::debugWriterefs()) {
+      SbString tmp;
+      tmp.sprintf(" # writeref: %d\n",
+                  SoWriterefCounter::instance(out)->getWriteref(this));
+      out->write(tmp.getString());
+    }
   }
   else {
     if (name.getLength() || multiref) {
@@ -1310,10 +1333,19 @@ SoBase::writeHeader(SoOutput * out, SbBool isgroup, SbBool isengine) const
       SbString nodename(this->getFileFormatName());
       if (nodename.getLength() > 4) {
         SbString vrml = nodename.getSubString(0, 3);
+        const char vrml2headerprefix[] = "#VRML V2.0 utf8";
+        const size_t len = sizeof(vrml2headerprefix) - 1;
+        const SbString fullheader = SoOutput_getHeaderString(out->pimpl);
+        const SbString fileid = ((size_t)fullheader.getLength() < len) ?
+          fullheader : fullheader.getSubString(0, len - 1);
         // FIXME: using a temporary workaround to test if we're
         // exporting a VRML97 file. pederb, 2003-03-18
-        if ((vrml == "VRML") &&
-            (SoOutput_getHeaderString(out->pimpl) == "#VRML V2.0 utf8")) {
+        //
+        // UPDATE 20060207 mortene: a better solution would be to
+        // carry along the format information in the SoOutput (or an
+        // internal private SoOutputP class?) as an enum or something,
+        // methinks.
+        if ((vrml == "VRML") && (fileid == vrml2headerprefix)) {
           SbString substring = nodename.getSubString(4);
           out->write(substring.getString());
         }
@@ -1336,6 +1368,12 @@ SoBase::writeHeader(SoOutput * out, SbBool isgroup, SbBool isengine) const
     }
     else {
       out->write(" {");
+      if (SoWriterefCounter::debugWriterefs()) {
+        SbString tmp;
+        tmp.sprintf(" # writeref: %d\n",
+                    SoWriterefCounter::instance(out)->getWriteref(this));
+        out->write(tmp.getString());
+      }
       out->write(END_OF_LINE);
       out->incrementIndent();
     }
@@ -1344,7 +1382,7 @@ SoBase::writeHeader(SoOutput * out, SbBool isgroup, SbBool isengine) const
   int writerefcount = SoWriterefCounter::instance(out)->getWriteref(this);
   
 #if COIN_DEBUG
-  if (debug_writerefs()) {
+  if (SoWriterefCounter::debugWriterefs()) {
     SoDebugError::postInfo("SoBase::writeHeader",
                            "%p/%s/'%s': %d -> %d",
                            this,
@@ -1778,37 +1816,6 @@ SoBase::flushInput(SoInput * in)
 #endif // obsoleted
 }
 
-//
-// helper function for connectRoute(). First test the actual fieldname,
-// then set set_<fieldname>, then <fieldname>_changed.
-//
-static SoField *
-find_field(SoNode * node, const SbName & fieldname)
-{
-  SoField * field = node->getField(fieldname);
-
-  if (!field) {
-    if (strncmp(fieldname.getString(), "set_", 4) == 0) {
-      SbName newname = fieldname.getString() + 4;
-      field = node->getField(newname);
-    }
-    else {
-      SbString str = fieldname.getString();
-      int len = str.getLength();
-      const char CHANGED[] = "_changed";
-      const int changedsize = sizeof(CHANGED) - 1;
-
-      if (len > changedsize && strcmp(str.getString()+len-changedsize,
-                                      CHANGED) == 0) {
-        SbString substr = str.getSubString(0, len-(changedsize+1));
-        SbName newname = substr.getString();
-        field = node->getField(newname);
-      }
-    }
-  }
-  return field;
-}
-
 /*!
   Connect a route from the node named \a fromnodename's field \a
   fromfieldname to the node named \a tonodename's field \a
@@ -1827,67 +1834,13 @@ SoBase::connectRoute(SoInput * in,
   SoNode * fromnode = SoNode::getByName(fromnodename);
   SoNode * tonode = SoNode::getByName(tonodename);
   if (fromnode && tonode) {
-    SoField * from = find_field(fromnode, fromfieldname);
-    SoField * to = find_field(tonode, tofieldname);
-    SoEngineOutput * output = NULL;
-    if (from == NULL && fromnode->isOfType(SoNodeEngine::getClassTypeId())) {
-      output = ((SoNodeEngine*) fromnode)->getOutput(fromfieldname);
-    }
-
-    if (to && (from || output)) {
-      SbBool notnotify = FALSE;
-      SbBool append = FALSE;
-      if (output || from->getFieldType() == SoField::EVENTOUT_FIELD) {
-        notnotify = TRUE;
-      }
-      if (to->getFieldType() == SoField::EVENTIN_FIELD) append = TRUE;
-
-      // Check if we're already connected.
-      SoFieldList fl;
-      if (from) from->getForwardConnections(fl);
-      else output->getForwardConnections(fl);
-      int idx = fl.find(to);
-      if (idx != -1) {
-        SoReadError::post(in,
-                          "Tried to connect a ROUTE multiple times "
-                          "(from %s.%s to %s.%s)",
-                          fromnodename.getString(), fromfieldname.getString(),
-                          tonodename.getString(), tofieldname.getString());
-        return FALSE;
-      }
-
-      // Check that there exists a field converter, if one is needed.
-      SoType totype = to->getTypeId();
-      SoType fromtype = from ? from->getTypeId() : output->getConnectionType();
-      if (totype != fromtype) {
-        SoType convtype = SoDB::getConverter(fromtype, totype);
-        if (convtype == SoType::badType()) {
-          SoReadError::post(in,
-                            "Tried to connect a ROUTE between entities "
-                            "that can not be connected (due to lack of "
-                            "field type converter): %s.%s is of type "
-                            "%s, and %s.%s is of type %s",
-                            fromnodename.getString(), fromfieldname.getString(),
-                            fromtype.getName().getString(),
-                            tonodename.getString(), tofieldname.getString(),
-                            totype.getName().getString());
-          return FALSE;
-        }
-      }
-
-      SbBool ok;
-      if (from) ok = to->connectFrom(from, notnotify, append);
-      else ok = to->connectFrom(output, notnotify, append);
-      // Both known possible failure points are caught above.
-      assert(ok && "unexpected connection error");
-
-      return TRUE;
-    }
+    SoDB::createRoute(fromnode, fromfieldname.getString(), 
+                      tonode, tofieldname.getString());
+    return TRUE;
   }
   return FALSE;
 }
 
-
 /*!
   \COININTERNAL
 
