@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -55,6 +55,7 @@
 #include <Inventor/lists/SbList.h>
 #include <Inventor/misc/SbHash.h>
 #include <Inventor/C/threads/threadsutilp.h>
+#include <Inventor/C/glue/glp.h>
 #include <stdlib.h>
 
 // *************************************************************************
@@ -141,7 +142,6 @@ socontexthandler_cleanup(void)
   socontexthandler_hashlist = NULL;
   socontexthandler_idx = 0;
   CC_MUTEX_DESTRUCT(socontexthandler_mutex);
-  socontexthandler_mutex = NULL;
 }
 
 // *************************************************************************
@@ -195,6 +195,9 @@ SoContextHandler::destructingContext(uint32_t contextid)
     const socontexthandler_cbitem & item = listcopy[i];
     item.func(contextid, item.closure);
   }
+
+  // tell glglue that this context is dead
+  coin_glglue_destruct(contextid);
 }
 
 // *************************************************************************
@@ -220,7 +223,7 @@ SoContextHandler::addContextDestructionCallback(ContextDestructionCB * func,
     socontexthandler_hashlist = new SbHash <uint32_t, socontexthandler_cbitem> (64);
     // make this callback trigger after the SoGLCacheContext cleanup function
     // by setting priority to -1
-    coin_atexit((coin_atexit_f *)socontexthandler_cleanup, -1);
+    coin_atexit((coin_atexit_f *)socontexthandler_cleanup, CC_ATEXIT_NORMAL - 1);
   }
   socontexthandler_cbitem item;
   item.func = func;
