@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -65,6 +65,7 @@
 
 #include <Inventor/C/threads/storage.h>
 #include <Inventor/C/tidbits.h>
+#include <Inventor/C/tidbitsp.h>
 #include <Inventor/SbImage.h>
 #include <Inventor/elements/SoGLCacheContextElement.h>
 #include <Inventor/elements/SoGLDisplayList.h>
@@ -161,6 +162,12 @@ public:
 
 SoType SoGLBigImageP::classTypeId STATIC_SOTYPE_INIT;
 
+static void soglbigimagep_cleanup(void)
+{
+  SoGLBigImageP::classTypeId STATIC_SOTYPE_INIT;
+  CHANGELIMIT = 4;
+}
+
 static void
 soglbigimagetls_construct(void * closure)
 {
@@ -226,6 +233,7 @@ SoGLBigImage::initClass(void)
   assert(SoGLBigImageP::classTypeId.isBad());
   SoGLBigImageP::classTypeId =
     SoType::createType(SoGLImage::getClassTypeId(), SbName("GLBigImage"));
+  coin_atexit((coin_atexit_f*) soglbigimagep_cleanup, CC_ATEXIT_NORMAL);
 }
 
 // Doc in superclass.
