@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -427,6 +427,7 @@ static void
 SoGlyph_cleanup(void)
 {
   delete activeGlyphs;
+  activeGlyphs = NULL;
   CC_MUTEX_DESTRUCT(SoGlyph_mutex);
 }
 
@@ -460,7 +461,7 @@ SoGlyph::getGlyph(const char character, const SbName & font)
 
   if (activeGlyphs == NULL) {
     activeGlyphs = new SbList <coin_glyph_info>;
-    coin_atexit((coin_atexit_f *)SoGlyph_cleanup, 0);
+    coin_atexit((coin_atexit_f *)SoGlyph_cleanup, CC_ATEXIT_NORMAL);
   }
 
   int i, n = activeGlyphs->getLength();
@@ -562,7 +563,7 @@ SoGlyph::getGlyph(SoState * state,
   
   if (activeGlyphs == NULL) {
     activeGlyphs = new SbList <coin_glyph_info>;
-    coin_atexit((coin_atexit_f *)SoGlyph_cleanup, 0);
+    coin_atexit((coin_atexit_f *)SoGlyph_cleanup, CC_ATEXIT_NORMAL);
   }
 
   int i, n = activeGlyphs->getLength();
