@@ -1,23 +1,23 @@
 /**************************************************************************\
- *  
+ *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
- *  
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  ("GPL") version 2 as published by the Free Software Foundation.
  *  See the file LICENSE.GPL at the root directory of this source
  *  distribution for additional information about the GNU GPL.
- *    
+ *
  *  For using Coin with software that can not be combined with the GNU
  *  GPL, and for taking advantage of the additional benefits of our
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -25,117 +25,73 @@
 #include <config.h>
 #endif // HAVE_CONFIG_H
 
-#ifdef COIN_HAVE_JAVASCRIPT 
+#ifdef COIN_HAVE_JAVASCRIPT
   
 #include <Inventor/misc/SoJavaScriptEngine.h>
-#include <Inventor/C/glue/spidermonkey.h>
 #include <Inventor/errors/SoDebugError.h>
 #include <Inventor/C/tidbits.h>
 
-// FIXME: Do this in a nicer way. 20050714 erikgors.
-#include "../vrml97/JS_VRMLClasses.cpp"
+#include "../vrml97/JS_VRMLClasses.h"
 
-JSRuntime * SoJavaScriptEngine::runtime = NULL;
-size_t SoJavaScriptEngine::CONTEXT_STACK_CHUNK_SIZE = 8192; /* stack chunk size */
+// *************************************************************************
 
+#undef PRIVATE
+#undef PUBLIC
+#define PRIVATE(p) (p->pimpl)
+#define PUBLIC(p) (p->master)
+
+class SoJavaScriptEngineP {
+public:
+  SoJavaScriptEngineP(SoJavaScriptEngine * node) : master(node) {};
+  SbBool executeJSScript(JSScript * script) const;
+
+  static size_t CONTEXT_STACK_CHUNK_SIZE; /* stack chunk size */
+  static JSRuntime * runtime;
+  static JSClass globalclass;
+  JSContext * context;
+  JSObject * global;
+
+  struct JavascriptHandler {
+    SoType type;
+    SoJSWrapperInitFunc * init;
+    SoJSfield2jsvalFunc * field2jsval;
+    SoJSjsval2field2Func * jsval2field;
+  };
+
+  SbList<JavascriptHandler> handlerList;
+  SoJavaScriptEngine * master;
+  SbBool autonodeunref;
+};
+
+JSRuntime * SoJavaScriptEngineP::runtime = NULL;
+size_t SoJavaScriptEngineP::CONTEXT_STACK_CHUNK_SIZE = 8192; /* stack chunk size */
+JSClass SoJavaScriptEngineP::globalclass;
+
+// *************************************************************************
 
 /*!
-  Returns spidermonkey runtime instance for this class.
+  Execute a compiled script.
  */
-JSRuntime *
-SoJavaScriptEngine::getRuntime(void)
-{
-  return SoJavaScriptEngine::runtime;
-}
-    
-/*!
-  Set the spidermonkey runtime for this class
- */
-void
-SoJavaScriptEngine::setRuntime(JSRuntime * runtime)
-{
-  SoJavaScriptEngine::runtime = runtime;
-}
-
-/*!
-  Returns spidermonkey context for this object.
- */
-JSContext *
-SoJavaScriptEngine::getContext(void)
-{
-  return this->context;
-}
-    
-/*!
-  Set the spidermonkey context for this object.
- */
-void
-SoJavaScriptEngine::setContext(JSContext * context)
-{
-  this->context = context;
-}
-
-/*!
-  Returns spidermonkey global object for this object.
- */
-JSObject *
-SoJavaScriptEngine::getGlobal(void)
-{
-  return this->global;
-}
-    
-/*!
-  Set the spidermonkey global object for this object.
- */
-void
-SoJavaScriptEngine::setGlobal(JSObject * global)
-{
-  this->global = global;
-}
-
-/*!
- Init the spidermonkey runtime.
- */
-void
-SoJavaScriptEngine::init(uint32_t maxBytes)
-{
-  assert(SoJavaScriptEngine::getRuntime() == NULL);
-  JSRuntime * rt = spidermonkey()->JS_NewRuntime(maxBytes);
-
-  if (rt == NULL) {
-    SoDebugError::postWarning("SoJavaScriptEngine::init",
-                              "SpiderMonkey Javascript engine available, "
-                              "but failed to instantiate a JSRuntime!");
-    return;
-  }
-  SoJavaScriptEngine::setRuntime(rt);
-}
-
-/*!
- Shutdown the spidermonkey runtime.
- */
-void
-SoJavaScriptEngine::shutdown(void)
-{
-  JSRuntime * rt = SoJavaScriptEngine::getRuntime();
-  if (rt != NULL) {
-    spidermonkey()->JS_DestroyRuntime(rt);
-  }
-
-  spidermonkey()->JS_ShutDown();
-  SoJavaScriptEngine::setRuntime(NULL);
-}
-
 SbBool
-SoJavaScriptEngine::debug(void)
+SoJavaScriptEngineP::executeJSScript(JSScript * script) const
 {
-  static int d = -1;
-  if (d == -1) {
-    const char * env = coin_getenv("COIN_DEBUG_VRMLSCRIPT");
-    d = (env && (atoi(env) > 0)) ? 1 : 0;
-
+  jsval rval;
+  JSBool ok = spidermonkey()->JS_ExecuteScript(this->context, this->global, script, &rval);
+  if (ok) {
+    if (SoJavaScriptEngine::debug()) {
+      JSString * str = spidermonkey()->JS_ValueToString(this->context, rval);
+      SoDebugError::postInfo("SoJavaScriptEngineP::executeJSScript",
+                             "script result: '%s'",
+                             spidermonkey()->JS_GetStringBytes(str));
+    }
+    return TRUE;
   }
-  return d ? TRUE : FALSE;
+  else {
+    // FIXME: improve on this. 20050526 mortene.
+    SoDebugError::postWarning("SoJavaScriptEngine::executeJSScript",
+                              "Script evaluation failed!");
+  }
+  return FALSE;
 }
 
 // FIXME: imported from SquirrelMonkey/src/jsutils.cpp
@@ -144,8 +100,7 @@ SoJavaScriptEngine::debug(void)
   Prints a stacktrace for the pending exception.
   Does nothing if there aren't any pending exceptions.
 */
-static void
-printJSException(JSContext *cx)
+static void printJSException(JSContext *cx)
 {
   jsval val, stack;
   JSObject * obj;
@@ -164,16 +119,18 @@ printJSException(JSContext *cx)
     return;
   }
   /* root the string */
-  if (!spidermonkey()->JS_AddRoot(cx, s)) {
+  if (!spidermonkey()->JS_AddRoot(cx, &s)) {
     SoDebugError::postWarning("printJSException", "could not root string");
     return;
   }
 
+  SbBool ok;
   /* Todo: we loose unicode information here */
   cstr = spidermonkey()->JS_GetStringBytes(s);
   if (!cstr) {
     SoDebugError::postWarning("printJSException", "could not get string bytes");
-    assert(spidermonkey()->JS_RemoveRoot(cx, s));
+    ok = spidermonkey()->JS_RemoveRoot(cx, &s);
+    assert(ok && "JS_RemoveRoot failed");
     return;
   }
   len = spidermonkey()->JS_GetStringLength(s);
@@ -185,18 +142,20 @@ printJSException(JSContext *cx)
   // FIXME: this looks ugly. 20050719 erikgors.
   fwrite(cstr, 1, spidermonkey()->JS_GetStringLength(s), stderr);
   fprintf(stderr, "\n");
-  assert(spidermonkey()->JS_RemoveRoot(cx, s));
+  ok = spidermonkey()->JS_RemoveRoot(cx, &s);
+  assert(ok && "JS_RemoveRoot failed");
 }
 
 /*!
   Default Error Handler for Coin
  */
-static void
-SpiderMonkey_ErrorHandler(JSContext * cx, const char * message, JSErrorReport * report)
+static void SpiderMonkey_ErrorHandler(JSContext * cx, const char * message, 
+                                      JSErrorReport * report)
 {                               
   SoDebugError::postWarning("SpiderMonkey_ErrorHandler",
                             "%s:%d: %s:\n  %s\n",
-                            report->filename, report->lineno, message, report->linebuf);
+                            report->filename, report->lineno, message, 
+                            report->linebuf);
   printJSException(cx);
 }   
 
@@ -205,7 +164,7 @@ SpiderMonkey_ErrorHandler(JSContext * cx, const char * message, JSErrorReport *
   print("hello", "world", 123, obj) will return "hello world 123 [some obj]"
  */
 static JSBool JavascriptPrint(JSContext * cx, JSObject * obj, 
-                    uintN argc, jsval * argv, jsval * rval)
+                              uintN argc, jsval * argv, jsval * rval)
 {
   SbString out;
 
@@ -223,14 +182,19 @@ static JSBool JavascriptPrint(JSContext * cx, JSObject * obj,
   return JS_TRUE;
 }
 
+
 /*!
   Constructor. Will create a new context and global spidermonkey object
   for this object.
  */
 SoJavaScriptEngine::SoJavaScriptEngine()
 {
-  JSContext * cx = this->context =
-    spidermonkey()->JS_NewContext(SoJavaScriptEngine::getRuntime(), SoJavaScriptEngine::CONTEXT_STACK_CHUNK_SIZE);
+  PRIVATE(this) = new SoJavaScriptEngineP(this); 
+  PRIVATE(this)->autonodeunref = TRUE;
+
+  JSContext * cx = PRIVATE(this)->context = 
+    spidermonkey()->JS_NewContext(SoJavaScriptEngine::getRuntime(), 
+                                  SoJavaScriptEngineP::CONTEXT_STACK_CHUNK_SIZE);
   if (!cx) {
     SoDebugError::postWarning("SoJavaScriptEngine::SoJavaScriptEngine",
                               "SpiderMonkey Javascript engine available, "
@@ -241,23 +205,11 @@ SoJavaScriptEngine::SoJavaScriptEngine()
 
   (void)spidermonkey()->JS_SetErrorReporter(cx, SpiderMonkey_ErrorHandler);
 
-  static JSClass jclass = {
-    "Coin SoJavaScriptEngine global object class", 0,
-    spidermonkey()->JS_PropertyStub,
-    spidermonkey()->JS_PropertyStub,
-    spidermonkey()->JS_PropertyStub,
-    spidermonkey()->JS_PropertyStub,
-    spidermonkey()->JS_EnumerateStub,
-    spidermonkey()->JS_ResolveStub,
-    spidermonkey()->JS_ConvertStub, 
-    spidermonkey()->JS_FinalizeStub,
-    NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0
-  };
-
   // FIXME: add global as an argument, so more than one context can share
   // the same global object? 20050719 erikgors.
-  JSObject * global = this->global =
-    spidermonkey()->JS_NewObject(cx, &jclass, NULL, NULL);
+  JSObject * global = PRIVATE(this)->global = 
+    spidermonkey()->JS_NewObject(cx, &SoJavaScriptEngineP::globalclass, 
+                                 NULL, NULL);
   if (!global) {
     SoDebugError::postWarning("SoJavaScriptEngine::SoJavaScriptEngine",
                               "SpiderMonkey Javascript engine available, "
@@ -289,12 +241,167 @@ SoJavaScriptEngine::SoJavaScriptEngine()
 
 /*!
   Destructor. Will destroy the spidermonkey context
+
+  FIXME: Should we destroy a context that was set externally 
+  using setContext()? (assuming that setContext will be made public)
+  kintel 20050920
  */
 SoJavaScriptEngine::~SoJavaScriptEngine()
 {
-  spidermonkey()->JS_DestroyContext(this->context);
+  spidermonkey()->JS_DestroyContext(PRIVATE(this)->context);
+  delete PRIVATE(this);
 }
 
+/*!
+  Returns spidermonkey runtime instance for this class.
+ */
+JSRuntime *
+SoJavaScriptEngine::getRuntime(void)
+{
+  return SoJavaScriptEngineP::runtime;
+}
+    
+/*!
+  Set the spidermonkey runtime for this class
+ */
+void
+SoJavaScriptEngine::setRuntime(JSRuntime * runtime)
+{
+  SoJavaScriptEngineP::runtime = runtime;
+}
+
+/*!
+  Returns spidermonkey context for this object.
+ */
+JSContext *
+SoJavaScriptEngine::getContext(void) const
+{
+  return PRIVATE(this)->context;
+}
+    
+/*!
+  Set the spidermonkey context for this object.
+ */
+void
+SoJavaScriptEngine::setContext(JSContext * context)
+{
+  // FIXME: reassociate this with the given context?  kintel 20050920
+  PRIVATE(this)->context = context;
+}
+
+/*!
+  Returns spidermonkey global object for this object.
+ */
+JSObject *
+SoJavaScriptEngine::getGlobal(void) const
+{
+  // FIXME: Get using JS_GetGlobalObject()? kintel 20050920
+  return PRIVATE(this)->global;
+}
+    
+/*!
+  Set the spidermonkey global object for this object.
+ */
+void
+SoJavaScriptEngine::setGlobal(JSObject * global)
+{
+  PRIVATE(this)->global = global;
+  // FIXME: Also set using JS_SetGlobalObject()? kintel 20050920
+}
+
+/*!
+ Init the spidermonkey runtime.
+ */
+SbBool
+SoJavaScriptEngine::init(uint32_t maxBytes)
+{
+  assert(SoJavaScriptEngine::getRuntime() == NULL);
+
+  if (!spidermonkey()->available) {
+    SoDebugError::postWarning("SoJavaScriptEngine::init",
+                              "Coin configured to support Javascript for "
+                              "VRML Script nodes, but SpiderMonkey Javascript "
+                              "engine not available. Javascript scripts will "
+                              "be ignored.");
+    return FALSE;
+  }
+
+  JSRuntime * rt = spidermonkey()->JS_NewRuntime(maxBytes);
+
+  if (rt == NULL) {
+    SoDebugError::post("SoJavaScriptEngine::init",
+                       "SpiderMonkey Javascript engine available, "
+                       "but failed to instantiate a JSRuntime!");
+    return FALSE;
+  }
+
+  SoJavaScriptEngine::setRuntime(rt);
+
+  SoJavaScriptEngineP::globalclass.name = "SoJavaScriptEngine_global";
+  SoJavaScriptEngineP::globalclass.flags = 0;
+  SoJavaScriptEngineP::globalclass.addProperty = spidermonkey()->JS_PropertyStub;
+  SoJavaScriptEngineP::globalclass.delProperty = spidermonkey()->JS_PropertyStub;
+  SoJavaScriptEngineP::globalclass.getProperty = spidermonkey()->JS_PropertyStub;
+  SoJavaScriptEngineP::globalclass.setProperty = spidermonkey()->JS_PropertyStub;
+  SoJavaScriptEngineP::globalclass.enumerate = spidermonkey()->JS_EnumerateStub;
+  SoJavaScriptEngineP::globalclass.resolve = spidermonkey()->JS_ResolveStub;
+  SoJavaScriptEngineP::globalclass.convert = spidermonkey()->JS_ConvertStub;
+  SoJavaScriptEngineP::globalclass.finalize = spidermonkey()->JS_FinalizeStub;
+  
+  return TRUE;
+}
+
+/*!
+ Shutdown the spidermonkey runtime.
+ */
+void
+SoJavaScriptEngine::shutdown(void)
+{
+  JSRuntime * rt = SoJavaScriptEngine::getRuntime();
+  if (rt != NULL) {
+    spidermonkey()->JS_DestroyRuntime(rt);
+  }
+
+  spidermonkey()->JS_ShutDown();
+  SoJavaScriptEngine::setRuntime(NULL);
+}
+
+SbBool
+SoJavaScriptEngine::debug(void)
+{
+  static int d = -1;
+  if (d == -1) {
+    const char * env = coin_getenv("COIN_DEBUG_VRMLSCRIPT");
+    d = (env && (atoi(env) > 0)) ? 1 : 0;
+
+  }
+  return d ? TRUE : FALSE;
+}
+
+/*!
+  Turn on/off the automatic reference-count handling of Coin nodes
+  created in JavaScript. When switched off, the programmer must
+  explicitly call ref()/unref() for the Coin nodes JavaScript.
+  
+  Default state is TRUE.
+*/
+void
+SoJavaScriptEngine::enableAutoNodeUnref(SbBool onoff)
+{
+  PRIVATE(this)->autonodeunref = onoff;
+}
+
+/*!
+  Returns the current state (on/off) of the automatic ref/unref
+  mechanism for Coin nodes created by JavaScript.
+*/
+SbBool
+SoJavaScriptEngine::getAutoNodeUnrefState() const
+{
+  return PRIVATE(this)->autonodeunref;
+}
+
+
 /*!
   Compile and execute a string containing a script.
  */
@@ -309,12 +416,12 @@ SoJavaScriptEngine::executeScript(const SbName & name, const SbString & script)
   // FIXME: should set a correct linenum offset, for better error messages.
   // 20050728 erikgors.
   JSScript * jsscript =
-    spidermonkey()->JS_CompileScript(this->context,
-                                     this->global,
+    spidermonkey()->JS_CompileScript(PRIVATE(this)->context,
+                                     PRIVATE(this)->global,
                                      script.getString(), script.getLength(),
                                      name.getString(), 1);
 
-  return this->executeJSScript(jsscript);
+  return PRIVATE(this)->executeJSScript(jsscript);
 }
 
 /*!
@@ -327,58 +434,34 @@ SoJavaScriptEngine::executeFile(const SbName & filename) const
     SoDebugError::postInfo("SoJavaScriptEngine::executeFile", "filename=='%s'", filename.getString());
   }
 
-  JSScript * script = spidermonkey()->JS_CompileFile(this->context, this->global, filename);
+  JSScript * script = spidermonkey()->JS_CompileFile(PRIVATE(this)->context, PRIVATE(this)->global, filename);
 
-  return this->executeJSScript(script);
-}
-
-/*!
-  Execute a compiled script.
- */
-SbBool
-SoJavaScriptEngine::executeJSScript(JSScript * script) const
-{
-  jsval rval;
-  JSBool ok = spidermonkey()->JS_ExecuteScript(this->context, this->global, script, &rval);
-  if (ok) {
-    if (SoJavaScriptEngine::debug()) {
-      JSString * str = spidermonkey()->JS_ValueToString(this->context, rval);
-      SoDebugError::postInfo("SoJavaScriptEngine::executeJSScript",
-                             "script result: '%s'",
-                             spidermonkey()->JS_GetStringBytes(str));
-    }
-    return TRUE;
-  }
-  else {
-    // FIXME: improve on this. 20050526 mortene.
-    SoDebugError::postWarning("SoJavaScriptEngine::executeJSScript",
-                              "Script evaluation failed!");
-  }
-  return FALSE;
+  return PRIVATE(this)->executeJSScript(script);
 }
 
 /*!
   Execute a function in the global spidermonkey object.
  */
 SbBool
-SoJavaScriptEngine::executeFunction(const SbName & name,
-                                    int argc, const SoField * argv, SoField * rval) const
+SoJavaScriptEngine::executeFunction(const SbName & name, 
+                                    int argc, const SoField * argv, 
+                                    SoField * rval) const
 {
   jsval * jsargv = new jsval[argc];
 
   for (int i=0; i<argc; ++i) {
-    this->field2jsval(this->context, &argv[i], &jsargv[i]);
+    field2jsval(&argv[i], &jsargv[i]);
   }
 
   jsval rjsval;
   JSBool ok =
-    spidermonkey()->JS_CallFunctionName(this->context, this->global,
+    spidermonkey()->JS_CallFunctionName(PRIVATE(this)->context, PRIVATE(this)->global,
                                         name.getString(), argc, jsargv, &rjsval);
   delete [] jsargv;
 
   if (ok) {
     if (SoJavaScriptEngine::debug()) {
-      JSString * str = spidermonkey()->JS_ValueToString(this->context, rjsval);
+      JSString * str = spidermonkey()->JS_ValueToString(PRIVATE(this)->context, rjsval);
       SoDebugError::postInfo("SoJavaScriptEngine::executeFunction",
                              "function: \"%s\" "
                              "result: '%s'",
@@ -387,7 +470,7 @@ SoJavaScriptEngine::executeFunction(const SbName & name,
 
     SbBool ok2 = TRUE;
     if (rval != NULL) {
-      ok2 = this->jsval2field(this->context, rjsval, rval);
+      ok2 = jsval2field(rjsval, rval);
     }
     return ok2;
   }
@@ -403,16 +486,16 @@ SoJavaScriptEngine::executeFunction(const SbName & name,
   Convert a SoField object to a native spidermonkey value.
  */
 SbBool
-SoJavaScriptEngine::field2jsval(const SoField * f, jsval * v)
+SoJavaScriptEngine::field2jsval(const SoField * f, jsval * v) const
 {
-  int n = this->handlerList.getLength();
+  int n = PRIVATE(this)->handlerList.getLength();
 
   // go backwards. new handlers has precedence. 20050719 erikgors.
   while (n --> 0) {
-    const SoJavaScriptEngine::JavascriptHandler & handler = this->handlerList[n];
+    const SoJavaScriptEngineP::JavascriptHandler & handler = PRIVATE(this)->handlerList[n];
 
     if (handler.field2jsval != NULL && f->isOfType(handler.type)) {
-      handler.field2jsval(this->context, f, v);
+      handler.field2jsval(PRIVATE(this)->context, f, v);
       return TRUE;
     }
   }
@@ -421,32 +504,43 @@ SoJavaScriptEngine::field2jsval(const SoField * f, jsval * v)
   return FALSE;
 }
 
-SbBool
-SoJavaScriptEngine::field2jsval(JSContext * cx, const SoField * f, jsval * v)
+/*!
+  Returns the SoJavaScriptEngine associated with the given context.
+  If the context isn't associated with an SoJavaScriptEngine it will 
+  return NULL.
+
+  NB! Setting the context private data (using JS_SetContextPrivate()) will
+  overwrite this information and cause this method to return a garbage
+  pointer.
+*/
+SoJavaScriptEngine *
+SoJavaScriptEngine::getEngine(JSContext * cx)
 {
-  SoJavaScriptEngine * engine = (SoJavaScriptEngine *)spidermonkey()->JS_GetContextPrivate(cx);
-  assert(engine != NULL);
-  return engine->field2jsval(f, v);
+  // FIXME: should use an internal SbHash dict instead, so we don't
+  // cripple the GetContextPrivate() / SetContextPrivate()
+  // functionality. (Which would give one less way of f*cking up for
+  // those using the SoJavaScriptEngine interface.)  20060207 mortene.
+  return (SoJavaScriptEngine *)spidermonkey()->JS_GetContextPrivate(cx);
 }
 
 /*!
   Convert a native spidermonkey value to a SoField object.
  */
 SbBool 
-SoJavaScriptEngine::jsval2field(const jsval v, SoField * f)
+SoJavaScriptEngine::jsval2field(const jsval v, SoField * f) const
 {
-  int n = this->handlerList.getLength();
+  int n = PRIVATE(this)->handlerList.getLength();
 
   // go backwards. new handlers has precedence. 20050719 erikgors.
   while (n --> 0) {
-    const SoJavaScriptEngine::JavascriptHandler & handler = this->handlerList[n];
+    const SoJavaScriptEngineP::JavascriptHandler & handler = PRIVATE(this)->handlerList[n];
 
     if (handler.jsval2field != NULL && f->isOfType(handler.type)) {
-      if (handler.jsval2field(this->context, v, f)) {
+      if (handler.jsval2field(PRIVATE(this)->context, v, f)) {
         return TRUE;
       }
       else {
-        JSString * jsstr = spidermonkey()->JS_ValueToString(this->context, v); 
+        JSString * jsstr = spidermonkey()->JS_ValueToString(PRIVATE(this)->context, v); 
         const char * str = spidermonkey()->JS_GetStringBytes(jsstr);
         SoDebugError::postWarning("SoJavaScriptEngine::jsval2field",
                                   "convertion of '%s' to SoField type '%s' failed",
@@ -459,31 +553,33 @@ SoJavaScriptEngine::jsval2field(const jsval v, SoField * f)
   return FALSE;
 }
 
-SbBool
-SoJavaScriptEngine::jsval2field(JSContext * cx, const jsval v, SoField * f)
-{
-  SoJavaScriptEngine * engine = (SoJavaScriptEngine *)spidermonkey()->JS_GetContextPrivate(cx);
-  assert(engine != NULL);
-  return engine->jsval2field(v, f);
-}
-
 /*!
-  Add a handler. init, field2jsval and jsval2field can be NULL.
-  init, if not NULL, will be called.
- */
+  Adds a JavaScript handler for an SoField subtype.
+
+  \a init is the class init function and will be called immediately 
+  if specified.
+  \a field2jsval and \a jsval2field will convert an SoField to a jsval 
+  or vice versa. Setting these to NULL is allowed but will result in the 
+  fields not being accessible from JavaScript.
+
+  New handlers will get precedence over old handlers.
+*/
 void
-SoJavaScriptEngine::addHandler(const SoType & type, CoinJSinit_t init, CoinJSfield2jsval_t field2jsval, CoinJSjsval2field_t jsval2field)
+SoJavaScriptEngine::addHandler(const SoType & type, 
+                               SoJSWrapperInitFunc * init, 
+                               SoJSfield2jsvalFunc * field2jsval, 
+                               SoJSjsval2field2Func * jsval2field)
 {
-  SoJavaScriptEngine::JavascriptHandler handler;
+  SoJavaScriptEngineP::JavascriptHandler handler;
   handler.type = type;
   handler.init = init;
   handler.field2jsval = field2jsval;
   handler.jsval2field = jsval2field;
 
-  this->handlerList.append(handler);
+  PRIVATE(this)->handlerList.append(handler);
 
   if (handler.init != NULL) {
-    handler.init(this->context, this->global);
+    handler.init(PRIVATE(this)->context, PRIVATE(this)->global);
   }
 }
 
@@ -494,10 +590,10 @@ SbBool
 SoJavaScriptEngine::setScriptField(const SbName & name, const SoField * f) const
 {
   jsval initval;
-  SoJavaScriptEngine::field2jsval(this->context, f, &initval);
+  field2jsval(f, &initval);
   const JSBool ok =
-    spidermonkey()->JS_SetProperty(this->context,
-                                   this->global,
+    spidermonkey()->JS_SetProperty(PRIVATE(this)->context,
+                                   PRIVATE(this)->global,
                                    name.getString(), &initval);
   if (!ok) {
     SoDebugError::post("SoJavaScriptEngine::setScriptField",
@@ -514,8 +610,8 @@ SbBool
 SoJavaScriptEngine::unsetScriptField(const SbName & name) const
 {
   const JSBool ok =
-    spidermonkey()->JS_DeleteProperty(this->context,
-                                      this->global,
+    spidermonkey()->JS_DeleteProperty(PRIVATE(this)->context,
+                                      PRIVATE(this)->global,
                                       name.getString());
   if (!ok) {
     SoDebugError::post("SoJavaScriptEngine::unsetScriptField",
@@ -532,7 +628,7 @@ SbBool
 SoJavaScriptEngine::getScriptField(const SbName & name, SoField * f) const
 {
     jsval val;
-    const JSBool ok = spidermonkey()->JS_GetProperty(this->context, this->global,
+    const JSBool ok = spidermonkey()->JS_GetProperty(PRIVATE(this)->context, PRIVATE(this)->global,
                                                      name.getString(), &val);
     if (!ok) {
       SoDebugError::post("SoJavaScriptEngine::getScriptField",
@@ -541,24 +637,27 @@ SoJavaScriptEngine::getScriptField(const SbName & name, SoField * f) const
       return FALSE;
     }
     if (SoJavaScriptEngine::debug()) {
-      JSString * str = spidermonkey()->JS_ValueToString(this->context, val);
+      JSString * str = spidermonkey()->JS_ValueToString(PRIVATE(this)->context, val);
       SoDebugError::postInfo("SoJavaScriptEngine::getScriptField", "trying to convert"
                              " \"%s\" to type %s", spidermonkey()->JS_GetStringBytes(str),
                              f->getTypeId().getName().getString());
     }
 
 
-    return SoJavaScriptEngine::jsval2field(this->context, val, f);
+    return jsval2field(val, f);
 }
 
 SbBool
 SoJavaScriptEngine::hasScriptField(const SbName & name) const
 {
   jsval val;
-  assert(spidermonkey()->JS_GetProperty(this->context, this->global,
+  assert(spidermonkey()->JS_GetProperty(PRIVATE(this)->context, PRIVATE(this)->global,
                                         name.getString(), &val));
 
   return JSVAL_IS_VOID(val) ? FALSE : TRUE;
 }
 
-#endif // !COIN_HAVE_JAVASCRIPT
+#undef PRIVATE
+#undef PUBLIC
+
+#endif // !COIN_HAVE_JAVASCRIPT 
