@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -40,6 +40,7 @@
 
 #include <Inventor/lock/SoLockMgr.h>
 #include <Inventor/SbString.h>
+#include <Inventor/C/tidbitsp.h>
 #include <string.h>
 
 class SoLockManager_pimpl {
@@ -50,7 +51,13 @@ public:
   char * unlockstr;
 };
 
-SoLockManager_pimpl * SoLockManager::pimpl = NULL;
+static SoLockManager_pimpl * solockmanager_pimpl = NULL;
+
+static void solockmanager_cleanup(void)
+{
+  delete solockmanager_pimpl;
+  solockmanager_pimpl = NULL;
+}
 
 /*!
   A void method provided just for sourcecode compatibility in client
@@ -62,11 +69,13 @@ SoLockManager_pimpl * SoLockManager::pimpl = NULL;
 void
 SoLockManager::SetUnlockString(char * unlockstr)
 {
-  if (!SoLockManager::pimpl) // FIXME: deallocate on exit. 20000518 mortene.
-    SoLockManager::pimpl = new SoLockManager_pimpl;
-  delete SoLockManager::pimpl->unlockstr;
-  SoLockManager::pimpl->unlockstr = new char[strlen(unlockstr) + 1];
-  (void)strcpy(SoLockManager::pimpl->unlockstr, unlockstr);
+  if (!solockmanager_pimpl) {
+    solockmanager_pimpl = new SoLockManager_pimpl;
+    coin_atexit((coin_atexit_f*)solockmanager_cleanup, CC_ATEXIT_NORMAL);
+  }
+  delete solockmanager_pimpl->unlockstr;
+  solockmanager_pimpl->unlockstr = new char[strlen(unlockstr) + 1];
+  (void)strcpy(solockmanager_pimpl->unlockstr, unlockstr);
 }
 
 /*!
@@ -78,5 +87,5 @@ SoLockManager::SetUnlockString(char * unlockstr)
 char *
 SoLockManager::GetUnlockString(void)
 {
-  return SoLockManager::pimpl ? SoLockManager::pimpl->unlockstr : NULL;
+  return solockmanager_pimpl ? solockmanager_pimpl->unlockstr : NULL;
 }
