@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -28,18 +28,33 @@
 #ifdef HAVE_GLX
 
 #include "SoOffscreenGLXData.h"
+#include <Inventor/C/tidbitsp.h>
 #include <Inventor/errors/SoDebugError.h>
 
 // *************************************************************************
 
 Display * SoOffscreenGLXData::display = NULL;
+static SbBool display_initialized = FALSE;
+
+void 
+SoOffscreenGLXData::cleanup(void) 
+{
+  display_initialized = FALSE;
+  // FIXME: Disabled since this might cause problems according to the
+  // comment below (see getDisplay). Not sure if *not* closing the
+  // display cannot also lead to problems though... 20060208 kyrah
+#if 0 
+  XCloseDisplay(display);
+  display = NULL;
+#endif
+}
 
 Display *
 SoOffscreenGLXData::getDisplay(void)
 {
-  static SbBool first = TRUE;
-  if (first) {
-    first = FALSE;
+  if (!display_initialized) {
+    display_initialized = TRUE;
+    coin_atexit((coin_atexit_f*)SoOffscreenGLXData::cleanup, CC_ATEXIT_NORMAL);
 
     // Keep a single static display-ptr.
     // 
