@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -95,6 +95,7 @@
 #include <Inventor/misc/SoProtoInstance.h>
 #include <Inventor/nodes/SoGroup.h>
 #include <Inventor/nodes/SoSeparator.h>
+#include "../io/SoWriterefCounter.h"
 
 // *************************************************************************
 
@@ -112,7 +113,6 @@ soproto_cleanup(void)
   delete protolist;
   protolist = NULL;
   CC_MUTEX_DESTRUCT(soproto_mutex);
-  soproto_mutex = NULL;
 }
 
 static SoProto *
@@ -244,7 +244,8 @@ SoProto::initClass(void)
                                     SbName("SoProto"), NULL,
                                     SoNode::nextActionMethodIndex++);
   protolist = new SbList<SoProto*>;
-  coin_atexit((coin_atexit_f*) soproto_cleanup, 0);
+
+  coin_atexit((coin_atexit_f*) soproto_cleanup, CC_ATEXIT_NORMAL);
   // this will set a default callback
   SoProto::setFetchExternProtoCallback(NULL, NULL);
 }
@@ -409,12 +410,22 @@ SoProto::write(SoWriteAction * action)
     if (PRIVATE(this)->defroot && !PRIVATE(this)->externurl) {
       this->writeDefinition(action);
     }
+    this->writeInterface(out);
   }
   else if (out->getStage() == SoOutput::WRITE) {
-    out->indent();
+    int writerefcount = SoWriterefCounter::instance(out)->getWriteref(this);
+
     out->write(PRIVATE(this)->externurl ? "EXTERNPROTO " : "PROTO ");
     out->write(PRIVATE(this)->name.getString());
-    out->write(" [\n");
+    if (SoWriterefCounter::debugWriterefs()) {
+      SbString tmp;
+      tmp.sprintf(" [ # writeref: %d\n",
+                  writerefcount);
+      out->write(tmp.getString());
+    }
+    else {
+      out->write(" [\n");
+    }
     out->incrementIndent();
 
     this->writeInterface(out);
@@ -436,8 +447,23 @@ SoProto::write(SoWriteAction * action)
       out->resolveRoutes();
       out->decrementIndent();
       out->indent();
-      out->write("}\n");
+      out->write("}");
     }
+
+#if COIN_DEBUG
+    if (SoWriterefCounter::debugWriterefs()) {
+      int writerefcount = SoWriterefCounter::instance(out)->getWriteref(this);
+      SoDebugError::postInfo("SoProto::write",
+                             "%p/%s/'%s': %d -> %d",
+                             this,
+                             this->getTypeId().getName().getString(),
+                             this->getName().getString(),
+                             writerefcount, writerefcount - 1);
+    }
+#endif // COIN_DEBUG
+
+    writerefcount--;
+    SoWriterefCounter::instance(out)->setWriteref(this, writerefcount);
   }
   else assert(0 && "unknown stage");
 
@@ -452,58 +478,76 @@ SoProto::writeInterface(SoOutput * out)
 {
   const SoFieldData * fd = PRIVATE(this)->fielddata;
 
-  for (int i = 0; i < fd->getNumFields(); i++) {
-    out->indent();
-    SoField * f = fd->getField(this, i);
-    SoType t = f->getTypeId();
-    switch (f->getFieldType()) {
-    case SoField::NORMAL_FIELD:
-      out->write("field ");
-      out->write(t.getName().getString());
-      if (PRIVATE(this)->externurl) {
+  if (out->getStage() == SoOutput::COUNT_REFS) {
+    for (int i = 0; i < fd->getNumFields(); i++) {
+      SoField * f = fd->getField(this, i);
+      switch (f->getFieldType()) {
+      case SoField::NORMAL_FIELD:
+      case SoField::EXPOSED_FIELD:
+        if (!PRIVATE(this)->externurl) {
+          SbBool fieldwasdefault = f->isDefault();
+          if (fieldwasdefault) f->setDefault(FALSE);
+          f->write(out, fd->getFieldName(i));
+          if (fieldwasdefault) f->setDefault(TRUE);
+        }
+        break;
+      }
+    }
+  }
+  else {
+    for (int i = 0; i < fd->getNumFields(); i++) {
+      out->indent();
+      SoField * f = fd->getField(this, i);
+      SoType t = f->getTypeId();
+      switch (f->getFieldType()) {
+      case SoField::NORMAL_FIELD:
+        out->write("field ");
+        out->write(t.getName().getString());
+        if (PRIVATE(this)->externurl) {
+          out->write(' ');
+          out->write(fd->getFieldName(i).getString());
+          out->write("\n");
+        }
+        else {
+          // field values are tagged as default for proto interface instances,
+          // but needs to be written to file anyway -- 20040115 larsa
+          SbBool fieldwasdefault = f->isDefault();
+          if ( fieldwasdefault ) f->setDefault(FALSE);
+          f->write(out, fd->getFieldName(i));
+          if ( fieldwasdefault ) f->setDefault(TRUE);
+        }
+        break;
+      case SoField::EXPOSED_FIELD:
+        out->write("exposedField ");
+        out->write(t.getName().getString());
+        if (PRIVATE(this)->externurl) {
+          out->write(' ');
+          out->write(fd->getFieldName(i).getString());
+          out->write("\n");
+        }
+        else {
+          SbBool fieldwasdefault = f->isDefault();
+          if ( fieldwasdefault ) f->setDefault(FALSE);
+          f->write(out, fd->getFieldName(i));
+          if ( fieldwasdefault ) f->setDefault(TRUE);
+        }
+        break;
+      case SoField::EVENTIN_FIELD:
+        out->write("eventIn ");
+        out->write(t.getName().getString());
         out->write(' ');
         out->write(fd->getFieldName(i).getString());
-        out->write("\n");
-      }
-      else {
-        // field values are tagged as default for proto interface instances,
-        // but needs to be written to file anyway -- 20040115 larsa
-        SbBool fieldwasdefault = f->isDefault();
-        if ( fieldwasdefault ) f->setDefault(FALSE);
-        f->write(out, fd->getFieldName(i));
-        if ( fieldwasdefault ) f->setDefault(TRUE);
-      }
-      break;
-    case SoField::EXPOSED_FIELD:
-      out->write("exposedField ");
-      out->write(t.getName().getString());
-      if (PRIVATE(this)->externurl) {
+        break;
+      case SoField::EVENTOUT_FIELD:
+        out->write("eventOut ");
+        out->write(t.getName().getString());
         out->write(' ');
         out->write(fd->getFieldName(i).getString());
-        out->write("\n");
+        break;
+      default:
+        assert(0 && "invalid field type");
+        break;
       }
-      else {
-        SbBool fieldwasdefault = f->isDefault();
-        if ( fieldwasdefault ) f->setDefault(FALSE);
-        f->write(out, fd->getFieldName(i));
-        if ( fieldwasdefault ) f->setDefault(TRUE);
-      }
-      break;
-    case SoField::EVENTIN_FIELD:
-      out->write("eventIn ");
-      out->write(t.getName().getString());
-      out->write(' ');
-      out->write(fd->getFieldName(i).getString());
-      break;
-    case SoField::EVENTOUT_FIELD:
-      out->write("eventOut ");
-      out->write(t.getName().getString());
-      out->write(' ');
-      out->write(fd->getFieldName(i).getString());
-      break;
-    default:
-      assert(0 && "invalid field type");
-      break;
     }
   }
   return TRUE;
@@ -912,13 +956,30 @@ SoProto::connectISRefs(SoProtoInstance * inst, SoNode * src, SoNode * dst) const
 
       continue;
     }
+
     node = dst;
 
+    int k;
     // browse path to find the correct (copied) node.
-    for (int k = 1; k < path->getLength(); k++) {
+    for (k = 1; k < path->getLength(); k++) {
       int idx = path->getIndex(k);
+      if (!node->getChildren()) break;
       node = (*(node->getChildren()))[idx];
+
+      // do some extra tests here to be more robust on invalid input
+      // files (for instance multiple IS refs to/from the same field)
+      SoNode * tstnode = path->getNode(k);
+      if (!node || tstnode->getTypeId() != node->getTypeId()) break;
     }
+
+    if (k < path->getLength()) {
+      SoDebugError::postWarning("SoProto::connectISRefs",
+                                "Unable to resolve '%s' from '%s' in '%s' PROTO",
+                                fieldname.getString(), iname.getString(), PRIVATE(this)->name.getString());
+
+      continue;
+    }
+
     if (dstfield) {
       if (isprotoinstance) {
         node = SoProtoInstance::findProtoInstance(node);
@@ -949,13 +1010,29 @@ SoProto::connectISRefs(SoProtoInstance * inst, SoNode * src, SoNode * dst) const
         // is to have to locate the PROTO instance object yourself, and
         // modify the fields on it directly - 20040115 larsa
         SbBool srcisdefault = srcfield->isDefault();
+        srcfield->setDefault(FALSE);
         dstfield->connectFrom(srcfield);
+#if 0 // start of problematic code
+        // this piece of code causes problems when writing PROTO
+        // instances, since the PROTO instance is counted once for
+        // each IS connection. The code is enabled for now, but I'll
+        // investigate more if this bidirectional connection is really
+        // necessary and if we should handle this case when counting
+        // write references. pederb, 2005-11-15
+        
+        // update 2005-12-16, pederb:
+        // This bidirectional thingie also causes bugs when importing
+        // gator_1.wrl (the connections are not set up correctly, or
+        // are messed up). It seems like we probably should disable
+        // this code since it causes a lot of problems.
+
         // propagate value immediately, before setting up reverse connection
         dstfield->evaluate();
         srcfield->connectFrom(dstfield, FALSE, TRUE);
         // propagate value immediately, so we can tag field as default
         srcfield->evaluate();
         if ( srcisdefault ) srcfield->setDefault(TRUE);
+#endif // end of problemetic code
       }
     }
     else {
