@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -97,7 +97,7 @@ SoProtoInstance::initClass(void)
 
   protoinstance_dict = new SoNode2SoProtoInstanceMap;
   CC_MUTEX_CONSTRUCT(protoinstance_mutex);
-  coin_atexit((coin_atexit_f*) SoProtoInstance::cleanupClass, 0);
+  coin_atexit((coin_atexit_f*) SoProtoInstance::cleanupClass, CC_ATEXIT_NORMAL);
 }
 
 void
@@ -253,6 +253,7 @@ SoProtoInstance::copyFieldData(const SoFieldData * src)
       cp->copyFrom(*f);
       cp->fixCopy(TRUE);
     }
+    cp->setFieldType(f->getFieldType());
     cp->setDefault(f->isDefault());
   }
   SoFieldContainer::copyDone();
