@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -47,6 +47,7 @@
 #endif // HAVE_CONFIG_H
 
 #include <Inventor/C/tidbits.h>
+#include <Inventor/C/tidbitsp.h>
 #include <Inventor/SoDB.h>
 #include <Inventor/actions/SoAudioRenderAction.h>
 #include <Inventor/actions/SoGLRenderAction.h>
@@ -54,6 +55,7 @@
 #include <Inventor/errors/SoDebugError.h>
 #include <Inventor/fields/SoSFTime.h>
 #include <Inventor/misc/SoAudioDevice.h>
+#include <Inventor/misc/SoState.h>
 #include <Inventor/nodes/SoNode.h>
 #include <Inventor/sensors/SoNodeSensor.h>
 #include <Inventor/sensors/SoOneShotSensor.h>
@@ -72,6 +74,8 @@ public:
     FLAG_ACTIVE = 0x0002
   };
 
+  SbBool isActive(void) const { return this->flags & SoSceneManagerP::FLAG_ACTIVE; }
+
   static void nodesensorCB(void * data, SoSensor * sensor);
   static void redrawshotTriggeredCB(void * data, SoSensor * sensor);
 
@@ -97,6 +101,9 @@ public:
   uint32_t redrawpri;
   static void prerendercb(void * userdata, SoGLRenderAction * action);
 
+  static SbBool cleanupfunctionset;
+  static void cleanup(void);
+
 #ifdef COIN_THREADSAFE
   SbMutex mutex;
 #endif // COIN_THREADSAFE
@@ -113,6 +120,7 @@ public:
 };
 
 SbBool SoSceneManagerP::touchtimer = TRUE;
+SbBool SoSceneManagerP::cleanupfunctionset = FALSE;
 
 #define PRIVATE(p) (p->pimpl)
 
@@ -366,9 +374,23 @@ SoSceneManager::processEvent(const SoEvent * const event)
 {
   assert(PRIVATE(this)->handleeventaction);
 
-  PRIVATE(this)->handleeventaction->setEvent(event);
-  if (PRIVATE(this)->scene) PRIVATE(this)->handleeventaction->apply(PRIVATE(this)->scene);
-  return PRIVATE(this)->handleeventaction->isHandled();
+  SbBool handled = FALSE;
+  if ( PRIVATE(this)->handleeventaction->getState() != NULL &&
+       PRIVATE(this)->handleeventaction->getState()->getDepth() != 0 ) {
+    // recursive invocation - action currently in use
+#if COIN_DEBUG
+    SoDebugError::post("SoSceneManager::processEvent",
+                       "Recursive invocation detected. Delay processing event "
+                       "until the current event is finished processing.");
+#endif // COIN_DEBUG
+  } else if ( PRIVATE(this)->scene == NULL ) {
+    // nothing
+  } else {
+    PRIVATE(this)->handleeventaction->setEvent(event);
+    PRIVATE(this)->handleeventaction->apply(PRIVATE(this)->scene);
+    handled = PRIVATE(this)->handleeventaction->isHandled();
+  }
+  return handled;
 }
 
 /*!
@@ -391,7 +413,7 @@ void
 SoSceneManager::scheduleRedraw(void)
 {
   PRIVATE(this)->lock();
-  if ((PRIVATE(this)->flags & SoSceneManagerP::FLAG_ACTIVE) && PRIVATE(this)->rendercb) {
+  if (PRIVATE(this)->isActive() && PRIVATE(this)->rendercb) {
     if (!PRIVATE(this)->redrawshot) {
       PRIVATE(this)->redrawshot =
         new SoOneShotSensor(SoSceneManagerP::redrawshotTriggeredCB, this);
@@ -414,7 +436,7 @@ SoSceneManager::scheduleRedraw(void)
 int
 SoSceneManager::isActive(void) const
 {
-  return (PRIVATE(this)->flags & SoSceneManagerP::FLAG_ACTIVE) != 0;
+  return PRIVATE(this)->isActive();
 }
 
 /*!
@@ -883,6 +905,10 @@ void
 SoSceneManager::enableRealTimeUpdate(const SbBool flag)
 {
   SoSceneManagerP::touchtimer = flag;
+  if (!SoSceneManagerP::cleanupfunctionset) {
+    coin_atexit((coin_atexit_f*) SoSceneManagerP::cleanup, CC_ATEXIT_NORMAL);
+    SoSceneManagerP::cleanupfunctionset = TRUE;
+  }
 }
 
 /*!
@@ -905,7 +931,11 @@ SoSceneManagerP::redrawshotTriggeredCB(void * data, SoSensor * /* sensor */)
   SoDebugError::postInfo("SoSceneManager::redrawshotTriggeredCB", "start");
 #endif // debug
 
-  ((SoSceneManager *)data)->redraw();
+  SoSceneManager * sm = (SoSceneManager *)data;
+
+  // Need to recheck the "active" flag, as it could have changed since
+  // it was tested in the SoSceneManager::scheduleRedraw() call.
+  if (PRIVATE(sm)->isActive()) { sm->redraw(); }
 
 #if COIN_DEBUG && 0 // debug
   SoDebugError::postInfo("SoSceneManager::redrawshotTriggeredCB", "done\n\n");
@@ -944,4 +974,10 @@ SoSceneManagerP::prerendercb(void * userdata, SoGLRenderAction * action)
   glClear(mask);
 }
 
+void
+SoSceneManagerP::cleanup(void)
+{
+  SoSceneManagerP::touchtimer = TRUE;
+  SoSceneManagerP::cleanupfunctionset = FALSE;
+}
 #undef PRIVATE
