@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -76,7 +76,10 @@
   given name.
   On UNIX, extensions nodes are regular .so files.
   On Win32, extension nodes are built as DLLs.
-  On Mac OS X systems, extension nodes are built as .dylib files.
+  On Mac OS X systems, extension nodes are built as .dylib files. (Note:
+  The extension nodes have to be built using the flag "-dynamiclib", 
+  not "-bundle".)
+
   Whether the dynamically loadable objects should be named with
   or without the "lib" prefix is optional.  Both schemes will work.
   
@@ -197,9 +200,7 @@ static Name2HandleMap * module_dict = NULL;
 void
 SoType::init(void)
 {
-  // Debugging for memory leaks will be easier if we can clean up the
-  // resource usage.
-  coin_atexit((coin_atexit_f *)SoType::clean, 0);
+  coin_atexit((coin_atexit_f *)SoType::clean, CC_ATEXIT_NORMAL);
 
   // If any of these assert fails, it is probably because
   // SoType::init() has been called for a second time. --mortene
@@ -678,6 +679,43 @@ SoType::isDerivedFrom(const SoType parent) const
 
   NB: do not write code which depends in any way on the order of the
   elements returned in \a list.
+
+  Here is a small, stand-alone example which shows how this method can
+  be used for introspection, listing all subclasses of the SoBase
+  superclass:
+
+  \code
+  #include <stdio.h>
+  #include <Inventor/SoDB.h>
+  #include <Inventor/lists/SoTypeList.h>
+  
+  static void
+  list_subtypes(SoType t, unsigned int indent = 0)
+  {
+    SoTypeList tl;
+    SoType::getAllDerivedFrom(t, tl);
+  
+    for (unsigned int i=0; i < indent; i++) { printf("  "); } 
+    printf("%s\n", t.getName().getString());
+  
+    indent++;
+    for (int j=0; j < tl.getLength(); j++) {
+      if (tl[j].getParent() == t) { // only interested in direct descendents
+        list_subtypes(tl[j], indent);
+      }
+    }
+  }
+  
+  int
+  main(void)
+  {
+    SoDB::init();
+  
+    list_subtypes(SoType::fromName("SoBase"));
+  
+    return 0;
+  }
+  \endcode
 */
 int
 SoType::getAllDerivedFrom(const SoType type, SoTypeList & list)
