@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -33,6 +33,7 @@
 */
 
 #include <Inventor/nodekits/SoNodeKit.h>
+#include <Inventor/SbBasic.h>
 #include <Inventor/SoDB.h>
 #include <Inventor/nodekits/SoAppearanceKit.h>
 #include <Inventor/nodekits/SoCameraKit.h>
@@ -41,10 +42,14 @@
 #include <Inventor/nodekits/SoSceneKit.h>
 #include <Inventor/nodekits/SoShapeKit.h>
 #include <Inventor/nodekits/SoWrapperKit.h>
+#include <Inventor/C/tidbitsp.h>
 
+static SbBool nodekit_isinitialized = FALSE;
 
-SbBool SoNodeKit::isinitialized = FALSE;
-
+static void nodekit_cleanup(void)
+{
+  nodekit_isinitialized = FALSE;
+}
 
 /*!
   Initialize the nodekit system.
@@ -60,7 +65,7 @@ SbBool SoNodeKit::isinitialized = FALSE;
 void
 SoNodeKit::init(void)
 {
-  if (SoNodeKit::isinitialized) return;
+  if (nodekit_isinitialized) return;
 
   if (!SoDB::isInitialized()) SoDB::init();
 
@@ -75,5 +80,5 @@ SoNodeKit::init(void)
   SoShapeKit::initClass();
   SoWrapperKit::initClass();
 
-  SoNodeKit::isinitialized = TRUE;
+  nodekit_isinitialized = TRUE;
 }
