@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -49,12 +49,22 @@
 #include <Inventor/actions/SoGetBoundingBoxAction.h>
 #include <Inventor/actions/SoGetMatrixAction.h>
 #include <Inventor/actions/SoAudioRenderAction.h>
+#include <Inventor/C/tidbitsp.h>
 #include <assert.h>
 
 #if COIN_DEBUG
 #include <Inventor/errors/SoDebugError.h>
 #endif // COIN_DEBUG
 
+
+static SoTypeList * sonodekitlistpart_deflist = NULL;
+
+static void sonodekitlistpart_atexit_cleanup(void)
+{
+  delete sonodekitlistpart_deflist;
+  sonodekitlistpart_deflist = NULL;
+}
+
 /*!
   \var SoChildList * SoNodeKitListPart::children
   \COININTERNAL
@@ -170,9 +180,12 @@ SoNodeKitListPart::getChildTypes(void) const
 
   // Dynamically allocated to avoid problems on systems which doesn't
   // handle static constructors.
-  static SoTypeList * deflist = new SoTypeList; // FIXME: should deallocate on exit. 20000406 mortene.
-  if (deflist->getLength() == 0) deflist->append(SoNode::getClassTypeId());
-  return *deflist;
+  if (!sonodekitlistpart_deflist) {
+    sonodekitlistpart_deflist = new SoTypeList;
+    sonodekitlistpart_deflist->append(SoNode::getClassTypeId());
+    coin_atexit((coin_atexit_f*)sonodekitlistpart_atexit_cleanup, CC_ATEXIT_NORMAL);
+  }
+  return *sonodekitlistpart_deflist;
 }
 
 /*!
