@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -48,7 +48,8 @@
 #include <Inventor/elements/SoGLLazyElement.h>
 #include <Inventor/elements/SoDiffuseColorElement.h>
 #include <Inventor/elements/SoOverrideElement.h>
-
+#include <Inventor/elements/SoGLVBOElement.h>
+#include "../misc/SoVBO.h"
 
 /*!
   \var SoMFColor SoBaseColor::rgb
@@ -59,8 +60,11 @@
 #ifndef DOXYGEN_SKIP_THIS
 
 class SoBaseColorP {
-public:
+ public:
+  SoBaseColorP() : vbo(NULL) { }
+  ~SoBaseColorP() { delete this->vbo; }
   SoColorPacker colorpacker;
+  SoVBO * vbo;
 };
 
 #endif // DOXYGEN_SKIP_THIS
@@ -116,11 +120,34 @@ void
 SoBaseColor::doAction(SoAction * action)
 {
   SoState * state = action->getState();
-
+  
   if (!this->rgb.isIgnored() && this->rgb.getNum() &&
       !SoOverrideElement::getDiffuseColorOverride(state)) {
-    SoLazyElement::setDiffuse(state, this, this->rgb.getNum(),
+    const int num = this->rgb.getNum();
+    SoLazyElement::setDiffuse(state, this, num,
                               this->rgb.getValues(0), &THIS->colorpacker);
+    
+#ifdef COIN_NEXT_MINOR
+    if (state->isElementEnabled(SoGLVBOElement::getClassStackIndex())) {
+      SbBool setvbo = FALSE;
+      SoBase::staticDataLock();
+      if (SoGLVBOElement::shouldCreateVBO(state, num)) {
+        setvbo = TRUE;
+        if (THIS->vbo == NULL) {
+          THIS->vbo = new SoVBO(GL_ARRAY_BUFFER, GL_STATIC_DRAW);
+        }
+      }
+      else if (THIS->vbo) {
+        THIS->vbo->setBufferData(NULL, 0, 0);
+      }
+      // don't fill in any data in the VBO. Data will be filled in
+      // using the ColorPacker right before the VBO is used
+      SoBase::staticDataUnlock();
+      if (setvbo) {
+        SoGLVBOElement::setColorVBO(state, THIS->vbo);
+      }
+    }
+#endif // COIN_NEXT_MINOR
     if (this->isOverride()) {
       SoOverrideElement::setDiffuseColorOverride(state, this, TRUE);
     }
