@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -51,6 +51,8 @@
 #include <Inventor/misc/SoChildList.h>
 #include <Inventor/SoOutput.h>
 #include <Inventor/errors/SoDebugError.h>
+#include <Inventor/sensors/SoOneShotSensor.h>
+#include "../misc/PointerMap.h"
 
 // *************************************************************************
 
@@ -68,23 +70,60 @@
 
 SO_NODE_SOURCE(SoBlinker);
 
+// *************************************************************************
+
+#define PRIVATE(_obj_) ((SoBlinkerP*) PointerMap::get(_obj_))
+
+class SoBlinkerP {
+public:
+  SoBlinkerP(SoBlinker * master) : master(master) { }
+
+  static void whichChildCB(void * closure, SoSensor * sensor) {
+    SoBlinkerP * thisp = (SoBlinkerP*) closure;
+    thisp->counter->reset.setValue(thisp->whichvalue);
+
+    // if sensor/blinker isn't enabled, we need to manually set the whichChild field
+    if (!thisp->counter->on.getValue()) {
+      SbBool old = thisp->master->whichChild.enableNotify(FALSE);
+      thisp->master->whichChild = thisp->whichvalue;
+      thisp->master->whichChild.enableNotify(old);
+    }
+  }
+  SoBlinker * master;
+  int whichvalue;
+  SoTimeCounter * counter;
+  SoOneShotSensor * whichChildSensor;
+};
+
+// *************************************************************************
+
 /*!
   Constructor.
 */
 SoBlinker::SoBlinker(void)
 {
+  this->counter = NULL; // this->counter is now obsoleted (pimpl version is used)
+
+  SoBlinkerP * pimpl = new SoBlinkerP(this);
+  PointerMap::add(this, pimpl);
+
+  pimpl->counter = new SoTimeCounter;
+  pimpl->counter->ref();
+  pimpl->counter->min = SO_SWITCH_NONE;
+  pimpl->counter->max = SO_SWITCH_NONE;
+  pimpl->counter->frequency.connectFrom(&this->speed);
+  pimpl->counter->on.connectFrom(&this->on);
+  pimpl->whichChildSensor = 
+    new SoOneShotSensor(SoBlinkerP::whichChildCB, pimpl);
+  pimpl->whichChildSensor->setPriority(1);
+  pimpl->whichvalue = SO_SWITCH_NONE;
+
   SO_NODE_INTERNAL_CONSTRUCTOR(SoBlinker);
 
   SO_NODE_ADD_FIELD(speed, (1));
   SO_NODE_ADD_FIELD(on, (TRUE));
-
-  this->counter = new SoTimeCounter;
-  this->counter->ref();
-  this->counter->min = SO_SWITCH_NONE;
-  this->counter->max = SO_SWITCH_NONE;
-  this->counter->frequency.connectFrom(&this->speed);
-  this->counter->on.connectFrom(&this->on);
-  this->whichChild.connectFrom(&this->counter->output);
+  
+  this->whichChild.connectFrom(&pimpl->counter->output, TRUE);
 }
 
 /*!
@@ -92,7 +131,12 @@ SoBlinker::SoBlinker(void)
 */
 SoBlinker::~SoBlinker()
 {
-  this->counter->unref();
+  SoBlinkerP * pimpl = PRIVATE(this);
+  
+  delete pimpl->whichChildSensor;
+  pimpl->counter->unref();
+  PointerMap::remove(this);
+  delete pimpl;
 }
 
 // doc in parent
@@ -136,35 +180,43 @@ SoBlinker::write(SoWriteAction * action)
 void
 SoBlinker::notify(SoNotList * nl)
 {
+  SoBlinkerP * pimpl = PRIVATE(this);
+
   // See if the whichChild field was "manually" set.
   if (nl->getFirstRec()->getBase() == this &&
       nl->getLastField() == &this->whichChild) {
-    this->counter->enableNotify(FALSE); // Wrap to avoid recursive invocation.
-    this->counter->reset.setValue(this->whichChild.getValue());
-    this->counter->enableNotify(TRUE);
+    // delay whichChild reset with the one shot sensor (to enable
+    // children to be added before the reset is actually done)
+
+    // disable connection while reading whichChild to get the actual value set
+    SbBool old = this->whichChild.isConnectionEnabled();
+    this->whichChild.enableConnection(FALSE);
+    pimpl->whichvalue = this->whichChild.getValue();
+    this->whichChild.enableConnection(old);
+    pimpl->whichChildSensor->schedule();
   }
 
   // Check if a child was added or removed.
   int lastchildidx = this->getNumChildren() - 1;
-  if (this->counter->max.getValue() != lastchildidx) {
+
+  if (pimpl->counter->max.getValue() != lastchildidx) {
     // Wrap to avoid recursive invocation.
-    this->counter->enableNotify(FALSE);
+    pimpl->counter->enableNotify(FALSE);
 
     // Note that if we have one child, the counting should go from -1
     // to 0 (so the child is toggled on and off).
-    this->counter->min.setValue(lastchildidx > 0 ? 0 : SO_SWITCH_NONE);
-    this->counter->max.setValue(lastchildidx >= 0 ? lastchildidx : SO_SWITCH_NONE);
+    pimpl->counter->min.setValue(lastchildidx > 0 ? 0 : SO_SWITCH_NONE);
+    pimpl->counter->max.setValue(lastchildidx >= 0 ? lastchildidx : SO_SWITCH_NONE);
 
     // To avoid SoSwitch getting an out-of-range whichChild value, in
     // case whichChild was at the end.
     if (lastchildidx < this->whichChild.getValue()) {
-      this->counter->reset.setValue(lastchildidx);
+      pimpl->counter->reset.setValue(lastchildidx);
       this->whichChild.setDirty(TRUE); // Force evaluate() on the field.
     }
-
-    this->counter->enableNotify(TRUE);
+    pimpl->counter->enableNotify(TRUE);
   }
-
+  
   inherited::notify(nl);
 }
 
@@ -202,10 +254,13 @@ SoBlinker::copy(SbBool copyconnections) const
 void
 SoBlinker::deconnectInternalEngine(void)
 {
-  this->whichChild.disconnect(&this->counter->output);
-  this->counter->on.disconnect(&this->on);
-  this->counter->on = FALSE;
-  this->counter->frequency.disconnect(&this->speed);
+  // Do this first, to avoid field being set due to subsequent engine
+  // input value change.
+  this->whichChild.disconnect(&PRIVATE(this)->counter->output);
+
+  PRIVATE(this)->counter->on.disconnect(&this->on);
+  PRIVATE(this)->counter->on = FALSE;
+  PRIVATE(this)->counter->frequency.disconnect(&this->speed);
 }
 
 
@@ -213,7 +268,10 @@ SoBlinker::deconnectInternalEngine(void)
 void
 SoBlinker::reconnectInternalEngine(void)
 {
-  this->counter->frequency.connectFrom(&this->speed);
-  this->counter->on.connectFrom(&this->on);
-  this->whichChild.connectFrom(&this->counter->output);
+  PRIVATE(this)->counter->frequency.connectFrom(&this->speed);
+  PRIVATE(this)->counter->on.connectFrom(&this->on);
+
+  this->whichChild.connectFrom(&PRIVATE(this)->counter->output, TRUE);
 }
+
+#undef PRIVATE
