@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -59,6 +59,8 @@
 #include <Inventor/nodes/SoSubNodeP.h>
 
 #include <Inventor/actions/SoGLRenderAction.h>
+#include <Inventor/actions/SoCallbackAction.h>
+#include <Inventor/actions/SoRayPickAction.h>
 #include <Inventor/elements/SoBumpMapCoordinateElement.h>
 
 
@@ -98,13 +100,16 @@ SoBumpMapCoordinate::initClass(void)
   SO_NODE_INTERNAL_INIT_CLASS(SoBumpMapCoordinate, SO_FROM_COIN_2_2);
 
   SO_ENABLE(SoGLRenderAction, SoBumpMapCoordinateElement);
+  SO_ENABLE(SoCallbackAction, SoBumpMapCoordinateElement);
+  SO_ENABLE(SoPickAction, SoBumpMapCoordinateElement);
 }
 
 // Doc from superclass.
 void
 SoBumpMapCoordinate::doAction(SoAction * action)
 {
-  // only SoGLRenderAction has activated the element for now
+  SoBumpMapCoordinateElement::set(action->getState(), this,
+                                  point.getNum(), point.getValues(0));
 }
 
 // Doc from superclass.
