@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -107,6 +107,8 @@ SoBumpMapTransform::initClass(void)
   SO_NODE_INTERNAL_INIT_CLASS(SoBumpMapTransform, SO_FROM_COIN_2_2);
 
   SO_ENABLE(SoGLRenderAction, SoBumpMapMatrixElement);
+  SO_ENABLE(SoCallbackAction, SoBumpMapMatrixElement);
+  SO_ENABLE(SoPickAction, SoBumpMapMatrixElement);
 }
 
 
@@ -130,7 +132,7 @@ SoBumpMapTransform::doAction(SoAction *action)
 void
 SoBumpMapTransform::callback(SoCallbackAction *action)
 {
-  // do nothing
+  SoBumpMapTransform::doAction(action);
 }
 
 // Documented in superclass.
@@ -144,7 +146,7 @@ SoBumpMapTransform::getMatrix(SoGetMatrixAction * action)
 void
 SoBumpMapTransform::pick(SoPickAction * action)
 {
-  // do nothing
+  SoBumpMapTransform::doAction(action);
 }
 
 //
