@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -41,6 +41,8 @@
   \endcode
 */
 
+// *************************************************************************
+
 #include <Inventor/nodes/SoCallback.h>
 #include <Inventor/nodes/SoSubNodeP.h>
 #include <Inventor/actions/SoActions.h> // SoCallback uses all of them.
@@ -49,13 +51,19 @@
 #include <Inventor/elements/SoGLTexture3EnabledElement.h>
 #include <Inventor/elements/SoShapeStyleElement.h>
 
+// *************************************************************************
+
 /*!
   \typedef void SoCallbackCB(void * userdata, SoAction * action)
   Signature that callback functions need to have.
 */
 
+// *************************************************************************
+
 SO_NODE_SOURCE(SoCallback);
 
+// *************************************************************************
+
 /*!
   Constructor.
 */
@@ -99,6 +107,38 @@ SoCallback::initClass(void)
   }
 
   \endcode
+
+  Please note that SoCallback nodes under a Separator might be
+  included in a cache. Cached nodes are not traversed, and you'll not
+  receive any callbacks. If you want to make sure that your callback
+  is called every time the scene graph is rendered, you must
+  invalidate the render cache in your callback:
+
+  \code
+  
+  void mycallback(void * userdata, SoAction * action)
+  {
+    if (action->isOfType(SoGLRenderAction::getClassTypeId())) {
+      SoCacheElement::invalidate(action->getState());
+    }
+  }
+
+  \endcode
+
+  If you want to invalidate all caches (for instance also the bounding
+  box cache), you can do this in your callback:
+
+  \code
+  
+  void mycallback(void * userdata, SoAction * action)
+  {
+    SoState * state = action->getState();
+    if (state->isElementEnabled(SoCacheElement::getClassStackIndex())) {
+      SoCacheElement::invalidate(state);
+    }
+  }
+
+  \endcode
 */
 void
 SoCallback::setCallback(SoCallbackCB * function, void * userdata)
@@ -125,6 +165,11 @@ SoCallback::callback(SoCallbackAction * action)
 void
 SoCallback::GLRender(SoGLRenderAction * action)
 {
+  // FIXME: we've had a user report which indicates that SGI Inventor
+  // may also cache OpenGL calls made from within a callback into
+  // renderlists. Investigate, and consider whether or not we should
+  // follow suit. 20051110 mortene.
+
   SoCallback::doAction(action);
 }
 
