@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -403,7 +403,13 @@ void
 SoCamera::pointAt(const SbVec3f & targetpoint)
 {
   SbVec3f dir = targetpoint - this->position.getValue();
-  if (dir.normalize() == 0.0f) return;
+  if (dir.normalize() == 0.0f) {
+#if COIN_DEBUG
+    SoDebugError::postInfo("SoCamera::pointAt",
+                           "targetpoint == camera position.");
+#endif // debug
+    return;
+  }
 
   SbVec3f up(0.0f, 1.0f, 0.0f);
 
@@ -422,7 +428,13 @@ void
 SoCamera::pointAt(const SbVec3f & targetpoint, const SbVec3f & upvector)
 {
   SbVec3f dir = targetpoint - this->position.getValue();
-  if (dir.normalize() == 0.0f) return;
+  if (dir.normalize() == 0.0f) {
+#if COIN_DEBUG
+    SoDebugError::postInfo("SoCamera::pointAt",
+                           "targetpoint == camera position.");
+#endif // debug
+    return;
+  }
   this->lookAt(dir, upvector);
 }
 
@@ -552,7 +564,7 @@ SoCamera::GLRender(SoGLRenderAction * action)
       float offset = this->stereoadjustment * 0.5f;
       if (this->stereomode == LEFT_VIEW) offset = -offset;
       SbVec3f r = vv.getProjectionDirection().cross(vv.getViewUp());
-      r.normalize();
+      (void) r.normalize();
 
       // get the current camera transformation/size
       vv.getMatrices(affine, proj);
@@ -980,6 +992,33 @@ SoCamera::getStereoMode(void) const
 /*!
   Sets the stereo adjustment. This is the distance between the left
   and right "eye" when doing stereo rendering.
+
+  When doing stereo rendering, Coin will render two views, one for the
+  left eye, and one for the right eye. The stereo adjustment is, a bit
+  simplified, how much the camera is translated along the local X-axis
+  between the left and the right view.
+
+  The default distance is 0.1, which is chosen since it's the
+  approximate distance between the human eyes.
+
+  To create a nice looking and visible stereo effect, the application
+  programmer will often have to adjust this value. If all you want to
+  do is examine simple stand-alone 3D objects, it is possible to
+  calculate a stereo offset based on the bounding box of the 3D model
+  (or scale the model down to an appropriate size).
+  
+  However, if you have a large scene, where you want to fly around in             
+  the scene, and see stereo on different objects as you approach them,            
+  you can't calculate the stereo offset based on the bounding box                 
+  of the scene, but rather use a stereo offset based on the scale of              
+  the individual objects/details you want to examine. 
+
+  Please note that it's important to set a sensible focal distance
+  when doing stereo rendering. See setBalanceAdjustment() for
+  information about how the focal distance affects the stereo
+  rendering.
+
+  \sa setBalanceAdjustment() 
 */
 void
 SoCamera::setStereoAdjustment(float adjustment)
@@ -1000,11 +1039,14 @@ SoCamera::getStereoAdjustment(void) const
 
 /*!
   Sets the stereo balance adjustment. This is a factor that enables you to
-  move the zero parallax plane. Objects in front of the zero parallax plane
-  appears to be in front of the screen.
+  move the zero parallax plane. Geometry in front of the zero parallax plane
+  will appear to be in front of the screen.
 
-  The default value is 1.0, and the zero parallax plane is then at the focal
-  point (see SoCamera::focalDistance).
+  The balance adjustment is multiplied with the focal distance to find
+  the zero parallax plane. The default value is 1.0, and the zero
+  parallax plane is then at the focal point.
+
+  \sa SoCamera::focalDistance
 */
 void
 SoCamera::setBalanceAdjustment(float adjustment)
@@ -1036,8 +1078,17 @@ SoCamera::lookAt(const SbVec3f & dir, const SbVec3f & up)
   y = z.cross(x);
 
   // normalize x and y to create an orthonormal coord system
-  y.normalize();
-  x.normalize();
+  if ((y.normalize() == 0.0f) ||
+      (x.normalize() == 0.0f)) {
+#if COIN_DEBUG
+    SoDebugError::postInfo("SoCamera::lookAt",
+                           "Unable to create a rotation matrix "
+                           "(dir = %g %g %g, up = %g %g %g)\n",
+                           dir[0], dir[1], dir[2],
+                           up[0], up[1], up[2]);
+#endif // debug
+    return;
+  }
 
   // create a rotation matrix
   SbMatrix rot = SbMatrix::identity();
