@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -53,13 +53,23 @@
 #include <Inventor/actions/SoGetPrimitiveCountAction.h>
 #include <Inventor/actions/SoPickAction.h>
 #include <Inventor/elements/SoGLCoordinateElement.h>
-
+#include <Inventor/elements/SoGLVBOElement.h>
+#include "../misc/SoVBO.h"
+#include "../misc/PointerMap.h"
 
 /*!
   \var SoMFVec3f SoCoordinate3::point
   Coordinate set of 3D points.
 */
 
+class SoCoordinate3P {
+ public:
+  SoCoordinate3P() : vbo(NULL) { }
+  ~SoCoordinate3P() { delete this->vbo; }
+  SoVBO * vbo;
+};
+
+#define PRIVATE(obj) ((SoCoordinate3P*) PointerMap::get(obj))
 
 // *************************************************************************
 
@@ -70,6 +80,7 @@ SO_NODE_SOURCE(SoCoordinate3);
 */
 SoCoordinate3::SoCoordinate3(void)
 {
+  PointerMap::add(this, new SoCoordinate3P);
   SO_NODE_INTERNAL_CONSTRUCTOR(SoCoordinate3);
 
   SO_NODE_ADD_FIELD(point, (0.0f, 0.0f, 0.0f));
@@ -80,6 +91,9 @@ SoCoordinate3::SoCoordinate3(void)
 */
 SoCoordinate3::~SoCoordinate3()
 {
+  SoCoordinate3P * pimpl = PRIVATE(this);
+  PointerMap::remove(this);
+  delete pimpl;  
 }
 
 // Doc from superclass.
@@ -90,6 +104,9 @@ SoCoordinate3::initClass(void)
 
   SO_ENABLE(SoGetBoundingBoxAction, SoCoordinateElement);
   SO_ENABLE(SoGLRenderAction, SoGLCoordinateElement);
+#ifdef COIN_NEXT_MINOR
+  SO_ENABLE(SoGLRenderAction, SoGLVBOElement);
+#endif // COIN_NEXT_MINOR
   SO_ENABLE(SoPickAction, SoCoordinateElement);
   SO_ENABLE(SoCallbackAction, SoCoordinateElement);
   SO_ENABLE(SoGetPrimitiveCountAction, SoCoordinateElement);
@@ -108,6 +125,37 @@ void
 SoCoordinate3::GLRender(SoGLRenderAction * action)
 {
   SoCoordinate3::doAction(action);
+#ifdef COIN_NEXT_MINOR
+  SoCoordinate3P * pimpl = PRIVATE(this);
+  SoState * state = action->getState();
+  const int num = this->point.getNum();
+  SbBool setvbo = FALSE;
+  SoBase::staticDataLock();
+  if (SoGLVBOElement::shouldCreateVBO(state, num)) {
+    SbBool dirty = FALSE;
+    setvbo = TRUE;
+    if (pimpl->vbo == NULL) {
+      pimpl->vbo = new SoVBO(GL_ARRAY_BUFFER, GL_STATIC_DRAW); 
+      dirty =  TRUE;
+    }
+    else if (pimpl->vbo->getBufferDataId() != this->getNodeId()) {
+      dirty = TRUE;
+    }
+    if (dirty) {
+      pimpl->vbo->setBufferData(this->point.getValues(0),
+                                num*sizeof(SbVec3f),
+                                this->getNodeId());
+    }
+  }
+  else if (pimpl->vbo && pimpl->vbo->getBufferDataId()) {
+    // clear buffers to deallocate VBO memory
+    pimpl->vbo->setBufferData(NULL, 0, 0);
+  }
+  SoBase::staticDataUnlock();
+  if (setvbo) {
+    SoGLVBOElement::setVertexVBO(state, pimpl->vbo);
+  }
+#endif // COIN_NEXT_MINOR
 }
 
 // Doc from superclass.
@@ -137,3 +185,5 @@ SoCoordinate3::getPrimitiveCount(SoGetPrimitiveCountAction * action)
 {
   SoCoordinate3::doAction(action);
 }
+
+#undef PRIVATE
