@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -58,6 +58,9 @@
 #include <Inventor/actions/SoGetPrimitiveCountAction.h>
 #include <Inventor/actions/SoPickAction.h>
 #include <Inventor/elements/SoGLCoordinateElement.h>
+#include <Inventor/elements/SoGLVBOElement.h>
+#include "../misc/SoVBO.h"
+#include "../misc/PointerMap.h"
 
 /*!
   \var SoMFVec4f SoCoordinate4::point
@@ -66,6 +69,15 @@
   coordinate plus normalization value).
 */
 
+class SoCoordinate4P {
+ public:
+  SoCoordinate4P() : vbo(NULL) { }
+
+  ~SoCoordinate4P() { delete this->vbo; }
+  SoVBO * vbo;
+};
+
+#define PRIVATE(obj) ((SoCoordinate4P*) PointerMap::get(obj))
 
 // *************************************************************************
 
@@ -76,6 +88,7 @@ SO_NODE_SOURCE(SoCoordinate4);
 */
 SoCoordinate4::SoCoordinate4(void)
 {
+  PointerMap::add(this, new SoCoordinate4P);
   SO_NODE_INTERNAL_CONSTRUCTOR(SoCoordinate4);
 
   SO_NODE_ADD_FIELD(point, (SbVec4f(0.0f, 0.0f, 0.0f, 1.0f)));
@@ -86,6 +99,9 @@ SoCoordinate4::SoCoordinate4(void)
 */
 SoCoordinate4::~SoCoordinate4()
 {
+  SoCoordinate4P * pimpl = PRIVATE(this);
+  PointerMap::remove(this);
+  delete pimpl;
 }
 
 // Doc from superclass.
@@ -121,6 +137,37 @@ void
 SoCoordinate4::GLRender(SoGLRenderAction * action)
 {
   SoCoordinate4::doAction(action);
+#ifdef COIN_NEXT_MINOR
+  SoCoordinate4P * pimpl = PRIVATE(this);
+  SoState * state = action->getState();
+  const int num = this->point.getNum();
+  SbBool setvbo = FALSE;
+  SoBase::staticDataLock();
+  if (SoGLVBOElement::shouldCreateVBO(state, num)) {
+    SbBool dirty = FALSE;
+    setvbo = TRUE;
+    if (pimpl->vbo == NULL) {
+      pimpl->vbo = new SoVBO(GL_ARRAY_BUFFER, GL_STATIC_DRAW);
+      dirty =  TRUE;
+    }
+    else if (pimpl->vbo->getBufferDataId() != this->getNodeId()) {
+      dirty = TRUE;
+    }
+    if (dirty) {
+      pimpl->vbo->setBufferData(this->point.getValues(0),
+                                num*sizeof(SbVec4f),
+                                this->getNodeId());
+    }
+  }
+  else if (pimpl->vbo && pimpl->vbo->getBufferDataId()) {
+    // clear buffers to deallocate VBO memory
+    pimpl->vbo->setBufferData(NULL, 0, 0);
+  }
+  SoBase::staticDataUnlock();
+  if (setvbo) {
+    SoGLVBOElement::setVertexVBO(state, pimpl->vbo);
+  }
+#endif // COIN_NEXT_MINOR
 }
 
 // Doc from superclass.
@@ -143,3 +190,5 @@ SoCoordinate4::getPrimitiveCount(SoGetPrimitiveCountAction * action)
 {
   SoCoordinate4::doAction(action);
 }
+
+#undef PRIVATE
