@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -50,28 +50,54 @@
         direction 0 0 -1
     }
   \endcode
+
+
+  A common thing to do with an SoDirectionalLight is to connect it to
+  a camera, so it works in the style of a head light to that camera.
+  This can easily be accomplished by linking an SoRotation::rotation
+  field, influencing the light, to the SoCamera::orientation
+  field. Here is a complete example iv-file demonstrating the
+  technique:
+
+  \verbatim
+  #Inventor V2.1 ascii
+  
+  DEF mycam PerspectiveCamera { }
+  
+  TransformSeparator {
+     SoRotation { rotation = USE mycam.orientation }
+     DirectionalLight { direction 0 0 -1 }
+  }
+  
+  Cube { }
+  \endverbatim
+
+  (The SoTransformSeparator is included to keep the effect of the
+  SoRotation node within a scope where it will only influence the
+  light, and not the geometry following the light in the scene graph.)
 */
 
+// *************************************************************************
+
 #include <Inventor/nodes/SoDirectionalLight.h>
-#include <Inventor/nodes/SoSubNodeP.h>
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif // HAVE_CONFIG_H
 
 #include <Inventor/SbColor4f.h>
 #include <Inventor/SbVec4f.h>
 #include <Inventor/actions/SoGLRenderAction.h>
 #include <Inventor/elements/SoEnvironmentElement.h>
 #include <Inventor/elements/SoGLLightIdElement.h>
+#include <Inventor/elements/SoLightElement.h>
 #include <Inventor/elements/SoModelMatrixElement.h>
 #include <Inventor/elements/SoViewingMatrixElement.h>
-#include <Inventor/elements/SoLightElement.h>
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif // HAVE_CONFIG_H
+#include <Inventor/errors/SoDebugError.h>
+#include <Inventor/nodes/SoSubNodeP.h>
 #include <Inventor/system/gl.h>
 
-#if COIN_DEBUG
-#include <Inventor/errors/SoDebugError.h>
-#endif // COIN_DEBUG
+// *************************************************************************
 
 /*!
   \var SoSFVec3f SoDirectionalLight::direction
@@ -80,11 +106,12 @@
   negative z-axis.
 */
 
-
 // *************************************************************************
 
 SO_NODE_SOURCE(SoDirectionalLight);
 
+// *************************************************************************
+
 /*!
   Constructor.
 */
@@ -109,6 +136,8 @@ SoDirectionalLight::initClass(void)
   SO_NODE_INTERNAL_INIT_CLASS(SoDirectionalLight, SO_FROM_INVENTOR_1|SoNode::VRML1);
 }
 
+// *************************************************************************
+
 // Doc from superclass.
 void
 SoDirectionalLight::GLRender(SoGLRenderAction * action)
@@ -125,7 +154,8 @@ SoDirectionalLight::GLRender(SoGLRenderAction * action)
 #endif // COIN_DEBUG
     return;
   }
-  
+
+
   SoLightElement::add(state, this, SoModelMatrixElement::get(state) * 
                       SoViewingMatrixElement::get(state));
   
@@ -143,8 +173,12 @@ SoDirectionalLight::GLRender(SoGLRenderAction * action)
 
   // GL directional light is specified towards light source
   SbVec3f dir = - this->direction.getValue();
-  dir.normalize();
-
+  if (dir.normalize() == 0.0f) {
+#if COIN_DEBUG
+    SoDebugError::postWarning("SoDirectionalLight::GLRender",
+                              "Direction is a null vector.");
+#endif // COIN_DEBUG
+  }
   // directional when w = 0.0
   SbVec4f dirvec(dir[0], dir[1], dir[2], 0.0f);
   glLightfv(light, GL_POSITION, dirvec.getValue());
@@ -155,3 +189,5 @@ SoDirectionalLight::GLRender(SoGLRenderAction * action)
   glLightf(light, GL_LINEAR_ATTENUATION, 0);
   glLightf(light, GL_QUADRATIC_ATTENUATION, 0);
 }
+
+// *************************************************************************
