@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -348,7 +348,7 @@ public:
     SbList<SbVec2s> coords;
     SoTimerSensor * updatetimer;
 
-    SelectionState::SelectionState(SoExtSelection * t)
+    SelectionState(SoExtSelection * t)
     {
       this->updatetimer = new SoTimerSensor(&SoExtSelectionP::timercallback, t);
       this->updatetimer->setBaseTime(SbTime::zero());
@@ -356,7 +356,7 @@ public:
       this->reset();
     }
 
-    SelectionState::~SelectionState()
+    ~SelectionState()
     {
       delete this->updatetimer;
     }
@@ -394,6 +394,7 @@ public:
     SbBool allhit;
     SbBool onlyrect;
     SbBool allshapes;
+    SbBool hasgeometry;
   } primcbdata_t;
   primcbdata_t primcbdata;
 
@@ -435,6 +436,7 @@ public:
   unsigned int drawcounter;
   SoPathList *visitedshapepaths;
   SbBool somefacesvisible;
+  SoPathList dummypathlist;
 
 private:
   SoExtSelection * master;
@@ -1217,6 +1219,88 @@ SoExtSelection::GLRenderBelowPath(SoGLRenderAction * action)
   state->pop();
 }
 
+#ifdef COIN_NEXT_MINOR
+/*!
+  Simulate lasso selection programmatically.
+
+  This function is currently just stubbed.
+*/
+
+void
+SoExtSelection::select(SoNode * root, int numcoords, SbVec3f * lasso, const SbViewportRegion & vp, SbBool shiftpolicy)
+{
+  // FIXME: Implement this for TGS compatibility...
+#ifndef COIN_BETA_VERSION
+#error start including this function only at the point of a minor release.
+#endif
+  COIN_STUB_ONCE();
+}
+
+/*!
+  Simulate lasso selection programmatically.
+
+  This function is currently just stubbed.
+*/
+
+void
+SoExtSelection::select(SoNode * root, int numcoords, SbVec2f * lasso, const SbViewportRegion & vp, SbBool shiftpolicy)
+{
+  // FIXME: Implement this for TGS compatibility...
+#ifndef COIN_BETA_VERSION
+#error start including this function only at the point of a minor release.
+#endif
+  COIN_STUB_ONCE();
+}
+
+/*!
+  Returns lasso coordinates in device coordinates.
+
+  This function is currently just stubbed.
+*/
+const SbVec2s *
+SoExtSelection::getLassoCoordsDC (int &numCoords)
+{
+  // FIXME: Implement this for TGS compatibility...
+#ifndef COIN_BETA_VERSION
+#error start including this function only at the point of a minor release.
+#endif
+  COIN_STUB_ONCE();
+  return NULL;
+}
+
+/*!
+  Returns lasso coordinates in world coordinates.
+
+  This function is currently just stubbed.
+*/
+const SbVec3f *
+SoExtSelection::getLassoCoordsWC (int &numCoords)
+{
+  // FIXME: Implement this for TGS compatibility...
+#ifndef COIN_BETA_VERSION
+#error start including this function only at the point of a minor release.
+#endif
+  COIN_STUB_ONCE();
+  return NULL;
+}
+
+/*!
+  Returns a pathlist containing selected objects.
+
+  This function is currently just stubbed.
+*/
+const SoPathList &
+SoExtSelection::getSelectionPathList () const
+{
+  // FIXME: Implement this for TGS compatibility...
+#ifndef COIN_BETA_VERSION
+#error start including this function only at the point of a minor release.
+#endif
+  COIN_STUB_ONCE();
+  return PRIVATE(this)->dummypathlist;
+}
+#endif // COIN_NEXT_MINOR
+
 /*!
 
   The lasso selection filter callback is called when a node is about
@@ -1333,6 +1417,11 @@ SoExtSelectionP::preShapeCallback(void *data, SoCallbackAction *action, const So
 
   PRIVATE(ext)->somefacesvisible = FALSE;
 
+  if (SoPickStyleElement::get(action->getState()) == 
+      SoPickStyleElement::UNPICKABLE) {
+    return SoCallbackAction::PRUNE;
+  }
+
   return PRIVATE(ext)->testShape(action, (const SoShape*) node);
 }
 
@@ -1346,10 +1435,13 @@ SoExtSelectionP::postShapeCallback(void *data, SoCallbackAction *action, const S
   SbBool hit = FALSE;
   switch (ext->lassoPolicy.getValue()) {
   case SoExtSelection::FULL:
-    hit = PRIVATE(ext)->primcbdata.allhit;
+    hit = (PRIVATE(ext)->primcbdata.allhit && 
+           PRIVATE(ext)->primcbdata.hasgeometry);
+    PRIVATE(ext)->primcbdata.allhit = FALSE;
     break;
   case SoExtSelection::PART:
     hit = PRIVATE(ext)->primcbdata.hit;
+    PRIVATE(ext)->primcbdata.hit = FALSE;
     break;
   default:
     break;
@@ -1402,25 +1494,11 @@ SoExtSelectionP::cameraCB(void * data,
   float right = float(rectbbox.getMax()[0] - org[0]) / float(siz[0]);
   float top = float(rectbbox.getMax()[1] - org[1]) / float(siz[1]);
 
-  // use an epsilon to avoid an empty view volume
-  const float EPS = 0.001f;
-
-  if (right - left < EPS) {
-    right = left + EPS;
-    if (right > 1.0f) {
-      right = 1.0f;
-      left = right - EPS;
-    }
+  if (vv.getDepth() > 0.0f && vv.getWidth() > 0.0f && vv.getHeight() > 0.0f &&
+      (right - left) > 0.0f && (top - bottom) > 0.0f) {
+    vv = vv.narrow(left, bottom, right, top);
+    SoCullElement::setViewVolume(state, vv);
   }
-  if (top - bottom < EPS) {
-    top = bottom + EPS;
-    if (top > 1.0f) {
-      top = 1.0f;
-      bottom = top - EPS;
-    }
-  }
-  vv = vv.narrow(left, bottom, right, top);
-  SoCullElement::setViewVolume(state, vv);
   return SoCallbackAction::CONTINUE;
 }
 
@@ -1585,6 +1663,7 @@ SoExtSelectionP::testPrimitives(SoCallbackAction * action,
   this->primcbdata.vpsize = SoViewportRegionElement::get(action->getState()).getViewportSizePixels();
   this->primcbdata.abort = FALSE;
   this->primcbdata.onlyrect = (this->runningselection.mode == SelectionState::LASSO);
+  this->primcbdata.hasgeometry = FALSE;
   // signal to callback action that we want to generate primitives for
   // this shape
   return SoCallbackAction::CONTINUE;
@@ -1629,6 +1708,7 @@ SoExtSelectionP::triangleCB(void * userData,
 
   SoExtSelectionP * thisp = ((SoExtSelection*)userData)->pimpl;
 
+  thisp->primcbdata.hasgeometry = TRUE;
   thisp->drawcallbackcounter++;
 
   if (!thisp->applyonlyonselectedtriangles) {
@@ -1844,6 +1924,8 @@ SoExtSelectionP::lineSegmentCB(void *userData,
                                const SoPrimitiveVertex * v2)
 {
   SoExtSelectionP * thisp = ((SoExtSelection*)userData)->pimpl;
+
+  thisp->primcbdata.hasgeometry = TRUE;
   thisp->drawcallbackcounter++;
 
   if (!thisp->applyonlyonselectedtriangles) {
@@ -2010,6 +2092,8 @@ SoExtSelectionP::pointCB(void *userData,
                          const SoPrimitiveVertex * v)
 {
   SoExtSelectionP * thisp = ((SoExtSelection*)userData)->pimpl;
+ 
+  thisp->primcbdata.hasgeometry = TRUE;
   thisp->drawcallbackcounter++;
 
   if (!thisp->applyonlyonselectedtriangles) {
