@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -174,9 +174,11 @@ SoLOD::commonConstructor(void)
   SO_NODE_ADD_FIELD(center, (SbVec3f(0, 0, 0)));
   SO_NODE_ADD_FIELD(range, (0.0f));
 
-  // Make multivalue field empty.
+  // Make multivalue field empty, as that is the default.
   this->range.setNum(0);
-
+  // So it's not written in its default state on SoWriteAction
+  // traversal.
+  this->range.setDefault(TRUE);
 }
 
 /*!
