@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -30,13 +30,31 @@
   geometry in the scene graph will use values from the material "pool"
   of the traversal state set up from nodes of this type.
 
+  For detailed information on the various components, see the OpenGL
+  color model, presented in the chapter "Colors and Coloring" (chapter
+  2.13 in the OpenGL 1.4 specification).
+
   Note that values from a material node will \e replace the previous
-  values from the traversal state, they will \e not accumulate.
+  values from the traversal state, they will \e not accumulate. That's
+  the case even when e.g. material changes are \e implicit in an
+  iv-file, as illustrated by the following example:
 
-  See also the information on the OpenGL color model, presented in the
-  chapter "Colors and Coloring" (chapter 2.13 in the OpenGL 1.4
-  specification).
+  \verbatim
+  #Inventor V2.1 ascii
+  
+  Material { ambientColor 1 0 0 }
+  Cone { }
+  
+  Translation { translation 5 0 0 }
+  
+  Material { }
+  Sphere { }
+  \endverbatim
 
+  (The SoSphere will not "inherit" the SoMaterial::ambientColor from
+  the first SoMaterial node, even though it is not explicitly set in
+  the second material node. The default value of
+  SoMaterial::ambientColor will be used.)
 
   Note that nodes imported as part of a VRML V1.0 file has a special
   case, where the fields SoMaterial::ambientColor,
@@ -131,6 +149,8 @@
 #include <Inventor/elements/SoShininessElement.h>
 #include <Inventor/elements/SoTransparencyElement.h>
 #include <Inventor/elements/SoLightModelElement.h>
+#include <Inventor/elements/SoGLVBOElement.h>
+#include "../misc/SoVBO.h"
 #include <Inventor/errors/SoDebugError.h>
 #include <Inventor/C/tidbits.h>
 #include <stdlib.h>
@@ -223,9 +243,13 @@
 
 class SoMaterialP {
 public:
+  SoMaterialP() : vbo(NULL) { }
+  ~SoMaterialP() { delete this->vbo; }
+  
   int materialtype;
   int transparencyflag;
   SoColorPacker colorpacker;
+  SoVBO * vbo;
 };
 
 #endif // DOXYGEN_SKIP_THIS
@@ -407,10 +431,11 @@ SoMaterial::doAction(SoAction * action)
     }
 #endif // COIN_DEBUG
 
+    const int numtransp = this->transparency.getNum();
     SoLazyElement::setMaterials(state, this, bitmask, 
                                 &THIS->colorpacker,
                                 diffuseptr, numdiffuse, 
-                                this->transparency.getValues(0), this->transparency.getNum(),
+                                this->transparency.getValues(0), numtransp,
                                 bitmask & SoLazyElement::AMBIENT_MASK ? 
                                 this->ambientColor[0] : dummycolor,
                                 bitmask & SoLazyElement::EMISSIVE_MASK ? 
@@ -420,6 +445,27 @@ SoMaterial::doAction(SoAction * action)
                                 bitmask & SoLazyElement::SHININESS_MASK ?
                                 SbClamp(this->shininess[0], 0.0f, 1.0f) : dummyval,
                                 istransparent);
+#ifdef COIN_NEXT_MINOR
+    if (state->isElementEnabled(SoGLVBOElement::getClassStackIndex())) {
+      SoBase::staticDataLock();
+      SbBool setvbo = FALSE;
+      if (SoGLVBOElement::shouldCreateVBO(state, numdiffuse)) {
+        setvbo = TRUE;
+        if (THIS->vbo == NULL) {
+          THIS->vbo = new SoVBO(GL_ARRAY_BUFFER, GL_STATIC_DRAW);
+        }
+      }
+      else if (THIS->vbo) {
+        THIS->vbo->setBufferData(NULL, 0, 0);
+      }
+      // don't fill in any data in the VBO. Data will be filled in
+      // using the ColorPacker right before the VBO is used
+      SoBase::staticDataUnlock();
+      if (setvbo) {
+        SoGLVBOElement::setColorVBO(state, THIS->vbo);
+      }
+    }
+#endif // COIN_NEXT_MINOR
   }
 }
 
