@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -193,6 +193,8 @@
 #include <Inventor/nodes/SoSubNodeP.h>
 #include <Inventor/nodes/SoNodes.h>
 #include <Inventor/nodes/SoUnknownNode.h>
+#include <Inventor/engines/SoNodeEngine.h>
+#include <Inventor/lists/SoEngineOutputList.h>
 
 #include <Inventor/system/gl.h> // glGetError
 
@@ -499,7 +501,7 @@ SoNode::initClass(void)
 
   // initialize the compatibility dict
   compatibility_dict = new Int16ToUInt32Map;
-  coin_atexit((coin_atexit_f*)SoNode::cleanupClass, 0);
+  coin_atexit((coin_atexit_f*)SoNode::cleanupClass, CC_ATEXIT_NORMAL);
 
   SoNode::setCompatibilityTypes(SoNode::getClassTypeId(),
                                 SO_FROM_INVENTOR_1);
@@ -1187,13 +1189,42 @@ SoNode::write(SoWriteAction * action)
 {
   SoOutput * out = action->getOutput();
 
+  SoNode * node = this;
+
+  SoProtoInstance * proto = SoProtoInstance::findProtoInstance(this);
+  if (proto) { node = proto; }
+
   if (out->getStage() == SoOutput::COUNT_REFS) {
-    this->addWriteReference(out, FALSE);
+    node->addWriteReference(out, FALSE);
   }
   else if (out->getStage() == SoOutput::WRITE) {
-    if (this->writeHeader(out, FALSE, FALSE)) return;
-    this->getFieldData()->write(out, this);
-    this->writeFooter(out);
+    if (node->writeHeader(out, FALSE, FALSE)) return;
+
+    // check for special case where we actually have to write out an
+    // SoEngineOutput "field". An engine output might be connected via
+    // an IS reference in a PROTO, and we then need to write back this
+    // IS reference when exporting the VRML file.
+    SoProto * proto = out->getCurrentProto();    
+    if (proto && node->isOfType(SoNodeEngine::getClassTypeId())) {
+      SoEngineOutputList l;
+      const int num = ((SoNodeEngine*)node)->getOutputs(l);
+      
+      for (int i = 0; i < num; i++) {
+        SbName name;
+        if (((SoNodeEngine*)node)->getOutputName(l[i], name)) {
+          SbName pname = proto->findISReference(node, name);
+          if (pname.getLength()) {
+            out->indent();
+            out->write(name.getString());
+            out->write(" IS ");
+            out->write(pname.getString());
+            out->write("\n");
+          }
+        }
+      }
+    }
+    node->getFieldData()->write(out, node);
+    node->writeFooter(out);
   }
   else assert(0 && "unknown stage");
 }
@@ -1274,13 +1305,7 @@ SoNode::getNodeId(void) const
   return this->uniqueId;
 }
 
-// Note that this documentation will also be used for all subclasses
-// which reimplements the method, so keep the doc "generic enough".
-/*!
-  This method is called from write() if the actual writing pass of the
-  write action is taking place. It dumps the node to the given \a out
-  output stream.
-*/
+// Documentation in superclass.
 void
 SoNode::writeInstance(SoOutput * out)
 {
@@ -1496,7 +1521,11 @@ init_action_methods(void)
   SoRayPickAction::addMethod(SoLOD::getClassTypeId(), SoNode::rayPickS);
   SoRayPickAction::addMethod(SoLevelOfDetail::getClassTypeId(), SoNode::rayPickS);
   SoRayPickAction::addMethod(SoShape::getClassTypeId(), SoNode::rayPickS);
-
+  SoRayPickAction::addMethod(SoTexture2::getClassTypeId(), SoNode::rayPickS);
+  SoRayPickAction::addMethod(SoBumpMap::getClassTypeId(), SoNode::rayPickS);
+  SoRayPickAction::addMethod(SoImage::getClassTypeId(), SoNode::rayPickS);
+  SoRayPickAction::addMethod(SoSceneTexture2::getClassTypeId(), SoNode::rayPickS);
+ 
   SoSearchAction::addMethod(SoNode::getClassTypeId(), SoNode::searchS);
   SoWriteAction::addMethod(SoNode::getClassTypeId(), SoNode::writeS);
 
