@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -52,6 +52,9 @@
 #include <Inventor/actions/SoPickAction.h>
 #include <Inventor/elements/SoNormalElement.h>
 #include <Inventor/elements/SoOverrideElement.h>
+#include <Inventor/elements/SoGLVBOElement.h>
+#include "../misc/SoVBO.h"
+#include "../misc/PointerMap.h"
 
 // *************************************************************************
 
@@ -62,6 +65,16 @@
 
 // *************************************************************************
 
+class SoNormalP {
+ public:
+  SoNormalP() : vbo(NULL) { }
+  ~SoNormalP() { delete this->vbo; }
+
+  SoVBO * vbo;
+};
+
+#define PRIVATE(obj) ((SoNormalP*) PointerMap::get(obj))
+
 SO_NODE_SOURCE(SoNormal);
 
 /*!
@@ -69,6 +82,7 @@ SO_NODE_SOURCE(SoNormal);
 */
 SoNormal::SoNormal(void)
 {
+  PointerMap::add(this, new SoNormalP);
   SO_NODE_INTERNAL_CONSTRUCTOR(SoNormal);
 
   SO_NODE_ADD_FIELD(vector, (NULL));
@@ -79,6 +93,9 @@ SoNormal::SoNormal(void)
 */
 SoNormal::~SoNormal()
 {
+  SoNormalP * pimpl = PRIVATE(this);
+  PointerMap::remove(this);
+  delete pimpl;
 }
 
 // Doc in superclass.
@@ -103,6 +120,39 @@ SoNormal::GLRender(SoGLRenderAction * action)
   // SoGLNormalizeElement to optimize rendering (pederb)
   //
   SoNormal::doAction(action);
+
+#ifdef COIN_NEXT_MINOR
+  SoNormalP * pimpl = PRIVATE(this);
+  SoState * state = action->getState();
+  
+  SoBase::staticDataLock();
+  SbBool setvbo = FALSE;
+  const int num = this->vector.getNum();
+  if (SoGLVBOElement::shouldCreateVBO(state, num)) {
+    setvbo = TRUE;
+    SbBool dirty = FALSE;
+    if (pimpl->vbo == NULL) {
+      pimpl->vbo = new SoVBO(GL_ARRAY_BUFFER, GL_STATIC_DRAW); 
+      dirty =  TRUE;
+    }
+    else if (pimpl->vbo->getBufferDataId() != this->getNodeId()) {
+      dirty = TRUE;
+    }
+    if (dirty) {
+      pimpl->vbo->setBufferData(this->vector.getValues(0),
+                                num*sizeof(SbVec3f),
+                                this->getNodeId());
+    }
+  }
+  else if (pimpl->vbo && pimpl->vbo->getBufferDataId()) {
+    // clear buffers to deallocate VBO memory
+    pimpl->vbo->setBufferData(NULL, 0, 0);
+  }
+  SoBase::staticDataUnlock();
+  if (setvbo) {
+    SoGLVBOElement::setNormalVBO(state, pimpl->vbo);
+  }
+#endif // COIN_NEXT_MINOR
 }
 
 // Doc in superclass.
@@ -140,3 +190,5 @@ SoNormal::getPrimitiveCount(SoGetPrimitiveCountAction * action)
 {
   SoNormal::doAction(action);
 }
+
+#undef PRIVATE
