@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -103,7 +103,7 @@ SoPendulum::SoPendulum(void)
   this->interpolator->input0.connectFrom(&this->rotation0);
   this->interpolator->input1.connectFrom(&this->rotation1);
   this->interpolator->alpha.connectFrom(&this->calculator->oa);
-  this->rotation.connectFrom(&this->interpolator->output);
+  this->rotation.connectFrom(&this->interpolator->output, TRUE);
 }
 
 /*!
@@ -164,30 +164,31 @@ SoPendulum::copy(SbBool copyconnections) const
   return cp;
 }
 
-// Remove connections to and from internal engine.
+// Remove connections to and from internal engines.
 void
 SoPendulum::deconnectInternalEngine(void)
 {
-  // Decouple connections to/from internal engine to avoid them being
-  // written.
+  // Do this first, to avoid field being set due to subsequent engine
+  // input value change.
+  this->rotation.disconnect(&this->interpolator->output);
+
   this->timer->on.disconnect(&this->on);
   this->timer->on = FALSE;
   this->calculator->b.disconnect(&this->speed);
   this->interpolator->input0.disconnect(&this->rotation0);
   this->interpolator->input1.disconnect(&this->rotation1);
-  this->rotation.disconnect(&this->interpolator->output);
 }
 
 
-// Reset connections to and from internal engine.
+// Reenable all connections to/from internal engines.
 void
 SoPendulum::reconnectInternalEngine(void)
 {
-  // Reenable all connections to/from internal engine.
   this->timer->on.connectFrom(&this->on);
   this->calculator->b.connectFrom(&this->speed);
   this->interpolator->input0.connectFrom(&this->rotation0);
   this->interpolator->input1.connectFrom(&this->rotation1);
   this->interpolator->alpha.connectFrom(&this->calculator->oa);
-  this->rotation.connectFrom(&this->interpolator->output);
+
+  this->rotation.connectFrom(&this->interpolator->output, TRUE);
 }
