@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -67,34 +67,55 @@
 
   \endverbatim
 
+  One word of notice with regard to the above quote from the OpenGL
+  documentation: it doesn't really make sense to set "factor" and
+  "units" to values with different signs, i.e. "factor" to a negative
+  value and "units" to a positive value, or vice versa.
 
-  Below is a simple usage example:
+  The pixels would then be "pushed back" in z-order by one part of the
+  equation, but at the same time be "pushed forward" by the other part
+  of the equation. This would most likely give very inconsistent
+  results, but which may at first look ok.
+
+  We mention this potential for making a mistake, as it seems to be a
+  quite common error.
+
+
+  Below is a simple, correct usage example:
 
   \verbatim
   #Inventor V2.1 ascii
   
   Separator {
      Coordinate3 { point [ -1 -1 0, 1 -1 0, 1 1 0, -1 1 0 ] }
-  
-     BaseColor { rgb 1 1 0 }
-     IndexedLineSet { coordIndex [ 0, 1, 2, 3, 0, 2, -1, 1, 3 -1 ] }
-  
-     PolygonOffset {
-       styles FILLED
-       factor 1.0
-       units 1.0
+     
+     Separator {
+        BaseColor { rgb 1 1 0 }
+        # needs to draw polygons-as-line, and not "real" lines -- see
+        # documentation below on why this is so:
+        DrawStyle { style LINES }
+        # draw two triangles, to get a line crossing the face of the
+        # polygon:
+        IndexedFaceSet { coordIndex [ 0, 1, 2, -1, 0, 2, 3 -1 ] }
      }
-  
+     
+     PolygonOffset {
+        styles FILLED
+        factor 1.0
+        units 1.0
+     }
+     
      BaseColor { rgb 0 0.5 0 }
      FaceSet { numVertices [ 4 ] }
   }
   \endverbatim
 
-  Without the polygonoffset node in the above example, the line will
-  look irregularly stippled, as parts of it will show through the
-  faceset, others not. This happen on seemingly random parts, as the
-  z-buffer floating point calculations will be fickle with regard to
-  whether or not the polygon or the line will be closer to the camera.
+  Without the polygonoffset node in the above example, the lines may
+  look irregularly stippled with some graphics card drivers, as parts
+  of it will show through the faceset, others not. This happen on
+  seemingly random parts, as the z-buffer floating point calculations
+  will be fickle with regard to whether or not the polygon or the line
+  will be closer to the camera.
 
   See the API documentation of the SoPolygonOffset::styles field below
   for a discussion of one important limitation of OpenGL's Z-buffer
