@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -149,6 +149,7 @@
 #include <Inventor/lists/SoCallbackList.h>
 #include <Inventor/SoPickedPoint.h>
 #include <Inventor/events/SoMouseButtonEvent.h>
+#include <Inventor/C/tidbitsp.h>
 
 // *************************************************************************
 
@@ -901,7 +902,7 @@ SoSelection::searchNode(SoNode * node) const
   if (soselection_searchAction == NULL) {
     soselection_searchAction = new SoSearchAction;
     soselection_searchAction->setInterest(SoSearchAction::FIRST);
-    coin_atexit((coin_atexit_f*) soselection_cleanup, 0);
+    coin_atexit((coin_atexit_f*) soselection_cleanup, CC_ATEXIT_NORMAL);
   }
   soselection_searchAction->setNode(node);
   soselection_searchAction->apply((SoNode *)this);
