@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -392,9 +392,12 @@ SoSeparator::commonConstructor(void)
   SO_NODE_SET_SF_ENUM_TYPE(renderCulling, CacheEnabled);
   SO_NODE_SET_SF_ENUM_TYPE(pickCulling, CacheEnabled);
 
+  static long int maxcaches = -1;
+  if (maxcaches == -1) {
+    maxcaches = -2; // so we don't request the envvar later if it is not set
   const char * maxcachesstr = coin_getenv("IV_SEPARATOR_MAX_CACHES");
   if (maxcachesstr) {
-    long int maxcaches = strtol(maxcachesstr, NULL, 10);
+      maxcaches = strtol(maxcachesstr, NULL, 10);
     if ((maxcaches == LONG_MIN) || (maxcaches == LONG_MAX) || (maxcaches < 0)) {
       SoDebugError::post("SoSeparator::commonConstructor",
                          "Environment variable IV_SEPARATOR_MAX_CACHES "
@@ -402,6 +405,7 @@ SoSeparator::commonConstructor(void)
     }
     else {
       SoSeparator::setNumRenderCaches(maxcaches);
+      }
     }
   }
 
@@ -440,6 +444,7 @@ SoSeparator::initClass(void)
 
   SO_ENABLE(SoGetBoundingBoxAction, SoCacheElement);
   SO_ENABLE(SoGLRenderAction, SoCacheElement);
+  SoSeparator::numrendercaches = 2;
 }
 
 // Doc from superclass.
