@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -76,8 +76,12 @@
   Toggles animation on or off. Defauls to \c TRUE.
 */
 
+// *************************************************************************
+
 SO_NODE_SOURCE(SoShuttle);
 
+// *************************************************************************
+
 /*!
   Constructor.
 */
@@ -104,7 +108,8 @@ SoShuttle::SoShuttle(void)
   this->interpolator->input0.connectFrom(&this->translation0);
   this->interpolator->input1.connectFrom(&this->translation1);
   this->interpolator->alpha.connectFrom(&this->calculator->oa);
-  this->translation.connectFrom(&this->interpolator->output);
+
+  this->translation.connectFrom(&this->interpolator->output, TRUE);
 }
 
 /*!
@@ -169,25 +174,26 @@ SoShuttle::copy(SbBool copyconnections) const
 void
 SoShuttle::deconnectInternalEngines(void)
 {
-  // Decouple connections to/from internal engines to avoid them being
-  // written.
+  // Do this first, to avoid field being set due to subsequent engine
+  // input value change.
+  this->translation.disconnect(&this->interpolator->output);
+
   this->timer->on.disconnect(&this->on);
   this->timer->on = FALSE;
   this->calculator->b.disconnect(&this->speed);
   this->interpolator->input0.disconnect(&this->translation0);
   this->interpolator->input1.disconnect(&this->translation1);
-  this->translation.disconnect(&this->interpolator->output);
 }
 
 
-// Reset connections to and from internal engines.
+// Reenable all connections to/from internal engines.
 void
 SoShuttle::reconnectInternalEngines(void)
 {
-  // Reenable all connections to/from internal engines.
   this->timer->on.connectFrom(&this->on);
   this->calculator->b.connectFrom(&this->speed);
   this->interpolator->input0.connectFrom(&this->translation0);
   this->interpolator->input1.connectFrom(&this->translation1);
-  this->translation.connectFrom(&this->interpolator->output);
+
+  this->translation.connectFrom(&this->interpolator->output, TRUE);
 }
