@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -69,6 +69,7 @@
 #include <Inventor/actions/SoCallbackAction.h>
 #include <Inventor/actions/SoGLRenderAction.h>
 #include <Inventor/elements/SoGLTexture3EnabledElement.h>
+#include <Inventor/elements/SoGLCacheContextElement.h>
 #include <Inventor/elements/SoGLTextureImageElement.h>
 #include <Inventor/elements/SoTextureQualityElement.h>
 #include <Inventor/elements/SoTextureOverrideElement.h>
@@ -79,6 +80,7 @@
 #include <Inventor/lists/SbStringList.h>
 #include <Inventor/errors/SoDebugError.h>
 #include <Inventor/SbImage.h>
+#include <Inventor/C/glue/gl.h>
 #include <assert.h>
 #include <string.h>
 
@@ -245,9 +247,24 @@ translateWrap(const SoTexture3::Wrap wrap)
 void
 SoTexture3::GLRender(SoGLRenderAction * action)
 {
-  // FIXME: consider sharing textures among contexts, pederb
   SoState * state = action->getState();
 
+  const cc_glglue * glue = cc_glglue_instance((uint32_t) SoGLCacheContextElement::get(state)); 
+  
+  if (!cc_glglue_has_3d_textures(glue)) {
+    static SbBool first = TRUE;
+    if (first) {
+      SoDebugError::postWarning("SoTexture3::GLRender",
+                                "The current OpenGL context does not support 3D textures "
+                                "(This warning message is only shown once, but "
+                                "there could be more cases of this in the "
+                                "scene graph.).");
+      first = FALSE;
+    }
+    return;
+  }
+
+
   if (SoTextureOverrideElement::getImageOverride(state))
     return;
 
