@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -105,9 +105,12 @@ Separator {
 #include <Inventor/elements/SoGLCacheContextElement.h>
 #include <Inventor/elements/SoGLMultiTextureCoordinateElement.h>
 #include <Inventor/elements/SoTextureUnitElement.h>
+#include <Inventor/elements/SoGLVBOElement.h>
 #include <Inventor/actions/SoCallbackAction.h>
 #include <Inventor/actions/SoPickAction.h>
 #include <Inventor/C/glue/gl.h>
+#include "../misc/SoVBO.h"
+#include "../misc/PointerMap.h"
 
 // *************************************************************************
 
@@ -118,16 +121,25 @@ Separator {
   empty set.
 
   Texture coordinates are usually specified in normalized coordinates,
-  ie in the range [0, 1]. (0, 0) is the lower left corner, while 
+  ie in the range [0, 1]. (0, 0) is the lower left corner, while
   (1, 1) is the upper right corner of the texture image. Coordinates
   outside the [0, 1] range can be used to repeat the texture across a
   surface.
 
-  \sa SoTexure2::wrapS, SoTexture2::wrapT 
+  \sa SoTexure2::wrapS, SoTexture2::wrapT
 */
 
 // *************************************************************************
 
+class SoTextureCoordinate2P {
+ public:
+  SoTextureCoordinate2P() : vbo(NULL) { }
+  ~SoTextureCoordinate2P() { delete this->vbo; }
+  SoVBO * vbo;
+};
+
+#define PRIVATE(obj) ((SoTextureCoordinate2P*) PointerMap::get(obj))
+
 SO_NODE_SOURCE(SoTextureCoordinate2);
 
 /*!
@@ -135,6 +147,7 @@ SO_NODE_SOURCE(SoTextureCoordinate2);
 */
 SoTextureCoordinate2::SoTextureCoordinate2(void)
 {
+  PointerMap::add(this, new SoTextureCoordinate2P);
   SO_NODE_INTERNAL_CONSTRUCTOR(SoTextureCoordinate2);
   SO_NODE_ADD_FIELD(point, (NULL));
 }
@@ -144,6 +157,9 @@ SoTextureCoordinate2::SoTextureCoordinate2(void)
 */
 SoTextureCoordinate2::~SoTextureCoordinate2()
 {
+  SoTextureCoordinate2P * pimpl = PRIVATE(this);
+  PointerMap::remove(this);
+  delete pimpl;  
 }
 
 // Documented in superclass.
@@ -154,25 +170,40 @@ SoTextureCoordinate2::initClass(void)
 
   SO_ENABLE(SoGLRenderAction, SoGLTextureCoordinateElement);
   SO_ENABLE(SoCallbackAction, SoTextureCoordinateElement);
+  SO_ENABLE(SoGLRenderAction, SoGLMultiTextureCoordinateElement);
+  SO_ENABLE(SoCallbackAction, SoMultiTextureCoordinateElement);
+
   SO_ENABLE(SoPickAction, SoTextureCoordinateElement);
+  SO_ENABLE(SoPickAction, SoMultiTextureCoordinateElement);
 }
 
 // Documented in superclass.
 void
 SoTextureCoordinate2::doAction(SoAction * action)
 {
-  SoTextureCoordinateElement::set2(action->getState(), this,
-                                   point.getNum(),
-                                   point.getValues(0));
+  SoState * state = action->getState();
+  int unit = SoTextureUnitElement::get(state);
+
+  if (unit == 0) {
+    SoTextureCoordinateElement::set2(action->getState(), this,
+                                     this->point.getNum(),
+                                     this->point.getValues(0));
+  }
+  else {
+    SoMultiTextureCoordinateElement::set2(action->getState(), this, unit,
+                                          this->point.getNum(),
+                                          this->point.getValues(0));
+  }
 }
 
 // Documented in superclass.
 void
 SoTextureCoordinate2::GLRender(SoGLRenderAction * action)
 {
+  SoTextureCoordinate2P * pimpl = PRIVATE(this);
   SoState * state = action->getState();
   int unit = SoTextureUnitElement::get(state);
-
+  
   if (unit == 0) {
     SoGLTextureCoordinateElement::setTexGen(action->getState(), this, NULL);
     SoTextureCoordinate2::doAction((SoAction *)action);
@@ -180,7 +211,7 @@ SoTextureCoordinate2::GLRender(SoGLRenderAction * action)
   else {
     const cc_glglue * glue = cc_glglue_instance(SoGLCacheContextElement::get(state));
     int maxunits = cc_glglue_max_texture_units(glue);
-    
+
     if (unit < maxunits) {
       SoGLMultiTextureCoordinateElement::setTexGen(action->getState(), this, unit, NULL);
       SoMultiTextureCoordinateElement::set2(action->getState(), this, unit,
@@ -188,6 +219,36 @@ SoTextureCoordinate2::GLRender(SoGLRenderAction * action)
                                             point.getValues(0));
     }
   }
+
+#ifdef COIN_NEXT_MINOR
+  SoBase::staticDataLock();
+  const int num = this->point.getNum();
+  SbBool setvbo = FALSE;
+  if (SoGLVBOElement::shouldCreateVBO(state, num)) {
+    setvbo = TRUE;
+    SbBool dirty = FALSE;
+    if (pimpl->vbo == NULL) {
+      pimpl->vbo = new SoVBO(GL_ARRAY_BUFFER, GL_STATIC_DRAW);
+      dirty =  TRUE;
+    }
+    else if (pimpl->vbo->getBufferDataId() != this->getNodeId()) {
+      dirty = TRUE;
+    }
+    if (dirty) {
+      pimpl->vbo->setBufferData(this->point.getValues(0),
+                                        num*sizeof(SbVec2f),
+                                        this->getNodeId());
+    }
+  }
+  else if (pimpl->vbo && pimpl->vbo->getBufferDataId()) {
+    // clear buffers to deallocate VBO memory
+    pimpl->vbo->setBufferData(NULL, 0, 0);
+  }
+  SoBase::staticDataUnlock();
+  if (setvbo) {
+    SoGLVBOElement::setVertexVBO(state, pimpl->vbo);
+  }
+#endif // COIN_NEXT_MINOR
 }
 
 // Documented in superclass.
@@ -203,3 +264,5 @@ SoTextureCoordinate2::pick(SoPickAction * action)
 {
   SoTextureCoordinate2::doAction((SoAction *)action);
 }
+
+#undef PRIVATE
