@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -60,6 +60,16 @@
 #include <Inventor/actions/SoGLRenderAction.h>
 #include <Inventor/elements/SoGLTextureCoordinateElement.h>
 #include <Inventor/actions/SoCallbackAction.h>
+#include <Inventor/elements/SoGLCacheContextElement.h>
+#include <Inventor/elements/SoGLMultiTextureCoordinateElement.h>
+#include <Inventor/elements/SoTextureUnitElement.h>
+#include <Inventor/elements/SoGLVBOElement.h>
+#include <Inventor/actions/SoPickAction.h>
+#include <Inventor/C/glue/gl.h>
+#include "../misc/SoVBO.h"
+#include "../misc/PointerMap.h"
+
+// *************************************************************************
 
 /*!
   \var SoMFVec3f SoTextureCoordinate3::point
@@ -76,6 +86,15 @@
 
 // *************************************************************************
 
+class SoTextureCoordinate3P {
+ public:
+  SoTextureCoordinate3P() : vbo(NULL) { }
+  ~SoTextureCoordinate3P() { delete this->vbo; }
+  SoVBO * vbo;
+};
+
+#define PRIVATE(obj) ((SoTextureCoordinate3P*) PointerMap::get(obj))
+
 SO_NODE_SOURCE(SoTextureCoordinate3);
 
 /*!
@@ -83,6 +102,7 @@ SO_NODE_SOURCE(SoTextureCoordinate3);
 */
 SoTextureCoordinate3::SoTextureCoordinate3(void)
 {
+  PointerMap::add(this, new SoTextureCoordinate3P);
   SO_NODE_INTERNAL_CONSTRUCTOR(SoTextureCoordinate3);
   SO_NODE_ADD_FIELD(point, (NULL));
 }
@@ -92,6 +112,9 @@ SoTextureCoordinate3::SoTextureCoordinate3(void)
 */
 SoTextureCoordinate3::~SoTextureCoordinate3()
 {
+  SoTextureCoordinate3P * pimpl = PRIVATE(this);
+  PointerMap::remove(this);
+  delete pimpl;  
 }
 
 // Documented in superclass.
@@ -101,7 +124,6 @@ SoTextureCoordinate3::initClass(void)
   SO_NODE_INTERNAL_INIT_CLASS(SoTextureCoordinate3, SO_FROM_INVENTOR_2_6|SO_FROM_COIN_2_0);
 
   SO_ENABLE(SoGLRenderAction, SoGLTextureCoordinateElement);
-
   SO_ENABLE(SoCallbackAction, SoTextureCoordinateElement);
 }
 
@@ -109,17 +131,74 @@ SoTextureCoordinate3::initClass(void)
 void
 SoTextureCoordinate3::doAction(SoAction * action)
 {
-  SoTextureCoordinateElement::set3(action->getState(), this,
-                                   point.getNum(),
-                                   point.getValues(0));
+  SoState * state = action->getState();
+  int unit = SoTextureUnitElement::get(state);
+  
+  if (unit == 0) {
+    SoTextureCoordinateElement::set3(action->getState(), this,
+                                     this->point.getNum(),
+                                     this->point.getValues(0));
+  }
+  else {
+    SoMultiTextureCoordinateElement::set3(action->getState(), this, unit,
+                                          this->point.getNum(),
+                                          this->point.getValues(0));
+  }
 }
 
 // Documented in superclass.
 void
 SoTextureCoordinate3::GLRender(SoGLRenderAction * action)
 {
-  SoGLTextureCoordinateElement::setTexGen(action->getState(), this, NULL);
-  SoTextureCoordinate3::doAction((SoAction *)action);
+  SoState * state = action->getState();
+  int unit = SoTextureUnitElement::get(state);
+  
+  if (unit == 0) {
+    SoGLTextureCoordinateElement::setTexGen(action->getState(), this, NULL);
+    SoTextureCoordinate3::doAction((SoAction *)action);
+  }
+  else {
+    const cc_glglue * glue = cc_glglue_instance(SoGLCacheContextElement::get(state));
+    int maxunits = cc_glglue_max_texture_units(glue);
+
+    if (unit < maxunits) {
+      SoGLMultiTextureCoordinateElement::setTexGen(action->getState(), this, unit, NULL);
+      SoMultiTextureCoordinateElement::set3(action->getState(), this, unit,
+                                            this->point.getNum(),
+                                            this->point.getValues(0));
+    }
+  }
+  
+#ifdef COIN_NEXT_MINOR
+  SoTextureCoordinate3P * pimpl = PRIVATE(this);
+  SoBase::staticDataLock();
+  const int num = this->point.getNum();
+  SbBool setvbo = FALSE;
+  if (SoGLVBOElement::shouldCreateVBO(state, num)) {
+    setvbo = TRUE;
+    SbBool dirty = FALSE;
+    if (pimpl->vbo == NULL) {
+      pimpl->vbo = new SoVBO(GL_ARRAY_BUFFER, GL_STATIC_DRAW); 
+      dirty =  TRUE;
+    }
+    else if (pimpl->vbo->getBufferDataId() != this->getNodeId()) {
+      dirty = TRUE;
+    }
+    if (dirty) {
+      pimpl->vbo->setBufferData(this->point.getValues(0),
+                                num*sizeof(SbVec3f),
+                                this->getNodeId());
+    }
+  }
+  else if (pimpl->vbo && pimpl->vbo->getBufferDataId()) {
+    // clear buffers to deallocate VBO memory
+    pimpl->vbo->setBufferData(NULL, 0, 0);
+  }
+  SoBase::staticDataUnlock();
+  if (setvbo) {
+    SoGLVBOElement::setVertexVBO(state, pimpl->vbo);
+  }
+#endif // COIN_NEXT_MINOR
 }
 
 // Documented in superclass.
@@ -135,3 +214,5 @@ SoTextureCoordinate3::pick(SoPickAction * action)
 {
   SoTextureCoordinate3::doAction((SoAction *)action);
 }
+
+#undef PRIVATE
