@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -123,6 +123,8 @@ SoTextureUnit::initClass(void)
   SO_NODE_INTERNAL_INIT_CLASS(SoTextureUnit, SO_FROM_COIN_2_2);
 
   SO_ENABLE(SoGLRenderAction, SoTextureUnitElement);
+  SO_ENABLE(SoPickAction, SoTextureUnitElement);
+  SO_ENABLE(SoCallbackAction, SoTextureUnitElement);
   SO_ENABLE(SoGLRenderAction, SoGLMultiTextureCoordinateElement);
   SO_ENABLE(SoGLRenderAction, SoGLMultiTextureImageElement);
   SO_ENABLE(SoGLRenderAction, SoGLMultiTextureEnabledElement);
@@ -167,20 +169,14 @@ SoTextureUnit::doAction(SoAction * action)
 void
 SoTextureUnit::callback(SoCallbackAction * action)
 {
-  // So far only SoGLRenderAction supports SoTextureUnitElement.  We
-  // may never support multiple texture units for SoCallbackAction,
-  // but we reimplement the method just in case
-  inherited::callback(action);
+  SoTextureUnit::doAction(action);
 }
 
 // Doc from superclass.
 void
 SoTextureUnit::pick(SoPickAction * action)
 {
-  // So far only SoGLRenderAction supports SoTextureUnitElement.  We
-  // may never support multiple texture units for SoPickAction, but we
-  // reimplement the method just in case
-  inherited::pick(action);
+  SoTextureUnit::doAction(action);
 }
 
 /*!
