@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -54,6 +54,9 @@
 #include <Inventor/actions/SoGLRenderAction.h>
 #include <Inventor/actions/SoGetMatrixAction.h>
 #include <Inventor/elements/SoModelMatrixElement.h>
+#if COIN_DEBUG
+#include <Inventor/errors/SoDebugError.h>
+#endif // COIN_DEBUG
 
 // *************************************************************************
 
@@ -145,10 +148,17 @@ SoTransform::pointAt(const SbVec3f & frompoint, const SbVec3f & topoint)
   
   this->translation = frompoint;  
   SbVec3f dir = topoint - frompoint;
-  dir.normalize();
-  
-  SbRotation rot(SbVec3f(0.0f, 0.0f, -1.0f), dir);
-  this->rotation = rot;
+  if (dir.normalize() != 0.0f) {
+    SbRotation rot(SbVec3f(0.0f, 0.0f, -1.0f), dir);
+    this->rotation = rot;
+  }
+#if COIN_DEBUG
+  else {
+    SoDebugError::postWarning("SoTransform::pointAt",
+                              "frompt == topoint");
+
+  }
+#endif // COIN_DEBUG
 }
 
 /*!
