@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -38,11 +38,13 @@
 #include <Inventor/actions/SoPickAction.h>
 #include <Inventor/actions/SoSearchAction.h>
 #include <Inventor/actions/SoWriteAction.h>
+#include <Inventor/errors/SoDebugError.h>
 #include <Inventor/errors/SoReadError.h>
 #include <Inventor/fields/SoSFNode.h>
 #include <Inventor/misc/SoChildList.h>
 #include <Inventor/nodes/SoGroup.h>
 #include "../io/SoWriterefCounter.h"
+#include "../io/SoInputP.h"
 
 // *************************************************************************
 
@@ -120,6 +122,13 @@ SoUnknownNode::initClass(void)
 SbBool
 SoUnknownNode::readInstance(SoInput * in, unsigned short flags)
 {
+  if (SoInputP::debug()) {
+    SoDebugError::postInfo("SoUnknownNode::readInstance",
+                           "Reading extension node ``%s'' as SoUnknownNode.",
+                           PRIVATE(this)->classname.getString());
+  }
+
+
   SbBool notbuiltin;
   // The "error on unknown field" is FALSE, in case we are a group
   // node with children specified in the file.
