@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -100,6 +100,10 @@
 #include <Inventor/elements/SoNormalBindingElement.h>
 #include <Inventor/elements/SoOverrideElement.h>
 #include <Inventor/elements/SoShapeStyleElement.h>
+#include <Inventor/elements/SoGLVBOElement.h>
+#include <Inventor/C/tidbits.h>
+#include "../misc/SoVBO.h"
+#include "../misc/PointerMap.h"
 
 /*!
   \enum SoVertexProperty::Binding
@@ -130,6 +134,12 @@
 
   \sa SoTextureCoordinate2
 */
+
+// FIXME: this field was added between TGS Inventor 2.5 and 2.6, and
+// between Coin 1.0 and Coin 2.0. This means it must get special
+// handling when exporting .iv-files, with regard to what header we
+// can put on the output. See also item #003 in the Coin/docs/todo.txt
+// file. 20030519 mortene.
 /*!
   \var SoMFVec3f SoVertexProperty::texCoord3
 
@@ -140,6 +150,7 @@
   \since Coin 2.0
   \since TGS Inventor 2.6
 */
+
 /*!
   \var SoMFVec3f SoVertexProperty::normal
 
@@ -182,6 +193,36 @@
   \sa SoMaterialBinding
 */
 
+class SoVertexPropertyP {
+ public:
+  SoVertexPropertyP(void)
+    : vertexvbo(NULL),
+      normalvbo(NULL),
+      texcoordvbo(NULL),
+      colorvbo(NULL)
+  {
+    this->checktransparent = FALSE;
+    this->transparent = FALSE;
+  }
+  ~SoVertexPropertyP() {
+    delete this->vertexvbo;
+    delete this->normalvbo;
+    delete this->texcoordvbo;
+    delete this->colorvbo;
+  }
+
+  SoVertexProperty * master;
+  SbBool transparent;
+  SbBool checktransparent;
+
+  SoVBO * vertexvbo;
+  SoVBO * normalvbo;
+  SoVBO * texcoordvbo;
+  SoVBO * colorvbo;
+
+};
+
+#define PRIVATE(obj) ((SoVertexPropertyP*) PointerMap::get(obj))
 
 SO_NODE_SOURCE(SoVertexProperty);
 
@@ -190,6 +231,8 @@ SO_NODE_SOURCE(SoVertexProperty);
 */
 SoVertexProperty::SoVertexProperty(void)
 {
+  PointerMap::add(this, new SoVertexPropertyP);
+  
   SO_NODE_INTERNAL_CONSTRUCTOR(SoVertexProperty);
 
   SO_NODE_ADD_FIELD(vertex, (0));
@@ -208,6 +251,14 @@ SoVertexProperty::SoVertexProperty(void)
   this->normal.setNum(0);
   this->orderedRGBA.setNum(0);
 
+  // So they are not written in their default state on SoWriteAction
+  // traversal.
+  this->vertex.setDefault(TRUE);
+  this->texCoord.setDefault(TRUE);
+  this->texCoord3.setDefault(TRUE);
+  this->normal.setDefault(TRUE);
+  this->orderedRGBA.setDefault(TRUE);
+
   SO_NODE_ADD_FIELD(normalBinding, (SoVertexProperty::PER_VERTEX_INDEXED));
   SO_NODE_ADD_FIELD(materialBinding, (SoVertexProperty::OVERALL));
 
@@ -222,8 +273,6 @@ SoVertexProperty::SoVertexProperty(void)
   SO_NODE_SET_SF_ENUM_TYPE(normalBinding, Binding);
   SO_NODE_SET_SF_ENUM_TYPE(materialBinding, Binding);
 
-  this->checktransparent = FALSE;
-  this->transparent = FALSE;
 }
 
 /*!
@@ -231,6 +280,9 @@ SoVertexProperty::SoVertexProperty(void)
 */
 SoVertexProperty::~SoVertexProperty()
 {
+  SoVertexPropertyP * pimpl = PRIVATE(this);
+  PointerMap::remove(this);
+  delete pimpl;  
 }
 
 // Documented in superclass.
@@ -287,56 +339,187 @@ SoVertexProperty::GLRender(SoGLRenderAction * action)
 void
 SoVertexProperty::doAction(SoAction *action)
 {
+  SoVertexPropertyP * pimpl = PRIVATE(this);
   SoState * state = action->getState();
 
   uint32_t overrideflags = SoOverrideElement::getFlags(state);
 #define TEST_OVERRIDE(bit) ((SoOverrideElement::bit & overrideflags) != 0)
-
+  
   SbBool glrender = action->isOfType(SoGLRenderAction::getClassTypeId());
-
-  if (this->checktransparent) {
-    this->checktransparent = FALSE;
-    this->transparent = FALSE;
+  
+  if (pimpl->checktransparent) {
+    pimpl->checktransparent = FALSE;
+    pimpl->transparent = FALSE;
     int n = this->orderedRGBA.getNum();
     for (int i = 0; i < n; i++) {
       if ((this->orderedRGBA[i] & 0xff) != 0xff) {
-        this->transparent = TRUE;
+        pimpl->transparent = TRUE;
         break;
       }
     }
   }
-
-  if (this->vertex.getNum() > 0)
-    SoCoordinateElement::set3(state, this, this->vertex.getNum(),
+  if (glrender) SoBase::staticDataLock();
+  int num = this->vertex.getNum();
+  const int numvertex = num;
+#ifdef COIN_NEXT_MINOR
+  const SbBool shouldcreatevbo = glrender ? SoGLVBOElement::shouldCreateVBO(state, numvertex) : FALSE;
+#else // COIN_NEXT_MINOR 
+  const SbBool shouldcreatevbo = FALSE;
+#endif // ! COIN_NEXT_MINOR
+  if (num > 0) {
+    SoCoordinateElement::set3(state, this, num,
                               this->vertex.getValues(0));
-
-  if (this->texCoord3.getNum() > 0) {
+    
+    SbBool setvbo = FALSE;
+#ifdef COIN_NEXT_MINOR
+    if (glrender) {
+      if (shouldcreatevbo) {
+        SbBool dirty = FALSE;
+        setvbo = TRUE;
+        if (pimpl->vertexvbo == NULL) {
+          pimpl->vertexvbo = new SoVBO(GL_ARRAY_BUFFER, GL_STATIC_DRAW);
+          dirty =  TRUE;
+        }
+        else if (pimpl->vertexvbo->getBufferDataId() != this->getNodeId()) {
+          dirty = TRUE;
+        }
+        if (dirty) {
+          pimpl->vertexvbo->setBufferData(this->vertex.getValues(0),
+                                          num*sizeof(SbVec3f),
+                                          this->getNodeId());
+        }
+      }
+      else if (pimpl->vertexvbo && pimpl->vertexvbo->getBufferDataId()) {
+        // clear buffers to deallocate VBO memory
+        pimpl->vertexvbo->setBufferData(NULL, 0, 0);
+      }
+      if (setvbo) {
+        SoGLVBOElement::setVertexVBO(state, pimpl->vertexvbo);
+      }
+    }
+#endif // COIN_NEXT_MINOR
+  }
+  num = this->texCoord3.getNum();
+  if (num > 0) {
     if (glrender) {
       if (SoGLTexture3EnabledElement::get(state)) {
+        // it's important to call this _before_ setting the coordinates
+        // on the state.
         SoGLTextureCoordinateElement::setTexGen(state,
                                                 this, NULL);
       }
     }
-    SoTextureCoordinateElement::set3(state, this, this->texCoord3.getNum(),
+    SoTextureCoordinateElement::set3(state, this, num,
                                      this->texCoord3.getValues(0));
+#ifdef COIN_NEXT_MINOR
+    if (glrender) {
+      SbBool setvbo = FALSE;
+      if ((num == numvertex) && shouldcreatevbo) {
+        SbBool dirty = FALSE;
+        setvbo = TRUE;
+        if (pimpl->texcoordvbo == NULL) {
+          pimpl->texcoordvbo = new SoVBO(GL_ARRAY_BUFFER, GL_STATIC_DRAW);
+          dirty =  TRUE;
+        }
+        else if (pimpl->texcoordvbo->getBufferDataId() != this->getNodeId()) {
+          dirty = TRUE;
+        }
+        if (dirty) {
+          pimpl->texcoordvbo->setBufferData(this->texCoord3.getValues(0),
+                                                    num*sizeof(SbVec3f),
+                                            this->getNodeId());
+        }
+      }
+      else if (pimpl->texcoordvbo && pimpl->texcoordvbo->getBufferDataId()) {
+        // clear buffers to deallocate VBO memory
+        pimpl->texcoordvbo->setBufferData(NULL, 0, 0);
+      }
+      if (setvbo) {
+        SoGLVBOElement::setTexCoordVBO(state, 0, pimpl->texcoordvbo);
+      }
+    }
+#endif // COIN_NEXT_MINOR
   }
   else {
-    if (this->texCoord.getNum() > 0) {
+    num = this->texCoord.getNum();
+    if (num > 0) {
       if (glrender) {
+        // it's important to call this _before_ setting the coordinates
+        // on the state.
         SoGLTextureCoordinateElement::setTexGen(state,
                                                 this, NULL);
       }
-      SoTextureCoordinateElement::set2(state, this, this->texCoord.getNum(),
+      SoTextureCoordinateElement::set2(state, this, num,
                                        this->texCoord.getValues(0));
+      
+#ifdef COIN_NEXT_MINOR
+      if (glrender) {
+        SbBool setvbo = FALSE;
+        if ((num == numvertex) && shouldcreatevbo) {
+          SbBool dirty = FALSE;
+          setvbo = TRUE;
+          if (pimpl->texcoordvbo == NULL) {
+            pimpl->texcoordvbo = new SoVBO(GL_ARRAY_BUFFER, GL_STATIC_DRAW);
+            dirty =  TRUE;
+          }
+          else if (pimpl->texcoordvbo->getBufferDataId() != this->getNodeId()) {
+            dirty = TRUE;
+          }
+          if (dirty) {
+            pimpl->texcoordvbo->setBufferData(this->texCoord.getValues(0),
+                                                      num*sizeof(SbVec2f),
+                                                    this->getNodeId());
+          }
+        }
+        else if (pimpl->texcoordvbo && pimpl->texcoordvbo->getBufferDataId()) {
+          // clear buffers to deallocate VBO memory
+          pimpl->texcoordvbo->setBufferData(NULL, 0, 0);
+        }
+        if (setvbo) {
+          SoGLVBOElement::setTexCoordVBO(state, 0, pimpl->texcoordvbo);
+        }
+      }
+#endif // COIN_NEXT_MINOR
     }
   }
-
-  if (this->normal.getNum() > 0 && !TEST_OVERRIDE(NORMAL_VECTOR)) {
-    SoNormalElement::set(state, this, this->normal.getNum(),
+  
+  num = this->normal.getNum();
+  if (num > 0 && !TEST_OVERRIDE(NORMAL_VECTOR)) {
+    SoNormalElement::set(state, this, num,
                          this->normal.getValues(0));
     if (this->isOverride()) {
       SoOverrideElement::setNormalVectorOverride(state, this, TRUE);
     }
+
+    SbBool setvbo = FALSE;
+
+#ifdef COIN_NEXT_MINOR
+    if (glrender) {
+      if ((num == numvertex) && shouldcreatevbo) {
+        SbBool dirty = FALSE;
+        setvbo = TRUE;
+        if (pimpl->normalvbo == NULL) {
+          pimpl->normalvbo = new SoVBO(GL_ARRAY_BUFFER, GL_STATIC_DRAW);
+          dirty =  TRUE;
+        }
+        else if (pimpl->normalvbo->getBufferDataId() != this->getNodeId()) {
+          dirty = TRUE;
+        }
+        if (dirty) {
+          pimpl->normalvbo->setBufferData(this->normal.getValues(0),
+                                          num*sizeof(SbVec3f),
+                                          this->getNodeId());
+        }
+      }
+      else if (pimpl->normalvbo && pimpl->normalvbo->getBufferDataId()) {
+        // clear buffers to deallocate VBO memory
+        pimpl->normalvbo->setBufferData(NULL, 0, 0);
+      }
+      if (setvbo) {
+        SoGLVBOElement::setNormalVBO(state, pimpl->normalvbo);
+      }
+    }
+#endif // COIN_NEXT_MINOR
   }
   if (this->normal.getNum() > 0 && !TEST_OVERRIDE(NORMAL_BINDING)) {
     SoNormalBindingElement::set(state, this,
@@ -346,16 +529,64 @@ SoVertexProperty::doAction(SoAction *action)
       SoOverrideElement::setNormalBindingOverride(state, this, TRUE);
     }
   }
-  if (this->orderedRGBA.getNum() > 0 && 
+  
+  num = this->orderedRGBA.getNum();
+  if (num > 0 &&
       !TEST_OVERRIDE(DIFFUSE_COLOR)) {
-    
-    SoLazyElement::setPacked(state, this, this->orderedRGBA.getNum(),
+
+    SoLazyElement::setPacked(state, this, num,
                              this->orderedRGBA.getValues(0),
-                             this->transparent);
+                             pimpl->transparent);
     if (this->isOverride()) {
       SoOverrideElement::setDiffuseColorOverride(state, this, TRUE);
     }
+#ifdef COIN_NEXT_MINOR
+    if (glrender) {
+      SbBool setvbo = FALSE;
+      if ((num == numvertex) && shouldcreatevbo) {
+        SbBool dirty = FALSE;
+        setvbo = TRUE;
+        if (pimpl->colorvbo == NULL) {
+          pimpl->colorvbo = new SoVBO(GL_ARRAY_BUFFER, GL_STATIC_DRAW);
+          dirty = TRUE;
+        }
+        else if (pimpl->colorvbo->getBufferDataId() != this->getNodeId()) {
+          dirty = TRUE;
+        }
+        if (dirty) {
+          if (coin_host_get_endianness() == COIN_HOST_IS_BIGENDIAN) {
+            pimpl->colorvbo->setBufferData(this->orderedRGBA.getValues(0),
+                                           num*sizeof(uint32_t),
+                                           this->getNodeId());
+          }
+          else {
+            const uint32_t * src = this->orderedRGBA.getValues(0);
+            uint32_t * dst = (uint32_t*)
+              pimpl->colorvbo->allocBufferData(num*sizeof(uint32_t),
+                                               this->getNodeId());
+            for (int i = 0; i < num; i++) {
+              uint32_t tmp = src[i];
+              dst[i] =
+                (tmp << 24) |
+                ((tmp & 0xff00) << 8) |
+                ((tmp & 0xff0000) >> 8) |
+                (tmp >> 24);
+            }
+          }
+        }
+      }
+      else if (pimpl->colorvbo) {
+        pimpl->colorvbo->setBufferData(NULL, 0, 0);
+      }
+      if (setvbo) {
+        SoGLVBOElement::setColorVBO(state, pimpl->colorvbo);
+      }
+    }
+#endif // COIN_NEXT_MINOR
   }
+  
+  if (glrender) SoBase::staticDataUnlock();
+  
   if (this->orderedRGBA.getNum() && !TEST_OVERRIDE(MATERIAL_BINDING)) {
     SoMaterialBindingElement::set(state, this,
                                   (SoMaterialBindingElement::Binding)
@@ -395,7 +626,9 @@ SoVertexProperty::notify(SoNotList *list)
 {
   SoField *f = list->getLastField();
   if (f == &this->orderedRGBA) {
-    this->checktransparent = TRUE;
+    PRIVATE(this)->checktransparent = TRUE;
   }
   inherited::notify(list);
 }
+
+#undef PRIVATE
