@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -69,6 +69,7 @@
 #include <Inventor/events/SoMouseButtonEvent.h>
 #include <Inventor/actions/SoHandleEventAction.h>
 #include <Inventor/SoPickedPoint.h>
+#include <Inventor/C/tidbitsp.h>
 #include <coindefs.h>
 
 
@@ -123,13 +124,20 @@ class SoWWWAnchorP {
   static void * fetchdata;
   static SoWWWAnchorCB * highlightfunc;
   static void * highlightdata;
+
+  static void atexit_cleanup() {
+    SoWWWAnchorP::fetchfunc = NULL;
+    SoWWWAnchorP::fetchdata = NULL;
+    SoWWWAnchorP::highlightfunc = NULL;
+    SoWWWAnchorP::highlightdata = NULL;
+  }
 };
 
 // static members
-SoWWWAnchorCB * SoWWWAnchorP::fetchfunc;
-void * SoWWWAnchorP::fetchdata;
-SoWWWAnchorCB * SoWWWAnchorP::highlightfunc;
-void * SoWWWAnchorP::highlightdata;
+SoWWWAnchorCB * SoWWWAnchorP::fetchfunc = NULL;
+void * SoWWWAnchorP::fetchdata = NULL;
+SoWWWAnchorCB * SoWWWAnchorP::highlightfunc = NULL;
+void * SoWWWAnchorP::highlightdata = NULL;
 
 #endif // DOXYGEN_SKIP_THIS
 
@@ -170,6 +178,7 @@ void
 SoWWWAnchor::initClass(void)
 {
   SO_NODE_INTERNAL_INIT_CLASS(SoWWWAnchor, SO_FROM_INVENTOR_2_1|SoNode::VRML1);
+  coin_atexit((coin_atexit_f*)SoWWWAnchorP::atexit_cleanup, CC_ATEXIT_NORMAL);
 }
 
 
