@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -34,6 +34,7 @@
 */
 
 #include <Inventor/projectors/SbCylinderSectionProjector.h>
+#include <float.h>
 
 #if COIN_DEBUG
 #include <Inventor/errors/SoDebugError.h>
@@ -184,6 +185,7 @@ SbBool
 SbCylinderSectionProjector::isWithinTolerance(const SbVec3f & point)
 {
   if (this->needSetup) this->setupTolerance();
+
   // check if behind tolerance plane
   if (!this->tolPlane.isInHalfSpace(point)) return FALSE;
 
@@ -217,7 +219,17 @@ SbCylinderSectionProjector::setupTolerance(void)
 
   // find plane direction perpendicular to line
   this->planeDir = somept - ptonaxis;
-  this->planeDir.normalize();
+  if (this->planeDir.normalize() < FLT_EPSILON) {
+    // the cylinder axis is parallel to the view direction. This is a
+    // special case, and not really supported by the projector. Just
+    // create a tilted plane to make it possible to rotate the
+    // cylinder even for this case.
+    this->planeDir = this->viewVol.getViewUp() + 
+      this->viewVol.getProjectionDirection();
+    this->worldToWorking.multDirMatrix(this->planeDir, this->planeDir);    
+    (void) this->planeDir.normalize();
+  }
+  
   if (!this->intersectFront) {
     this->planeDir = -this->planeDir;
   }
