@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -33,6 +33,7 @@
 // should of course be used. 20000308 mortene.
 
 #include <Inventor/projectors/SbCylinderSheetProjector.h>
+#include <float.h> // FLT_EPSILON
 
 #if COIN_DEBUG
 #include <Inventor/errors/SoDebugError.h>
@@ -196,7 +197,18 @@ SbCylinderSheetProjector::setupPlane(void)
   SbVec3f ptOnAxis = axis.getClosestPoint(somePt);
 
   this->planeDir = somePt - ptOnAxis;
-  this->planeDir.normalize();
+
+  if (this->planeDir.normalize() < FLT_EPSILON) {
+    // the cylinder axis is parallel to the view direction. This is a
+    // special case, and not really supported by the projector. Just
+    // create a tilted plane to make it possible to rotate the
+    // cylinder even for this case.
+    this->planeDir = this->viewVol.getViewUp() + 
+      this->viewVol.getProjectionDirection();
+    this->worldToWorking.multDirMatrix(this->planeDir, this->planeDir);    
+    (void) this->planeDir.normalize();
+  }
+
   if (!this->intersectFront) this->planeDir = -this->planeDir;
 
   this->tolPlane = SbPlane(this->planeDir, axis.getPosition());
