@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -32,6 +32,15 @@
 
   The priority values can be used to queue events by their importance,
   so the sensors are triggered in the sequence you want.
+
+  The queue of delay sensors (i.e. instances of subclasses of
+  SoDelayQueueSensor) will be processed as soon as either the run-time
+  system is idle, or if it is continually busy they will be processed
+  within a fixed amount of time.
+
+  This time interval is by default 1/12th of a second, but can be
+  controlled with the SoSensorManager::setDelaySensorTimeout()
+  interface.
 */
 
 #include <assert.h>
