@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -194,6 +194,8 @@
 #include <Inventor/elements/SoCacheElement.h>
 #include <Inventor/elements/SoNormalElement.h>
 #include <Inventor/elements/SoCreaseAngleElement.h>
+#include <Inventor/elements/SoGLVBOElement.h>
+#include <Inventor/elements/SoGLLazyElement.h>
 #include <Inventor/caches/SoNormalCache.h>
 #include <Inventor/misc/SoGL.h>
 #include <assert.h>
@@ -201,6 +203,9 @@
 #include <Inventor/bundles/SoTextureCoordinateBundle.h>
 #include "../caches/normalcache_numcoords_hack.h"
 
+#include "../misc/SoVertexArrayIndexer.h"
+#include "../misc/SoVBO.h"
+
 #ifdef COIN_THREADSAFE
 #include <Inventor/threads/SbRWMutex.h>
 #endif // COIN_THREADSAFE
@@ -210,6 +215,8 @@
 #define STATUS_CONVEX  1
 #define STATUS_CONCAVE 2
 
+#define LOCK_VAINDEXER(obj) SoBase::staticDataLock()
+#define UNLOCK_VAINDEXER(obj) SoBase::staticDataUnlock()
 
 #ifndef DOXYGEN_SKIP_THIS
 class SoIndexedFaceSetP {
@@ -219,6 +226,7 @@ public:
     : convexmutex(SbRWMutex::READ_PRECEDENCE)
 #endif // COIN_THREADSAFE 
   { }
+  SoVertexArrayIndexer * vaindexer;
   SoConvexDataCache * convexCache;
   int concavestatus;
 #ifdef COIN_THREADSAFE
@@ -259,6 +267,7 @@ SO_NODE_SOURCE(SoIndexedFaceSet);
 SoIndexedFaceSet::SoIndexedFaceSet()
 {
   THIS = new SoIndexedFaceSetP;
+  THIS->vaindexer = NULL;
   THIS->convexCache = NULL;
   THIS->concavestatus = STATUS_UNKNOWN;
 
@@ -271,6 +280,7 @@ SoIndexedFaceSet::SoIndexedFaceSet()
 */
 SoIndexedFaceSet::~SoIndexedFaceSet()
 {
+  delete THIS->vaindexer;
   if (THIS->convexCache) THIS->convexCache->unref();
   delete THIS;
 }
@@ -368,7 +378,15 @@ SoIndexedFaceSet::notify(SoNotList * list)
   if (THIS->convexCache) THIS->convexCache->invalidate();
   THIS->readUnlockConvexCache();
   SoField *f = list->getLastField();
-  if (f == &this->coordIndex) THIS->concavestatus = STATUS_UNKNOWN;
+  if (f == &this->coordIndex) {
+    THIS->concavestatus = STATUS_UNKNOWN;
+    LOCK_VAINDEXER(this);
+    if (THIS->vaindexer) {
+      delete THIS->vaindexer;
+      THIS->vaindexer = NULL;
+    }
+    UNLOCK_VAINDEXER(this);
+  }
   inherited::notify(list);
 }
 
@@ -379,13 +397,16 @@ SoIndexedFaceSet::GLRender(SoGLRenderAction * action)
   if (this->coordIndex.getNum() < 3) return;
   SoState * state = action->getState();
 
+  SbBool hasvp = FALSE;
+  
   if (this->vertexProperty.getValue()) {
+    hasvp = TRUE;
     state->push();
     this->vertexProperty.getValue()->GLRender(action);
   }
 
   if (!this->shouldGLRender(action)) {
-    if (this->vertexProperty.getValue())
+    if (hasvp)
       state->pop();
     return;
   }
@@ -475,19 +496,112 @@ SoIndexedFaceSet::GLRender(SoGLRenderAction * action)
 
   mb.sendFirst(); // make sure we have the correct material
 
-  sogl_render_faceset((SoGLCoordinateElement *)coords,
-                      cindices,
-                      numindices,
-                      normals,
-                      nindices,
-                      &mb,
-                      mindices,
-                      &tb,
-                      tindices,
-                      (int)nbind,
-                      (int)mbind,
-                      doTextures?1:0);
+#if 0
+  fprintf(stderr,"numindices: %d, convex: %d, ncache: %d, nbind: %d, mbind: %d, tbind: %d, va: %d\n",
+          numindices, convexcacheused, normalCacheUsed, nbind, mbind, tbind,
+          cc_glglue_has_vertex_array(sogl_glue_instance(state)));
+#endif
 
+#ifdef COIN_NEXT_MINOR
+  SoGLLazyElement * lelem = NULL;
+  const uint32_t contextid = action->getCacheContext();
+  SbBool dova = 
+    SoVBO::shouldRenderAsVertexArrays(contextid, numindices) &&
+    !convexcacheused && !normalCacheUsed &&
+    ((nbind == OVERALL) || ((nbind == PER_VERTEX_INDEXED) && ((nindices == cindices) || (nindices == NULL)))) &&
+    ((tbind == NONE) || ((tbind == PER_VERTEX_INDEXED) && ((tindices == cindices) || (tindices == NULL)))) &&
+    ((mbind == NONE) || ((mbind == PER_VERTEX_INDEXED) && ((mindices == cindices) || (mindices == NULL)))) &&
+    cc_glglue_has_vertex_array(sogl_glue_instance(state));
+
+  const SoGLVBOElement * vboelem = SoGLVBOElement::getInstance(state);
+  SoVBO * colorvbo = NULL;
+
+  if (dova && (mbind != OVERALL)) {
+    dova = FALSE;
+    if ((mbind == PER_VERTEX_INDEXED) && ((mindices == cindices) || (mindices == NULL))) {
+      lelem = (SoGLLazyElement*) SoLazyElement::getInstance(state);
+      colorvbo = vboelem->getColorVBO();
+      if (colorvbo) dova = TRUE;
+      else {
+        // we might be able to do VA-rendering, but need to check the
+        // diffuse color type first.
+        if (!lelem->isPacked() && lelem->getNumTransparencies() <= 1) {
+          dova = TRUE;
+        }
+      }
+    }
+  }
+  if (dova) {
+    SbBool dovbo = this->startVertexArray(action,
+                                          coords,
+                                          (nbind != OVERALL) ? normals : NULL,
+                                          doTextures,
+                                          mbind != OVERALL);
+    LOCK_VAINDEXER(this);
+    if (THIS->vaindexer == NULL) {
+      SoVertexArrayIndexer * indexer = new SoVertexArrayIndexer;
+      int i = 0;
+      while (i < numindices) {
+        int cnt = 0;
+        while (i + cnt < numindices && cindices[i+cnt] >= 0) cnt++;
+        
+        switch (cnt) {
+        case 3:
+          indexer->addTriangle(cindices[i],cindices[i+1], cindices[i+2]);
+          break;
+        case 4:
+          indexer->addQuad(cindices[i],cindices[i+1],cindices[i+2],cindices[i+3]);
+          break;
+        default:
+          if (cnt > 4) {
+            indexer->beginTarget(GL_POLYGON);
+            for (int j = 0; j < cnt; j++) {
+              indexer->targetVertex(GL_POLYGON, cindices[i+j]);
+            }
+            indexer->endTarget(GL_POLYGON);
+          }
+        }
+        i += cnt + 1;
+      }
+      indexer->close();
+      if (indexer->getNumVertices()) {
+        THIS->vaindexer = indexer;
+      }
+      else {
+        delete indexer;
+      }
+#if 0
+      fprintf(stderr,"XXX: create VertexArrayIndexer: %d\n", indexer->getNumVertices());
+#endif
+    }
+
+    if (THIS->vaindexer) {
+      THIS->vaindexer->render(sogl_glue_instance(state), dovbo, contextid);
+    }
+    UNLOCK_VAINDEXER(this);
+    this->finishVertexArray(action,
+                            dovbo,
+                            (nbind != OVERALL),
+                            doTextures,
+                            mbind != OVERALL);
+  }
+#else // COIN_NEXT_MINOR
+  if (0) { }
+#endif // !COIN_NEXT_MINOR
+  else {
+    sogl_render_faceset((SoGLCoordinateElement *)coords,
+                        cindices,
+                        numindices,
+                        normals,
+                        nindices,
+                        &mb,
+                        mindices,
+                        &tb,
+                        tindices,
+                        (int)nbind,
+                        (int)mbind,
+                        doTextures?1:0);
+  }
   if (normalCacheUsed) {
     this->readUnlockNormalCache();
   }
@@ -496,7 +610,7 @@ SoIndexedFaceSet::GLRender(SoGLRenderAction * action)
     THIS->readUnlockConvexCache();
   }
 
-  if (this->vertexProperty.getValue()) {
+  if (hasvp) {
     state->pop();
   }
   // send approx number of triangles for autocache handling
@@ -697,7 +811,7 @@ SoIndexedFaceSet::generatePrimitives(SoAction *action)
       currnormal = &normals[normnr++];
       vertex.setNormal(*currnormal);
     }
-    else if (nbind == PER_VERTEX || nbind == PER_VERTEX_INDEXED) {
+    else if (nbind == PER_FACE_INDEXED || nbind == PER_VERTEX_INDEXED) {
       pointDetail.setNormalIndex(*nindices);
       currnormal = &normals[*nindices++];
       vertex.setNormal(*currnormal);
@@ -967,3 +1081,5 @@ SoIndexedFaceSet::generateDefaultNormals(SoState * state,
 #undef STATUS_UNKNOWN
 #undef STATUS_CONVEX
 #undef STATUS_CONCAVE
+#undef LOCK_VAINDEXER
+#undef UNLOCK_VAINDEXER
