@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -74,13 +74,20 @@
 #include <Inventor/elements/SoTextureCoordinateBindingElement.h>
 #include <Inventor/elements/SoDrawStyleElement.h>
 #include <Inventor/elements/SoGLCoordinateElement.h>
-#include <Inventor/elements/SoLazyElement.h>
+#include <Inventor/elements/SoGLLazyElement.h>
+#include <Inventor/elements/SoGLCacheContextElement.h>
+#include <Inventor/elements/SoGLVBOElement.h>
+#include <Inventor/elements/SoMultiTextureCoordinateElement.h>
+#include <Inventor/elements/SoMultiTextureEnabledElement.h>
 
 #include <assert.h>
 
 #include <Inventor/bundles/SoTextureCoordinateBundle.h>
 #include <Inventor/details/SoLineDetail.h>
 #include <Inventor/caches/SoBoundingBoxCache.h>
+#include "../misc/SoVertexArrayIndexer.h"
+#include "../misc/SoVBO.h"
+#include "../misc/PointerMap.h"
 
 #if COIN_DEBUG
 #include <Inventor/errors/SoDebugError.h>
@@ -89,11 +96,27 @@
 
 SO_NODE_SOURCE(SoIndexedLineSet);
 
+
+#define PRIVATE(obj) ((SoIndexedLineSetP*) PointerMap::get(obj))
+#define LOCK_VAINDEXER(obj) SoBase::staticDataLock()
+#define UNLOCK_VAINDEXER(obj) SoBase::staticDataUnlock()
+
+class SoIndexedLineSetP {
+public:
+  SoIndexedLineSetP() : vaindexer(NULL), nodeid(0) {}
+  ~SoIndexedLineSetP() { delete this->vaindexer; }
+  
+  SoVertexArrayIndexer * vaindexer;
+  uint32_t nodeid;
+};
+
 /*!
   Constructor.
 */
 SoIndexedLineSet::SoIndexedLineSet()
 {
+  PointerMap::add(this, new SoIndexedLineSetP);
+
   SO_NODE_INTERNAL_CONSTRUCTOR(SoIndexedLineSet);
 }
 
@@ -102,6 +125,9 @@ SoIndexedLineSet::SoIndexedLineSet()
 */
 SoIndexedLineSet::~SoIndexedLineSet()
 {
+  SoIndexedLineSetP * pimpl = PRIVATE(this);
+  PointerMap::remove(this);
+  delete pimpl;
 }
 
 // doc from parent
@@ -208,6 +234,8 @@ SoIndexedLineSet::GLRender(SoGLRenderAction * action)
   SoState * state = action->getState();
 
   if (!this->shouldGLRender(action)) return;
+
+  SoIndexedLineSetP * pimpl = PRIVATE(this);
   
   SbBool didpush = FALSE;
 
@@ -281,19 +309,102 @@ SoIndexedLineSet::GLRender(SoGLRenderAction * action)
   SbBool drawPoints =
     SoDrawStyleElement::get(state) == SoDrawStyleElement::POINTS;
 
-  sogl_render_lineset((SoGLCoordinateElement*)coords,
-                      cindices,
-                      numindices,
-                      normals,
-                      nindices,
-                      &mb,
-                      mindices,
-                      &tb,
-                      tindices,
-                      (int)nbind,
-                      (int)mbind,
-                      doTextures ? 1 : 0,
-                      drawPoints ? 1 : 0);
+#ifdef COIN_NEXT_MINOR  
+  SoGLLazyElement * lelem = NULL;
+  const uint32_t contextid = action->getCacheContext();
+  SbBool dova = 
+    !drawPoints &&
+    SoVBO::shouldRenderAsVertexArrays(contextid, numindices) &&
+    ((nbind == OVERALL) || ((nbind == PER_VERTEX_INDEXED) && ((nindices == cindices) || (nindices == NULL)))) &&
+    (!doTextures || (tindices == cindices)) &&
+    ((mbind == OVERALL) || ((mbind == PER_VERTEX_INDEXED) && ((mindices == cindices) || (mindices == NULL)))) &&
+    cc_glglue_has_vertex_array(sogl_glue_instance(state));
+  
+  const SoGLVBOElement * vboelem = SoGLVBOElement::getInstance(state);
+  SoVBO * colorvbo = NULL;
+
+  if (dova && (mbind != OVERALL)) {
+    dova = FALSE;
+    if ((mbind == PER_VERTEX_INDEXED) && ((mindices == cindices) || (mindices == NULL))) {
+      lelem = (SoGLLazyElement*) SoLazyElement::getInstance(state);
+      colorvbo = vboelem->getColorVBO();
+      if (colorvbo) dova = TRUE;
+      else {
+        // we might be able to do VA-rendering, but need to check the
+        // diffuse color type first.
+        if (!lelem->isPacked() && lelem->getNumTransparencies() <= 1) {
+          dova = TRUE;
+        }
+      }
+    }
+  }
+  if (dova) {
+    SbBool dovbo = this->startVertexArray(action,
+                                          coords,
+                                          nbind != OVERALL ? normals : NULL,
+                                          doTextures,
+                                          mbind != OVERALL);
+    LOCK_VAINDEXER(this);
+    if (pimpl->vaindexer == NULL || pimpl->nodeid != this->getNodeId()) {
+      delete pimpl->vaindexer;
+      pimpl->vaindexer = NULL;
+      pimpl->nodeid = this->getNodeId();
+      
+      SoVertexArrayIndexer * indexer = new SoVertexArrayIndexer;
+      
+      int i = 0;
+      while (i < numindices) {
+        int cnt = 0;
+        while (i + cnt < numindices && cindices[i+cnt] >= 0) cnt++;
+        if (cnt >= 2) {
+          for (int j = 1; j < cnt;j++) {
+            indexer->addLine(cindices[i+j-1],
+                             cindices[i+j]);
+          }
+        }
+        i += cnt + 1;
+      }
+      indexer->close();
+      if (indexer->getNumVertices()) {
+        pimpl->vaindexer = indexer;
+      }
+      else {
+        delete indexer;
+      }
+#if 0
+      fprintf(stderr,"XXX: ILS create VertexArrayIndexer: %d\n", indexer->getNumVertices());
+#endif
+    }
+    
+    if (pimpl->vaindexer) {
+      pimpl->vaindexer->render(sogl_glue_instance(state), dovbo, contextid);
+    }
+    UNLOCK_VAINDEXER(this);
+
+    this->finishVertexArray(action,
+                            dovbo,
+                            nbind != OVERALL,
+                            doTextures,
+                            mbind != OVERALL);
+  }
+#else // COIN_NEXT_MINOR
+  if (0) { }
+#endif // !COIN_NEXT_MINOR
+  else {
+    sogl_render_lineset((SoGLCoordinateElement*)coords,
+                        cindices,
+                        numindices,
+                        normals,
+                        nindices,
+                        &mb,
+                        mindices,
+                        &tb,
+                        tindices,
+                        (int)nbind,
+                        (int)mbind,
+                        doTextures ? 1 : 0,
+                        drawPoints ? 1 : 0);
+  }
 
   if (didpush) {
     state->pop();
@@ -662,3 +773,7 @@ SoIndexedLineSet::generatePrimitives(SoAction *action)
     state->pop();
   }
 }
+
+#undef LOCK_VAINDEXER
+#undef PRIVATE
+#undef UNLOCK_VAINDEXER
