@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -26,10 +26,49 @@
   \brief The SoNurbsCurve class is a node for representing smooth curves.
   \ingroup nodes
 
-  Explaining NURBS is beyond the scope of this documentation. If you
-  are unfamiliar with the principles of representing smooth curves and
-  surfaces when doing 3D visualization, we recommend finding a good
-  book on the subject.
+  A general explanation of NURBS is beyond the scope of the Coin
+  documentation. For detailed information, refer to the specialized
+  literature on the topic (for example "An Introduction to NURBS: With
+  Historical Perspective" by David F. Rogers). A basic overview of
+  curve and surface rendering using NURBS can be found in chapter 8 of
+  "The Inventor Mentor".
+
+  Note that knot values should be specified as [0, 1, 2,..., a] rather
+  than [0, 1/a, 2/a,..., 1] to avoid tesselation errors due to
+  floating point precision problems. (Even if the rendered curve
+  <i>looks</i> correct, such issues might surface when e.g. doing
+  picking, since the tesselated representation used internally is not
+  the same as the one you see rendered by OpenGL on-screen.)
+
+  A small usage example (drawing a circle using a rational curve):
+
+  \code
+  #Inventor V2.1 ascii
+  
+  Coordinate4 {
+     point [
+       0.0 -5.0     0.0 1.0,
+       2.5 -2.5     0.0 0.5,
+       2.5 -0.66987 0.0 1.0,
+       0.0  1.94013 0.0 0.5,
+      -2.5 -0.66987 0.0 1.0,
+      -2.5 -2.5     0.0 0.5,
+       0.0 -5.0     0.0 1.0
+     ]
+  }
+  
+  NurbsCurve {
+     numControlPoints 7
+     knotVector [ 0, 0, 0, 1, 1, 2, 2, 3, 3, 3 ]
+  }
+  
+  # debug: show all the control points
+  BaseColor { rgb 1 1 0 }
+  DrawStyle { pointSize 3 }
+  PointSet { numPoints 7 }
+  \endcode
+
+  <center><img src="http://doc.coin3d.org/images/Coin/nodes/nurbscurve-circle.png"></center>
 
   <b>FILE FORMAT/DEFAULTS:</b>
   \code
@@ -40,36 +79,35 @@
   \endcode
 
 */
-// FIXME: recommend the book pederb bought about the subject? If so,
-// also add to class doc for SoNurbsSurface (and others?). Plus, add a
-// usage exaple. 20010909 mortene.
 
 #include <Inventor/nodes/SoNurbsCurve.h>
-#include <Inventor/nodes/SoSubNodeP.h>
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif // HAVE_CONFIG_H
+
+#include <Inventor/C/glue/GLUWrapper.h>
 #include <Inventor/SoPrimitiveVertex.h>
 #include <Inventor/actions/SoGLRenderAction.h>
 #include <Inventor/actions/SoGetBoundingBoxAction.h>
 #include <Inventor/actions/SoRayPickAction.h>
 #include <Inventor/bundles/SoMaterialBundle.h>
 #include <Inventor/caches/SoBoundingBoxCache.h>
+#include <Inventor/elements/SoComplexityTypeElement.h>
 #include <Inventor/elements/SoCoordinateElement.h>
 #include <Inventor/elements/SoDrawStyleElement.h>
+#include <Inventor/elements/SoGLCacheContextElement.h>
 #include <Inventor/elements/SoGLTextureEnabledElement.h>
-#include <Inventor/elements/SoPickStyleElement.h>
 #include <Inventor/elements/SoLazyElement.h>
+#include <Inventor/elements/SoPickStyleElement.h>
 #include <Inventor/errors/SoDebugError.h>
 #include <Inventor/misc/SoGL.h>
 #include <Inventor/misc/SoState.h>
-#include <Inventor/elements/SoGLCacheContextElement.h>
-#include <Inventor/elements/SoComplexityTypeElement.h>
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif // HAVE_CONFIG_H
-#include <coindefs.h> // COIN_OBSOLETED()
+#include <Inventor/nodes/SoSubNodeP.h>
 #include <Inventor/system/gl.h>
-#include <Inventor/C/glue/GLUWrapper.h>
+#include <coindefs.h> // COIN_OBSOLETED()
+
+// *************************************************************************
 
 /*!
   \var SoSFInt32 SoNurbsCurve::numControlPoints
@@ -77,7 +115,11 @@
 */
 /*!
   \var SoMFFloat SoNurbsCurve::knotVector
-  The knot vector.
+  The knot vector. 
+
+  Values should be specified as [0, 1, 2,..., a] rather than [0, 1/a,
+  2/a,..., 1] to avoid tesselation errors due to floating point
+  precision problems. 
 */
 
 // *************************************************************************
@@ -392,7 +434,11 @@ SoNurbsCurveP::tessBegin(int type, void * data)
     break;
   case GL_LINE_LOOP:
     shapetype = SoShape::LINE_STRIP; // will not be closed...
-    // FIXME: implement this functionality. 20010909 mortene.
+    // FIXME: implement this functionality. 20010909 mortene.  
+    // Update 20060130 kyrah: According to gluNurbsCallback(3), the
+    // only valid arguments we can expect here are GL_LINES or
+    // GL_LINE_STRIP, so this should really be an assert. (See also
+    // FIXME from 20010909 below - yes, it should be an assert.)
 
     // FIXME: un-commenting the following line exposes a _weird_ HP-UX
     // aCC bug -- should investigate closer.  (Detected with aCC
