@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -26,10 +26,47 @@
   \brief The SoNurbsSurface class is used to render smooth surfaces.
   \ingroup nodes
 
-  Explaining NURBS is beyond the scope of this documentation. If you
-  are unfamiliar with the principles of representing smooth curves and
-  surfaces when doing 3D visualization, we recommend finding a good
-  book on the subject.
+  A general explanation of NURBS is beyond the scope of the Coin
+  documentation. For detailed information, refer to the specialized
+  literature on the topic (for example "An Introduction to NURBS: With
+  Historical Perspective" by David F. Rogers). A basic overview of
+  curve and surface rendering using NURBS can be found in chapter 8 of
+  "The Inventor Mentor".
+
+  Note that knot values should be specified as [0, 1, 2,..., a] rather
+  than [0, 1/a, 2/a,..., 1] to avoid tesselation errors due to
+  floating point precision problems. (Even if the rendered surface
+  <i>looks</i> correct, such issues might surface when e.g. doing
+  picking, since the tesselated representation used internally is not
+  the same as the one you see rendered by OpenGL on-screen.)
+
+  A basic usage example:
+
+  \code
+  #Inventor V2.1 ascii
+
+  ShapeHints {
+    vertexOrdering COUNTERCLOCKWISE
+  }
+
+  Coordinate3 {
+     point [ 
+       -3 -3 -3, -3 -1 -3, -3  1 -3, -3  3 -3,
+       -1 -3 -3, -1 -1  3, -1  1  3, -1  3 -3,
+        1 -3 -3,  1 -1  3,  1  1  3,  1  3 -3,
+        3 -3 -3,  3 -1 -3,  3  1 -3,  3  3 -3
+      ]
+  }
+
+  NurbsSurface {
+     numUControlPoints 4
+     numVControlPoints 4
+     uKnotVector [ 0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0 ]
+     vKnotVector [ 0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0 ]
+  }
+  \endcode
+
+  <center><img src="http://doc.coin3d.org/images/Coin/nodes/nurbssurface.png"></center>
 
   <b>FILE FORMAT/DEFAULTS:</b>
   \code
@@ -46,9 +83,6 @@
   \endcode
 
 */
-// FIXME: recommend the book pederb bought about the subject? If so,
-// also add to class doc for SoNurbsSurface (and others?). Plus, add a
-// usage exaple. 20010909 mortene.
 
 #include <Inventor/nodes/SoNurbsSurface.h>
 #include <Inventor/nodes/SoSubNodeP.h>
