@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -91,9 +91,11 @@
 #include <Inventor/elements/SoMaterialBindingElement.h>
 #include <Inventor/bundles/SoMaterialBundle.h>
 #include <Inventor/elements/SoGLLazyElement.h>
+#include <Inventor/elements/SoGLVBOElement.h>
 #include <Inventor/caches/SoNormalCache.h>
 #include <Inventor/details/SoPointDetail.h>
 #include <Inventor/misc/SoGL.h>
+#include "../misc/SoVBO.h"
 
 #if COIN_DEBUG
 #include <Inventor/errors/SoDebugError.h>
@@ -235,19 +237,57 @@ SoPointSet::GLRender(SoGLRenderAction * action)
 
   int32_t idx = this->startIndex.getValue();
   if (numpts < 0) numpts = coords->getNum() - idx;
+  
+#ifdef COIN_NEXT_MINOR
+  const cc_glglue * glue = sogl_glue_instance(state);
+  const uint32_t contextid = action->getCacheContext();
 
-  sogl_render_pointset(coords,
-                       nbind != OVERALL ? normals : NULL,
-                       mbind != OVERALL ? &mb : NULL,
-                       doTextures ? &tb : NULL,
-                       numpts, idx);
-
+  SbBool dova = 
+    SoVBO::shouldRenderAsVertexArrays(contextid, numpts) && 
+    cc_glglue_has_vertex_array(glue);
+  
+  if (dova && (mbind == PER_VERTEX)) {
+    const SoGLVBOElement * vboelem = SoGLVBOElement::getInstance(state);
+    if (vboelem->getColorVBO() == NULL) {
+      dova = FALSE;
+      // we might be able to do VA-rendering, but need to check the
+      // diffuse color type first.
+      SoGLLazyElement * lelem = (SoGLLazyElement*) SoLazyElement::getInstance(state);
+      if (!lelem->isPacked() && lelem->getNumTransparencies() <= 1) {
+        dova = TRUE;
+      }
+    }
+  }
+  if (dova) {
+    SbBool vbo = this->startVertexArray(action,
+                                        coords,
+                                        (needNormals && (nbind != OVERALL)) ? normals : NULL,
+                                        doTextures,
+                                        mbind == PER_VERTEX);
+    
+    cc_glglue_glDrawArrays(glue, GL_POINTS, idx, numpts);
+    this->finishVertexArray(action, vbo,
+                            (needNormals && (nbind != OVERALL)),
+                            doTextures,
+                            mbind == PER_VERTEX);
+  }
+#else // COIN_NEXT_MINOR
+  if (0) {}
+#endif // !COIN_NEXT_MINOR
+  else {
+    sogl_render_pointset(coords,
+                         nbind != OVERALL ? normals : NULL,
+                         mbind != OVERALL ? &mb : NULL,
+                         doTextures ? &tb : NULL,
+                         numpts, idx);
+  }
   if (didpush)
     state->pop();
 
   // send approx number of points for autocache handling. Divide
   // by three so that three points is the same as one triangle.
   sogl_autocache_update(state, numpts/3);
+
 }
 
 // Documented in superclass.
@@ -283,7 +323,10 @@ SoPointSet::getPrimitiveCount(SoGetPrimitiveCountAction *action)
   if (!this->shouldPrimitiveCount(action)) return;
   int num = this->numPoints.getValue();
   if (num < 0) {
-    SoVertexProperty *vp = (SoVertexProperty*) this->vertexProperty.getValue();
+    SoNode *vpnode = this->vertexProperty.getValue();
+    SoVertexProperty *vp = 
+      (vpnode && vpnode->isOfType(SoVertexProperty::getClassTypeId())) ?
+      (SoVertexProperty *)vpnode : NULL;
     if (vp && vp->vertex.getNum()) {
       num = vp->vertex.getNum() - this->startIndex.getValue();
     }
