@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -74,6 +74,7 @@
 #include <Inventor/elements/SoLightModelElement.h>
 #include <Inventor/elements/SoModelMatrixElement.h>
 #include <Inventor/elements/SoNormalElement.h>
+#include <Inventor/elements/SoCoordinateElement.h>
 #include <Inventor/elements/SoProjectionMatrixElement.h>
 #include <Inventor/elements/SoShapeStyleElement.h>
 #include <Inventor/elements/SoTextureCoordinateElement.h>
@@ -89,7 +90,13 @@
 #include <Inventor/nodes/SoLight.h>
 #include <Inventor/nodes/SoSubNodeP.h>
 #include <Inventor/nodes/SoVertexShape.h>
+#include <Inventor/nodes/SoVertexProperty.h>
 #include <Inventor/threads/SbStorage.h>
+#include "../misc/SoVBO.h"
+#include <Inventor/elements/SoGLVBOElement.h>
+#include <Inventor/elements/SoMultiTextureEnabledElement.h>
+#include <Inventor/elements/SoMultiTextureCoordinateElement.h>
+
 #include <coindefs.h> // COIN_OBSOLETED()
 
 // SoShape.cpp grew too big, so I had to move some code into new
@@ -348,7 +355,7 @@ SoShape::initClass(void)
     soshape_use_gl_vertex_arrays = atoi(env);
   }
   SoShapeP::calibrateBBoxCache();
-  coin_atexit((coin_atexit_f*) soshape_cleanup, 0);
+  coin_atexit((coin_atexit_f*) soshape_cleanup, CC_ATEXIT_NORMAL);
 }
 
 // Doc in parent.
@@ -374,10 +381,25 @@ SoShape::GLRender(SoGLRenderAction * action)
   // implement the GLRender() method.  pederb, 20000612
 
   if (!this->shouldGLRender(action)) return;
+
+  // test for SoVertexShape node and push data onto the state before
+  // calling generatePrimitives(). This is needed for SoMaterialBundle
+  // to work correctly.
+  SoVertexProperty * vp = NULL;
+  if (this->isOfType(SoVertexShape::getClassTypeId())) {
+    vp = (SoVertexProperty*) ((SoVertexShape*)this)->vertexProperty.getValue();
+  }
+
+  if (vp) {
+    action->getState()->push();
+    vp->doAction(action);
+  }
   SoMaterialBundle mb(action);
   mb.sendFirst();
   soshape_get_staticdata()->currentbundle = &mb;  // needed in the primitive callbacks
   this->generatePrimitives(action);
+
+  if (vp) action->getState()->pop();
 }
 
 // Doc in parent.
@@ -537,8 +559,8 @@ SoShape::shouldGLRender(SoGLRenderAction * action)
 
   // test if we should sort triangles before rendering
   if (transparent && (shapestyleflags & SoShapeStyleElement::TRANSP_SORTED_TRIANGLES)) {
-    soshape_staticdata * shapedata = soshape_get_staticdata();
-
+    soshape_staticdata * shapedata = soshape_get_staticdata();    
+#if 0 // old code
     // do this before generating triangles to get correct
     // material for lines and point (only triangles are sorted).
     SoMaterialBundle mb(action);
@@ -554,6 +576,57 @@ SoShape::shouldGLRender(SoGLRenderAction * action)
     shapedata->rendermode = NORMAL;
 
     shapedata->trianglesort->endShape(state, mb); // this will render the triangles
+#else // new code that used the pvcache to render
+    // lock mutex since pvcache is shared among all threads
+    PRIVATE(this)->lock();
+    if (PRIVATE(this)->pvcache == NULL ||
+        !PRIVATE(this)->pvcache->isValid(state)) {
+      if (PRIVATE(this)->pvcache) {
+        PRIVATE(this)->pvcache->unref();
+      }
+      SbBool storedinvalid = SoCacheElement::setInvalid(FALSE);
+      // must push state to make cache dependencies work
+      state->push();
+      PRIVATE(this)->pvcache = new SoPrimitiveVertexCache(state);
+      PRIVATE(this)->pvcache->ref();
+      SoCacheElement::set(state, PRIVATE(this)->pvcache);
+      shapedata->rendermode = PVCACHE;
+      this->generatePrimitives(action);
+      shapedata->rendermode = NORMAL;
+      // this _must_ be called after creating the pvcache
+      state->pop();
+      SoCacheElement::setInvalid(storedinvalid);
+      PRIVATE(this)->pvcache->fit();
+      PRIVATE(this)->testSetupShapeHints(this);
+    }
+
+    int arrays = SoPrimitiveVertexCache::NORMAL|SoPrimitiveVertexCache::COLOR;
+    SoGLTextureImageElement::Model model;
+    SbColor blendcolor;
+    SoGLImage * glimage = SoGLTextureImageElement::get(state, model, blendcolor);
+    if (glimage) arrays |= SoPrimitiveVertexCache::TEXCOORD;
+    SoMaterialBundle mb(action);
+    mb.sendFirst();
+    PRIVATE(this)->setupShapeHints(this, state);
+    PRIVATE(this)->pvcache->depthSortTriangles(state);
+    PRIVATE(this)->pvcache->renderTriangles(state, arrays);
+    if (PRIVATE(this)->pvcache->getNumLineIndices() ||
+        PRIVATE(this)->pvcache->getNumPointIndices()) {
+      const SoNormalElement * nelem = SoNormalElement::getInstance(state);
+      if (nelem->getNum() == 0) {
+        glPushAttrib(GL_LIGHTING_BIT);
+        glDisable(GL_LIGHTING);
+        arrays &= SoPrimitiveVertexCache::NORMAL;
+      }
+      PRIVATE(this)->pvcache->renderLines(state, arrays);
+      PRIVATE(this)->pvcache->renderPoints(state, arrays);
+
+      if (nelem->getNum() == 0) {
+        glPopAttrib();
+      }
+    }
+    PRIVATE(this)->unlock();    
+#endif // end of new sorted triangles transparency rendering
     return FALSE; // tell shape _not_ to render
   }
   
@@ -726,7 +799,8 @@ SoShape::shouldGLRender(SoGLRenderAction * action)
   // wait some frames before trying to create the vertex array
   // cache. This will give Coin a chance to create display list caches
   // for parts of the scene graph.
-  if (soshape_use_gl_vertex_arrays && 
+  if (((shapestyleflags & SoShapeStyleElement::VERTEXARRAY) ||
+       soshape_use_gl_vertex_arrays) && 
       ((PRIVATE(this)->flags & SoShapeP::DISABLE_VERTEX_ARRAY_CACHE) == 0) &&
       cc_glglue_has_vertex_array(glue) &&
       (PRIVATE(this)->rendercnt >= SoShapeP::VERTEXARRAY_WAITCNT) &&
@@ -922,7 +996,8 @@ SoShape::createTriangleDetail(SoRayPickAction * action,
   if (shapedata->primdata->faceDetail) {
     return shapedata->primdata->createPickDetail();
   }
-#if COIN_DEBUG
+  // don't warn here. SoDetail instances are optional for extension nodes.
+#if COIN_DEBUG && 0
   SoDebugError::postInfo("SoShape::createTriangleDetail",
                          "Unable to create triangle detail.");
 #endif // COIN_DEBUG
@@ -955,7 +1030,8 @@ SoShape::createLineSegmentDetail(SoRayPickAction * action,
   if (shapedata->primdata->lineDetail) {
     return shapedata->primdata->createPickDetail();
   }
-#if COIN_DEBUG
+  // don't warn here. SoDetail instances are optional for extension nodes.
+#if COIN_DEBUG && 0
   SoDebugError::postInfo("SoShape::createLineSegmentDetail",
                          "Unable to create line segment detail.");
 #endif // COIN_DEBUG
@@ -1021,6 +1097,7 @@ SoShape::invokeTriangleCallbacks(SoAction * const action,
             v1->getTextureCoords() * barycentric[0] +
             v2->getTextureCoords() * barycentric[1] +
             v3->getTextureCoords() * barycentric[2];
+
           pp->setObjectTextureCoords(tc);
 
           // material index need to be approximated, since there is no
@@ -1230,7 +1307,7 @@ SoShape::invokePointCallbacks(SoAction * const action,
   beginShape() and endShape(). For instance, to generate primitives
   for a triangle you'd do something like this:
 
-  \begin verbatim
+  \verbatim
   SoPrimitiveVertex vertex;
 
   this->beginShape(action, SoShape::POLYGON);
@@ -1241,7 +1318,7 @@ SoShape::invokePointCallbacks(SoAction * const action,
   vertex.setPoint(SbVec3f(1.0f, 1.0f, 0.0f));
   this->shapeVertex(&vertex);
   this->endShape();
-  \end verbatim
+  \endverbatim
 
   Note that the SoPrimitiveVertex instance can simply be placed on the
   stack and not allocated. SoShape will copy the needed information
@@ -1474,6 +1551,8 @@ SoShape::getBBox(SoAction * action, SbBox3f & box, SbVec3f & center)
     PRIVATE(this)->bboxcache->unref();
     PRIVATE(this)->bboxcache = NULL;
     PRIVATE(this)->unlock();
+    // don't create bbox caches for shapes that change
+    PRIVATE(this)->flags &= ~SoShapeP::SHOULD_BBOX_CACHE;
   }
   
   SbBool shouldcache = (PRIVATE(this)->flags & SoShapeP::SHOULD_BBOX_CACHE) != 0;
@@ -1549,4 +1628,211 @@ SoShapeP::calibrateBBoxCache(void)
   SoShapeP::bboxcachetimelimit = end.getValue() - begin.getValue();
 }
 
+#ifdef COIN_NEXT_MINOR
+
+/*!
+  Convenience method that enables vertex arrays and/or VBOs
+  Returns \e TRUE if VBO is used.
+  
+  \sa finishVertexArray()
+  \since Coin 2.5
+*/
+SbBool 
+SoShape::startVertexArray(SoGLRenderAction * action,
+                          const SoCoordinateElement * coords,
+                          const SbVec3f * pervertexnormals,
+                          const SbBool texpervertex,
+                          const SbBool colorpervertex)
+{
+  SoState * state = action->getState();
+  const cc_glglue * glue = sogl_glue_instance(state);
+  const SoGLVBOElement * vboelem = SoGLVBOElement::getInstance(state);
+  const uint32_t contextid = action->getCacheContext();
+  SoVBO * vertexvbo = vboelem->getVertexVBO();
+  SbBool dovbo = TRUE;
+  if (!vertexvbo) dovbo = FALSE;
+  SbBool didbind = FALSE;
+
+  if (colorpervertex) {
+    const GLvoid * dataptr = NULL;
+    SoVBO * colorvbo = dovbo ? vboelem->getColorVBO() : NULL;
+    SoGLLazyElement * lelem = (SoGLLazyElement*) SoLazyElement::getInstance(state);
+    if (colorvbo) {
+      lelem->updateColorVBO(colorvbo);
+      colorvbo->bindBuffer(contextid);
+      didbind = TRUE;
+    }
+    else {
+      if (didbind) {
+        cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, 0);
+        didbind = FALSE;
+      }
+      dataptr = (const GLvoid*) lelem->getDiffusePointer();
+    }
+    if (colorvbo) {
+      cc_glglue_glColorPointer(glue, 4, GL_UNSIGNED_BYTE, 0, dataptr);
+    }
+    else {
+      cc_glglue_glColorPointer(glue, 3, GL_FLOAT, 0, dataptr);
+    }
+    cc_glglue_glEnableClientState(glue, GL_COLOR_ARRAY);
+  }
+  if (texpervertex) {
+    const SoTextureCoordinateElement * telem = NULL;
+    const SoMultiTextureCoordinateElement * mtelem = NULL;
+    const SbBool * enabledunits = NULL;
+    int lastenabled;
+    
+    telem = SoTextureCoordinateElement::getInstance(state);
+    enabledunits = SoMultiTextureEnabledElement::getEnabledUnits(state, lastenabled);
+    if (enabledunits) {
+      mtelem = SoMultiTextureCoordinateElement::getInstance(state);
+    }
+    SoVBO * vbo;
+    if (telem->getNum()) {
+      int dim = telem->getDimension();
+      const GLvoid * tptr;
+      switch (dim) {
+      default:
+      case 2: tptr = (const GLvoid*) telem->getArrayPtr2(); break;
+      case 3: tptr = (const GLvoid*) telem->getArrayPtr3(); break;
+      case 4: tptr = (const GLvoid*) telem->getArrayPtr4(); break;
+      }
+      vbo = dovbo ? vboelem->getTexCoordVBO(0) : NULL;
+      if (vbo) {
+        vbo->bindBuffer(contextid);
+        didbind = TRUE;
+        tptr = NULL;
+      }
+      else {
+        if (didbind) {
+          cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, 0);
+          didbind = FALSE;
+        }
+      }
+      cc_glglue_glTexCoordPointer(glue, dim, GL_FLOAT, 0, tptr);
+      cc_glglue_glEnableClientState(glue, GL_TEXTURE_COORD_ARRAY);
+    }
+    for (int i = 1; i <= lastenabled; i++) {
+      if (enabledunits[i] && mtelem->getNum(i)) {
+        int dim = mtelem->getDimension(i);
+        const GLvoid * tptr;
+        switch (dim) {
+        default:
+        case 2: tptr = (const GLvoid*) mtelem->getArrayPtr2(i); break;
+        case 3: tptr = (const GLvoid*) mtelem->getArrayPtr3(i); break;
+        case 4: tptr = (const GLvoid*) mtelem->getArrayPtr4(i); break;
+        }
+        cc_glglue_glClientActiveTexture(glue, GL_TEXTURE0 + i);
+        vbo = dovbo ? vboelem->getTexCoordVBO(i) : NULL;
+        if (vbo) {
+          vbo->bindBuffer(contextid);
+          didbind = TRUE;
+          tptr = NULL;
+        }
+        else {
+          if (didbind) {
+            cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, 0);
+            didbind = FALSE;
+          }
+        }
+        cc_glglue_glTexCoordPointer(glue, dim, GL_FLOAT, 0, tptr);
+        cc_glglue_glEnableClientState(glue, GL_TEXTURE_COORD_ARRAY);
+      }
+    }
+  }
+  if (pervertexnormals != NULL) {
+    SoVBO * vbo = dovbo ? vboelem->getNormalVBO() : NULL;
+    const GLvoid * dataptr = NULL;
+    if (vbo) {
+      vbo->bindBuffer(contextid);
+      didbind = TRUE;
+    }
+    else {
+      dataptr = (const GLvoid*) pervertexnormals;
+      if (didbind) {
+        cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, 0);
+        didbind = FALSE;
+      }
+    }
+    cc_glglue_glNormalPointer(glue, GL_FLOAT, 0, dataptr);
+    cc_glglue_glEnableClientState(glue, GL_NORMAL_ARRAY);
+  }
+  const GLvoid * dataptr = NULL;
+  if (vertexvbo) {
+    vertexvbo->bindBuffer(contextid);
+  }
+  else {
+    dataptr = coords->is3D() ? 
+      ((const GLvoid *)coords->getArrayPtr3()) : 
+      ((const GLvoid *)coords->getArrayPtr4());
+  }
+  cc_glglue_glVertexPointer(glue, coords->is3D() ? 3 : 4, GL_FLOAT, 0,
+                            dataptr);
+  cc_glglue_glEnableClientState(glue, GL_VERTEX_ARRAY);
+  
+  return dovbo;
+}
+
+/*!  
+  Should be called after rendering with vertex arrays. This method
+  will disable arrays and VBOs enabled in the startVertexArray()
+  function.
+  
+  \sa startVertexArray()
+  \since Coin 2.5
+*/
+void 
+SoShape::finishVertexArray(SoGLRenderAction * action,
+                           const SbBool vbo,
+                           const SbBool normpervertex,
+                           const SbBool texpervertex,
+                           const SbBool colorpervertex)
+{
+  SoState * state = action->getState();
+  const cc_glglue * glue = sogl_glue_instance(state);
+
+  if (vbo) {
+    if (!coin_glglue_vbo_in_displaylist_supported(glue)) {
+      SoCacheElement::invalidate(state);
+      SoGLCacheContextElement::shouldAutoCache(state, 
+                                               SoGLCacheContextElement::DONT_AUTO_CACHE);
+    }
+    // unset VBO buffer
+    cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, 0);
+  }
+  cc_glglue_glDisableClientState(glue, GL_VERTEX_ARRAY);
+  if (normpervertex) {
+    cc_glglue_glDisableClientState(glue, GL_NORMAL_ARRAY);
+  }
+  if (texpervertex) {
+    const SoTextureCoordinateElement * telem = NULL;
+    const SoMultiTextureCoordinateElement * mtelem = NULL;
+    const SbBool * enabledunits = NULL;
+    int lastenabled;
+
+    telem = SoTextureCoordinateElement::getInstance(state);
+    enabledunits = SoMultiTextureEnabledElement::getEnabledUnits(state, lastenabled);
+    
+    for (int i = 1; i <= lastenabled; i++) {
+      if (enabledunits[i] && mtelem->getNum(i)) {
+        cc_glglue_glClientActiveTexture(glue, GL_TEXTURE0 + i);
+        cc_glglue_glDisableClientState(glue, GL_TEXTURE_COORD_ARRAY);
+      }
+    }
+    cc_glglue_glClientActiveTexture(glue, GL_TEXTURE0);
+    if (telem->getNum()) {
+      cc_glglue_glDisableClientState(glue, GL_TEXTURE_COORD_ARRAY);
+    }
+  }
+  if (colorpervertex) {
+    SoGLLazyElement * lelem = (SoGLLazyElement*) SoLazyElement::getInstance(state);
+    lelem->reset(state, SoLazyElement::DIFFUSE_MASK);
+    
+    cc_glglue_glDisableClientState(glue, GL_COLOR_ARRAY);
+  }
+}
+
+#endif // COIN_NEXT_MINOR
+
 #undef PRIVATE
