@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -123,6 +123,9 @@
 #include <Inventor/sensors/SoFieldSensor.h>
 #include <Inventor/caches/SoGlyphCache.h>
 #include <Inventor/elements/SoCacheElement.h>
+#include <Inventor/elements/SoGLTextureEnabledElement.h>
+#include <Inventor/elements/SoGLTexture3EnabledElement.h>
+#include <Inventor/elements/SoGLMultiTextureEnabledElement.h>
 
 #ifdef COIN_THREADSAFE
 #include <Inventor/threads/SbMutex.h>
@@ -150,8 +153,6 @@
 
 #include "../fonts/glyph2d.h"
 
-static const unsigned int NOT_AVAILABLE = UINT_MAX;
-
 /*!
   \enum SoText2::Justification
 
@@ -206,6 +207,8 @@ public:
   SoGlyphCache * cache;
   SoFieldSensor * spacingsensor;
   SoFieldSensor * stringsensor;
+  unsigned char * pixel_buffer;
+  int pixel_buffer_size;
 
   static void sensor_cb(void * userdata, SoSensor * s) {
     SoText2P * thisp = (SoText2P*) userdata;
@@ -264,6 +267,8 @@ SoText2::SoText2(void)
   PRIVATE(this)->spacingsensor->attach(&this->spacing);
   PRIVATE(this)->spacingsensor->setPriority(0);
   PRIVATE(this)->cache = NULL;
+  PRIVATE(this)->pixel_buffer = NULL;
+  PRIVATE(this)->pixel_buffer_size = 0;
 }
 
 /*!
@@ -272,6 +277,7 @@ SoText2::SoText2(void)
 SoText2::~SoText2()
 {
   if (PRIVATE(this)->cache) PRIVATE(this)->cache->unref();
+  delete[] PRIVATE(this)->pixel_buffer;
   delete PRIVATE(this)->stringsensor;
   delete PRIVATE(this)->spacingsensor;
 
@@ -347,6 +353,27 @@ SoText2::GLRender(SoGLRenderAction * action)
     
     const int nrlines = this->string.getNum();
 
+    // get the current glColor
+    float ftglColor[4];
+    glGetFloatv(GL_CURRENT_COLOR, ftglColor);
+
+    unsigned char red   = (unsigned char) (ftglColor[0] * 255.0f);
+    unsigned char green = (unsigned char) (ftglColor[1] * 255.0f);
+    unsigned char blue  = (unsigned char) (ftglColor[2] * 255.0f);
+
+    state->push();
+    
+    // disable textures for all units
+    SoGLTextureEnabledElement::set(state, this, FALSE);
+    SoGLTexture3EnabledElement::set(state, this, FALSE);
+#ifdef COIN_NEXT_MINOR
+    SoGLMultiTextureEnabledElement::disableAll(state);
+#endif // COIN_NEXT_MINOR
+    glPushAttrib(GL_ENABLE_BIT | GL_PIXEL_MODE_BIT | GL_COLOR_BUFFER_BIT);
+    glPushClientAttrib(GL_CLIENT_PIXEL_STORE_BIT);
+    
+    SbBool didenableblend = FALSE;
+
     for (int i = 0; i < nrlines; i++) {
       SbString str = this->string[i];
       switch (this->justification.getValue()) {
@@ -378,7 +405,8 @@ SoText2::GLRender(SoGLRenderAction * action)
         int kerningx, kerningy;
         if (strcharidx > 0) {
           cc_glyph2d_getkerning(prevglyph, glyph, &kerningx, &kerningy);
-        } else {
+        } 
+        else {
           kerningx = 0;
           kerningy = 0;          
         }
@@ -399,34 +427,35 @@ SoText2::GLRender(SoGLRenderAction * action)
 
         if (buffer) {
           if (cc_glyph2d_getmono(glyph)) {
+            if (didenableblend) {
+              glDisable(GL_BLEND);
+              glDisable(GL_ALPHA_TEST);
+              didenableblend = FALSE;
+            }
             glBitmap(ix,iy,0,0,0,0,(const GLubyte *)buffer);
-          } else {
-            glPushAttrib(GL_ENABLE_BIT | GL_PIXEL_MODE_BIT | GL_COLOR_BUFFER_BIT);
-            glPushClientAttrib(GL_CLIENT_PIXEL_STORE_BIT);
+          } 
+          else {
+            if (!didenableblend) {
+              glEnable(GL_ALPHA_TEST);
+              glAlphaFunc(GL_GREATER, 0.3f);
           
-            glEnable(GL_ALPHA_TEST);
-            glAlphaFunc(GL_GREATER, 0.3f);
-          
-            glEnable(GL_BLEND);
-            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-          
-            glDisable(GL_TEXTURE_2D);
-
+              glEnable(GL_BLEND);
+              glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+              didenableblend = TRUE;
+            }
             glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
         
-            // get the current glColor
-            float ftglColor[4];
-            glGetFloatv(GL_CURRENT_COLOR, ftglColor);
-
-            unsigned char red   = (unsigned char) (ftglColor[0] * 255.0f);
-            unsigned char green = (unsigned char) (ftglColor[1] * 255.0f);
-            unsigned char blue  = (unsigned char) (ftglColor[2] * 255.0f);
-
             int numpixels = ix * iy;
-            unsigned char * data = (unsigned char *) malloc(numpixels * 4);
-            unsigned char * dst = data;
+ 
+            if (numpixels > PRIVATE(this)->pixel_buffer_size) {
+              delete[] PRIVATE(this)->pixel_buffer;
+              PRIVATE(this)->pixel_buffer = new unsigned char[numpixels*4];
+              PRIVATE(this)->pixel_buffer_size = numpixels;
+            }
+            unsigned char * dst = PRIVATE(this)->pixel_buffer;
             const unsigned char * src = buffer;
 
+            // Ouch. This must lead to pretty slow rendering
             if (ftglColor[3] == 1.0f) {
               for (int i = 0; i < numpixels; i++) {
                 *dst++ = red; *dst++ = green; *dst++ = blue;
@@ -440,13 +469,8 @@ SoText2::GLRender(SoGLRenderAction * action)
                 *dst++ = (((unsigned int)(ftglColor[3] * 256.0f)) * *src++) >> 8;
               }
             }
+            glDrawPixels(ix,iy,GL_RGBA,GL_UNSIGNED_BYTE,(const GLubyte *)PRIVATE(this)->pixel_buffer);
 
-            glDrawPixels(ix,iy,GL_RGBA,GL_UNSIGNED_BYTE,(const GLubyte *)data);
-
-            free(data);
-
-            glPopClientAttrib();
-            glPopAttrib();
           }
         }
 
@@ -468,9 +492,14 @@ SoText2::GLRender(SoGLRenderAction * action)
       // instance
       cc_glyph2d_unref(prevglyph);
     }
+
+    // pop old state
+    glPopClientAttrib();
+    glPopAttrib();
+    state->pop();
       
     glPixelStorei(GL_UNPACK_ALIGNMENT,4);
-    // Pop old GL state.
+    // Pop old GL matrix state.
     glMatrixMode(GL_PROJECTION);
     glPopMatrix();
     glMatrixMode(GL_MODELVIEW);
