@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -403,6 +403,8 @@ soshape_bumprender::renderBumpSpecular(SoState * state,
   // A check for fragment- and vertex-program support has already
   // been done in SoShape::shouldGLRender().
   //
+  const int n = cache->getNumIndices();
+  if (n == 0) return;
 
   const cc_glglue * glue = sogl_glue_instance(state);
   const SbColor spec = SoLazyElement::getSpecular(state);
@@ -496,7 +498,6 @@ soshape_bumprender::renderBumpSpecular(SoState * state,
 
   cc_glglue_glActiveTexture(glue, GL_TEXTURE0);
 
-  const int n = cache->getNumIndices();
   const SbVec3f * cmptr = this->cubemaplist.getArrayPtr();
   const SbVec3f * tptr = this->tangentlist.getArrayPtr();
 
@@ -582,6 +583,9 @@ soshape_bumprender::renderBump(SoState * state,
                                const SoPrimitiveVertexCache * cache,
                                SoLight * light, const SbMatrix & toobjectspace)
 {
+  const int n = cache->getNumIndices();
+  if (n == 0) return;
+
   this->initLight(light, toobjectspace);
 
   const cc_glglue * glue = sogl_glue_instance(state);
@@ -647,7 +651,6 @@ soshape_bumprender::renderBump(SoState * state,
   glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_DOT3_RGB);
   glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB, GL_PREVIOUS);
 
-  const int n = cache->getNumIndices();
   const SbVec3f * cmptr = this->cubemaplist.getArrayPtr();
   const SbVec3f * tsptr = this->tangentlist.getArrayPtr();
 
@@ -769,6 +772,9 @@ void
 soshape_bumprender::calcTangentSpace(const SoPrimitiveVertexCache * cache)
 {
   int i;
+  const int numi = cache->getNumIndices();
+  if (numi == 0) return;
+
 
   const int numv = cache->getNumVertices();
   const int32_t * idxptr = cache->getIndices();
@@ -782,7 +788,6 @@ soshape_bumprender::calcTangentSpace(const SoPrimitiveVertexCache * cache)
     this->tangentlist.append(SbVec3f(0.0f, 0.0f, 0.0f));
     this->tangentlist.append(SbVec3f(0.0f, 0.0f, 0.0f));
   }
-  const int numi = cache->getNumIndices();
 
   SbVec3f sTangent;
   SbVec3f tTangent;
