@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,22 +14,29 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
 #include "soshape_primdata.h"
-#include <Inventor/actions/SoAction.h>
+
+#include <string.h>
+
+#include <Inventor/SbTesselator.h>
 #include <Inventor/SoPrimitiveVertex.h>
+#include <Inventor/actions/SoAction.h>
 #include <Inventor/details/SoFaceDetail.h>
 #include <Inventor/details/SoLineDetail.h>
 #include <Inventor/details/SoPointDetail.h>
-#include <Inventor/SbTesselator.h>
+#include <Inventor/elements/SoMaterialBindingElement.h>
+#include <Inventor/elements/SoNormalBindingElement.h>
 #include <Inventor/elements/SoShapeHintsElement.h>
-#include <string.h>
+#include "../base/SbGLUTessellator.h"
+
+// *************************************************************************
 
 soshape_primdata::soshape_primdata(void)
 {
@@ -40,9 +47,20 @@ soshape_primdata::soshape_primdata(void)
   this->arraySize = 4;
   this->vertsArray = new SoPrimitiveVertex[this->arraySize];
   this->pointDetails = new SoPointDetail[this->arraySize];
-  this->tess = new SbTesselator();
   this->faceDetail = NULL;
   this->lineDetail = NULL;
+  this->matPerFace = FALSE;
+  this->normPerFace = FALSE;
+
+  this->tess = NULL;
+  this->glutess = NULL;
+
+  if (SbGLUTessellator::preferred()) {
+    this->glutess = new SbGLUTessellator(soshape_primdata::tess_callback, this);
+  }
+  else {
+    this->tess = new SbTesselator(soshape_primdata::tess_callback, this);
+  }
 }
 
 soshape_primdata::~soshape_primdata()
@@ -50,8 +68,11 @@ soshape_primdata::~soshape_primdata()
   delete[] this->vertsArray;
   delete[] this->pointDetails;
   delete this->tess;
+  delete this->glutess;
 }
 
+// *************************************************************************
+
 void
 soshape_primdata::beginShape(SoShape * shapeptr, SoAction * actionptr,
                                SoShape::TriangleShape shapetypearg,
@@ -65,6 +86,22 @@ soshape_primdata::beginShape(SoShape * shapeptr, SoAction * actionptr,
   this->faceDetail = (SoFaceDetail *)detail;
   this->lineDetail = (SoLineDetail *)detail;
   this->counter = 0;
+
+  SoState * state = action->getState();
+
+  SoMaterialBindingElement::Binding mbind = SoMaterialBindingElement::get(state);
+  SoNormalBindingElement::Binding nbind = SoNormalBindingElement::get(state);
+
+  // need to test for PER_FACE bindings since special rules need to followed 
+  // to get the correct per vertex material and normal indices in these cases
+  // (basically the same rules as when sending geometry to OpenGL)
+  this->matPerFace = 
+    (mbind == SoMaterialBindingElement::PER_FACE) ||
+    (mbind == SoMaterialBindingElement::PER_FACE_INDEXED);
+  this->normPerFace = 
+    (nbind == SoNormalBindingElement::PER_FACE) ||
+    (nbind == SoNormalBindingElement::PER_FACE_INDEXED);
+    
 }
 
 void
@@ -83,12 +120,24 @@ soshape_primdata::endShape(void)
       }
     }
     else {
-      this->tess->setCallback(soshape_primdata::tess_callback, this);
-      this->tess->beginPolygon(TRUE);
-      for (int i = 0; i < counter; i++) {
-        this->tess->addVertex(vertsArray[i].getPoint(), &vertsArray[i]);
+      if (SbGLUTessellator::preferred()) {
+        this->glutess->beginPolygon();
+        for (int i = 0; i < counter; i++) {
+          this->glutess->addVertex(vertsArray[i].getPoint(), &vertsArray[i]);
+        }
+        this->glutess->endPolygon();
+      }
+      else {
+        // FIXME: the keepVertices==TRUE setting may not be necessary,
+        // according to pederb. (The flag causes us to get callbacks
+        // even on empty polygons -- probably not useful in Coin, as
+        // it was for Rational Reducer.)  20060216 mortene.
+        this->tess->beginPolygon(TRUE);
+        for (int i = 0; i < counter; i++) {
+          this->tess->addVertex(vertsArray[i].getPoint(), &vertsArray[i]);
+        }
+        this->tess->endPolygon();
       }
-      this->tess->endPolygon();
     }
   }
 }
@@ -180,6 +229,8 @@ soshape_primdata::shapeVertex(const SoPrimitiveVertex * const v)
   case SoShape::QUAD_STRIP:
     this->setVertex(this->counter++, v);
     if (counter == 4) {
+      if (this->matPerFace) this->copyMaterialIndex(3);
+      if (this->normPerFace) this->copyNormalIndex(3);
       // can't use handleFaceDetail(), because of the vertex
       // order.
       if (this->faceDetail) {
@@ -260,6 +311,11 @@ soshape_primdata::setVertex(const int idx, const SoPrimitiveVertex * const v)
 void
 soshape_primdata::handleFaceDetail(const int numv)
 {
+  // if PER_FACE binding, copy indices from the last vertex we
+  // received to the other vertices
+  if (this->matPerFace) this->copyMaterialIndex(numv-1);
+  if (this->normPerFace) this->copyNormalIndex(numv-1);
+
   if (this->faceDetail) {
     this->faceDetail->setNumPoints(numv);
     for (int i = 0; i < numv; i++) {
@@ -275,6 +331,8 @@ soshape_primdata::handleLineDetail(void)
   if (this->lineDetail) {
     this->lineDetail->setPoint0(&this->pointDetails[0]);
     this->lineDetail->setPoint1(&this->pointDetails[1]);
+    this->vertsArray[0].setDetail(this->lineDetail);
+    this->vertsArray[1].setDetail(this->lineDetail);
   }
 }
 
@@ -348,3 +406,24 @@ soshape_primdata::tess_callback(void * v0, void * v1, void * v2, void * data)
                                         (SoPrimitiveVertex *)v1,
                                         (SoPrimitiveVertex *)v2);
 }
+
+void 
+soshape_primdata::copyMaterialIndex(const int lastvertex)
+{
+  int i;
+  int matidx = this->vertsArray[lastvertex].getMaterialIndex();
+  for (i = 0; i < lastvertex; i++) {
+    this->vertsArray[i].setMaterialIndex(matidx);
+    this->pointDetails[i].setMaterialIndex(matidx);
+  } 
+}
+
+void 
+soshape_primdata::copyNormalIndex(const int lastvertex)
+{
+  int i;
+  int normidx = this->pointDetails[lastvertex].getNormalIndex();
+  for (i = 0; i < lastvertex; i++) {
+    this->pointDetails[i].setNormalIndex(normidx);
+  } 
+}
