@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -17,10 +17,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -39,7 +39,6 @@ class SoPrimitiveVertex;
 class SoPointDetail;
 class SoFaceDetail;
 class SoLineDetail;
-class SbTesselator;
 
 class soshape_primdata {
 public:
@@ -64,6 +63,9 @@ private:
   SoDetail * createPickDetail(void);
   static void tess_callback(void * v0, void * v1, void * v2, void * data);
 
+  void copyMaterialIndex(const int lastvertex);
+  void copyNormalIndex(const int lastvertex);
+
 private:
   friend class SoShape;
 
@@ -76,8 +78,12 @@ private:
   SoLineDetail * lineDetail;
   int arraySize;
   int counter;
-  SbTesselator * tess;
+  class SbTesselator * tess;
+  class SbGLUTessellator * glutess;
   int faceCounter;
+
+  SbBool matPerFace;
+  SbBool normPerFace;
 };
 
 
