@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,13 +14,23 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
+/* FIXME: should provide dummy implementations of the (few) internal
+   public cc_mutex_*() calls, so one can include the header files
+   mutex.h and SbMutex.h without #ifdef checks, and also declare
+   e.g. SbMutex instances when thread-support is missing.
+
+   This would clean up source code everywhere we're using mutex'es.
+
+   20050516 mortene.
+*/
+
 #include <Inventor/C/threads/mutex.h>
 #include <Inventor/C/threads/mutexp.h>
 
@@ -39,8 +49,8 @@
 #ifdef USE_W32THREAD
 /* we test if Win32 TryEnterCriticalSection exists, and use Win32
    critical section if it does, and Win32 mutex if it doesn't */
-typedef BOOL cc_mutex_TryEnterCriticalSection_func(LPCRITICAL_SECTION);
-static cc_mutex_TryEnterCriticalSection_func * cc_mutex_TryEnterCriticalSection = NULL; 
+typedef BOOL (WINAPI * cc_mutex_TryEnterCriticalSection_func)(LPCRITICAL_SECTION);
+static cc_mutex_TryEnterCriticalSection_func cc_mutex_TryEnterCriticalSection = NULL; 
 #include "mutex_win32mutex.ic" 
 #include "mutex_win32cs.ic" 
 #endif /* USE_W32THREAD */
@@ -198,16 +208,20 @@ cc_mutex_init(void)
   /* This function is unsupported in Win95/98/Me and NT <=3.51, but we
      still want to use it if it's available, since it can provide
      major speed-ups for certain aspects of Win32 mutex handling. */
-  cc_mutex_TryEnterCriticalSection = (cc_mutex_TryEnterCriticalSection_func *)
+  cc_mutex_TryEnterCriticalSection = (cc_mutex_TryEnterCriticalSection_func)
     GetProcAddress(h, "TryEnterCriticalSection");
   
 #endif /* USE_W32THREAD */
 
   if (cc_global_mutex == NULL) {
     cc_global_mutex = cc_mutex_construct();
-    /* use -1000 as atexit priority to make this callback trigger
-       after other cleanup functions. */
-    coin_atexit((coin_atexit_f*) cc_mutex_cleanup, -1000);
+    /* atexit priority makes this callback trigger after other cleanup
+       functions. */
+    /* FIXME: not sure if this really needs the "- 1", but I added it
+       to keep the same order wrt the other thread-related clean-up
+       functions, since before I changed hard-coded numbers for
+       enumerated values for coin_atexit() invocations. 20060301 mortene. */
+    coin_atexit((coin_atexit_f*) cc_mutex_cleanup, CC_ATEXIT_THREADING_SUBSYSTEM - 1);
   }
 }
 
