@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,23 +14,48 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
 #include <Inventor/C/threads/sched.h>
-#include <Inventor/C/threads/schedp.h>
-#include <Inventor/C/threads/wpool.h>
-#include <Inventor/C/threads/mutex.h>
-#include <Inventor/C/threads/condvar.h>
-#include <Inventor/C/threads/thread.h>
+#include <assert.h>
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif /* HAVE_CONFIG_H */
+
+#ifndef HAVE_THREADS
+
+/* FIXME: instead of disallowing the use of these completely when
+   thread support is not available (that's why there are asserts
+   within them), could we perhaps implement in such a manner that they
+   still work, but within only the calling thread?
+
+   20051202 mortene. */
+
+cc_sched * cc_sched_construct(int numthreads) { assert(FALSE); return NULL; }
+void cc_sched_destruct(cc_sched * sched) { assert(FALSE); }
+void cc_sched_set_num_threads(cc_sched * sched, int num) { assert(FALSE); }
+int cc_sched_get_num_threads(cc_sched * sched) { assert(FALSE); return 0; }
+void cc_sched_schedule(cc_sched * sched, 
+                       void (*workfunc)(void *), void * closure,
+                       unsigned int priority) { assert(FALSE); }
+void cc_sched_wait_all(cc_sched * sched) { assert(FALSE); }
+
+#else /* HAVE_THREADS */
+
+#include <stdlib.h>
 
 #include <Inventor/C/errors/debugerror.h>
-#include <stdlib.h>
-#include <assert.h>
+#include <Inventor/C/threads/condvar.h>
+#include <Inventor/C/threads/mutex.h>
+#include <Inventor/C/threads/schedp.h>
+#include <Inventor/C/threads/thread.h>
+#include <Inventor/C/threads/wpool.h>
 
 /* ********************************************************************** */
 
@@ -191,3 +216,5 @@ cc_sched_wait_all(cc_sched * sched)
 #ifdef __cplusplus
 } /* extern "C" */
 #endif /* __cplusplus */
+
+#endif /* HAVE_THREADS */
