@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -50,6 +50,7 @@ sync_cleanup(void)
 {
   cc_dict_apply(sync_hash_table, sync_hash_cb, NULL);
   cc_dict_destruct(sync_hash_table);
+  sync_hash_table = NULL;
 }
 
 /*
@@ -59,10 +60,10 @@ void
 cc_sync_init(void)
 {
   if (sync_hash_table == NULL) {
-    /* use -100 as atexit priority to make this callback trigger after
-       normal cleanup function which might still use a cc_sync
+    /* the priority is set so to make this callback trigger late,
+       after normal cleanup function which might still use a cc_sync
        instance */
-    coin_atexit((coin_atexit_f*) sync_cleanup, -100);
+    coin_atexit((coin_atexit_f*) sync_cleanup, CC_ATEXIT_THREADING_SUBSYSTEM);
     sync_hash_table = cc_dict_construct(256, 0.75f);    
   }
 }
