@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -38,6 +38,7 @@
 #include <locale.h>
 #include <string.h> /* strncasecmp() */
 #include <stdio.h>
+#include <stdlib.h> /* atio() */
 #include <ctype.h> /* tolower() */
 #include <stdlib.h> /* atexit(), putenv(), qsort(), atof() */
 #ifdef HAVE_WINDOWS_H
@@ -67,6 +68,14 @@
 #include <Inventor/C/base/list.h>
 #include <Inventor/C/errors/debugerror.h>
 
+#ifdef HAVE__STRDUP
+#define strdup(a) _strdup(a)
+#endif /* HAVE__STRDUP */
+
+#ifdef HAVE__FDOPEN
+#define fdopen(a,b) _fdopen(a,b)
+#endif /* HAVE__FDOPEN */
+
 /**************************************************************************/
 
 /* FIXME: we should grab some BSD/PD-style licensed code to provide a
@@ -148,6 +157,10 @@ extern "C" {
 static cc_mutex * atexit_list_monitor = NULL;
 #endif /* COIN_THREADSAFE */
 
+
+static int COIN_DEBUG_EXTRA = -1;
+static int COIN_DEBUG_NORMALIZE = -1;
+
 /* ********************************************************************** */
 
 /* Called from SoDB::init() exactly once, a call which is guaranteed
@@ -155,9 +168,25 @@ static cc_mutex * atexit_list_monitor = NULL;
 void
 coin_init_tidbits(void)
 {
+  const char * env;
 #ifdef COIN_THREADSAFE
   atexit_list_monitor = cc_mutex_construct();
 #endif /* COIN_THREADSAFE */
+
+  env  = coin_getenv("COIN_DEBUG_EXTRA");
+  if (env && atoi(env) == 1) {
+    COIN_DEBUG_EXTRA = 1;
+  }
+  else {
+    COIN_DEBUG_EXTRA = 0;
+  }
+  env = coin_getenv("COIN_DEBUG_NORMALIZE");
+  if (env && atoi(env) == 1) {
+    COIN_DEBUG_NORMALIZE = 1;
+  }
+  else {
+    COIN_DEBUG_NORMALIZE = 0;
+  }
 }
 
 /* ********************************************************************** */
@@ -261,8 +290,7 @@ coin_common_vsnprintf(func_vsnprintf * func,
 int
 coin_vsnprintf(char * dst, unsigned int n, const char * fmtstr, va_list args)
 {
-  return coin_common_vsnprintf((func_vsnprintf *) vsnprintf, dst,
-                               (size_t)n, fmtstr, args);
+  return coin_common_vsnprintf(vsnprintf, dst, (size_t)n, fmtstr, args);
 }
 
 #elif defined HAVE__VSNPRINTF
@@ -441,6 +469,8 @@ envlist_cleanup(void)
     ptr = ptr->next;
     free(tmp);
   }
+  envlist_head = NULL;
+  envlist_tail = NULL;
 }
 
 #else
@@ -504,9 +534,25 @@ envlist_append(struct envvar_data * item)
 const char *
 coin_getenv(const char * envname)
 {
+  /* FIXME: with recent version(s) of Cygwin, it seems the Win32
+     GetEnvironmentVariable() is no longer working properly, if Coin
+     has been built with the Cygwin-port of GCC. We've had one bug
+     report to this effect, plus handegar had to fix wrapmsvc.cpp,
+     which exhibited the exact same problem.
+
+     Not 100% sure how to fix it, but perhaps always preferring to use
+     getenv() over GetEnvironmentVariable() would be sufficient? Or
+     would that just exhibit the opposite problem when building Coin
+     with MSVC?
+
+     20060314 mortene. */
+
   /* Important note: this code is identical to the getenv() code in
      So@Gui@/.../common/SoAny.cpp.in. If you do bugfixes or whatever,
      keep them in sync! */
+  /* FIXME: this code from tidbits should be moved under
+     Coin/src/share/, so the actual source code is shared instead of
+     having to copy it around. 20060314 mortene. */
 #ifdef HAVE_GETENVIRONMENTVARIABLE
   int neededsize;
   neededsize = GetEnvironmentVariable(envname, NULL, 0);
@@ -1146,7 +1192,11 @@ coin_atexit_cleanup(void)
   for (i = n-1; i >= 0; i--) {
     data = (tb_atexit_data*) cc_list_get(atexit_list, i);
     if (debug) {
-      cc_debugerror_postinfo("coin_atexit_cleanup", "invoking %s()", data->name);
+      /* Can't use cc_debugerror_postinfo() here, since this will
+          allocate static data that need to be cleaned up, resulting
+          in a call to coin_atexit() while we are already exiting...
+      */
+      fprintf(stdout, "coin_atexit_cleanup: invoking %s()\n", data->name);
     }
     data->func();
     free(data->name);
@@ -1161,7 +1211,9 @@ coin_atexit_cleanup(void)
   atexit_list_monitor = NULL;
 #endif /* COIN_THREADSAFE */
 
-  if (debug) { cc_debugerror_postinfo("coin_atexit_cleanup", "fini"); }
+  if (debug) {
+    fprintf(stdout, "coin_atexit_cleanup: fini\n");
+  }
 }
 
 /*
@@ -1250,8 +1302,11 @@ coin_atexit_func(const char * name, coin_atexit_f * f, int32_t priority)
 
 #ifdef COIN_NEXT_MINOR
 /*
-  Public version of the coin_atexit function which always sets the priority
-  to -2.
+  Public version of the coin_atexit function which always sets the
+  priority such that external clean-up functions are run \e first.
+
+  (Which is of course important because they may have dependencies
+  into Coin functionality.)
 
   Note that the registered atexit functions will only be called when
   SoDB::finish() is invoked from the application code.
@@ -1259,7 +1314,7 @@ coin_atexit_func(const char * name, coin_atexit_f * f, int32_t priority)
 void
 cc_coin_atexit(coin_atexit_f * f)
 {
-  coin_atexit(f, -2);
+  coin_atexit(f, CC_ATEXIT_EXTERNAL);
 }
 #endif // COIN_NEXT_MINOR
 
@@ -1685,7 +1740,63 @@ coin_geq_prime_number(unsigned long num)
       return coin_prime_table[i];
     }
   }
-  return coin_prime_table[31];
+  /* just return num if we can't find a bigger prime number */
+  return num;
+}
+
+/**************************************************************************/
+
+int 
+coin_runtime_os(void)
+{
+  /*
+   * FIXME: this implementation should be replaced by something that
+   * does a runtime check. kyrah suggested using sysctl() or uname()
+   * (both part of the C runtime library) which sounds like a good
+   * idea. pederb, 20051101
+   *
+   */
+#if defined(__APPLE__)
+  return COIN_OS_X;
+#elif defined(HAVE_WIN32_API)
+  return COIN_MSWINDOWS;
+#else
+  return COIN_UNIX;
+#endif
+}
+
+/**************************************************************************/
+
+/*
+ * Will return TRUE if extra debugging information is enabled. These
+ * are typically debugging messages extra for Coin and not found in
+ * SGI Inventor. Also, these debugging message will not necessarily
+ * mean that anything is wrong, but they can be useful for debugging
+ * anyway.
+ */
+int 
+coin_debug_extra(void)
+{
+#if COIN_DEBUG
+  return COIN_DEBUG_EXTRA;
+#else /* COIN_DEBUG */
+  return 0;
+#endif /* !COIN_DEBUG */ 
+}
+
+/*
+ * Will return TRUE if SbVec*::normalize() debugging is enabled.
+ * This can be enabled using the COIN_DEBUG_NORMALIZE environment
+ * variable.
+ */
+int 
+coin_debug_normalize(void)
+{
+#if COIN_DEBUG
+  return COIN_DEBUG_NORMALIZE;
+#else /* COIN_DEBUG */
+  return 0;
+#endif /* !COIN_DEBUG */
 }
 
 /**************************************************************************/
@@ -1693,3 +1804,4 @@ coin_geq_prime_number(unsigned long num)
 #ifdef __cplusplus
 } /* extern "C" */
 #endif
+
