@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -63,11 +63,11 @@
   children are chosen. A description of how multiple Anchors and
   pointing-device sensors are resolved on activation is contained in
   4.6.7, Sensor nodes
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.7>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.7>).
 
   More details on the children, addChildren, and removeChildren fields
   and eventIns can be found in 4.6.5, Grouping and children nodes
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.5>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.5>).
 
   The description field in the Anchor node specifies a textual
   description of the Anchor node. This may be used by browser-specific
@@ -130,7 +130,7 @@
   the Viewpoint is not found, no action occurs on activation.  More
   details on the url field are contained in 4.5, VRML and the World
   Wide Web
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.5>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.5>).
   The bboxCenter and bboxSize fields specify a bounding box
   that encloses the Anchor's children. This is a hint that may be used
   for optimization purposes. The results are undefined if the
@@ -139,7 +139,7 @@
   implies that the bounding box is not specified and if needed shall
   be calculated by the browser. More details on the bboxCenter and
   bboxSize fields can be found in 4.6.4, Bounding boxes
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.4>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.4>).
 
 */
 
@@ -194,6 +194,8 @@ void
 SoVRMLAnchor::initClass(void) // static
 {
   SO_NODE_INTERNAL_INIT_CLASS(SoVRMLAnchor, SO_VRML97_NODE_TYPE);
+  SoVRMLAnchor::fetchurlcb = NULL;
+  SoVRMLAnchor::userdata = NULL;
 }
 
 
