@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -51,7 +51,7 @@
   (all lights are ignored during rendering of the object that
   references this Appearance) and the unlit object colour is (1, 1,
   1). Details of the VRML lighting model are in 4.14, Lighting model
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.14>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.14>).
 
   The texture field, if specified, shall contain one of the various
   types of texture nodes (VRMLImageTexture, VRMLMovieTexture, or
@@ -93,11 +93,12 @@
 #include <Inventor/elements/SoTextureQualityElement.h>
 #include <Inventor/elements/SoTextureImageElement.h>
 
-#ifdef COIN_THREADSAFE
+#ifdef HAVE_THREADS
 #include <Inventor/threads/SbMutex.h>
-#endif // COIN_THREADSAFE
+#endif // HAVE_THREADS
+
+// *************************************************************************
 
-#ifndef DOXYGEN_SKIP_THIS
 class SoVRMLAppearanceP {
 public:
   SoChildList * childlist;
@@ -105,25 +106,23 @@ public:
 
 #ifdef COIN_THREADSAFE
   SbMutex mutex;
-#endif // COIN_THREADSAFE
-  void lock(void) {
-#ifdef COIN_THREADSAFE
-    this->mutex.lock();
-#endif // COIN_THREADSAFE
-  }
-  void unlock(void) {
-#ifdef COIN_THREADSAFE
-    this->mutex.unlock();
-#endif // COIN_THREADSAFE
-  }
+  void lock(void) { this->mutex.lock(); }
+  void unlock(void) { this->mutex.unlock(); }
+#else // !COIN_THREADSAFE
+  void lock(void) {  }
+  void unlock(void) { }
+#endif // !COIN_THREADSAFE
   uint32_t fakecolor;
 };
-#endif // DOXYGEN_SKIP_THIS
 
 #define PRIVATE(thisp) ((thisp)->pimpl)
 
+// *************************************************************************
+
 SO_NODE_SOURCE(SoVRMLAppearance);
 
+// *************************************************************************
+
 // doc in parent
 void
 SoVRMLAppearance::initClass(void)
