@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -60,20 +60,20 @@
   sound.  The url field specifies the URL from which the sound is
   loaded.  Browsers shall support at least the wavefile format in
   uncompressed PCM format (see
-  http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/bibliography.html#[WAV]).
+  http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/bibliography.html#[WAV]).
   It is recommended that browsers also support the MIDI file type 1
   sound format
-  (see http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/references.html#[MIDI]);
+  (see http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/references.html#[MIDI]);
   MIDI files are presumed to use the
   General MIDI patch set. Subclause 4.5, VRML and the World Wide Web
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.5>),
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.5>),
   contains details on the url field. The results are undefined when no
   URLs refer to supported data types.
 
   The loop, startTime, and stopTime exposedFields and the isActive
   eventOut, and their effects on the AudioClip node, are discussed in
   detail in 4.6.9, Time-dependent nodes
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.9>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.9>).
   The "cycle" of an AudioClip is the length of time in seconds for one
   playing of the audio at the specified pitch.  The pitch field
   specifies a multiplier for the rate at which sampled sound is
@@ -308,7 +308,7 @@ SoVRMLAudioClip::initClass(void) // static
 {
   SO_NODE_INTERNAL_INIT_CLASS(SoVRMLAudioClip, SO_VRML97_NODE_TYPE);
   SoVRMLAudioClipP::staticdata = new SoVRMLAudioClipP::StaticData;
-  coin_atexit((coin_atexit_f*) cleanup_audioclip, 0);
+  coin_atexit((coin_atexit_f*) cleanup_audioclip, CC_ATEXIT_NORMAL);
 
   const char * env = coin_getenv("COIN_SOUND_INTRO_PAUSE");
   float intropause = env ? (float) atof(env) : 0.0f;
