@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -58,7 +58,7 @@
   local coordinate system and are affected by the accumulated rotation
   of their ancestors as described below.  Background nodes are
   bindable nodes as described in 4.6.10, Bindable children nodes
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.10>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.10>).
   There exists a Background stack, in which the top-most Background on
   the stack is the currently active Background. To move a Background
   to the top of the stack, a TRUE value is sent to the set_bind
@@ -66,7 +66,7 @@
   view. A FALSE value sent to set_bind removes the Background from the
   stack and unbinds it from the browser's view. More detail on the
   bind stack is described in 4.6.10, Bindable children nodes
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.10>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.10>).
 
   The backdrop is conceptually a partial sphere (the ground) enclosed
   inside of a full sphere (the sky) in the local coordinate system
@@ -131,7 +131,7 @@
   orientation as if the image were displayed normally in 2D.
 
   <center>
-  <img src="http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/Images/background.gif">
+  <img src="http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/Images/background.gif">
   Figure 6.1
   </center>
 
@@ -149,7 +149,7 @@
   GIF (see E.[GIF]) format is recommended (including transparency).
   More detail on the url fields can be found in 4.5, VRML and the
   World Wide Web
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.5>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.5>).
 
 
 */
