@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -88,7 +88,7 @@
   undefined.  Multiple instances of Billboard nodes (DEF/USE) operate
   as expected: each instance rotates in its unique coordinate system
   to face the viewer.  Subclause 4.6.5, Grouping and children nodes
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.5>),
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.5>),
   provides a description of the children, addChildren, and
   removeChildren fields and eventIns.  The bboxCenter and bboxSize
   fields specify a bounding box that encloses the Billboard node's
@@ -99,9 +99,11 @@
   is not specified and if needed shall be calculated by the browser. A
   description of the bboxCenter and bboxSize fields is contained in
   4.6.4, Bounding boxes
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.4>),
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.4>),
 */
 
+// *************************************************************************
+
 /*!
   \var SoSFVec3f SoVRMLBillboard::axisOfRotation
 
@@ -118,6 +120,8 @@
   The bounding box size hint. Default value is (-1, -1, -1).
 */
 
+// *************************************************************************
+
 #include <Inventor/VRMLnodes/SoVRMLBillboard.h>
 
 #include <math.h>
@@ -141,6 +145,17 @@
 #include <Inventor/system/gl.h>
 #include <Inventor/C/glue/glp.h>
 
+#include "../misc/PointerMap.h"
+
+// *************************************************************************
+
+class SoVRMLBillboardP {
+public:
+  SbRotation previousrotation;
+};
+
+#define PRIVATE(obj) ((SoVRMLBillboardP *) PointerMap::get(this))
+
 // *************************************************************************
 
 SO_NODE_SOURCE(SoVRMLBillboard);
@@ -159,6 +174,8 @@ SoVRMLBillboard::initClass(void)
 */
 SoVRMLBillboard::SoVRMLBillboard(void)
 {
+  PointerMap::add(this, new SoVRMLBillboardP);
+
   SO_VRMLNODE_INTERNAL_CONSTRUCTOR(SoVRMLBillboard);
 
   SO_VRMLNODE_ADD_EXPOSED_FIELD(axisOfRotation, (0.0f, 0.0f, 0.0f));
@@ -179,6 +196,8 @@ SoVRMLBillboard::SoVRMLBillboard(int numchildren)
 */
 SoVRMLBillboard::~SoVRMLBillboard()
 {
+  delete PRIVATE(this);
+  PointerMap::remove(this);
 }
 
 // *************************************************************************
@@ -367,22 +386,11 @@ SoVRMLBillboard::performRotation(SoState * state) const
 {
   SbRotation rot;
   const SbMatrix & mm = SoModelMatrixElement::get(state);
-  SbMatrix imm = mm.inverse();
 
   SbVec3f toviewer;
-  SbVec3f cameray(0.0f, 1.0f, 0.0f);
   const SbViewVolume & vv = SoViewVolumeElement::get(state);
 
-  if (0 && vv.getProjectionType() == SbViewVolume::PERSPECTIVE) {
-    SbVec3f worldpos(0.0f, 0.0f, 0.0f);
-    mm.multVecMatrix(worldpos, worldpos);
-    toviewer = vv.getProjectionPoint() - worldpos;
-    imm.multDirMatrix(toviewer, toviewer);
-  }
-  else { // ORTHOGRAPHIC
-    toviewer = - vv.getProjectionDirection();
-  }
-
+  toviewer = - vv.getProjectionDirection();
   (void) toviewer.normalize();
 
   SbVec3f rotaxis = this->axisOfRotation.getValue();
@@ -406,16 +414,28 @@ SoVRMLBillboard::performRotation(SoState * state) const
     //    billboard-to-viewer vector.
     // 3. Rotate the local Z-axis of the billboard into the plane from 2.,
     //    pivoting around the axisOfRotation.
-    SbVec3f zaxis(0.0f, 0.0f, 1.0f);
-    // FIXME: this is buggy if axisOfRotation and toviewer are
-    // parallel. 20050104 mortene.
-    SbPlane plane(rotaxis.cross(toviewer), 0.0f);
-    const SbVec3f n = plane.getNormal();
-    SbVec3f vecinplane = zaxis - n * n[2];
-    (void) vecinplane.normalize();
-    if (vecinplane.dot(toviewer) < 0.0f) vecinplane = - vecinplane;
-    float angle = (float) acos(SbClamp(vecinplane.dot(zaxis), -1.0f, 1.0f));
-    rot.setValue(rotaxis, n[2] < 0.0f ? angle : - angle);
+    SbVec3f planenormal(rotaxis.cross(toviewer));
+    float length = planenormal.length();
+    if (length == 0) {
+      // Resulting rotation of billboard is undefined here; we are going
+      // to apply previous rotation so that eventually no sudden changes
+      // in orientation arise when this situation occurs while smoothly
+      // moving camera.
+      rot = PRIVATE(this)->previousrotation;
+    }
+    else {
+      // Calculate projection vector of local Z-axis to plane defined by
+      // the axisOfRotation and the billboard-to-viewer vector and then
+      // apply rotation around axisOfRotation that will rotate Z-axis to
+      // this projection.
+      planenormal /= length;
+      SbVec3f zaxis(0.0f, 0.0f, 1.0f);
+      SbVec3f vecinplane = zaxis - planenormal * planenormal[2];
+      (void) vecinplane.normalize();
+      if (vecinplane.dot(toviewer) < 0.0f) vecinplane = - vecinplane;
+      float angle = (float) acos(SbClamp(zaxis.dot(vecinplane), -1.0f, 1.0f));
+      rot.setValue(rotaxis, planenormal[2] < 0.0f ? angle : - angle);
+    }
   }
   SbVec3f translation, scale;
   SbRotation rotation, scaleorientation;
@@ -423,6 +443,10 @@ SoVRMLBillboard::performRotation(SoState * state) const
   mm.getTransform(translation, rotation, scale, scaleorientation);
   mm2.setTransform(translation, rot, scale);
   SoModelMatrixElement::set(state, (SoNode*) this, mm2);
+
+  PRIVATE(this)->previousrotation = rot;
 }
 
+#undef PRIVATE
+
 #endif // HAVE_VRML97
