@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -53,12 +53,25 @@
 
 #include <Inventor/VRMLnodes/SoVRMLCoordinate.h>
 #include <Inventor/nodes/SoSubNodeP.h>
+#include <Inventor/actions/SoGLRenderAction.h>
 #include <Inventor/VRMLnodes/SoVRMLMacros.h>
 #include <Inventor/elements/SoCoordinateElement.h>
+#include <Inventor/elements/SoGLVBOElement.h>
 #include <Inventor/actions/SoAction.h>
+#include "../misc/SoVBO.h"
+#include "../misc/PointerMap.h"
 
 SO_NODE_SOURCE(SoVRMLCoordinate);
 
+class SoVRMLCoordinateP {
+ public:
+  SoVRMLCoordinateP() : vbo(NULL) { }
+  ~SoVRMLCoordinateP() { delete this->vbo; }
+  SoVBO * vbo;
+};
+
+#define PRIVATE(obj) ((SoVRMLCoordinateP*) PointerMap::get(obj))
+
 // Doc in parent
 void
 SoVRMLCoordinate::initClass(void) // static
@@ -71,6 +84,7 @@ SoVRMLCoordinate::initClass(void) // static
 */
 SoVRMLCoordinate::SoVRMLCoordinate(void)
 {
+  PointerMap::add(this, new SoVRMLCoordinateP);
   SO_VRMLNODE_INTERNAL_CONSTRUCTOR(SoVRMLCoordinate);
 
   SO_VRMLNODE_ADD_EMPTY_EXPOSED_MFIELD(point);
@@ -81,6 +95,9 @@ SoVRMLCoordinate::SoVRMLCoordinate(void)
 */
 SoVRMLCoordinate::~SoVRMLCoordinate()
 {
+  SoVRMLCoordinateP * pimpl = PRIVATE(this);
+  PointerMap::remove(this);
+  delete pimpl;  
 }
 
 // Doc in parent
@@ -96,6 +113,37 @@ void
 SoVRMLCoordinate::GLRender(SoGLRenderAction * action)
 {
   SoVRMLCoordinate::doAction((SoAction*) action);
+#ifdef COIN_NEXT_MINOR
+  SoVRMLCoordinateP * pimpl = PRIVATE(this);
+  SoState * state = action->getState();
+  const int num = this->point.getNum();
+  SbBool setvbo = FALSE;
+  SoBase::staticDataLock();
+  if (SoGLVBOElement::shouldCreateVBO(state, num)) {
+    SbBool dirty = FALSE;
+    setvbo = TRUE;
+    if (pimpl->vbo == NULL) {
+      pimpl->vbo = new SoVBO(GL_ARRAY_BUFFER, GL_STATIC_DRAW); 
+      dirty =  TRUE;
+    }
+    else if (pimpl->vbo->getBufferDataId() != this->getNodeId()) {
+      dirty = TRUE;
+    }
+    if (dirty) {
+      pimpl->vbo->setBufferData(this->point.getValues(0),
+                                num*sizeof(SbVec3f),
+                                        this->getNodeId());
+    }
+  }
+  else if (pimpl->vbo && pimpl->vbo->getBufferDataId()) {
+    // clear buffers to deallocate VBO memory
+    pimpl->vbo->setBufferData(NULL, 0, 0);
+  }
+  SoBase::staticDataUnlock();
+  if (setvbo) {
+    SoGLVBOElement::setVertexVBO(state, pimpl->vbo);
+  }
+#endif // COIN_NEXT_MINOR
 }
 
 // Doc in parent
@@ -119,4 +167,5 @@ SoVRMLCoordinate::pick(SoPickAction * action)
   SoVRMLCoordinate::doAction((SoAction*) action);
 }
 
+#undef PRIVATE
 #endif // HAVE_VRML97
