@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -187,7 +187,7 @@
   is TRUE.  See Figure 6.5 for a depiction of the ElevationGrid node.
 
   <center>
-  <img src="http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/Images/ElevationGrid.gif">
+  <img src="http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/Images/ElevationGrid.gif">
   Figure 6.5
   </center>
 
@@ -280,58 +280,48 @@
 #include <Inventor/details/SoPointDetail.h>
 #include <Inventor/errors/SoDebugError.h>
 #include <Inventor/system/gl.h>
-
-#ifdef COIN_THREADSAFE
+#ifdef HAVE_THREADS
 #include <Inventor/threads/SbRWMutex.h>
-#endif // COIN_THREADSAFE
+#endif // HAVE_THREADS
 
-#ifndef DOXYGEN_SKIP_THIS
+// *************************************************************************
 
 class SoVRMLElevationGridP {
 public:
   SoVRMLElevationGridP(void)
     : dirty(TRUE),
-#ifdef COIN_THREADSAFE
-      mutex(SbRWMutex::READ_PRECEDENCE),
-#endif // COIN_THREADSAFE
       ngen(TRUE)
+#ifdef COIN_THREADSAFE
+      , mutex(SbRWMutex::READ_PRECEDENCE)
+#endif // COIN_THREADSAFE
   { }
 
   SbBool dirty;
-#ifdef COIN_THREADSAFE
-  SbRWMutex mutex;
-#endif // COIN_THREADSAFE
   SoNormalGenerator ngen;
   SoVRMLElevationGrid::Binding nbind;
 
-  void readLockNormalCache(void) {
 #ifdef COIN_THREADSAFE
-    this->mutex.readLock();
-#endif // COIN_THREADSAFE
-  }
-  void readUnlockNormalCache(void) {
-#ifdef COIN_THREADSAFE
-    this->mutex.readUnlock();
-#endif // COIN_THREADSAFE
-  }
-  void writeLockNormalCache(void) {
-#ifdef COIN_THREADSAFE
-    this->mutex.writeLock();
-#endif // COIN_THREADSAFE
-  }
-  void writeUnlockNormalCache(void) {
-#ifdef COIN_THREADSAFE
-    this->mutex.writeUnlock();
-#endif // COIN_THREADSAFE
-  }
+  SbRWMutex mutex;
+  void readLockNormalCache(void) { this->mutex.readLock(); }
+  void readUnlockNormalCache(void) { this->mutex.readUnlock(); }
+  void writeLockNormalCache(void) { this->mutex.writeLock(); }
+  void writeUnlockNormalCache(void) { this->mutex.writeUnlock(); }
+#else // ! COIN_THREADSAFE
+  void readLockNormalCache(void) { }
+  void readUnlockNormalCache(void) { }
+  void writeLockNormalCache(void) { }
+  void writeUnlockNormalCache(void) { }
+#endif // ! COIN_THREADSAFE
 };
 
-#endif // DOXYGEN_SKIP_THIS
-
 #define PRIVATE(obj) ((obj)->pimpl)
 
+// *************************************************************************
+
 SO_NODE_SOURCE(SoVRMLElevationGrid);
 
+// *************************************************************************
+
 // Doc in parent
 void
 SoVRMLElevationGrid::initClass(void)
