@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -98,7 +98,7 @@
   cross-section out of the SCP.
 
   <center>
-  <img src="http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/Images/Extrusion.gif">
+  <img src="http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/Images/Extrusion.gif">
   Figure 6.6
   </center>
 
@@ -253,14 +253,14 @@
 
   in that order. By default, normals for the sides are generated as
   described in 4.6.3, Shapes and geometry
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.3>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.3>).
 
   For instance, a circular crossSection with counter-clockwise
   ordering and the default spine form a cylinder. With solid TRUE and
   ccw TRUE, the cylinder is visible from the outside. Changing ccw to
   FALSE makes it visible from the inside.  The ccw, solid, convex, and
   creaseAngle fields are described in 4.6.3, Shapes and geometry
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.3>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.3>).
 
 */
 
@@ -328,6 +328,7 @@
 #include <Inventor/elements/SoCoordinateElement.h>
 #include <Inventor/elements/SoTextureCoordinateElement.h>
 #include <Inventor/elements/SoGLTextureEnabledElement.h>
+#include <Inventor/elements/SoShapeHintsElement.h>
 #include <Inventor/SbTesselator.h>
 #include <Inventor/actions/SoGLRenderAction.h>
 #include <Inventor/actions/SoGetPrimitiveCountAction.h>
@@ -339,10 +340,11 @@
 #include <Inventor/elements/SoMultiTextureEnabledElement.h>
 #include <Inventor/elements/SoMultiTextureCoordinateElement.h>
 #include <Inventor/SbBox2f.h>
-
-#ifdef COIN_THREADSAFE
+#ifdef HAVE_THREADS
 #include <Inventor/threads/SbRWMutex.h>
-#endif // COIN_THREADSAFE
+#endif // HAVE_THREADS
+
+// *************************************************************************
 
 //
 // needed to avoid warnings generated by SbVec3f::normalize
@@ -370,10 +372,10 @@ public:
      tcoord(32),
      idx(32),
      gen(TRUE),
-#ifdef COIN_THREADSAFE
-     rwmutex(SbRWMutex::READ_PRECEDENCE),
-#endif // COIN_THREADSAFE
      dirty(TRUE)
+#ifdef COIN_THREADSAFE
+     , rwmutex(SbRWMutex::READ_PRECEDENCE)
+#endif // COIN_THREADSAFE
   {
     this->tess.setCallback(tess_callback, this);
   }
@@ -387,31 +389,20 @@ public:
   static void tess_callback(void *, void *, void *, void *);
   void generateCoords(void);
   void generateNormals(void);
-#ifdef COIN_THREADSAFE
-  SbRWMutex rwmutex;
-#endif // COIN_THREADSAFE
   SbBool dirty;
 
-  void readLock(void) {
 #ifdef COIN_THREADSAFE
-    this->rwmutex.readLock();
-#endif // COIN_THREADSAFE
-  }
-  void readUnlock(void) {
-#ifdef COIN_THREADSAFE
-    this->rwmutex.readUnlock();
-#endif // COIN_THREADSAFE
-  }
-  void writeLock(void) {
-#ifdef COIN_THREADSAFE
-    this->rwmutex.writeLock();
-#endif // COIN_THREADSAFE
-  }
-  void writeUnlock(void) {
-#ifdef COIN_THREADSAFE
-    this->rwmutex.writeUnlock();
-#endif // COIN_THREADSAFE
-  }
+  SbRWMutex rwmutex;
+  void readLock(void) { this->rwmutex.readLock(); }
+  void readUnlock(void) { this->rwmutex.readUnlock(); }
+  void writeLock(void) { this->rwmutex.writeLock(); }
+  void writeUnlock(void) { this->rwmutex.writeUnlock(); }
+#else // !COIN_THREADSAFE
+  void readLock(void) { }
+  void readUnlock(void) { }
+  void writeLock(void) { }
+  void writeUnlock(void) { }
+#endif // !COIN_THREADSAFE
 };
 
 #define PRIVATE(obj) (obj)->pimpl
@@ -588,7 +579,10 @@ SoVRMLExtrusion::generatePrimitives(SoAction * action)
 
   SoState * state = action->getState();
   state->push();
-  
+
+  SbBool ccw = this->ccw.getValue();
+  SbBool solid = this->solid.getValue();
+
   if (SoTextureCoordinateElement::getType(state) !=
       SoTextureCoordinateElement::FUNCTION) {
     SoTextureCoordinateElement::set2(state, this, PRIVATE(this)->tcoord.getLength(),
@@ -609,14 +603,39 @@ SoVRMLExtrusion::generatePrimitives(SoAction * action)
       }
     }
   }
+  SoShapeHintsElement::set(state, this,
+                           this->ccw.getValue() ?
+                           SoShapeHintsElement::COUNTERCLOCKWISE :
+                           SoShapeHintsElement::CLOCKWISE,
+                           this->solid.getValue() ?
+                           SoShapeHintsElement::SOLID :
+                           SoShapeHintsElement::UNKNOWN_SHAPE_TYPE,
+                           this->convex.getValue() ?
+                           SoShapeHintsElement::CONVEX :
+                           SoShapeHintsElement::UNKNOWN_FACE_TYPE);
 
   SoTextureCoordinateBundle tb(action, FALSE, FALSE); 
   SbBool istexfunc = tb.isFunction();
   SoPrimitiveVertex vertex;
 
-  int idx;
   this->beginShape(action, TRIANGLES);
+  TriangleShape shapetype = LINES; // set it to some impossible value
+
+  int idx;
   while (iptr < endptr) {
+
+    // we generate either triangles or quads, so this test is safe
+    SbBool isquad = iptr[3] >= 0;
+    if (isquad && (shapetype != QUADS)) {
+      if (shapetype == TRIANGLES) this->endShape();
+      this->beginShape(action, QUADS);
+      shapetype = QUADS;
+    }
+    if (!isquad && (shapetype != TRIANGLES)) {
+      if (shapetype == QUADS) this->endShape();
+      this->beginShape(action, TRIANGLES);
+      shapetype = TRIANGLES;
+    }
     idx = *iptr++;
     while (idx >= 0) {
       vertex.setNormal(*normals);
@@ -632,14 +651,14 @@ SoVRMLExtrusion::generatePrimitives(SoAction * action)
       normals++;
     }
   }
-  this->endShape();
+  if ((shapetype == TRIANGLES) || (shapetype == QUADS)) this->endShape();
+
   state->pop();
   PRIVATE(this)->readUnlock();
 }
 
-//
 // private method that updates the coordinate and normal cache.
-// cache must be read locked when entering here!
+// cache must be read-locked when entering here!
 void
 SoVRMLExtrusion::updateCache(void)
 {
