@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -49,7 +49,7 @@
   identity transform.  More details on the children, addChildren, and
   removeChildren fields and eventIns can be found in 4.6.5, Grouping
   and children nodes
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.5>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.5>).
 
   The bboxCenter and bboxSize fields specify a bounding box that
   encloses the Group node's children. This is a hint that may be used
@@ -59,7 +59,7 @@
   implies that the bounding box is not specified and, if needed, is
   calculated by the browser. A description of the bboxCenter and
   bboxSize fields is contained in 4.6.4, Bounding boxes
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.4>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.4>).
 
 
 */
@@ -113,9 +113,9 @@
 #include <Inventor/system/gl.h>
 #include <Inventor/C/glue/glp.h>
 
-#ifdef COIN_THREADSAFE
+#ifdef HAVE_THREADS
 #include <Inventor/threads/SbMutex.h>
-#endif // COIN_THREADSAFE
+#endif // HAVE_THREADS
 
 // *************************************************************************
 
@@ -158,9 +158,6 @@ public:
   }
 
   SoBoundingBoxCache * bboxcache;
-#ifdef COIN_THREADSAFE
-  SbMutex mutex;
-#endif // !COIN_THREADSAFE
   SbStorage * glcachestorage;
   static void invalidate_gl_cache(void * tls, void *) {
     sovrmlgroup_storage * ptr = (sovrmlgroup_storage*) tls;
@@ -178,16 +175,14 @@ public:
     glcachestorage->applyToAll(invalidate_gl_cache, NULL);
   }
 
-  void lock(void) {
 #ifdef COIN_THREADSAFE
-    this->mutex.lock();
-#endif // COIN_THREADSAFE
-  }
-  void unlock(void) {
-#ifdef COIN_THREADSAFE
-    this->mutex.unlock();
-#endif // COIN_THREADSAFE
-  }
+  SbMutex mutex;
+  void lock(void) { this->mutex.lock(); }
+  void unlock(void) { this->mutex.unlock(); }
+#else // !COIN_THREADSAFE
+  void lock(void) {  }
+  void unlock(void) { }
+#endif // !COIN_THREADSAFE
 };
 
 SoGLCacheList *
@@ -217,6 +212,7 @@ SoVRMLGroup::initClass(void)
 
   SoType type = SoVRMLGroup::getClassTypeId();
   SoRayPickAction::addMethod(type, SoNode::rayPickS);
+  SoVRMLGroup::numRenderCaches = 2;
 }
 
 /*!
