@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -53,7 +53,7 @@
   Figure 6.9.
 
   <center>
-  <img src="http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/Images/ImageTexture.gif">
+  <img src="http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/Images/ImageTexture.gif">
   Figure 6.9
   </center>
 
@@ -68,11 +68,11 @@
   Wide Web.
 
   See 4.6.11, Texture maps
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.11>),
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.11>),
   for a general description of texture maps.
 
   See 4.14, Lighting model
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.14>),
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.14>),
   for a description of lighting equations and the interaction between
   textures, materials, and geometry appearance.
 
@@ -84,6 +84,9 @@
   [0.0, 1.0] range. The repeatT field is analogous to the repeatS
   field.
 
+
+  \ENDWEB3D
+
   One common flaw with many programs that has support for exporting
   VRML or Inventor files, is that the same texture file is exported
   several times, but as different nodes. This can cause excessive
@@ -160,52 +163,61 @@
   \endcode
 */
 
+// *************************************************************************
+
 /*!
   SoMFString SoVRMLImageTexture::url
   The texture file URL.
 */
 
-#include <Inventor/VRMLnodes/SoVRMLImageTexture.h>
-#include <Inventor/VRMLnodes/SoVRMLMacros.h>
-#include <Inventor/nodes/SoSubNodeP.h>
+// *************************************************************************
 
-#include <Inventor/actions/SoCallbackAction.h>
-#include <Inventor/actions/SoRayPickAction.h>
-#include <Inventor/actions/SoGLRenderAction.h>
-#include <Inventor/elements/SoGLTextureEnabledElement.h>
-#include <Inventor/elements/SoGLTexture3EnabledElement.h>
-#include <Inventor/elements/SoGLTextureImageElement.h>
-#include <Inventor/elements/SoTextureQualityElement.h>
-#include <Inventor/elements/SoTextureOverrideElement.h>
-#include <Inventor/elements/SoTextureScalePolicyElement.h>
-#include <Inventor/elements/SoCacheElement.h>
-#include <Inventor/errors/SoReadError.h>
-#include <Inventor/misc/SoGLBigImage.h>
-#include <Inventor/sensors/SoFieldSensor.h>
-#include <Inventor/lists/SbStringList.h>
-#include <Inventor/errors/SoDebugError.h>
-#include <Inventor/sensors/SoOneShotSensor.h>
+#include <Inventor/VRMLnodes/SoVRMLImageTexture.h>
+
+#include <assert.h>
+
+#include <Inventor/C/glue/simage_wrapper.h>
+#include <Inventor/C/tidbitsp.h>
 #include <Inventor/SbImage.h>
 #include <Inventor/SoInput.h>
-#include <Inventor/C/tidbitsp.h>
+#include <Inventor/VRMLnodes/SoVRMLMacros.h>
+#include <Inventor/actions/SoCallbackAction.h>
+#include <Inventor/actions/SoGLRenderAction.h>
+#include <Inventor/actions/SoRayPickAction.h>
+#include <Inventor/elements/SoCacheElement.h>
+#include <Inventor/elements/SoGLTexture3EnabledElement.h>
+#include <Inventor/elements/SoGLTextureEnabledElement.h>
+#include <Inventor/elements/SoGLTextureImageElement.h>
+#include <Inventor/elements/SoMultiTextureEnabledElement.h>
+#include <Inventor/elements/SoMultiTextureImageElement.h>
+#include <Inventor/elements/SoTextureOverrideElement.h>
+#include <Inventor/elements/SoTextureQualityElement.h>
+#include <Inventor/elements/SoTextureScalePolicyElement.h>
+#include <Inventor/elements/SoTextureUnitElement.h>
+#include <Inventor/errors/SoDebugError.h>
+#include <Inventor/errors/SoReadError.h>
+#include <Inventor/lists/SbStringList.h>
+#include <Inventor/misc/SoGLBigImage.h>
+#include <Inventor/nodes/SoSubNodeP.h>
+#include <Inventor/sensors/SoFieldSensor.h>
+#include <Inventor/sensors/SoOneShotSensor.h>
 #include <Inventor/sensors/SoTimerSensor.h>
-#include <assert.h>
-#include <Inventor/C/glue/simage_wrapper.h>
+#include <Inventor/C/threads/sched.h>
+
+// FIXME: should be able to include file without
+// #ifdef-wrapper. 20051202 mortene.
+#ifdef HAVE_THREADS
+#include <Inventor/threads/SbMutex.h>
+#endif // HAVE_THREADS
+
+// *************************************************************************
 
 static int imagedata_maxage = 0;
 static VRMLPrequalifyFileCallback * imagetexture_prequalify_cb = NULL;
 static void * imagetexture_prequalify_closure = NULL;
 static SbBool imagetexture_delay_fetch = TRUE;
-static SbBool imagetexture_is_exiting = FALSE;
 
-#ifdef COIN_THREADSAFE
-
-#include <Inventor/C/threads/sched.h>
-#include <Inventor/threads/SbMutex.h>
-
-static cc_sched * imagetexture_scheduler = NULL;
-
-#endif // COIN_THREADSAFE
+// *************************************************************************
 
 class SoVRMLImageTextureP {
 public:
@@ -214,7 +226,6 @@ public:
   
   int readstatus;
   class SoGLImage * glimage;
-  SbBool glimagevalid;
   SbImage image;
   SoFieldSensor * urlsensor;
   SbBool allowprequalifycb;
@@ -225,9 +236,7 @@ public:
 
   void readimage_cleanup(void);
   SbBool isdestructing;
-#ifdef COIN_THREADSAFE
-  SbMutex glimagemutex;
-#endif // COIN_THREADSAFE
+
   SbStringList searchdirs;
 
   void clearSearchDirs(void) {
@@ -244,44 +253,80 @@ public:
       this->searchdirs.append(new SbString(*sl[i]));
     }
   }
+
+  static SbBool is_exiting;
+  static cc_sched * scheduler;
+
+#ifdef COIN_THREADSAFE
+  static SbMutex * glimagemutex;
+  void lock_glimage(void) { this->glimagemutex->lock(); }
+  void unlock_glimage(void) { this->glimagemutex->unlock(); }
+#else // !COIN_THREADSAFE
+  void lock_glimage(void) { }
+  void unlock_glimage(void) { }
+#endif // !COIN_THREADSAFE
+
+  static void cleanup(void)
+  {
+    is_exiting = TRUE;
+
+#ifdef COIN_THREADSAFE
+    delete glimagemutex;
+    glimagemutex = NULL;
+#endif // COIN_THREADSAFE
+
+    if (scheduler) {
+      cc_sched_destruct(scheduler);
+      scheduler = NULL;
+    }
+
+    imagetexture_delay_fetch = TRUE;
+    imagetexture_prequalify_cb = NULL;
+    imagetexture_prequalify_closure = NULL;
+  }
 };
 
 #ifdef COIN_THREADSAFE
-#define LOCK_GLIMAGE(_thisp_) (_thisp_)->pimpl->glimagemutex.lock()
-#define UNLOCK_GLIMAGE(_thisp_) (_thisp_)->pimpl->glimagemutex.unlock()
-#else // COIN_THREADSAFE
-#define LOCK_GLIMAGE(_thisp_)
-#define UNLOCK_GLIMAGE(_thisp_)
+SbMutex * SoVRMLImageTextureP::glimagemutex = NULL;
 #endif // COIN_THREADSAFE
 
+cc_sched * SoVRMLImageTextureP::scheduler = NULL;
+SbBool SoVRMLImageTextureP::is_exiting = FALSE;
+
+// *************************************************************************
+
 SO_NODE_SOURCE(SoVRMLImageTexture);
 
-static void
-imagetexture_cleanup(void)
-{
-#ifdef COIN_THREADSAFE
-  imagetexture_is_exiting = TRUE;
-  if (imagetexture_scheduler) {
-    cc_sched_destruct(imagetexture_scheduler);
-  }
-#endif // COIN_THREADSAFE
-}
+// *************************************************************************
 
 // Doc in parent
 void
 SoVRMLImageTexture::initClass(void) // static
 {
+  SoVRMLImageTextureP::is_exiting = FALSE;
   SO_NODE_INTERNAL_INIT_CLASS(SoVRMLImageTexture, SO_VRML97_NODE_TYPE);
   imagedata_maxage = 500;
 
+  SoType type = SoVRMLImageTexture::getClassTypeId();
+  SoRayPickAction::addMethod(type, SoNode::rayPickS);
+
+  if (cc_thread_implementation() != CC_NO_THREADS) {
+    SoVRMLImageTextureP::scheduler = cc_sched_construct(1);
+  }
+
 #ifdef COIN_THREADSAFE
-  imagetexture_scheduler = cc_sched_construct(1);
-  coin_atexit((coin_atexit_f*) imagetexture_cleanup, 0);
+  SoVRMLImageTextureP::glimagemutex = new SbMutex;
 #endif // COIN_THREADSAFE
+
+  coin_atexit((coin_atexit_f *)SoVRMLImageTextureP::cleanup, CC_ATEXIT_NORMAL);
 }
 
+// *************************************************************************
+
 #define PRIVATE(x) (x)->pimpl
 
+// *************************************************************************
+
 /*!
   Constructor.
 */
@@ -293,12 +338,11 @@ SoVRMLImageTexture::SoVRMLImageTexture(void)
   SO_VRMLNODE_ADD_EMPTY_EXPOSED_MFIELD(url);
 
   PRIVATE(this)->glimage = NULL;
-  PRIVATE(this)->glimagevalid = FALSE;
   PRIVATE(this)->readstatus = 1;
   PRIVATE(this)->allowprequalifycb = TRUE;
   PRIVATE(this)->timersensor = 
     new SoTimerSensor(SoVRMLImageTextureP::timersensor_cb, PRIVATE(this));
-  PRIVATE(this)->timersensor->setInterval(SbTime(1.0));
+  PRIVATE(this)->timersensor->setInterval(SbTime(0.5));
   
   // use field sensor for url since we will load an image if
   // filename changes. This is a time-consuming task which should
@@ -315,13 +359,12 @@ SoVRMLImageTexture::SoVRMLImageTexture(void)
 SoVRMLImageTexture::~SoVRMLImageTexture()
 {
   delete PRIVATE(this)->timersensor;
-#ifdef COIN_THREADSAFE
+
   // just wait for all threads to finish reading
-  if (imagetexture_scheduler) {
+  if (SoVRMLImageTextureP::scheduler) {
     PRIVATE(this)->isdestructing = TRUE; // signal thread that we are destructing
-    cc_sched_wait_all(imagetexture_scheduler);
+    cc_sched_wait_all(SoVRMLImageTextureP::scheduler);
   }
-#endif // COIN_THREADSAFE  
 
   if (PRIVATE(this)->glimage) PRIVATE(this)->glimage->unref(NULL);
   PRIVATE(this)->clearSearchDirs();
@@ -329,6 +372,8 @@ SoVRMLImageTexture::~SoVRMLImageTexture()
   delete PRIVATE(this);
 }
 
+// *************************************************************************
+
 /*!
   Sets the prequalify callback for ImageTexture nodes. This is a callback
   that will be called when an image is about to be read.
@@ -373,44 +418,60 @@ void
 SoVRMLImageTexture::doAction(SoAction * action)
 {
   SoState * state = action->getState();
-
-  if (SoTextureOverrideElement::getImageOverride(state))
+  int unit = SoTextureUnitElement::get(state);
+  
+  if ((unit == 0) && SoTextureOverrideElement::getImageOverride(state))
     return;
 
-  SoTexture3EnabledElement::set(state, this, FALSE);
+  int nc;
+  SbVec2s size;
+  const unsigned char * bytes = PRIVATE(this)->image.getValue(size, nc);
+  
   if (!PRIVATE(this)->image.hasData()) {
     if (this->url.getNum()) {
       // texture has not been loaded yet. Supply a dummy image and
       // enable SoTextureImageElement so that texture coordinates are
       // generated in generatePrimitives()
-      SoTextureEnabledElement::set(state, this, TRUE);    
       static unsigned char dummydata[] = { 0xff, 0xff, 0xff, 0xff };
-      SoTextureImageElement::set(state, this,
-                                 SbVec2s(2,2), 1, dummydata,
-                                 imagetexture_translate_wrap(this->repeatS.getValue()),
-                                 imagetexture_translate_wrap(this->repeatT.getValue()),                                                           
-                                 SoTextureImageElement::MODULATE,
-                                 SbColor(1.0f, 1.0f, 1.0f));
-      
+      bytes = dummydata;
+      size = SbVec2s(2,2);
+      nc = 1;
+    }
+  }
+
+  if (unit == 0) {
+    SoTexture3EnabledElement::set(state, this, FALSE);
+    if (size == SbVec2s(0,0)) {
+      SoTextureEnabledElement::set(state, this, FALSE);    
     }
     else {
-      SoTextureEnabledElement::set(state, this, FALSE);    
-    }    
+      SoTextureEnabledElement::set(state, this, TRUE);    
+      SoTextureImageElement::set(state, this,
+                                 size, nc, bytes,
+                                 imagetexture_translate_wrap(this->repeatS.getValue()),
+                                 imagetexture_translate_wrap(this->repeatT.getValue()),
+                                 SoTextureImageElement::MODULATE,
+                                 SbColor(1.0f, 1.0f, 1.0f));      
+    }
+    if (this->isOverride()) {
+      SoTextureOverrideElement::setImageOverride(state, TRUE);
+    }
   }
   else {
-    int nc;
-    SbVec2s size;
-    const unsigned char * bytes = PRIVATE(this)->image.getValue(size, nc);
-    SoTextureImageElement::set(state, this,
-                               size, nc, bytes,
-                               imagetexture_translate_wrap(this->repeatS.getValue()),
-                               imagetexture_translate_wrap(this->repeatT.getValue()),                                                           
-                               SoTextureImageElement::MODULATE,
-                               SbColor(1.0f, 1.0f, 1.0f));
-    SoTextureEnabledElement::set(state, this, TRUE);
-  }
-  if (this->isOverride()) {
-    SoTextureOverrideElement::setImageOverride(state, TRUE);
+    if (size == SbVec2s(0,0)) {
+      SoMultiTextureEnabledElement::set(state, this, unit, FALSE);    
+    }
+    else {
+      SoMultiTextureEnabledElement::set(state, this, unit, TRUE);    
+      SoMultiTextureImageElement::set(state, this, unit,
+                                      size, nc, bytes,
+                                      (SoTextureImageElement::Wrap)
+                                      imagetexture_translate_wrap(this->repeatS.getValue()),
+                                      (SoTextureImageElement::Wrap)
+                                      imagetexture_translate_wrap(this->repeatT.getValue()),
+                                      SoTextureImageElement::MODULATE,
+                                      SbColor(1.0f, 1.0f, 1.0f));      
+    }
   }
 }
 
@@ -432,41 +493,35 @@ SoVRMLImageTexture::GLRender(SoGLRenderAction * action)
   SoGLTexture3EnabledElement::set(state, this, FALSE);
   float quality = SoTextureQualityElement::get(state);
 
-  LOCK_GLIMAGE(this);
-  
-  if (!PRIVATE(this)->glimagevalid) {
-    SoTextureScalePolicyElement::Policy scalepolicy =
-      SoTextureScalePolicyElement::get(state);      
-    SbBool needbig = (scalepolicy == SoTextureScalePolicyElement::FRACTURE);
+  PRIVATE(this)->lock_glimage();
 
-    if (needbig &&
-        (PRIVATE(this)->glimage == NULL ||
-         PRIVATE(this)->glimage->getTypeId() != SoGLBigImage::getClassTypeId())) {
-      if (PRIVATE(this)->glimage) {
-        PRIVATE(this)->glimage->setEndFrameCallback(NULL, NULL);
-        PRIVATE(this)->glimage->unref(state);
-      }
+  SoTextureScalePolicyElement::Policy scalepolicy =
+    SoTextureScalePolicyElement::get(state);      
+  SbBool needbig = (scalepolicy == SoTextureScalePolicyElement::FRACTURE);
+  SbBool isbig = 
+    PRIVATE(this)->glimage && 
+    PRIVATE(this)->glimage->getTypeId() == SoGLBigImage::getClassTypeId();
+  
+  if (!PRIVATE(this)->glimage || (needbig != isbig)) {
+    if (PRIVATE(this)->glimage) {
+      PRIVATE(this)->glimage->unref(state);
+    }
+    if (needbig) {
       PRIVATE(this)->glimage = new SoGLBigImage();
     }
-    else if (!needbig &&
-             (PRIVATE(this)->glimage == NULL ||
-              PRIVATE(this)->glimage->getTypeId() != SoGLImage::getClassTypeId())) {
-      if (PRIVATE(this)->glimage) {
-        PRIVATE(this)->glimage->setEndFrameCallback(NULL, NULL);
-        PRIVATE(this)->glimage->unref(state);
-      }
+    else {
       PRIVATE(this)->glimage = new SoGLImage();
     }
-
     if (scalepolicy == SoTextureScalePolicyElement::SCALE_DOWN) {
       PRIVATE(this)->glimage->setFlags(PRIVATE(this)->glimage->getFlags()|SoGLImage::SCALE_DOWN);
     }
 
-    PRIVATE(this)->glimagevalid = TRUE;
     PRIVATE(this)->glimage->setData(&PRIVATE(this)->image,
                                     imagetexture_translate_wrap(this->repeatS.getValue()),
                                     imagetexture_translate_wrap(this->repeatT.getValue()),
                                     quality);
+    PRIVATE(this)->glimage->setEndFrameCallback(glimage_callback, this);
+
     // don't cache while creating a texture object
     SoCacheElement::setInvalid(TRUE);
     if (state->isCacheOpen()) {
@@ -478,14 +533,14 @@ SoVRMLImageTexture::GLRender(SoGLRenderAction * action)
     SoCacheElement::invalidate(state);
   }
 
-  UNLOCK_GLIMAGE(this);
+  PRIVATE(this)->unlock_glimage();
 
   SoGLTextureImageElement::set(state, this,
-                               PRIVATE(this)->glimagevalid ? PRIVATE(this)->glimage : NULL,
+                               PRIVATE(this)->glimage,
                                SoTextureImageElement::MODULATE,
                                SbColor(1.0f, 1.0f, 1.0f));
 
-  SbBool enable = PRIVATE(this)->glimagevalid &&
+  SbBool enable = PRIVATE(this)->glimage &&
     quality > 0.0f &&
     PRIVATE(this)->glimage->getImage() &&
     PRIVATE(this)->glimage->getImage()->hasData();
@@ -554,7 +609,12 @@ SoVRMLImageTexture::setReadStatus(int status)
 SbBool
 SoVRMLImageTexture::loadUrl(void)
 {
-  PRIVATE(this)->glimagevalid = FALSE; // recreate GL image in next GLRender()
+  PRIVATE(this)->lock_glimage();
+  if (PRIVATE(this)->glimage) {
+    PRIVATE(this)->glimage->unref(NULL);
+    PRIVATE(this)->glimage = NULL;
+  }
+  PRIVATE(this)->unlock_glimage();
 
   SbBool retval = TRUE;
   if (this->url.getNum() && this->url[0].getLength()) {
@@ -583,6 +643,15 @@ SoVRMLImageTexture::loadUrl(void)
   return retval;
 }
 
+// sensor callback used for deleting old GLImage instances
+static void 
+imagetexture_glimage_delete(void * closure, SoSensor * s)
+{
+  SoGLImage * img = (SoGLImage*) closure;
+  img->unref(NULL);
+  delete s;
+}
+
 //
 // used for multithread loading.
 //
@@ -591,16 +660,28 @@ SoVRMLImageTexture::glimage_callback(void * closure)
 {
 #ifdef COIN_THREADSAFE
   SoVRMLImageTexture * thisp = (SoVRMLImageTexture*) closure;
+  PRIVATE(thisp)->lock_glimage();
   if (PRIVATE(thisp)->glimage) {
     int age = PRIVATE(thisp)->glimage->getNumFramesSinceUsed();
     if (age > imagedata_maxage) {
-      PRIVATE(thisp)->glimagevalid = FALSE;
-      assert(PRIVATE(thisp)->glimage);
-      PRIVATE(thisp)->glimage->setEndFrameCallback(NULL, NULL);
+      // we can't delete the glimage here, since it's locked by
+      // SoGLImage. Use a sensor to delete it the next time the
+      // delayqueue sensors are processed.
+      if (PRIVATE(thisp)->glimage) {
+        PRIVATE(thisp)->glimage->setEndFrameCallback(NULL, NULL);
+        // allocate new sensor. It will be deleted in the sensor callback
+        SoOneShotSensor * s = new SoOneShotSensor(imagetexture_glimage_delete, PRIVATE(thisp)->glimage);
+        s->schedule();
+        // clear the GLImage in this node
+        PRIVATE(thisp)->glimage = NULL;
+      }
+      PRIVATE(thisp)->unlock_glimage();
       PRIVATE(thisp)->image.setValue(SbVec2s(0,0), 0, NULL);
       (void) thisp->loadUrl();
+      return;
     }
   }
+  PRIVATE(thisp)->unlock_glimage();
 #endif // COIN_THREADSAFE
 }
 
@@ -609,7 +690,7 @@ SoVRMLImageTexture::default_prequalify_cb(const SbString & url,  void * closure,
                                           SoVRMLImageTexture * thisp)
 {
   SbBool ret = TRUE;
-  if (!imagetexture_is_exiting && !PRIVATE(thisp)->isdestructing) {
+  if (!SoVRMLImageTextureP::is_exiting && !PRIVATE(thisp)->isdestructing) {
     ret = PRIVATE(thisp)->image.readFile(url);
   }
   return ret;
@@ -654,13 +735,9 @@ SoVRMLImageTexture::image_read_cb(const SbString & filename, SbImage * image, vo
 {
   SoVRMLImageTexture * thisp = (SoVRMLImageTexture*) closure;
   assert(&PRIVATE(thisp)->image == image);
-  assert(PRIVATE(thisp)->glimage);
-  if (!PRIVATE(thisp)->glimage) {
-    return FALSE;
-  }
   
-  // start a timer sensor that triggers each second and tests if the
-  // thread that loads images has finished loading this image.
+  // start a timer sensor which polls the thread that loads images, to
+  // detect when it's done:
   PRIVATE(thisp)->finishedloading = FALSE; // this will be TRUE when finished
   PRIVATE(thisp)->timersensor->schedule();
 
@@ -668,17 +745,19 @@ SoVRMLImageTexture::image_read_cb(const SbString & filename, SbImage * image, vo
   data->thisp = thisp;
   data->filename = filename;
 
-#if defined(COIN_THREADSAFE)
-  // use a separate thread to load the image  
-  cc_sched_schedule(imagetexture_scheduler,
-                    read_thread, data, 0);
+  if (SoVRMLImageTextureP::scheduler) {
+    // use a separate thread to load the image
+    cc_sched_schedule(SoVRMLImageTextureP::scheduler,
+                      read_thread, data, 0);
+  }
+  else {
+    // schedule a sensor to read the image as soon as the delay sensor
+    // queue is processed (typically when the run-time system is idle)
+    SoOneShotSensor * sensor = new SoOneShotSensor(oneshot_readimage_cb, data);
+    sensor->schedule();
+  }
+
   return TRUE;
-#else // COIN_THREADSAFE
-  // trigger a sensor to read the image
-  SoOneShotSensor * sensor = new SoOneShotSensor(oneshot_readimage_cb, data);
-  sensor->schedule();
-  return TRUE;
-#endif // ! COIN_THREADSAFE
 }
 
 //
@@ -688,7 +767,14 @@ void
 SoVRMLImageTexture::urlSensorCB(void * data, SoSensor *)
 {
   SoVRMLImageTexture * thisp = (SoVRMLImageTexture*) data;
-
+  
+  PRIVATE(thisp)->lock_glimage();
+  if (PRIVATE(thisp)->glimage) {
+    PRIVATE(thisp)->glimage->unref(NULL);
+    PRIVATE(thisp)->glimage = NULL;
+  }
+  PRIVATE(thisp)->unlock_glimage();
+  
   thisp->setReadStatus(1);
   if (thisp->url.getNum() && thisp->url[0].getLength() &&
       !thisp->loadUrl()) {
@@ -701,14 +787,11 @@ SoVRMLImageTexture::urlSensorCB(void * data, SoSensor *)
     if (thisp->url.getNum() == 0 || thisp->url[0].getLength() == 0) {
       // wait for threads to finish in case a new thread is used to
       // load the previous image, and the thread has not finished yet.
-#ifdef COIN_THREADSAFE
-      cc_sched_wait_all(imagetexture_scheduler);
-#endif // COIN_THREADSAFE
-      thisp->pimpl->image.setValue(SbVec2s(0,0), 0, NULL);
-      thisp->pimpl->glimagevalid = FALSE;
-      if (PRIVATE(thisp)->glimage) {
-        PRIVATE(thisp)->glimage->setEndFrameCallback(NULL, NULL);
+      if (SoVRMLImageTextureP::scheduler) {
+        cc_sched_wait_all(SoVRMLImageTextureP::scheduler);
       }
+
+      thisp->pimpl->image.setValue(SbVec2s(0,0), 0, NULL);
     }
   }
 }
@@ -726,10 +809,13 @@ SoVRMLImageTexture::readImage(const SbString & filename)
   else {
     retval = default_prequalify_cb(filename, NULL, this); 
   }
+  PRIVATE(this)->lock_glimage();
   if (PRIVATE(this)->glimage) {
-    PRIVATE(this)->glimage->setEndFrameCallback(glimage_callback, this);
-    PRIVATE(this)->glimagevalid = FALSE;
+    PRIVATE(this)->glimage->unref(NULL);
+    PRIVATE(this)->glimage = NULL;
   }
+  PRIVATE(this)->unlock_glimage();
+
   // set flag that timer sensor will test. 
   PRIVATE(this)->finishedloading = TRUE;
   return retval;
@@ -743,7 +829,12 @@ void
 SoVRMLImageTexture::setImage(const SbImage & image)
 {
   PRIVATE(this)->image = image;
-  PRIVATE(this)->glimagevalid = FALSE;
+  PRIVATE(this)->lock_glimage();
+  if (PRIVATE(this)->glimage) {
+    PRIVATE(this)->glimage->unref(NULL);
+    PRIVATE(this)->glimage = NULL;
+  }
+  PRIVATE(this)->unlock_glimage();
   this->touch(); // destroy caches using this node
 }
 
@@ -778,7 +869,4 @@ SoVRMLImageTextureP::timersensor_cb(void * data, SoSensor * sensor)
 
 #undef PRIVATE
 
-#undef LOCK_GLIMAGE
-#undef UNLOCK_GLIMAGE
-
 #endif // HAVE_VRML97
