@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -124,7 +124,7 @@
   If the color field is NULL, the geometry shall be rendered normally
   using the Material and texture defined in the Appearance node (see
   4.14, Lighting model, for details
-  http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.5).
+  http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.5).
 
   If the normal field is not NULL, it shall contain a Normal node
   whose normals are applied to the vertices or faces of the
@@ -169,17 +169,17 @@
   texture image used on the IndexedFaceSet used in Figure 6.10.
 
   <center>
-  <img src="http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/Images/IFStexture.gif">
+  <img src="http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/Images/IFStexture.gif">
   Figure 6.10
   </center>
 
   <center>
-  <img src="http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/Images/IFStexture2.gif">
+  <img src="http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/Images/IFStexture2.gif">
   Figure 6.11
   </center>
 
   Subclause 4.6.3, Shapes and geometry
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.5>),
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.5>),
   provides a description of the ccw, solid, convex, and creaseAngle
   fields.
 
@@ -206,45 +206,52 @@
 */
 
 #include <Inventor/VRMLnodes/SoVRMLIndexedFaceSet.h>
-#include <Inventor/nodes/SoSubNodeP.h>
-#include <Inventor/actions/SoGLRenderAction.h>
-#include <Inventor/actions/SoGetPrimitiveCountAction.h>
-#include <Inventor/actions/SoCallbackAction.h>
-#include <Inventor/caches/SoNormalCache.h>
-#include <Inventor/caches/SoConvexDataCache.h>
-#include <Inventor/bundles/SoMaterialBundle.h>
-#include <Inventor/bundles/SoTextureCoordinateBundle.h>
-#include <Inventor/elements/SoCacheElement.h>
-#include <Inventor/elements/SoModelMatrixElement.h>
-#include <Inventor/elements/SoCoordinateElement.h>
-#include <Inventor/elements/SoOverrideElement.h>
-#include <Inventor/elements/SoMaterialBindingElement.h>
-#include <Inventor/elements/SoNormalBindingElement.h>
-#include <Inventor/misc/SoGL.h>
+
+#include "../caches/normalcache_numcoords_hack.h"
+#include "../misc/SoVBO.h"
+#include "../misc/SoVertexArrayIndexer.h"
+#include <Inventor/C/glue/glp.h>
+#include <Inventor/SoPrimitiveVertex.h>
 #include <Inventor/VRMLnodes/SoVRMLCoordinate.h>
 #include <Inventor/VRMLnodes/SoVRMLMacros.h>
-#include <Inventor/SoPrimitiveVertex.h>
+#include <Inventor/actions/SoCallbackAction.h>
+#include <Inventor/actions/SoGLRenderAction.h>
+#include <Inventor/actions/SoGetPrimitiveCountAction.h>
+#include <Inventor/bundles/SoMaterialBundle.h>
+#include <Inventor/bundles/SoTextureCoordinateBundle.h>
+#include <Inventor/caches/SoConvexDataCache.h>
+#include <Inventor/caches/SoNormalCache.h>
 #include <Inventor/details/SoFaceDetail.h>
-#include <Inventor/misc/SoGL.h>
-#include "../caches/normalcache_numcoords_hack.h"
-
-#include <Inventor/system/gl.h>
-#include <Inventor/C/glue/glp.h>
-
-#if COIN_DEBUG
+#include <Inventor/elements/SoCacheElement.h>
+#include <Inventor/elements/SoCoordinateElement.h>
+#include <Inventor/elements/SoGLLazyElement.h>
+#include <Inventor/elements/SoGLVBOElement.h>
+#include <Inventor/elements/SoMaterialBindingElement.h>
+#include <Inventor/elements/SoModelMatrixElement.h>
+#include <Inventor/elements/SoNormalBindingElement.h>
+#include <Inventor/elements/SoOverrideElement.h>
 #include <Inventor/errors/SoDebugError.h>
-#endif // COIN_DEBUG
+#include <Inventor/misc/SoGL.h>
+#include <Inventor/misc/SoGL.h>
+#include <Inventor/nodes/SoSubNodeP.h>
+#include <Inventor/system/gl.h>
 
-#ifdef COIN_THREADSAFE
+#ifdef HAVE_THREADS
 #include <Inventor/threads/SbRWMutex.h>
-#endif // COIN_THREADSAFE
+#endif // HAVE_THREADS
+
+// *************************************************************************
 
 // for concavestatus
 #define STATUS_UNKNOWN 0
 #define STATUS_CONVEX  1
 #define STATUS_CONCAVE 2
 
-#ifndef DOXYGEN_SKIP_THIS
+#define LOCK_VAINDEXER(obj) SoBase::staticDataLock()
+#define UNLOCK_VAINDEXER(obj) SoBase::staticDataUnlock()
+
+// *************************************************************************
+
 class SoVRMLIndexedFaceSetP {
 public:
   SoVRMLIndexedFaceSetP(void) 
@@ -252,40 +259,32 @@ public:
     : convexmutex(SbRWMutex::READ_PRECEDENCE)
 #endif // COIN_THREADSAFE 
   { }
+  SoVertexArrayIndexer * vaindexer;
   SoConvexDataCache * convexCache;
   int concavestatus;
+
 #ifdef COIN_THREADSAFE
   SbRWMutex convexmutex;
-#endif // COIN_THREADSAFE
-
-  void readLockConvexCache(void) {
-#ifdef COIN_THREADSAFE
-    this->convexmutex.readLock();
-#endif // COIN_THREADSAFE
-  }
-  void readUnlockConvexCache(void) {
-#ifdef COIN_THREADSAFE
-    this->convexmutex.readUnlock();
-#endif // COIN_THREADSAFE
-  }
-  void writeLockConvexCache(void) {
-#ifdef COIN_THREADSAFE
-    this->convexmutex.writeLock();
-#endif // COIN_THREADSAFE
-  }
-  void writeUnlockConvexCache(void) {
-#ifdef COIN_THREADSAFE
-    this->convexmutex.writeUnlock();
-#endif // COIN_THREADSAFE
-  }
+  void readLockConvexCache(void) { this->convexmutex.readLock(); }
+  void readUnlockConvexCache(void) { this->convexmutex.readUnlock(); }
+  void writeLockConvexCache(void) { this->convexmutex.writeLock(); }
+  void writeUnlockConvexCache(void) { this->convexmutex.writeUnlock(); }
+#else // !COIN_THREADSAFE
+  void readLockConvexCache(void) { }
+  void readUnlockConvexCache(void) { }
+  void writeLockConvexCache(void) { }
+  void writeUnlockConvexCache(void) { }
+#endif // !COIN_THREADSAFE
 };
-#endif // DOXYGEN_SKIP_THIS
 
 #define PRIVATE(obj) ((obj)->pimpl)
 
+// *************************************************************************
 
 SO_NODE_SOURCE(SoVRMLIndexedFaceSet);
 
+// *************************************************************************
+
 // Doc in parent
 void
 SoVRMLIndexedFaceSet::initClass(void) // static
@@ -301,6 +300,7 @@ SoVRMLIndexedFaceSet::SoVRMLIndexedFaceSet(void)
   PRIVATE(this) = new SoVRMLIndexedFaceSetP;
   PRIVATE(this)->convexCache = NULL;
   PRIVATE(this)->concavestatus = STATUS_UNKNOWN;
+  PRIVATE(this)->vaindexer = NULL;
 
   SO_VRMLNODE_INTERNAL_CONSTRUCTOR(SoVRMLIndexedFaceSet);
 
@@ -317,6 +317,7 @@ SoVRMLIndexedFaceSet::SoVRMLIndexedFaceSet(void)
 SoVRMLIndexedFaceSet::~SoVRMLIndexedFaceSet() // virtual, protected
 {
   if (PRIVATE(this)->convexCache) PRIVATE(this)->convexCache->unref();
+  delete PRIVATE(this)->vaindexer;
   delete PRIVATE(this);
 }
 
@@ -517,19 +518,107 @@ SoVRMLIndexedFaceSet::GLRender(SoGLRenderAction * action)
 
   this->setupShapeHints(state, this->ccw.getValue(), this->solid.getValue());
 
-  sogl_render_faceset((SoGLCoordinateElement *)coords,
-                      cindices,
-                      numindices,
-                      normals,
-                      nindices,
-                      &mb,
-                      mindices,
-                      &tb,
-                      tindices,
-                      (int)nbind,
-                      (int)mbind,
-                      doTextures?1:0);
+#ifdef COIN_NEXT_MINOR
+  SoGLLazyElement * lelem = NULL;
+  const uint32_t contextid = action->getCacheContext();
 
+  SbBool dova = 
+    SoVBO::shouldRenderAsVertexArrays(contextid, numindices) &&
+    !convexcacheused && !normalCacheUsed &&
+    ((nbind == OVERALL) || ((nbind == PER_VERTEX_INDEXED) && ((nindices == cindices) || (nindices == NULL)))) &&
+    ((tbind == NONE) || ((tbind == PER_VERTEX_INDEXED) && ((tindices == cindices) || (tindices == NULL)))) &&
+    ((mbind == NONE) || ((mbind == PER_VERTEX_INDEXED) && ((mindices == cindices) || (mindices == NULL)))) &&
+    cc_glglue_has_vertex_array(sogl_glue_instance(state));
+
+  const SoGLVBOElement * vboelem = SoGLVBOElement::getInstance(state);
+  SoVBO * colorvbo = NULL;
+
+  if (dova && (mbind != OVERALL)) {
+    dova = FALSE;
+    if ((mbind == PER_VERTEX_INDEXED) && ((mindices == cindices) || (mindices == NULL))) {
+      lelem = (SoGLLazyElement*) SoLazyElement::getInstance(state);
+      colorvbo = vboelem->getColorVBO();
+      if (colorvbo) dova = TRUE;
+      else {
+        // we might be able to do VA-rendering, but need to check the
+        // diffuse color type first.
+        if (!lelem->isPacked() && lelem->getNumTransparencies() <= 1) {
+          dova = TRUE;
+        }
+      }
+    }
+  }
+  if (dova) {
+    SbBool dovbo = this->startVertexArray(action,
+                                          coords,
+                                          (nbind != OVERALL) ? normals : NULL,
+                                          doTextures,
+                                          mbind != OVERALL);
+    LOCK_VAINDEXER(this);
+    if (PRIVATE(this)->vaindexer == NULL) {
+      SoVertexArrayIndexer * indexer = new SoVertexArrayIndexer;
+      int i = 0;
+      while (i < numindices) {
+        int cnt = 0;
+        while (i + cnt < numindices && cindices[i+cnt] >= 0) cnt++;
+        
+        switch (cnt) {
+        case 3:
+          indexer->addTriangle(cindices[i],cindices[i+1], cindices[i+2]);
+          break;
+        case 4:
+          indexer->addQuad(cindices[i],cindices[i+1],cindices[i+2],cindices[i+3]);
+          break;
+        default:
+          if (cnt > 4) {
+            indexer->beginTarget(GL_POLYGON);
+            for (int j = 0; j < cnt; j++) {
+              indexer->targetVertex(GL_POLYGON, cindices[i+j]);
+            }
+            indexer->endTarget(GL_POLYGON);
+          }
+        }
+        i += cnt + 1;
+      }
+      indexer->close();
+      if (indexer->getNumVertices()) {
+        PRIVATE(this)->vaindexer = indexer;
+      }
+      else {
+        delete indexer;
+      }
+#if 0
+      fprintf(stderr,"XXX: create VRML VertexArrayIndexer: %d\n", indexer->getNumVertices());
+#endif
+    }
+
+    if (PRIVATE(this)->vaindexer) {
+      PRIVATE(this)->vaindexer->render(sogl_glue_instance(state), dovbo, contextid);
+    }
+    UNLOCK_VAINDEXER(this);
+    this->finishVertexArray(action,
+                            dovbo,
+                            (nbind != OVERALL),
+                            doTextures,
+                            mbind != OVERALL);
+  }
+#else // COIN_NEXT_MINOR
+  if (0) { }
+#endif // !COIN_NEXT_MINOR
+  else {
+    sogl_render_faceset((SoGLCoordinateElement *)coords,
+                        cindices,
+                        numindices,
+                        normals,
+                        nindices,
+                        &mb,
+                        mindices,
+                        &tb,
+                        tindices,
+                        (int)nbind,
+                        (int)mbind,
+                        doTextures?1:0);
+  }
   if (normalCacheUsed) {
     this->readUnlockNormalCache();
   }
@@ -841,7 +930,15 @@ SoVRMLIndexedFaceSet::notify(SoNotList * list)
 {
   if (PRIVATE(this)->convexCache) PRIVATE(this)->convexCache->invalidate();
   SoField *f = list->getLastField();
-  if (f == &this->coordIndex) PRIVATE(this)->concavestatus = STATUS_UNKNOWN;
+  if (f == &this->coordIndex) {
+    PRIVATE(this)->concavestatus = STATUS_UNKNOWN;
+    LOCK_VAINDEXER(this);
+    if (PRIVATE(this)->vaindexer) {
+      delete PRIVATE(this)->vaindexer;
+      PRIVATE(this)->vaindexer = NULL;
+    }
+    UNLOCK_VAINDEXER(this);
+  }
   inherited::notify(list);
 }
 
