@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -99,7 +99,7 @@
   Material shall be used to draw the lines. Details on lighting
   equations as they affect IndexedLineSet nodes are described in 4.14,
   Lighting model
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.14>).  
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.14>).  
 
 */
 
@@ -130,22 +130,30 @@
 #include <Inventor/elements/SoGLTextureEnabledElement.h>
 #include <Inventor/elements/SoGLTexture3EnabledElement.h>
 #include <Inventor/elements/SoGLCoordinateElement.h>
-#include <Inventor/elements/SoLazyElement.h>
 #include <Inventor/elements/SoOverrideElement.h>
 #include <Inventor/elements/SoCacheElement.h>
+#include <Inventor/elements/SoGLLazyElement.h>
+#include <Inventor/elements/SoGLCacheContextElement.h>
+#include <Inventor/elements/SoGLVBOElement.h>
 #include <Inventor/elements/SoMaterialBindingElement.h>
 #include <Inventor/bundles/SoTextureCoordinateBundle.h>
 #include <Inventor/details/SoLineDetail.h>
 #include <Inventor/caches/SoBoundingBoxCache.h>
 #include <Inventor/SbColor4f.h>
 #include <Inventor/C/glue/glp.h>
+#include "../misc/SoVertexArrayIndexer.h"
+#include "../misc/SoVBO.h"
+#include "../misc/PointerMap.h"
 
 #if COIN_DEBUG
 #include <Inventor/errors/SoDebugError.h>
 #endif // COIN_DEBUG
 
 class SoVRMLIndexedLineSetP {
-public:
+ public:
+  SoVRMLIndexedLineSetP() : vaindexer(NULL), nodeid(0) { }
+  ~SoVRMLIndexedLineSetP() { delete this->vaindexer; }
+
   enum Binding {
     // Needs to be these specific values to match the rendering code
     // in SoGL.cpp.  FIXME: bad dependency. It looks like the same
@@ -159,8 +167,14 @@ public:
   };
 
   static Binding findMaterialBinding(SoVRMLIndexedLineSet * node, SoState * state);
+  SoVertexArrayIndexer * vaindexer;
+  uint32_t nodeid;
 };
 
+#define PRIVATE(obj) ((SoVRMLIndexedLineSetP*) PointerMap::get(this))
+#define LOCK_VAINDEXER(obj) SoBase::staticDataLock()
+#define UNLOCK_VAINDEXER(obj) SoBase::staticDataUnlock()
+
 
 SO_NODE_SOURCE(SoVRMLIndexedLineSet);
 
@@ -172,11 +186,15 @@ SoVRMLIndexedLineSet::initClass(void) // static
 
 SoVRMLIndexedLineSet::SoVRMLIndexedLineSet(void)
 {
+  PointerMap::add(this, new SoVRMLIndexedLineSetP);
   SO_VRMLNODE_INTERNAL_CONSTRUCTOR(SoVRMLIndexedLineSet);
 }
 
 SoVRMLIndexedLineSet::~SoVRMLIndexedLineSet()
 {
+  SoVRMLIndexedLineSetP * pimpl = PRIVATE(this);
+  PointerMap::remove(this);
+  delete pimpl;
 }
 
 SoVRMLIndexedLineSetP::Binding
@@ -242,6 +260,8 @@ SoVRMLIndexedLineSet::GLRender(SoGLRenderAction * action)
   if (!this->shouldGLRender(action)) 
     return;
 
+  SoVRMLIndexedLineSetP * pimpl = PRIVATE(this);
+  
   // If the coordIndex field is invalid by not including the
   // terminating -1, fix the field by adding it.
   //
@@ -296,19 +316,102 @@ SoVRMLIndexedLineSet::GLRender(SoGLRenderAction * action)
 
   SoMaterialBundle mb(action);
   mb.sendFirst(); // make sure we have the correct material
+ 
+#ifdef COIN_NEXT_MINOR
+  SoGLLazyElement * lelem = NULL;
+  const uint32_t contextid = action->getCacheContext();
 
-  sogl_render_lineset((SoGLCoordinateElement*)coords,
-                      cindices,
-                      numindices,
-                      NULL,
-                      NULL,
-                      &mb,
-                      mindices,
-                      NULL, 0,
-                      0,
-                      (int)mbind,
-                      0,
-                      drawPoints ? 1 : 0);
+  SbBool dova = 
+    !drawPoints &&
+    SoVBO::shouldRenderAsVertexArrays(contextid, numindices) &&
+    cc_glglue_has_vertex_array(sogl_glue_instance(state));
+  
+  const SoGLVBOElement * vboelem = SoGLVBOElement::getInstance(state);
+  SoVBO * colorvbo = NULL;
+  
+  if (dova && (mbind != SoVRMLIndexedLineSetP::OVERALL)) {
+    dova = FALSE;
+    if ((mbind == SoVRMLIndexedLineSetP::PER_VERTEX_INDEXED) && 
+        ((mindices == cindices) || (mindices == NULL))) {
+      lelem = (SoGLLazyElement*) SoLazyElement::getInstance(state);
+      colorvbo = vboelem->getColorVBO();
+      if (colorvbo) dova = TRUE;
+      else {
+        // we might be able to do VA-rendering, but need to check the
+        // diffuse color type first.
+        if (!lelem->isPacked() && lelem->getNumTransparencies() <= 1) {
+          dova = TRUE;
+        }
+      }
+    }
+  }
+  if (dova) {
+    SbBool dovbo = this->startVertexArray(action,
+                                          coords,
+                                          FALSE,
+                                          FALSE,
+                                          mbind != SoVRMLIndexedLineSetP::OVERALL);
+    LOCK_VAINDEXER(this);
+    if (pimpl->vaindexer == NULL || pimpl->nodeid != this->getNodeId()) {
+      delete pimpl->vaindexer;
+      pimpl->vaindexer = NULL;
+      pimpl->nodeid = this->getNodeId();
+      
+      SoVertexArrayIndexer * indexer = new SoVertexArrayIndexer;
+      
+      int i = 0;
+      while (i < numindices) {
+        int cnt = 0;
+        while (i + cnt < numindices && cindices[i+cnt] >= 0) cnt++;
+        if (cnt >= 2) {
+          for (int j = 1; j < cnt;j++) {
+            indexer->addLine(cindices[i+j-1],
+                             cindices[i+j]);
+          }
+        }
+        i += cnt + 1;
+      }
+      indexer->close();
+      if (indexer->getNumVertices()) {
+        pimpl->vaindexer = indexer;
+      }
+      else {
+        delete indexer;
+      }
+#if 0
+      fprintf(stderr,"XXX: ILS create VertexArrayIndexer: %d\n", indexer->getNumVertices());
+#endif
+    }
+    
+    if (pimpl->vaindexer) {
+      pimpl->vaindexer->render(sogl_glue_instance(state), dovbo, contextid);
+    }
+    UNLOCK_VAINDEXER(this);
+
+    this->finishVertexArray(action,
+                            dovbo,
+                            FALSE,
+                            FALSE,
+                            mbind != SoVRMLIndexedLineSetP::OVERALL);
+  }
+#else // COIN_NEXT_MINOR
+  if (0) {}
+#endif // !COIN_NEXT_MINOR
+  else {
+    
+    sogl_render_lineset((SoGLCoordinateElement*)coords,
+                        cindices,
+                        numindices,
+                        NULL,
+                        NULL,
+                        &mb,
+                        mindices,
+                        NULL, 0,
+                        0,
+                        (int)mbind,
+                        0,
+                        drawPoints ? 1 : 0);
+  }
   // send approx number of triangles for autocache handling
   sogl_autocache_update(state, this->coordIndex.getNum() / 2);
 }
@@ -438,4 +541,7 @@ SoVRMLIndexedLineSet::generatePrimitives(SoAction * action)
   state->pop();
 }
 
+#undef LOCK_VAINDEXER
+#undef PRIVATE
+#undef UNLOCK_VAINDEXER
 #endif // HAVE_VRML97
