@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -62,7 +62,7 @@
   to obtain, the higher preference VRML file. Details on the url field
   and preference order can be found in 4.5, VRML and the World Wide
   Web
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.5>).  
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.5>).  
 
   The results are undefined if the contents of the URL change after it
   has been loaded.  
@@ -75,7 +75,7 @@
   implies that the bounding box is not specified and if needed shall
   be calculated by the browser. A description of the bboxCenter and
   bboxSize fields is in 4.6.4, Bounding boxes
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.4>).  
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.4>).  
 
 */
 
@@ -158,10 +158,13 @@ static SbColor * sovrmlinline_bboxcolor = NULL;
 static SbBool sovrmlinline_readassofile = TRUE;
 
 static void
-sovrmlinline_cleanup_color(void)
+sovrmlinline_cleanup(void)
 {
   delete sovrmlinline_bboxcolor;
   sovrmlinline_bboxcolor = NULL;
+  sovrmlinline_bboxvisibility = SoVRMLInline::UNTIL_LOADED;
+  sovrmlinline_fetchurlcb = NULL;  
+  sovrmlinline_readassofile = TRUE;
 }
 
 SO_NODE_SOURCE(SoVRMLInline);
@@ -172,7 +175,7 @@ SoVRMLInline::initClass(void)
 {
   SO_NODE_INTERNAL_INIT_CLASS(SoVRMLInline, SO_VRML97_NODE_TYPE);
   sovrmlinline_bboxcolor = new SbColor(0.8f, 0.8f, 0.8f);
-  coin_atexit((coin_atexit_f*) sovrmlinline_cleanup_color, 0);
+  coin_atexit((coin_atexit_f*) sovrmlinline_cleanup, CC_ATEXIT_NORMAL);
   SoAudioRenderAction::addMethod(SoVRMLInline::getClassTypeId(),
                                  SoAudioRenderAction::callDoAction);
 }
