@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -40,7 +40,7 @@
   For more information, a detailed discussion of interpolators is
   available in section 4.6.8 of the VRML97 specification:
 
-  <http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.8>
+  <http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.8>
 */
 
 /*!
@@ -78,7 +78,7 @@ SO_NODEENGINE_ABSTRACT_SOURCE(SoVRMLInterpolator);
 void
 SoVRMLInterpolator::initClass(void) // static
 {
-  SO_NODEENGINE_INIT_ABSTRACT_CLASS(SoVRMLInterpolator, SoNodeEngine, "NodeEngine");
+  SO_NODEENGINE_INTERNAL_INIT_ABSTRACT_CLASS(SoVRMLInterpolator);
 }
 
 SoVRMLInterpolator::SoVRMLInterpolator(void) // protected
