@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -55,11 +55,24 @@
 #include <Inventor/VRMLnodes/SoVRMLNormal.h>
 #include <Inventor/VRMLnodes/SoVRMLMacros.h>
 #include <Inventor/nodes/SoSubNodeP.h>
-#include <Inventor/actions/SoAction.h>
+#include <Inventor/actions/SoGLRenderAction.h>
 #include <Inventor/elements/SoNormalElement.h>
+#include <Inventor/elements/SoGLVBOElement.h>
+#include "../misc/SoVBO.h"
+#include "../misc/PointerMap.h"
 
 SO_NODE_SOURCE(SoVRMLNormal);
 
+class SoVRMLNormalP {
+ public:
+  SoVRMLNormalP() : vbo(NULL) { }
+  ~SoVRMLNormalP() { delete this->vbo; }
+  
+  SoVBO * vbo;
+};
+
+#define PRIVATE(obj) ((SoVRMLNormalP*) PointerMap::get(obj))
+
 // Doc in parent
 void
 SoVRMLNormal::initClass(void)
@@ -72,6 +85,7 @@ SoVRMLNormal::initClass(void)
 */
 SoVRMLNormal::SoVRMLNormal(void)
 {
+  PointerMap::add(this, new SoVRMLNormalP);
   SO_VRMLNODE_INTERNAL_CONSTRUCTOR(SoVRMLNormal);
 
   SO_VRMLNODE_ADD_EMPTY_EXPOSED_MFIELD(vector);
@@ -82,6 +96,9 @@ SoVRMLNormal::SoVRMLNormal(void)
 */
 SoVRMLNormal::~SoVRMLNormal()
 {
+  SoVRMLNormalP * pimpl = PRIVATE(this);
+  PointerMap::remove(this);
+  delete pimpl;  
 }
 
 // Doc in parent
@@ -96,12 +113,39 @@ SoVRMLNormal::doAction(SoAction * action)
 void
 SoVRMLNormal::GLRender(SoGLRenderAction * action)
 {
-  //
-  // FIXME: code to test if all normals are unit length, and store
-  // this in some cached variable.  should be passed on to
-  // SoGLNormalizeElement to optimize rendering (pederb)
-  //
   SoVRMLNormal::doAction((SoAction*) action);
+#ifdef COIN_NEXT_MINOR
+  SoVRMLNormalP * pimpl = PRIVATE(this);
+  SoState * state = action->getState();
+  
+  SoBase::staticDataLock();
+  SbBool setvbo = FALSE;
+  const int num = this->vector.getNum();
+  if (SoGLVBOElement::shouldCreateVBO(state, num)) {
+    setvbo = TRUE;
+    SbBool dirty = FALSE;
+    if (pimpl->vbo == NULL) {
+      pimpl->vbo = new SoVBO(GL_ARRAY_BUFFER, GL_STATIC_DRAW); 
+      dirty =  TRUE;
+    }
+    else if (pimpl->vbo->getBufferDataId() != this->getNodeId()) {
+      dirty = TRUE;
+    }
+    if (dirty) {
+      pimpl->vbo->setBufferData(this->vector.getValues(0),
+                                num*sizeof(SbVec3f),
+                                this->getNodeId());
+    }
+  }
+  else if (pimpl->vbo && pimpl->vbo->getBufferDataId()) {
+    // clear buffers to deallocate VBO memory
+    pimpl->vbo->setBufferData(NULL, 0, 0);
+  }
+  SoBase::staticDataUnlock();
+  if (setvbo) {
+    SoGLVBOElement::setNormalVBO(state, pimpl->vbo);
+  }
+#endif // COIN_NEXT_MINOR
 }
 
 // Doc in parent
@@ -125,4 +169,5 @@ SoVRMLNormal::getPrimitiveCount(SoGetPrimitiveCountAction * action)
   SoVRMLNormal::doAction((SoAction*) action);
 }
 
+#undef PRIVATE
 #endif // HAVE_VRML97
