@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -44,9 +44,11 @@
 #include <Inventor/nodes/SoInfo.h>
 #include <Inventor/C/tidbitsp.h>
 
-#ifdef COIN_THREADSAFE
+#ifdef HAVE_THREADS
 #include <Inventor/threads/SbMutex.h>
-#endif // COIN_THREADSAFE
+#endif // HAVE_THREADS
+
+// *************************************************************************
 
 /*!
   \var SoMFNode SoVRMLParent::children
@@ -63,7 +65,7 @@
   An event in that is used to remove children from this node.
 */
 
-#ifndef DOXYGEN_SKIP_THIS
+// *************************************************************************
 
 class SoVRMLParentP {
 public:
@@ -73,21 +75,14 @@ public:
 
 #ifdef COIN_THREADSAFE
   SbMutex childlistmutex;
-#endif // COIN_THREADSAFE
-  void lockChildList(void) {
-#ifdef COIN_THREADSAFE
-    this->childlistmutex.lock();
-#endif // COIN_THREADSAFE
-  }
-  void unlockChildList(void) {
-#ifdef COIN_THREADSAFE
-    this->childlistmutex.unlock();
-#endif // COIN_THREADSAFE
-  }
+  void lockChildList(void) { this->childlistmutex.lock(); }
+  void unlockChildList(void) { this->childlistmutex.unlock(); }
+#else // !COIN_THREADSAFE
+  void lockChildList(void) { }
+  void unlockChildList(void) { }
+#endif // !COIN_THREADSAFE
 };
 
-#endif // DOXYGEN_SKIP_THIS
-
 static SoInfo * vrmlparent_nullnode = NULL;
 
 static void
@@ -97,8 +92,12 @@ vrmlparent_cleanup(void)
   vrmlparent_nullnode = NULL;
 }
 
+// *************************************************************************
+
 SO_NODE_ABSTRACT_SOURCE(SoVRMLParent);
 
+// *************************************************************************
+
 // Doc in parent
 void
 SoVRMLParent::initClass(void)
@@ -107,7 +106,7 @@ SoVRMLParent::initClass(void)
   // used when the 'children' field contains NULL-nodes
   vrmlparent_nullnode = new SoInfo;
   vrmlparent_nullnode->ref();
-  coin_atexit((coin_atexit_f*) vrmlparent_cleanup, 0);
+  coin_atexit((coin_atexit_f*) vrmlparent_cleanup, CC_ATEXIT_NORMAL);
 }
 
 #define PRIVATE(thisp) ((thisp)->pimpl)
