@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -52,17 +52,17 @@
   lower-left pixel of the pixel image corresponds to s=0.0, t=0.0, and
   the top-right pixel of the image corresponds to s = 1.0, t = 1.0.
   See 4.6.11, Texture maps
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.11>),
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.11>),
   for a general description of texture
   maps. Figure 6.13 depicts an example PixelTexture.
 
   <center>
-  <img src="http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/Images/PixelTexture.gif">
+  <img src="http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/Images/PixelTexture.gif">
   Figure 6.13 -- PixelTexture node
   </center>
 
   See 4.14, Lighting model
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.14>),
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.14>),
   for a description of how the texture values interact with the
   appearance of the geometry.  SoSFImage, describes the
   specification of an image.  The repeatS and repeatT fields specify
@@ -103,39 +103,42 @@
 #include <Inventor/errors/SoDebugError.h>
 #include <Inventor/SbImage.h>
 
-#ifdef COIN_THREADSAFE
+#ifdef HAVE_THREADS
 #include <Inventor/threads/SbMutex.h>
-#endif // COIN_THREADSAFE
+#endif // HAVE_THREADS
 
-#ifndef DOXYGEN_SKIP_THIS
+// *************************************************************************
 
 class SoVRMLPixelTextureP {
 public:
   SoGLImage * glimage;
   SbBool glimagevalid;
   int readstatus;
+
 #ifdef COIN_THREADSAFE
   SbMutex glimagemutex;
-#endif // COIN_THREADSAFE
+  void lock_glimage(void) { this->glimagemutex.lock(); }
+  void unlock_glimage(void) { this->glimagemutex.unlock(); }
+#else // !COIN_THREADSAFE
+  void lock_glimage(void) { }
+  void unlock_glimage(void) { }
+#endif // !COIN_THREADSAFE
 };
 
-#endif // DOXYGEN_SKIP_THIS
-
-#ifdef COIN_THREADSAFE
-#define LOCK_GLIMAGE(_thisp_) (_thisp_)->pimpl->glimagemutex.lock()
-#define UNLOCK_GLIMAGE(_thisp_) (_thisp_)->pimpl->glimagemutex.unlock()
-#else // COIN_THREADSAFE
-#define LOCK_GLIMAGE(_thisp_)
-#define UNLOCK_GLIMAGE(_thisp_)
-#endif // COIN_THREADSAFE
+// *************************************************************************
 
 SO_NODE_SOURCE(SoVRMLPixelTexture);
 
+// *************************************************************************
+
 // Doc in parent
 void
 SoVRMLPixelTexture::initClass(void)
 {
   SO_NODE_INTERNAL_INIT_CLASS(SoVRMLPixelTexture, SO_VRML97_NODE_TYPE);
+
+  SoType type = SoVRMLPixelTexture::getClassTypeId();
+  SoRayPickAction::addMethod(type, SoNode::rayPickS);
 }
 
 #define PRIVATE(obj) ((obj)->pimpl)
@@ -222,7 +225,7 @@ SoVRMLPixelTexture::GLRender(SoGLRenderAction * action)
 
   float quality = SoTextureQualityElement::get(state);
 
-  LOCK_GLIMAGE(this);
+  PRIVATE(this)->lock_glimage();
 
   if (!PRIVATE(this)->glimagevalid) {
     int nc;
@@ -264,7 +267,7 @@ SoVRMLPixelTexture::GLRender(SoGLRenderAction * action)
     }
   }
 
-  UNLOCK_GLIMAGE(this);
+  PRIVATE(this)->unlock_glimage();
 
   SoGLTextureImageElement::set(state, this,
                                PRIVATE(this)->glimagevalid ? PRIVATE(this)->glimage : NULL,
@@ -307,7 +310,5 @@ SoVRMLPixelTexture::notify(SoNotList * list)
 }
 
 #undef PRIVATE
-#undef LOCK_GLIMAGE
-#undef UNLOCK_GLIMAGE
 
 #endif // HAVE_VRML97
