@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -60,7 +60,7 @@
   this PointSet node, the emissiveColor of the Material node shall be
   used to draw the points. More details on lighting equations can be
   found in 4.14, Lighting model
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.14>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.14>).
 
 */
 
@@ -94,6 +94,9 @@
 #include <Inventor/SbColor4f.h>
 #include <Inventor/elements/SoOverrideElement.h>
 #include <Inventor/elements/SoMaterialBindingElement.h>
+#include <Inventor/elements/SoGLVBOElement.h>
+#include <Inventor/elements/SoGLLazyElement.h>
+#include "../misc/SoVBO.h"
 
 #if COIN_DEBUG
 #include <Inventor/errors/SoDebugError.h>
@@ -164,15 +167,54 @@ SoVRMLPointSet::GLRender(SoGLRenderAction * action)
     mb.sendFirst();
   }
 
-  sogl_render_pointset((SoGLCoordinateElement*) coords,
-                       NULL,
-                       matpervertex ? &mb : NULL,
-                       NULL,
-                       coords->getNum(), 0);
+  const cc_glglue * glue = sogl_glue_instance(state);
 
+  const int numpts = coords->getNum();
+  
+#ifdef COIN_NEXT_MINOR
+  const uint32_t contextid = action->getCacheContext();
+  SbBool dova = 
+    SoVBO::shouldRenderAsVertexArrays(contextid, numpts) && 
+    cc_glglue_has_vertex_array(glue);
+  
+  if (dova && matpervertex) {
+    const SoGLVBOElement * vboelem = SoGLVBOElement::getInstance(state);
+    if (vboelem->getColorVBO() == NULL) {
+      dova = FALSE;
+      // we might be able to do VA-rendering, but need to check the
+      // diffuse color type first.
+      SoGLLazyElement * lelem = (SoGLLazyElement*) SoLazyElement::getInstance(state);
+      if (!lelem->isPacked() && lelem->getNumTransparencies() <= 1) {
+        dova = TRUE;
+      }
+    }
+  }
+  if (dova) {
+    SbBool vbo = this->startVertexArray(action,
+                                        coords,
+                                        NULL,
+                                        FALSE,
+                                        matpervertex);
+    
+    cc_glglue_glDrawArrays(glue, GL_POINTS, 0, numpts);
+    this->finishVertexArray(action, vbo,
+                            FALSE,
+                            FALSE,
+                            matpervertex);
+  }
+#else // COIN_NEXT_MINOR
+  if (0) { }
+#endif // !COIN_NEXT_MINOR
+  else {
+    sogl_render_pointset((SoGLCoordinateElement*) coords,
+                         NULL,
+                         matpervertex ? &mb : NULL,
+                         NULL,
+                         numpts, 0);
+  }
 
   // send approx number of points for autocache handling
-  sogl_autocache_update(state, coords->getNum()); 
+  sogl_autocache_update(state, numpts); 
 }
 
 // Doc in parent
