@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -74,11 +74,11 @@
   Script node's function. That code is referred to as the "script" in
   the rest of this description. Details on the url field can be found
   in 4.5, VRML and the World Wide Web
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.5>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.5>).
 
   Browsers are not required to support any specific language. Detailed
   information on scripting languages is described in 4.12, Scripting
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.12>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.12>).
 
   Browsers supporting a scripting language for which a language
   binding is specified shall adhere to that language binding.
@@ -191,12 +191,12 @@ public:
   }
 
   static SbBool debug(void);
+  static void cleanup(void);
 
 #ifdef COIN_HAVE_JAVASCRIPT
   void initialize(void);
   void shutdown(void);
 
-  static void cleanup(void);
   static SbBool allowSpiderMonkey(void);
   static SbBool useSpiderMonkey(void);
 #endif // !COIN_HAVE_JAVASCRIPT
@@ -212,6 +212,8 @@ public:
   SoJavaScriptEngine * engine;
 #endif // !COIN_HAVE_JAVASCRIPT
 
+  static SbBool spidermonkey_init_failed;
+
 private:
   SoVRMLScript * master;
 };
@@ -219,10 +221,14 @@ private:
 #define PUBLIC(p) ((p)->master)
 #define PRIVATE(p) ((p)->pimpl)
 
-#ifdef COIN_HAVE_JAVASCRIPT
+SbBool SoVRMLScriptP::spidermonkey_init_failed = FALSE;
+
+// *************************************************************************
+
 void
 SoVRMLScriptP::cleanup(void)
 {
+#ifdef COIN_HAVE_JAVASCRIPT
   // FIXME: need to make sure this is added to atexit only after
   // the engine has started. 20050720 erikgors.
   if (SoJavaScriptEngine::getRuntime() == NULL)
@@ -232,9 +238,14 @@ SoVRMLScriptP::cleanup(void)
     // Destroy javascript runtime
     SoJavaScriptEngine::shutdown();
   }
-}
 #endif // !COIN_HAVE_JAVASCRIPT
 
+  // reset static var
+  SoVRMLScriptP::spidermonkey_init_failed = FALSE;
+  sovrmlscript_eval_cb = NULL;
+  sovrmlscript_eval_closure = NULL;
+}
+
 // *************************************************************************
 
 SbBool
@@ -297,10 +308,17 @@ SoVRMLScript::initClass(void) // static
 SoVRMLScript::SoVRMLScript(void)
   : fielddata(NULL)
 {
+  coin_atexit((coin_atexit_f *)SoVRMLScriptP::cleanup, CC_ATEXIT_NORMAL);
+
 #ifdef COIN_HAVE_JAVASCRIPT
-  if (!SoJavaScriptEngine::getRuntime() && SoVRMLScriptP::allowSpiderMonkey()) {
-    SoJavaScriptEngine::init();
-    coin_atexit((coin_atexit_f *)SoVRMLScriptP::cleanup, 0);
+  if (SoVRMLScriptP::allowSpiderMonkey() &&
+      !SoVRMLScriptP::spidermonkey_init_failed &&
+      // FIXME: next line is a hack-ish way of checking whether init()
+      // has already been done on the SoJavaScriptEngine class.
+      // 20060207 mortene.
+      (SoJavaScriptEngine::getRuntime() == NULL)) {
+    SbBool ok = SoJavaScriptEngine::init();
+    if (!ok) { SoVRMLScriptP::spidermonkey_init_failed = TRUE; }
   }
 #endif // !COIN_HAVE_JAVASCRIPT
 
@@ -482,12 +500,18 @@ SoVRMLScript::write(SoWriteAction * action)
         }
         else {
           out->write(fieldname.getString());
+          // the write() call below is needed to resolve
+          // ROUTES. SoField::write() will not write anything since if
+          // we get here, the field is either an EVENTIN or an
+          // EVENTOUT field
+          f->write(out, fieldname); 
         }
         out->write("\n");
-        ((SoField*)f)->setDefault(TRUE); // don't write again when we do a SoFieldData::write later. 
+      }
+      else if (f->shouldWrite()){
+        f->write(out, fieldname);
       }
     }
-    fd->write(out, this);
     this->writeFooter(out);
   }
   else assert(0 && "unknown stage");
@@ -707,10 +731,10 @@ void
 SoVRMLScriptP::initialize(void)
 {
   if (this->engine != NULL) {
-      if (SoVRMLScriptP::debug()) {
-        SoDebugError::postInfo("SoVRMLScriptP::initialize",
-                               "restarting script engine");
-      }
+    if (SoVRMLScriptP::debug()) {
+      SoDebugError::postInfo("SoVRMLScriptP::initialize",
+                             "restarting script engine");
+    }
     this->shutdown();
   }
 
@@ -727,7 +751,8 @@ SoVRMLScriptP::initialize(void)
       if (!SoVRMLScriptP::useSpiderMonkey()) {
         if (SoVRMLScriptP::debug()) {
           SoDebugError::postInfo("SoVRMLScriptP::initialize",
-                                 "SpiderMonkey Javascript engine not available");
+                                 "Only the SpiderMonkey Javascript engine "
+                                 "currently supported.");
         }
         continue;
       }
@@ -739,9 +764,9 @@ SoVRMLScriptP::initialize(void)
   }
 
   if (this->engine == NULL) {
-      SoDebugError::postWarning("SoVRMLScript::initialize",
-                                "No script language evaluation engine available.");
-      return;
+    SoDebugError::postWarning("SoVRMLScript::initialize",
+                              "No script language evaluation engine available.");
+    return;
   }
 
   // FIXME: should scriptFields be set before or after the script has been
