@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -48,11 +48,11 @@
   geometry. The geometry field contains a geometry node. The specified
   geometry node is rendered with the specified appearance nodes
   applied. See 4.6.3, Shapes and geometry
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.3>),
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.3>),
   and SoVRMLAppearance, for more information.
 
   4.14, Lighting model
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.14>),
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.14>),
   contains details of the VRML lighting model and the interaction
   between Appearance nodes and geometry nodes.  If the geometry field
   is NULL, the object is not drawn.
@@ -101,6 +101,7 @@
 
 #include <stddef.h>
 
+#include <Inventor/C/tidbitsp.h>
 #include <Inventor/VRMLnodes/SoVRMLShape.h>
 #include <Inventor/VRMLnodes/SoVRMLMacros.h>
 #include <Inventor/VRMLnodes/SoVRMLAppearance.h>
@@ -128,43 +129,47 @@
 
 #include <Inventor/system/gl.h>
 
-#ifdef COIN_THREADSAFE
+#ifdef HAVE_THREADS
 #include <Inventor/threads/SbMutex.h>
-#endif // COIN_THREADSAFE
+#endif // HAVE_THREADS
 
-#ifndef DOXYGEN_SKIP_THIS
+// *************************************************************************
 
 class SoVRMLShapeP {
 public:
   SoGLCacheList * cachelist;
   SoChildList * childlist;
   SbBool childlistvalid;
+
 #ifdef COIN_THREADSAFE
   SbMutex childlistmutex;
-#endif // COIN_THREADSAFE
-  void lockChildList(void) {
-#ifdef COIN_THREADSAFE
-    this->childlistmutex.lock();
-#endif // COIN_THREADSAFE
-  }
-  void unlockChildList(void) {
-#ifdef COIN_THREADSAFE
-    this->childlistmutex.unlock();
-#endif // COIN_THREADSAFE
-  }
+  void lockChildList(void) { this->childlistmutex.lock(); }
+  void unlockChildList(void) { this->childlistmutex.unlock(); }
+#else // !COIN_THREADSAFE
+  void lockChildList(void) { }
+  void unlockChildList(void) { }
+#endif // !COIN_THREADSAFE
 };
 
-#endif // DOXYGEN_SKIP_THIS
-
+// *************************************************************************
 
 SO_NODE_SOURCE(SoVRMLShape);
 
-int SoVRMLShape::numrendercaches;
+// *************************************************************************
+
+static int sovrmlshape_numrendercaches = 0;
+
+static void sovrmlshape_cleanup(void) {                                           
+  sovrmlshape_numrendercaches = 0;
+}
+
+// *************************************************************************
 
 void
 SoVRMLShape::initClass(void) // static
 {
   SO_NODE_INTERNAL_INIT_CLASS(SoVRMLShape, SO_VRML97_NODE_TYPE);
+  coin_atexit((coin_atexit_f*)sovrmlshape_cleanup, CC_ATEXIT_NORMAL); 
 
   SoType type = SoVRMLShape::getClassTypeId();
   SoRayPickAction::addMethod(type, SoNode::rayPickS);
@@ -208,13 +213,13 @@ SoVRMLShape::~SoVRMLShape()
 void
 SoVRMLShape::setNumRenderCaches(int num)
 {
-  SoVRMLShape::numrendercaches = num;
+  sovrmlshape_numrendercaches = num;
 }
 
 int
 SoVRMLShape::getNumRenderCaches(void)
 {
-  return SoVRMLShape::numrendercaches;
+  return sovrmlshape_numrendercaches;
 }
 
 SbBool
