@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -82,7 +82,7 @@
   sounds to play when there are more active Sound nodes than can be
   played at once due to either limited system resources or system
   load. 7.3.4, Sound priority, attenuation, and spatialization
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#7.3.4>),
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#7.3.4>),
   describes a recommended algorithm for determining which sounds to
   play under such circumstances. The priority field ranges from 0.0 to
   1.0, with 1.0 being the highest priority and 0.0 the lowest
@@ -135,7 +135,7 @@
   boundary (see Figure 6.14).
 
   <center>
-  <img src="http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/Images/Sound.gif">
+  <img src="http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/Images/Sound.gif">
   Figure 6.14 -- Sound node geometry
   </center>
 
@@ -147,7 +147,7 @@
   playback. Details outlining the minimum required spatialization
   functionality can be found in 7.3.4, Sound priority, attenuation,
   and spatialization
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#7.3.4>),
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#7.3.4>),
   If the spatialize field is FALSE, then directional effects are
   ignored, but the ellipsoid dimensions and intensity will still
   affect the loudness of the sound.  If the sound source is
@@ -411,6 +411,7 @@ SoVRMLSound::SoVRMLSound(void)
   PRIVATE(this)->exitthread = FALSE;
   PRIVATE(this)->errorInThread = FALSE;
   PRIVATE(this)->audioBuffer = NULL;
+  PRIVATE(this)->bufferLength = 0;
 
   this->setBufferingProperties(SoVRMLSoundP::defaultBufferLength,
                                SoVRMLSoundP::defaultNumBuffers,
