@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -43,7 +43,7 @@
 
   The Switch grouping node traverses zero or one of the nodes
   specified in the choice field.  4.6.5, Grouping and children nodes
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.5>),
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.5>),
   describes details on the types of nodes that are legal values for
   choice.
 
@@ -75,59 +75,55 @@
 #include <stddef.h>
 
 #include <Inventor/VRMLnodes/SoVRMLSwitch.h>
-#include <Inventor/VRMLnodes/SoVRMLMacros.h>
-#include <Inventor/VRMLnodes/SoVRMLParent.h>
-#include <Inventor/nodes/SoSubNodeP.h>
-
-#include <Inventor/actions/SoGetBoundingBoxAction.h>
-#include <Inventor/actions/SoSearchAction.h>
-#include <Inventor/actions/SoGetMatrixAction.h>
-#include <Inventor/actions/SoGLRenderAction.h>
-#include <Inventor/actions/SoPickAction.h>
-#include <Inventor/actions/SoHandleEventAction.h>
-#include <Inventor/actions/SoCallbackAction.h>
-#include <Inventor/actions/SoGetPrimitiveCountAction.h>
-#include <Inventor/actions/SoWriteAction.h>
-#include <Inventor/actions/SoAudioRenderAction.h>
-#include <Inventor/elements/SoSwitchElement.h>
-#include <Inventor/elements/SoSoundElement.h>
-#include <Inventor/SoOutput.h>
-#include <Inventor/misc/SoChildList.h>
-
-#if COIN_DEBUG
-#include <Inventor/errors/SoDebugError.h>
-#endif // COIN_DEBUG
-
-#ifdef COIN_THREADSAFE
-#include <Inventor/threads/SbMutex.h>
-#endif // COIN_THREADSAFE
 
 #include "../nodes/SoSoundElementHelper.h"
+#include <Inventor/SoOutput.h>
+#include <Inventor/VRMLnodes/SoVRMLMacros.h>
+#include <Inventor/VRMLnodes/SoVRMLParent.h>
+#include <Inventor/actions/SoAudioRenderAction.h>
+#include <Inventor/actions/SoCallbackAction.h>
+#include <Inventor/actions/SoGLRenderAction.h>
+#include <Inventor/actions/SoGetBoundingBoxAction.h>
+#include <Inventor/actions/SoGetMatrixAction.h>
+#include <Inventor/actions/SoGetPrimitiveCountAction.h>
+#include <Inventor/actions/SoHandleEventAction.h>
+#include <Inventor/actions/SoPickAction.h>
+#include <Inventor/actions/SoSearchAction.h>
+#include <Inventor/actions/SoWriteAction.h>
+#include <Inventor/elements/SoSoundElement.h>
+#include <Inventor/elements/SoSwitchElement.h>
+#include <Inventor/errors/SoDebugError.h>
+#include <Inventor/misc/SoChildList.h>
+#include <Inventor/nodes/SoSubNodeP.h>
+
+#ifdef HAVE_THREADS
+#include <Inventor/threads/SbMutex.h>
+#endif // HAVE_THREADS
+
+// *************************************************************************
 
-#ifndef DOXYGEN_SKIP_THIS
 class SoVRMLSwitchP : public SoSoundElementHelper {
 public:
   SbBool childlistvalid;
+
 #ifdef COIN_THREADSAFE
   SbMutex childlistmutex;
-#endif // COIN_THREADSAFE
-  void lockChildList(void) {
-#ifdef COIN_THREADSAFE
-    this->childlistmutex.lock();
-#endif // COIN_THREADSAFE
-  }
-  void unlockChildList(void) {
-#ifdef COIN_THREADSAFE
-    this->childlistmutex.unlock();
-#endif // COIN_THREADSAFE
-  }
+  void lockChildList(void) { this->childlistmutex.lock(); }
+  void unlockChildList(void) { this->childlistmutex.unlock(); }
+#else // !COIN_THREADSAFE
+  void lockChildList(void) { }
+  void unlockChildList(void) { }
+#endif // !COIN_THREADSAFE
 };
-#endif // DOXYGEN_SKIP_THIS
 
 #define PRIVATE(thisp) ((thisp)->pimpl)
 
+// *************************************************************************
+
 SO_NODE_SOURCE(SoVRMLSwitch);
 
+// *************************************************************************
+
 // Doc in parent
 void
 SoVRMLSwitch::initClass(void)
@@ -359,8 +355,8 @@ SoVRMLSwitch::doAction(SoAction * action)
         if (idx >= this->getNumChildren()) {
 #if COIN_DEBUG
           SoDebugError::post("SoVRMLSwitch::doAction",
-                             "whichChoice %d out of range (0-%d).",
-                             idx, this->getNumChildren());
+                             "whichChoice %d out of range (-1 - %d).",
+                             idx, this->getNumChildren()-1);
 #endif // COIN_DEBUG
         }
         else {
