@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -54,7 +54,7 @@
   values defined in the fontStyle field (see SoVRMLFontStyle).
   Text nodes may contain multiple text strings specified using the
   UTF-8 encoding as specified by ISO 10646-1:1993 (see
-  <http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/references.html#[UTF8]>).
+  <http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/references.html#[UTF8]>).
   The text strings are stored in the order in which the text mode
   characters are to be produced as defined by the parameters in the
   FontStyle node.
@@ -114,36 +114,36 @@
 #include <string.h>
 
 #include <Inventor/VRMLnodes/SoVRMLText.h>
-#include <Inventor/VRMLnodes/SoVRMLMacros.h>
-#include <Inventor/VRMLnodes/SoVRMLFontStyle.h>
-
-#include <Inventor/elements/SoFontNameElement.h>
-#include <Inventor/elements/SoFontSizeElement.h>
 
+#include "../fonts/glyph3d.h"
 #include <Inventor/SoPrimitiveVertex.h>
-#include <Inventor/nodes/SoSubNodeP.h>
+#include <Inventor/VRMLnodes/SoVRMLFontStyle.h>
+#include <Inventor/VRMLnodes/SoVRMLMacros.h>
 #include <Inventor/actions/SoGLRenderAction.h>
 #include <Inventor/actions/SoGetPrimitiveCountAction.h>
 #include <Inventor/bundles/SoMaterialBundle.h>
+#include <Inventor/caches/SoGlyphCache.h>
 #include <Inventor/details/SoTextDetail.h>
+#include <Inventor/elements/SoCacheElement.h>
+#include <Inventor/elements/SoComplexityTypeElement.h>
+#include <Inventor/elements/SoFontNameElement.h>
+#include <Inventor/elements/SoFontSizeElement.h>
+#include <Inventor/elements/SoGLCacheContextElement.h>
 #include <Inventor/elements/SoGLTexture3EnabledElement.h>
 #include <Inventor/elements/SoGLTextureEnabledElement.h>
 #include <Inventor/errors/SoDebugError.h>
 #include <Inventor/lists/SbList.h>
 #include <Inventor/misc/SoGlyph.h>
-#include <Inventor/system/gl.h>
-#include <Inventor/sensors/SoFieldSensor.h>
 #include <Inventor/nodes/SoAsciiText.h>
-#include <Inventor/caches/SoGlyphCache.h>
-#include <Inventor/elements/SoCacheElement.h>
-#include <Inventor/elements/SoComplexityTypeElement.h>
-#include <Inventor/elements/SoGLCacheContextElement.h>
+#include <Inventor/nodes/SoSubNodeP.h>
+#include <Inventor/sensors/SoFieldSensor.h>
+#include <Inventor/system/gl.h>
 
-#ifdef COIN_THREADSAFE
+#ifdef HAVE_THREADS
 #include <Inventor/threads/SbMutex.h>
-#endif // COIN_THREADSAFE
+#endif // HAVE_THREADS
 
-#include "../fonts/glyph3d.h"
+// *************************************************************************
 
 class SoVRMLTextP {
 public:
@@ -172,26 +172,26 @@ public:
   int fontfamily;
   int fontstyle;
 
-  void lock(void) {
 #ifdef COIN_THREADSAFE
-    this->mutex.lock();
-#endif // COIN_THREADSAFE
-  }
-  void unlock(void) {
-#ifdef COIN_THREADSAFE
-    this->mutex.unlock();
-#endif // COIN_THREADSAFE
-  }
+  void lock(void) { this->mutex.lock(); }
+  void unlock(void) { this->mutex.unlock(); }
 private:
-#ifdef COIN_THREADSAFE
   SbMutex mutex;
-#endif // COIN_THREADSAFE
+#else // !COIN_THREADSAFE
+  void lock(void) { }
+  void unlock(void) { }
+#endif // !COIN_THREADSAFE
 };
 
 #define PRIVATE(obj) (obj->pimpl)
+#define PUBLIC(obj) (obj->master)
+
+// *************************************************************************
 
 SO_NODE_SOURCE(SoVRMLText);
 
+// *************************************************************************
+
 // Doc in parent
 void
 SoVRMLText::initClass(void)
@@ -245,7 +245,7 @@ float
 SoVRMLTextP::getWidth(const int idx, const float fontsize)
 {
   float w = this->glyphwidths[idx];
-  float maxe = this->master->maxExtent.getValue();
+  float maxe = PUBLIC(this)->maxExtent.getValue();
   if (maxe > 0.0f && w > maxe) w = maxe;
   return w;
 }
@@ -348,10 +348,17 @@ SoVRMLText::GLRender(SoGLRenderAction * action)
       }
 
       switch (PRIVATE(this)->justificationminor) {
+        // FIXME: shouldn't the calculations below take the
+        // FontStyle::topToBottom field into account? 20051207 mortene.
+
       case SoAsciiText::LEFT:
         break;
       case SoAsciiText::CENTER:
-        ypos = (i * spacing) + ((n-1) * spacing) * 0.5f;
+        {
+          const float VERTICALSIZE = spacing * n;
+          const float FIRSTLINEPOS = -(VERTICALSIZE / 2);
+          ypos = FIRSTLINEPOS + (i * spacing);
+        }
         break;
       case SoAsciiText::RIGHT:
         ypos = (i * spacing) + ((n-1) * spacing);
@@ -954,48 +961,48 @@ SoVRMLText::generatePrimitives(SoAction * action)
   PRIVATE(this)->unlock();
 }
 
-#undef PRIVATE
-
 void
 SoVRMLTextP::updateFontStyle(void)
 {
-  SoVRMLFontStyle * fs = (SoVRMLFontStyle*) this->master->fontStyle.getValue();
-  if (!fs) {
-    this->textsize = 1.0f;
-    this->textspacing = 1.0f;
-    this->lefttorighttext = TRUE;
-    this->toptobottomtext = TRUE;
-    this->horizontaltext = TRUE;
-    this->justificationmajor = SoAsciiText::LEFT;
-    this->justificationminor = SoAsciiText::LEFT;
-    this->fontfamily = SoVRMLFontStyle::SERIF;
-    this->fontstyle = SoVRMLFontStyle::PLAIN;
-    return;
-  }
+  // the defaults
+  this->textsize = 1.0f;
+  this->textspacing = 1.0f;
+  this->lefttorighttext = TRUE;
+  this->toptobottomtext = TRUE;
+  this->horizontaltext = TRUE;
+  this->justificationmajor = SoAsciiText::LEFT;
+  this->justificationminor = SoAsciiText::LEFT;
+  this->fontfamily = SoVRMLFontStyle::SERIF;
+  this->fontstyle = SoVRMLFontStyle::PLAIN;
 
-  // Major mode
-  if (!strcmp(fs->justify[0].getString(),"BEGIN") || 
-      !strcmp(fs->justify[0].getString(),"FIRST") ||
-      (fs->justify[0].getLength() == 0)) {
-    this->justificationmajor = SoAsciiText::LEFT;  
-  } 
-  else if (!strcmp(fs->justify[0].getString(),"MIDDLE")) {
-    this->justificationmajor = SoAsciiText::CENTER;
-  } 
-  else if (!strcmp(fs->justify[0].getString(),"END")) {
-    this->justificationmajor = SoAsciiText::RIGHT;
+  SoVRMLFontStyle * fs = (SoVRMLFontStyle *)PUBLIC(this)->fontStyle.getValue();
+  if (!fs) { return; }
+
+  if (fs->justify.getNum() > 0) {
+    // Major mode
+    SbString s = fs->justify[0];
+    if ((s == "BEGIN") || (s == "FIRST") || (s == "")) {
+      this->justificationmajor = SoAsciiText::LEFT;  
+    } 
+    else if (s == "MIDDLE") {
+      this->justificationmajor = SoAsciiText::CENTER;
+    } 
+    else if (s == "END") {
+      this->justificationmajor = SoAsciiText::RIGHT;
+    }
+    // FIXME: else... Improve robustness & error reporting. 20051207 mortene.
   }
     
-  // Minor mode
   if (fs->justify.getNum() > 1) {
-    if (!strcmp(fs->justify[1].getString(),"BEGIN") || 
-        !strcmp(fs->justify[1].getString(),"FIRST") ||
-        (fs->justify[1].getLength() == 0))
+    // Minor mode
+    SbString s = fs->justify[1];
+    if ((s == "BEGIN") || (s == "FIRST") || (s == ""))
       this->justificationminor = SoAsciiText::LEFT;  
-    else if (!strcmp(fs->justify[1].getString(),"MIDDLE")) 
+    else if (s == "MIDDLE")
       this->justificationminor = SoAsciiText::CENTER;
-    else if (!strcmp(fs->justify[1].getString(),"END")) 
+    else if (s == "END")
       this->justificationminor = SoAsciiText::RIGHT;
+    // FIXME: else... Improve robustness & error reporting. 20051207 mortene.
   }
   
   this->lefttorighttext = fs->leftToRight.getValue();
@@ -1116,4 +1123,7 @@ SoVRMLTextP::setUpGlyphs(SoState * state, SoVRMLText * textnode)
   if (oldcache) oldcache->unref();
 }
 
+#undef PRIVATE
+#undef PUBLIC
+
 #endif // HAVE_VRML97
