@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -85,6 +85,18 @@
 #include <Inventor/nodes/SoSubNodeP.h>
 #include <Inventor/actions/SoGLRenderAction.h>
 #include <Inventor/elements/SoGLTextureCoordinateElement.h>
+#include <Inventor/elements/SoGLVBOElement.h>
+#include "../misc/SoVBO.h"
+#include "../misc/PointerMap.h"
+
+class SoVRMLTextureCoordinateP {
+ public:
+  SoVRMLTextureCoordinateP() : vbo(NULL) { }
+  ~SoVRMLTextureCoordinateP() { delete this->vbo; }
+  SoVBO * vbo;
+};
+
+#define PRIVATE(obj) ((SoVRMLTextureCoordinateP*) PointerMap::get(obj))
 
 SO_NODE_SOURCE(SoVRMLTextureCoordinate);
 
@@ -100,6 +112,7 @@ SoVRMLTextureCoordinate::initClass(void)
 */
 SoVRMLTextureCoordinate::SoVRMLTextureCoordinate(void)
 {
+  PointerMap::add(this, new SoVRMLTextureCoordinateP);
   SO_VRMLNODE_INTERNAL_CONSTRUCTOR(SoVRMLTextureCoordinate);
 
   SO_VRMLNODE_ADD_EMPTY_EXPOSED_MFIELD(point);
@@ -110,6 +123,9 @@ SoVRMLTextureCoordinate::SoVRMLTextureCoordinate(void)
 */
 SoVRMLTextureCoordinate::~SoVRMLTextureCoordinate()
 {
+  SoVRMLTextureCoordinateP * pimpl = PRIVATE(this);
+  PointerMap::remove(this);
+  delete pimpl;  
 }
 
 // Doc in parent
@@ -132,9 +148,42 @@ SoVRMLTextureCoordinate::callback(SoCallbackAction * action)
 void
 SoVRMLTextureCoordinate::GLRender(SoGLRenderAction * action)
 {
-  SoGLTextureCoordinateElement::setTexGen(action->getState(),
+  SoState * state = action->getState();
+  SoGLTextureCoordinateElement::setTexGen(state,
                                           this, NULL);
   SoVRMLTextureCoordinate::doAction((SoAction*)action);
+
+#ifdef COIN_NEXT_MINOR
+  SoVRMLTextureCoordinateP * pimpl = PRIVATE(this);
+
+  SoBase::staticDataLock();
+  const int num = this->point.getNum();
+  SbBool setvbo = FALSE;
+  if (SoGLVBOElement::shouldCreateVBO(state, num)) {
+    setvbo = TRUE;
+    SbBool dirty = FALSE;
+    if (pimpl->vbo == NULL) {
+      pimpl->vbo = new SoVBO(GL_ARRAY_BUFFER, GL_STATIC_DRAW); 
+      dirty =  TRUE;
+    }
+    else if (pimpl->vbo->getBufferDataId() != this->getNodeId()) {
+      dirty = TRUE;
+    }
+    if (dirty) {
+      pimpl->vbo->setBufferData(this->point.getValues(0),
+                                        num*sizeof(SbVec2f),
+                                        this->getNodeId());
+    }
+  }
+  else if (pimpl->vbo && pimpl->vbo->getBufferDataId()) {
+    // clear buffers to deallocate VBO memory
+    pimpl->vbo->setBufferData(NULL, 0, 0);
+  }
+  SoBase::staticDataUnlock();
+  if (setvbo) {
+    SoGLVBOElement::setTexCoordVBO(state, 0, pimpl->vbo);
+  }
+#endif // COIN_NEXT_MINOR
 }
 
 // Doc in parent
@@ -151,4 +200,5 @@ SoVRMLTextureCoordinate::getPrimitiveCount(SoGetPrimitiveCountAction * action)
   SoVRMLTextureCoordinate::doAction((SoAction*)action);
 }
 
+#undef PRIVATE
 #endif // HAVE_VRML97
