@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -82,7 +82,7 @@
   The e\ loop, \e startTime, and \e stopTime exposedFields and the
   isActive eventOut and their effects on the TimeSensor node are discussed
   in detail in 4.6.9, Time-dependent nodes
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.9>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.9>).
   The "cycle" of a TimeSensor node lasts for cycleInterval
   seconds. The value of cycleInterval shall be greater than zero.  
 
@@ -330,7 +330,9 @@ SoVRMLTimeSensor::write(SoWriteAction * action)
 
   // Re-connect to realTime field.
   if (connectfromrealTime) {
-    this->timeIn.connectFrom(connectfield);
+    // Don't send notification when reconnecting to preserve the state
+    // of the scenegraph between write passes.
+    this->timeIn.connectFrom(connectfield, TRUE);
     this->timeIn.setDefault(defaultflag);
   }
 }
