@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -51,7 +51,7 @@
   coordinate system from which the user may view the scene. Viewpoint
   nodes are bindable children nodes (see 4.6.10, Bindable children
   nodes:
-  http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.10)
+  http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.10)
   and thus there exists a Viewpoint node stack in the browser in which
   the top-most Viewpoint node on the stack is the currently active
   Viewpoint node.  If a TRUE value is sent to the set_bind eventIn of
@@ -67,7 +67,7 @@
   viewpoint stack, the user's view is re-parented to the next entry in
   the stack. More details on binding stacks can be found in 4.6.10,
   Bindable children nodes
-  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.10>).
+  (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.10>).
   When a Viewpoint node is moved to the top
   of the stack, the existing top of stack Viewpoint node sends an
   isBound FALSE event and is pushed down the stack.
@@ -123,7 +123,7 @@
   binding stack from a Viewpoint node with jump TRUE). The following
   summarizes the bind stack rules (see 4.6.10, Bindable children
   nodes:
-  http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.6.10)
+  http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.10)
   with additional rules regarding Viewpoint nodes (displayed in
   boldface type):
 
@@ -135,7 +135,7 @@
     strings passed to the Browser.createVrmlFromString() method, or
     within files passed to the Browser.createVrmlFromURL() method (see
     4.12.10, Browser script interface:
-    http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-IS-VRML97WithAmendment1/part1/concepts.html#4.12.10)
+    http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.12.10)
     are not candidates for the first encountered Viewpoint node. The
     first node within a prototype instance is a valid candidate for
     the first encountered Viewpoint node. The first encountered
